
' .ApplicationSource APL-Interface(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 August 1993 at 3:25:25 pm'!



'- comment for APL-Interface(1.0) -'!

(Message selector: #documentation: arguments: ((Array new: 1) at: 1 put: (Text string: 'This application defines the structure and functionality of the user interface of Application Management.' runs: (RunArray runs: #(82 23 ) values: #(nil #italic ))); yourself))!

'- installation of APL-Interface(1.0) -'!

"APL-Interface enhances some menus of system browsers; so flush the old menus"

Browser flushMenus!

'- required applications for APL-Interface(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('APL-ChangeSequence' 'APL-Framework' ) ))!

'- sources of APL-Interface(1.0) -'!

TextView subclass: #ApplicationTextView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Application'!


!ApplicationTextView methodsFor: 'model access'!

newText: aText
	"enhance default by updating info view"

	super newText: aText.
	model changed: #info! !



!ApplicationManagement methodsFor: 'browsing'!

selectAndUpgradeApplication
	"select an application which has new sources and upgrade it"

	| app |
	app := self upgradeApplications asSortedCollection: [:x :y | x identifier < y identifier].
	app := app select: 'Select an application to upgrade' fromUserIfEmpty: [^nil].
	self upgradeMinor: app!

selectApplication: prompt
	"select an application from the registered ones"

	| id |
	id := self sortedApplicationIdentifiers select: prompt fromUserIfEmpty: [^nil].
	^self applicationIdentified: id!

selectApplicationOrNew: prompt
	"select a registered application or create a new; prompt for 
	initialName if new. Return application or nil if none selected"

	| new list selectedName |
	(list := self sortedApplicationIdentifiers) addFirst: (new := 'NEW APPLICATION').
	^(selectedName := list select: prompt fromUserIfEmpty: [^nil]) == new
		ifTrue: [self newApplication]
		ifFalse: [self applicationIdentified: selectedName]! !



StandardSystemController subclass: #ApplicationController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Application'!
ApplicationController comment:
'ApplicationController installs an exception handler for ApplicationManagement changeSignal
around the control loop, returning the application of the model if the signal is raised.'!


!ApplicationController methodsFor: 'basic control sequence'!

startUp
	"put a handle block around the startUp routine to catch changes for 
	application management"

	ApplicationManagement changeSignal 
		handle: [:ex | ex proceedWith: self model application] 
		do: [super startUp]! !

!ApplicationController methodsFor: 'menu messages'!

close
	locked ifTrue: [^self].
	model releaseExternalReferences.
	super close! !



!Browser methodsFor: 'class list'!

classMenu
	"Answer a Menu of operations on classes that is to be displayed 
	when the operate menu button is pressed."
	"Browser flushMenus"

	className == nil ifTrue: [^nil].
	ClassMenu == nil ifTrue:
		[ClassMenu := PopUpMenu
			labels: 'file out as...\hardcopy\spawn\spawn hierarchy
hierarchy\definition\comment
inst var refs...\class var refs...\class refs\applications
move to...\rename as...\remove...' withCRs
			lines: #(4 7 11)
			values: #(#fileOutClass #printOutClass #spawnClass #spawnHierarchy #showHierarchy #editClass #editComment #browseFieldReferences #browseClassVariables #browseClassReferences),
				(Array with: ApplicationBrowser referencesMenu),
				#(#changeClassCategory #renameClass #removeClass )].
	^ClassMenu! !

!Browser methodsFor: 'selector list'!

selectorMenu
	"Answer a Menu of operations on message selectors to be 
	displayed when the operate menu button is pressed.
	Enhanced by Application Management cross reference features"

	"Browser flushMenus"
	selector == nil ifTrue: [^ nil].
	MessageMenu == nil ifTrue:
		[MessageMenu := PopUpMenu
			labels: 'file out as...\hardcopy\spawn\senders\implementors\messages...\applications\move to...\remove...' withCRs
			lines: #(3 7)
			values: #(fileOutMessage printOutMessage spawnMethod browseSenders browseImplementors browseMessages ),
				(Array with: ApplicationBrowser selectorRefsMenu),
				#(moveMethod removeMethod)].
	^ MessageMenu! !

!Browser methodsFor: 'private-class functions'!

showClassIncluders
	"show a list of applications which registered selectors of the selected class;
	if the user selects one of them open a browser on it"

	| app |
	app := (self applicationManagement applicationsIncludingClass: self selectedClass)
		select: (ApplicationBrowser promptFor: #selectThenBrowse)
		fromUserIfEmpty: [^self].
	app browseInitial: self selectedClass selector: nil!

showClassOwners
	"show a list of applications having the definition of this class;
	if the user selects one of them open a browser on it"

	| app |
	app := (self applicationManagement applicationsIncludingDefinition: self selectedClass)
		select: (ApplicationBrowser promptFor: #selectThenBrowse)
		fromUserIfEmpty: [^self].
	app browseInitial: self selectedClass selector: nil!

showClassUsers
	"show a list with all applications referring to the selected class;
	if the user selects an item, we proceed as if 'class refs' has been chosen
	(note the restriction on elements of the chosen application)"

	| calls app |
	calls := self applicationManagement applicationsCallingSelector: nil class: self selectedClass without: nil.
	app := calls keys select: (ApplicationBrowser promptFor: #selectThenBrowse) fromUserIfEmpty: [^self].
	calls := (calls at: app) collect: [:assoc | assoc key , ' ' , assoc value].

	MethodListBrowser
		openListBrowserOn: calls asSortedCollection
		label: 'Calls on ', self selectedClass asSymbol, ' in ', app printString
		initialSelection: self selectedClass asSymbol! !

!Browser methodsFor: 'private-selector functions'!

showMethodOwners
	"show a list of all applications having this method;
	if the user selects one open a browser on it"

	| app |
	app := self applicationManagement applicationsIncludingSelector: self selector asSymbol class: self selectedClass.
	app := app select: (ApplicationBrowser promptFor: #selectThenBrowse) fromUserIfEmpty: [^self].
	app browseInitial: self selectedClass selector: self selector asSymbol!

showMethodUsers
	"show a list of all applications calling this method;
	if the user selects an item, we proceed as if 'senders' has been chosen
	(note the restriction on elements of the chosen application)"

	| calls app |
	calls := self applicationManagement applicationsCallingSelector: self selector asSymbol class: self selectedClass without: nil.
	app := calls keys select: (ApplicationBrowser promptFor: #selectThenBrowse) fromUserIfEmpty: [^self].
	calls := (calls at: app) collect: [:assoc | assoc key , ' ' , assoc value].

	MethodListBrowser
		openListBrowserOn: calls asSortedCollection
		label: 'Calls on ', self selector asSymbol, ' in ', app printString
		initialSelection: self selector asSymbol! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!Browser class methodsFor: 'controller accessing'!

defaultControllerClass
	"use the default of ScheduledWindow"

	^StandardSystemController! !

!Browser class methodsFor: 'view creation'!

openCategoryBrowserOn: aBrowser withTextState: anArray
	"Create and schedule a browser on the category of classes selected by
	aBrowser.  anArray holds the initial text state;
	modified for the needs of application management (different controller)"

	| topView lineHeightBlock buttonOffset topWindow |
	lineHeightBlock := self lineHeightBlock.
	buttonOffset := self buttonOffset.
	topWindow := ScheduledWindow
			model: aBrowser
			label: (aBrowser category, ' Category Browser')
			minimumSize: 400@250.
	topView := DependentComposite new.
	self
		addCategoryViewTo: topView
			in: (LayoutFrame new rightFraction: 0.3; bottomOffset: lineHeightBlock)
			on: aBrowser readOnly: true;
		addClassViewTo: topView
			in: (LayoutFrame new
				topOffset: lineHeightBlock;
				rightFraction: 0.3;
				bottomFraction: 0.4 offset: buttonOffset)
			on: aBrowser readOnly: false;
		addClassAndInstanceButtonViewTo: topView
			in: (LayoutFrame new
				topFraction: 0.4 offset: buttonOffset;
				rightFraction: 0.3;
				bottomFraction: 0.4)
			on: aBrowser;
		addProtocolViewTo: topView in: (0.3@0 extent: 0.3@0.4) on: aBrowser readOnly: false;
		addSelectorViewTo: topView in: (0.6@0 extent: 0.4@0.4) on: aBrowser readOnly: false;
		addTextViewTo: topView in: (0@0.4 corner: 1 asPoint) on: aBrowser initialSelection: nil initialState: anArray.

	topWindow
		controller: aBrowser class defaultControllerClass new;
		component: topView;
		icon: (Icon constantNamed: #categoryBrowser);
		openWithExtent: 400@300!

openClassBrowserOn: aBrowser withTextState: anArray
	"Create and schedule a browser on the class currently selected by aBrowser.
	 anArray holds the initial text state;
	modified for the needs of application management (different controller)"

	| topView topWindow lineHeightBlock buttonBottom |
	lineHeightBlock := self lineHeightBlock.
	buttonBottom := [lineHeightBlock value + LabeledBooleanView defaultHeight].
	topWindow := ScheduledWindow model: aBrowser 
						label: aBrowser nonMetaClass name,' Class Browser' 
						minimumSize: 400@250.
	topView := DependentComposite new.
	self
		addClassViewTo: topView
			in: (LayoutFrame new rightFraction: 0.50; bottomOffset: lineHeightBlock)
			on: aBrowser readOnly: true;
		addClassAndInstanceButtonViewTo: topView
			in: (LayoutFrame new
				topOffset: lineHeightBlock;
				rightFraction: 0.5;
				bottomOffset: buttonBottom)
			on: aBrowser;
		addProtocolViewTo: topView
			in: (LayoutFrame new topOffset: buttonBottom; rightFraction: 0.5; bottomFraction: 0.4)
			on: aBrowser readOnly: false;
		addSelectorViewTo: topView
			in: (LayoutFrame new leftFraction: 0.5; rightFraction: 1; bottomFraction: 0.4)
			on: aBrowser readOnly: false;
		addTextViewTo: topView
			in: (0@0.4 corner: 1 asPoint)
			on: aBrowser initialSelection: nil initialState: anArray.

	topWindow
		controller: aBrowser class defaultControllerClass new;
		component: topView;
		icon: (Icon constantNamed: #classBrowser);
		openWithExtent: ((topWindow  minimumSize * 3 + topWindow maximumSize) // 4)!

openMethodBrowserOn: aBrowser withTextState: anArray
	"Create and schedule a browser on the method currently selected by
	aBrowser.  Set the topview's controller to a be aController.  The
	method may contain changes that haven't already been accepted;
	modified for the needs of application management (different controller)"

	| topView topWindow lineHeightBlock |
	lineHeightBlock := self lineHeightBlock.
	topWindow := ScheduledWindow
				model: aBrowser
				label: 'Method Browser on ' , aBrowser selectedClass name
				minimumSize: 250 @ 120.
	topView := DependentComposite new.
	self
		addSelectorViewTo: topView
			in: (LayoutFrame new
				rightFraction: 1;
				bottomOffset: lineHeightBlock)
			on: aBrowser
			readOnly: true;
		addTextViewTo: topView
			in: (LayoutFrame new
				topOffset: lineHeightBlock;
				rightFraction: 1;
				bottomFraction: 1)
			on: aBrowser
			initialSelection: nil
			initialState: anArray.

	topWindow
		controller: aBrowser class defaultControllerClass new;
		component: topView;
		icon: (Icon constantNamed: #methodBrowser);
		openWithExtent: 400@200!

openOn: aBrowser withTextState: anArray
	"Create and schedule a browser on an entire collection of organized 
	classes.   anArray holds the initial text state;
	modified for the needs of application management (different controller)"

	| topView divider topWindow buttonOffset |
	buttonOffset := self buttonOffset.
	topWindow := ScheduledWindow model: aBrowser label: 'System Browser' 
					minimumSize: 400@250.
	divider := 0.35.
	topView := DependentComposite new.
	self
		addCategoryViewTo: topView in: (0@0 extent: 0.25@divider) on: aBrowser readOnly: false;
		addClassViewTo: topView
			in: (LayoutFrame new 
				leftFraction: 0.25;
				topOffset: 0;
				rightFraction: 0.50;
				bottomFraction: divider offset: buttonOffset) on: aBrowser readOnly: false;
		addClassAndInstanceButtonViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.25;
				topFraction: divider offset:  buttonOffset;
				rightFraction: 0.5;
				bottomFraction: divider)
			on: aBrowser;
		addProtocolViewTo: topView in: (0.5@0 extent: 0.25@divider) on: aBrowser readOnly: false;
		addSelectorViewTo: topView in: (0.75@0 extent: 0.25@divider) on: aBrowser readOnly: false;
		addTextViewTo: topView in: (0@divider corner: 1.0@1.0) on: aBrowser initialSelection: nil initialState: anArray.

	topWindow
		controller: aBrowser class defaultControllerClass new;
		component: topView;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: ((topWindow  minimumSize * 3 + topWindow maximumSize) // 4)!

openProtocolBrowserOn: aBrowser withTextState: anArray
	"Create and schedule a browser on the category of messages selected by
	aBrowser.  anArray holds the initial text state;
	modified for the needs of application management (different controller)"

	| topView lineHeightBlock topWindow |
	lineHeightBlock := self lineHeightBlock.
	topWindow := ScheduledWindow
				model: aBrowser
				label: aBrowser selectedClass name, ' Protocol Browser'
				minimumSize: 400@200.
	topView := DependentComposite new.
	self
		addProtocolViewTo: topView
			in: (LayoutFrame new
				rightFraction: 1;
				bottomOffset: lineHeightBlock) on: aBrowser readOnly: true;
		addSelectorViewTo: topView
			in: (LayoutFrame new
				rightFraction: 1;
				topOffset: lineHeightBlock;
				bottomFraction: 0.4) on: aBrowser readOnly: false;
		addTextViewTo: topView in: (0@0.4 corner: 1.0@1.0) on: aBrowser initialSelection: nil initialState: anArray.

	topWindow
		controller: aBrowser class defaultControllerClass new;
		component: topView;
		icon: (Icon constantNamed: #protocolBrowser);
		open! !



Browser subclass: #ApplicationBrowser
	instanceVariableNames: 'application documentationMode systemMode inheritUpToClass '
	classVariableNames: 'Menus Prompts '
	poolDictionaries: ''
	category: 'Interface-Application'!
ApplicationBrowser comment:
'ApplicationBrowser extends system browsers with functionality dealing with applications. New menus introduce operator selectors for

	- manual construction and maintenance of applications (include/exclude)
	- editing applications and their components (e.g. changes)
	- enhanced cross reference utilities (e.g. method senders restricted to an application)
	- major or minor version release

The model of the browser is an application. Lists contents and operations are restricted to the information (i.e. classes and selectors) registered by the model (what you see is what you get). This restriction may be disabled by switching the ''system'' button on. Each user action (especially ''accept'') will be recorded by the model.

Furthermore application browsers allow to extend selector list contents. Select an inheritance level and all methods implemented by the selected class up to the inheritance level will be shown, thus giving easy overview about inherited behavior.

For more information see the Application Management manual.


Instance Variables:
	application	<Application>  currently edited application
	documentationMode	<Boolean>  mode which accepts text as documentation rather than code
	systemMode	<Boolean>  disables the filter defined by application membership
	inheritUpToClass	<Symbol | nil>  name of superclass (display inherited selectors) 

Class Variables:
	Menus	<Dictionary key: Symbol value: PopUpMenu>  cache of menus for each subview 
	Prompts	<Dictionary key: Symbol value: String>  prompts used in dialogs
'!


!ApplicationBrowser methodsFor: 'protocol list'!

protocol
	"Answer the receiver's currently selected message category."

	^ protocol notNil ifTrue: [protocol asText]!

protocol: selection 
	super protocol: (selection isNil ifFalse: [selection asSymbol])!

protocolList
	"list of protocols in selected class and the superclasses as specified in 
	inheritance pane (alphabetical order)"

	| set |
	className isNil ifTrue: [^nil].
	set := self collectListsWithSuperclasses: [:c | self systemMode 
			ifTrue: [c asClass organization categories]
			ifFalse: [(self application classSubsetOf: c ifAbsent: [ClassSubset new]) categories]].
	set := self emphasizeList: set criterium: [:p | self application includesProtocol: p class: self selectedClass].
	^set asSortedCollection!

protocolMenu
	^protocol == nil
		ifTrue: [super protocolMenu]
		ifFalse: [self class protocolMenu]! !

!ApplicationBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a minor version release via application management"

	anAspect == #version
		ifTrue: 
			[self changed: #text.
			self changed: #windowLabel with: self application identifier.
			^self applicationManagement changed: anAspect].
	super update: anAspect! !

!ApplicationBrowser methodsFor: 'selector list'!

selector
	^selector == nil ifFalse: [selector asText]!

selector: selection 
	super selector: (selection notNil ifTrue: [selection asSymbol])!

selectorList
	| list |
	protocol == nil ifTrue: [^nil].
	list := (self collectListsWithSuperclasses: (self systemMode
					ifTrue: [[:c | c asClass organization listAtCategoryNamed: self protocol asSymbol]]
					ifFalse: [[:c | self application selectorsInProtocol: self protocol asSymbol class: c]])) asSortedCollection.
	^self emphasizeList: list criterium: [:sel | self application includesSelector: sel class: self selectedClass name]!

selectorMenu
	^selector == nil ifFalse: [self class selectorMenu]! !

!ApplicationBrowser methodsFor: 'text'!

acceptDocumentation: aText from: aController 
	self application documentation: aText.
	^true!

acceptMethod: aText from: aController 
	^(super acceptMethod: aText from: aController)
		and: 
			[self newProtocolList: protocol.
			true]!

text
	"it is possible to add inherited selectors to the selector list, 
	so the selected class may be different from the implementing class;
	if there is no text, return a short info about the application"

	| text |
	self documentationMode
		ifTrue: [^self application documentation asText].
	textMode == #methodDefinition & selector notNil
		ifTrue: [^self classOfSelectedMethod sourceMethodAt: selector].
	^(text := super text) isEmpty
		ifTrue: [self applicationInformation]
		ifFalse: [text]!

textMode: aSymbol 
	super textMode: aSymbol.
	self changed: #info.
	self changed: #inheritedFromClass! !

!ApplicationBrowser methodsFor: 'initialize-release'!

on: anApplication 
	super on: Smalltalk organization.
	application := anApplication.
	systemMode := false.
	documentationMode := false.
	anApplication addDependent: self!

release
	self releaseExternalReferences.
	super release!

releaseExternalReferences
	self application removeDependent: self; releaseExternalReferences! !

!ApplicationBrowser methodsFor: 'private-selector functions'!

browseMessages
	Browser showMenuThenBrowse:
		(self classOfSelectedMethod compiledMethodAt: selector)
			allSymbolLiterals asSortedCollection!

excludeSelector
	self application
		excludeSelector: self selector asSymbol
		class: self classOfSelectedMethod;
		cleanUpProtocols.
	self newProtocolList: protocol!

fileOutMessage
	"class is determined by 'self classOfSelectedMethod' instead of 'self selectedClass' "
	| stream sel implementor |
	selector isKeyword
		ifTrue: 
			[sel := WriteStream on: (String new: 20).
			selector keywords do: [:each | sel nextPutAll: (each copyFrom: 1 to: each size - 1)].
			sel := sel contents]
		ifFalse: [sel := selector].
	implementor := self classOfSelectedMethod.
	stream := self
		requestFileName: 'File out selector on:'
		default: (implementor name, '-', sel, '.st')
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.
	
	[stream timeStamp; fileOutMessages: (Array with: selector) for: implementor]
		valueNowOrOnUnwindDo: [stream close]!

includeSelector
	self application
		includeSelector: self selector asSymbol
		protocol: self protocol asSymbol
		class: self classOfSelectedMethod.
	self newProtocolList: protocol!

moveMethod
	| otherProtocols newProtocol newString |
	otherProtocols := super protocolList asSortedCollection asOrderedCollection.
	otherProtocols isEmpty
		ifTrue: [super moveMethod]
		ifFalse: 
			[otherProtocols addLast: (newString := 'new protocol').
			newProtocol := otherProtocols selectFromUserIfEmpty: [^nil].
			newProtocol == newString
				ifTrue: [super moveMethod]
				ifFalse: [ | class |
					(class := self classOfSelectedMethod) organization classify: selector under: newProtocol.
					self logProtocolChange: class name , ' organization classify: ' , selector storeString , ' under: ' , newProtocol storeString.
					class reorganize.
					self newSelectorList: selector]]!

printOutMessage
	"Print the method."
	
	| stream |
	stream := TextStream on: (String new: 1000).
	self classOfSelectedMethod printMethod: selector on: stream.
	self hardcopyStream: stream!

removeMethod
	| c |
	c := self classOfSelectedMethod.
	(self changeRequest and: [DialogView confirm: 'Are you sure that you want to remove
this method in class ' , c name , ' ?'])
		ifTrue: 
			[c removeSelector: selector.
			self application removeSelector: selector class: c.
			self newSelectorList: nil.
			^true].
	^false!

showMethodOwners
	"show a list of all applications having this method; 
	if the user selects one open a browser on it"

	| app sel cls |
	sel := self selector asSymbol.
	cls := self classOfSelectedMethod.
	app := self applicationManagement applicationsIncludingSelector: sel class: cls.
	app := app select: (self class promptFor: #selectThenBrowse)
				fromUserIfEmpty: [^self].
	app browseInitial: cls selector: sel!

showMethodUsers
	"show a List with all applications calling this method;
	if the user selects an item, we proceed as if 'senders' has been chosen
	(note the restriction on elements of the chosen application)"

	| calls app |
	calls := self applicationManagement applicationsCallingSelector: self selector asSymbol class: self classOfSelectedMethod without: nil.
	app := calls keys select: (self class promptFor: #selectThenBrowse) fromUserIfEmpty: [^self].
	calls := (calls at: app) collect: [:assoc | assoc key , ' ' , assoc value].

	MethodListBrowser
		openListBrowserOn: calls asSortedCollection
		label: 'Calls on ', self selector asSymbol, ' in ', app printString
		initialSelection: self selector asSymbol!

spawnFullHistory
	"open a change sequence browser on all saved and temporary changes, 
	filtering on the selected method"

	self
		spawnChanges: self application fullHistory
		label: 'Full history from ' , self application identifier
		select: self selector notNil!

spawnMethodHierarchy
	"open an implementors browser, restricted to superclasses of the currently 
	selected class"

	| list |
	list := self superClassList select: [:c | c asClass includesSelector: self selector asSymbol].
	list size == 1
		ifTrue: [^DialogView userInfo: 'No superclasses implementing this selector'].

	MethodListBrowser
		openListBrowserOn: (list collect: [:e | e , ' ' , self selector])
		label: self className , ': Hierarchy of ' , self selector asSymbol!

spawnSelectorChanges
	"open a change sequence browser on all temporary changes, 
	filtering on the selected method"

	self
		spawnChanges: self application changes
		label: 'Changes from ' , self application identifier
		select: true! !

!ApplicationBrowser methodsFor: 'private-application functions'!

addRequiredApplication
	"prompt the user for an application which should be regarded as necessary for 
	running the underlying application and add it to the set of required applications"

	| app |
	(app := self applicationManagement selectApplication: 'Select a necessary application') == nil ifTrue: [^self].
	self application includeApplication: app!

fileIn
	"prompt for a name of a source code file, file in the sources, and log them all to 
	the application of this browser"

	| filename |
	(filename := DialogView
				requestFileName: 'Enter the name of the file to scan'
				default: ParagraphEditor currentSelection
				version: #mustBeOld) isEmpty ifTrue: [^self].
	self applicationManagement
		fileInFrom: filename asFilename readStream
		to: self application
		external: false!

fileOutApplication
	"file out the current state of all sources of the application;
	format: <time stamp> <pre installation> <source code> <post installation>"

	| stream |
	stream := self
		requestFileName: 'File out on'
		default: self application fileString , '.st'
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.

	Cursor write showWhile:
	[[self application
		fileOutPreInstallationOn: stream timeStamp;
		fileOutSourcesOn: stream;
		fileOutPostInstallationOn: stream]
		valueNowOrOnUnwindDo: [stream close]]!

fileOutChanges
	self fileOutChanges: #changes!

fileOutChanges: mode 
	"mode is one of #changes, #history, #fullHistory; 
	file out the corresponding set"

	| list fileName fileStream |
	(list := self application perform: mode) isEmpty
		ifTrue: [DialogView userInfo: 'Nothing to file out'. ^self].
	fileName := self
				requestFileName: 'File out ', mode, ' on'
				default: self application fileString , '.cs'
				ifQuit: [^nil].
	fileStream := fileName asFilename writeStream.
	
	Cursor write showWhile:
		[[list fileOutOn: fileStream timeStamp] valueNowOrOnUnwindDo: [fileStream close]]!

findRequiredApplications
	self application findRequiredApplications!

inspectApplication
	self application inspect!

newMajorVersion
	"create a new major version"

	(DialogView confirm: 'Do you really want to create a new version of
' , self application identifier , '?')
		ifFalse: [^self].

	"change the dependency entry according to the new version"
	self releaseExternalReferences.
	self on: self application newMajorVersion.

	"update window labels and application management ids"
	self update: #version!

newMinorVersion
	"append changes to source file, clear changes and increment minor version"

	self application changes isEmpty
		ifTrue: [(DialogView confirm: 'There are no changes.
Do you want a new minor version anyway?' initialAnswer: false)
			ifFalse: [^self]].

	self application newMinorVersion!

printOutApplication
	| stream |
	stream := TextStream on: (String new: 1000).
	self application printOutSourcesOn: stream.
	self hardcopyStream: stream!

setNameForDos
	self application setNameForDos!

setPath
	self application setPath!

spawnApplication
	self application
		browseInitial: self selectedClass
		selector: (self selector notNil ifTrue: [self selector asSymbol])!

spawnChanges
	"open an editor on the application's temporary changes"

	self
		spawnChanges: self application changes
		label: 'Changes from ' , self application identifier
		select: false!

spawnHistory
	"open a change sequence browser on all saved changes"

	self
		spawnChanges: self application history
		label: 'History from ' , self application identifier
		select: self selector notNil!

spawnInstallation
	"open an editor on the application's installation doits;
	label text must be in sync with EnhancedChangeList modes"

	self
		spawnChanges: self application preInstallation
		label: 'Pre installation doits for ' , self application identifier
		select: false!

spawnPostInstallation
	"open an editor on the application's installation doits;
	label text must be in sync with EnhancedChangeList modes"

	self
		spawnChanges: self application postInstallation
		label: 'Post installation doits for ' , self application identifier
		select: false!

spawnRequired
	"open an inspector on the set of required applications"

	self application necessary inspect!

spawnStatistics
	"open a workspace with statistical informations about the application"

	| s a |
	(s := TextStream on: (String new: 500))
		nextPutAll: 'classes:'; tab: 2; nextPutAll: (a := self application) classNames size printString; cr;
		nextPutAll: 'selectors:'; tab: 1; nextPutAll: a selectors size printString; cr.

	a path notNil ifTrue:
		[OSErrorHolder nonexistentSignal
			handle: [:ex | ex proceedWith: 0]
			do: [s
				nextPutAll: 'sources:'; tab: 2; nextPutAll: a instSourceFilename fileSize printString; cr;
				nextPutAll: 'binary:'; tab: 2; nextPutAll: a instBossFilename fileSize printString; cr]].

	s := s contents.
	ComposedTextView
		open: (ValueHolder with: s)
		label: 'Statistics on ' , a identifier storeString
		icon: (Icon constantNamed: #workspace)
		extent: s asComposedText extent + 30!

upgradeMinor
	"try to upgrade anApplication to the newest available minor version"

	self applicationManagement upgradeMinor: self application! !

!ApplicationBrowser methodsFor: 'private-source code control'!

withControlstring: code
	"add a source code control string to method code"

	| stream |
	stream := WriteStream on: (String new: 1000).
	stream nextPutAll: code; cr.
	stream nextPut: $".
	stream nextPutAll: '.SCC: '.
	stream nextPutAll: Date today printString , ', ', Time now "24hours" printString.
	(Smalltalk includesKey: #UserRegistry) ifTrue: [stream nextPut: $;; nextPut: $ ; nextPutAll: (Smalltalk at: #UserRegistry) username].
	stream nextPut: $".
	stream cr.
	^stream contents! !

!ApplicationBrowser methodsFor: 'doIt/accept/explain'!

acceptText: aText from: aController 
	self documentationMode
		ifTrue: [^self acceptDocumentation: aText from: aController].
	^super acceptText: aText from: aController!

explain: fullText fromController: aController 
	"copied from superclass, but passing 'selector' instead of 'self selector'
	(the latter returns a Text; construction of an Explainer expects a Symbol)"

	| reply |
	reply := (Explainer new
		class: self classOfSelectedMethod
		selector: selector
		instance: self doItReceiver
		context: nil
		methodText: self text) explain: aController selection string for: self.
	aController insertAndSelect: (' ' , reply) at: aController selectionStopIndex! !

!ApplicationBrowser methodsFor: 'accessing'!

application
	^application!

applicationInformation
	"return a text with application name, version, required applications"

	| info |
	info := TextStream on: (String new: 500).
	info nextPutAll: 'Application '; emphasis: #bold; nextPutAll: self application printString.
	self application necessary isEmpty
		ifFalse:
			[info cr; cr; emphasis: #italic; nextPutAll: 'supporting applications:'; emphasis: nil.
			self application necessary do: [:name | info crtab; nextPutAll: name]].
	^info contents!

info
	"return source file name for selected method or class"

	selector notNil 
		ifTrue: [Dictionary keyNotFoundSignal 
			handle: [:ex | ex returnWith: 0]
			do: [ | method index |
				method := self classOfSelectedMethod compiledMethodAt: self selector asSymbol.
				(index := method fileIndex) isInteger
					ifTrue: [^self sourceFilename: index].
				^index notNil ifTrue: [index sourceFileHolder identifier] ifFalse: [''].
				]].

	className notNil
		ifTrue: [ | cs cls owners |
			textMode == #comment
				ifTrue: [| src | 
					(src := self nonMetaClass organization globalComment) isNil
						ifTrue: [^''].
					^src class == SourceReference
						ifTrue: [src sourceFileHolder identifier]
						ifFalse: [self sourceFilename: src sourceFileNumber]].

			"test if class is defined in an application"
			(cls := self selectedClass) isNil
				ifTrue: [^''].
			(owners := self applicationManagement applicationsIncludingDefinition: cls) isEmpty
				ifFalse: [^owners first printString].

			"test if class is in system changes"
			(((cs := ChangeSet current) isNew: cls)
				or: [cs atClass: cls includes: #change])
				ifTrue: [^SourceFileManager default targetFile name]. 

			"class must be defined by system sources"
			^self sourceFilename: 1].
	^''! !

!ApplicationBrowser methodsFor: 'private-class functions'!

excludeClass
	"exclude all selectors and definition of selected class (class and metaclass)"

	Cursor execute showWhile: 
		[ | c |
		self application excludeClass: (c := self nonMetaClass); excludeClass: c class].
	self newClassList: className!

excludeClassDefinition
	"move selected class' definition to system application"

	| cls |
	self application
		excludeClassDefinition: (cls := self nonMetaClass);
		excludeClassDefinition: cls class.
	self newClassList: className!

fileOutClass
	| stream theClass |
	self systemMode ifTrue: [^super fileOutClass].
	stream := self
		requestFileName: 'File out on'
		default: (theClass := self nonMetaClass) name, '.st'
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.

	[self application fileOutClass: theClass on: stream timeStamp] 

		valueNowOrOnUnwindDo: [stream close]!

findMethodAndSelectAlphabetic
	"enhances default implementation by watching the inheritance switch"

	| sel selectors cls cat grepBlock |
	inheritUpToClass isNil ifTrue: [^super findMethodAndSelectAlphabetic].
	self changeRequest ifFalse: [^self].
	grepBlock := self systemMode
		ifTrue: [ [:c | c asClass organization elements] ]
		ifFalse: [ [:c | self application selectorsInClass: c] ].
	selectors := (self collectListsWithSuperclasses: grepBlock) asSet asSortedCollection.
	sel := selectors select: '' fromUserIfEmpty: [^self].
	cls := self selectedClass whichClassIncludesSelector: sel.

	(cat := cls category) = self category
		ifFalse: [self category: cat. self changed: #category].
	self newClassList: cls name.
	self newProtocolList: (cls whichCategoryIncludesSelector: sel).
	self newSelectorList: sel!

includeClass
	"include definition and all selectors of selected class (class and metaclass)"

	self includeClass: self nonMetaClass.
	self newClassList: className!

includeClass: aClass 
	"include aClass with definition and all selectors (class and metaclass)"

	Cursor execute
		showWhile: 
			[self application includeClass: aClass.
			aClass organization categories do: [:p | self includeProtocol: p class: aClass].
			self application includeClass: aClass class.
			aClass class organization categories do: [:p | self includeProtocol: p class: aClass class]]!

printOutClass
	| stream |
	self systemMode ifTrue: [^super printOutClass].
	stream := TextStream on: (String new: 1000).
	self application printOutClasses: (Array with: self nonMetaClass) on: stream.
	self hardcopyStream: stream!

removeClass
	"remove selected class from system (meta and soleInstance) 
	(restricted to the underlying application)"

	| class |
	self systemMode ifTrue: [^super removeClass].
	self changeRequest ifFalse: [^self].
	class := self nonMetaClass.
	(DialogView confirm: 'Are you certain that you
want to remove the class ' , class name , '?')
		ifFalse: [^self].
	self application removeFromSystem: class.
	self newClassList: className! !

!ApplicationBrowser methodsFor: 'private-category functions'!

addCategory
	"Prompt the user for a new category name. Insert the category into 
	the class organization before the currently selected category; 
	system mode must be ON, else new categories are invisible. Notify the user"

	super addCategory.
	self systemMode ifFalse: [DialogView userInfo: 'switch system ON to see new category']!

excludeCategory
	"exclude all class info organized by currently selected system category"

	self application excludeCategory: self category.
	self newCategoryList: category!

fileOutCategory
	| stream |
	self category == nil ifTrue: [^self].
	self systemMode ifTrue: [^super fileOutCategory].
	stream := self
		requestFileName: 'File out on'
		default: category, '.st'
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.

	[self application fileOutCategory: category on: stream]
		valueNowOrOnUnwindDo: [stream close]!

includeCategory
	"include category with all classes and methods (class and instance)"

	self application includeCategory: self category.
	(organization listAtCategoryNamed: self category)
		do: [:c | self includeClass: c asClass].
	self newClassList: className!

printOutCategory
	| stream |
	self category == nil ifTrue: [^self].
	self systemMode ifTrue: [^super printOutCategory].
	stream := TextStream on: (String new: 1000).
	self application printOutCategory: self category on: stream.
	self hardcopyStream: stream!

removeCategory
	"remove all classes in selected category from system (meta and soleInstance) 
	(restricted to the underlying application)"

	self systemMode ifTrue: [^super removeCategory].
	self changeRequest ifFalse: [^self].
	(DialogView confirm: 
'Are you certain that you
want to remove all classes in category ', self category, '?')
		ifFalse: [^self]. 
	self application removeCategory: self category.
	self newCategoryList: category!

updateCategories
	self changeRequest ifFalse: [^self].
	self application cleanUp.
	super updateCategories! !

!ApplicationBrowser methodsFor: 'private-protocol functions'!

addProtocol
	"if the user wants to add a protocol, that protocol might be in system, 
	invisible for the application. Therefore the user gets a list with hidden 
	protocols in the selected class. The user may choose between them or 
	create a real new one. Some standard protocols are included"

	self systemMode
		ifFalse: [ | class otherProtocols newProtocol newString |
			class := self selectedClass.
			otherProtocols := (self standardProtocols , super protocolList) asSet - (self application protocolsInClass: class).
			otherProtocols isEmpty
				ifTrue: [^super addProtocol].
			(otherProtocols := otherProtocols asSortedCollection asOrderedCollection) addLast: (newString := 'other protocol').
			newProtocol := otherProtocols selectFromUserIfEmpty: [^self].
			newProtocol == newString
				ifFalse: [ | sym | 
					newProtocol := newProtocol asSymbol.
					sym := self protocol isNil ifFalse: [self protocol asSymbol].
					class organization addCategory: newProtocol before: sym.
					self logProtocolChange: class name , ' organization addCategory: ' , newProtocol storeString , ' before: ' , sym storeString.
					class reorganize.
					^self newProtocolList: newProtocol]].
	^super addProtocol!

excludeProtocol
	"exclude all selectors in the currently selected protocol restricted to the 
	application"

	self application excludeProtocol: self protocol asSymbol class: self selectedClass.
	self newProtocolList: protocol!

fileOutProtocol
	| stream |
	self systemMode ifTrue: [^super fileOutProtocol].
	stream := self
		requestFileName: 'File out protocol on:'
		default: (self selectedClass name, '-', protocol, '.st')
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.

	[stream timeStamp.
	self application
		fileOutProtocol: protocol
		class: self selectedClass asSymbol
		on: stream]

	valueNowOrOnUnwindDo: [stream close]!

includeProtocol
	self includeProtocol: self protocol asSymbol class: self selectedClass.
	self newProtocolList: protocol!

includeProtocol: p class: aClass 
	"include each selector of aClass classified under the protocol p"

	self application includeProtocol: p class: aClass.
	(aClass organization listAtCategoryNamed: p)
		do: [:sel | self application
				includeSelector: sel
				protocol: p
				class: aClass]!

printOutProtocol
	| stream |
	self systemMode ifTrue: [^super printOutProtocol].
	stream := TextStream on: (String new: 1000).
	self application
		printOutProtocol: self protocol
		class: self selectedClass
		on: stream.
	self hardcopyStream: stream!

removeProtocol
	"remove all selectors from system found in selected protocol and class
	(restricted to the underlying application)"

	| class p ownSelectors |
	self systemMode ifTrue: [^super removeProtocol].
	ownSelectors := self application
		selectorsInProtocol: (p := self protocol asSymbol)
		class: (class := self selectedClass).
	super selectorList size == ownSelectors size ifTrue: [^super removeProtocol].
	self changeRequest ifFalse: [^self].
	ownSelectors isEmpty ifFalse:
		[(DialogView confirm: 'Are you certain that you want to
remove all registered methods in this protocol?') ifFalse: [^self]].
	ownSelectors do: [:sel | class removeSelector: sel].
	self application removeProtocol: p class: class.
	self newProtocolList: nil! !

!ApplicationBrowser methodsFor: 'def/doc switch'!

documentationMode
	^documentationMode!

documentationMode: aBoolean
	self changeRequest ifFalse: [^self].
	documentationMode := aBoolean.
	self changed: #text! !

!ApplicationBrowser methodsFor: 'inheritance pane'!

classOfSelectedMethod
	"return the class of the selected method, regarding inheritance pane"

	| c |
	^(c := self selectedClass whichClassIncludesSelector: selector) isNil
		ifTrue: ["the selected method has been removed elsewhere"
			self selectedClass]
		ifFalse: [c]!

collectListsWithSuperclasses: aBlock 
	"collect lists obtained from evaluating aBlock in selected class and 
	superclasses upt o limit as specified in inheritance pane"

	| list |
	list := Set new.
	self restrictedSuperClassList do: [:c | list addAll: (aBlock value: c)].
	^list!

inheritedFromClass
	"class in which selected method is implemented"

	^(className notNil & selector notNil
		ifTrue: [self classOfSelectedMethod name]
		ifFalse: [' ']) asText!

inheritUpToClass
	^inheritUpToClass isNil
		ifTrue: [self selectedClass asSymbol]
		ifFalse: [inheritUpToClass]!

inheritUpToClass: classname
	inheritUpToClass := classname.
	self newProtocolList: protocol!

restrictedSuperClassList
	"list of superclasses up to 'inheritUpTo'"

	| more |
	more := true.
	^self superClassList  reverse select: [:c | more
			and: 
				[c = self inheritUpToClass ifTrue: [more := false].
				true]]!

superClassList
	"return the set of superclasses"

	^self selectedClass withAllSuperclasses reverse collect: [:cls | cls asSymbol]!

superClassMenu
	"return a Menu with all superclasses of selected class. Initial selection is the 
	current inheritance class"

	| scl |
	self selectedClass isNil ifTrue: [^nil].
	scl := self superClassList.
	^(PopUpMenu labelList: (Array with: scl) values: scl )
		selection: (scl indexOf: self inheritUpToClass)! !

!ApplicationBrowser methodsFor: 'category list'!

categoryList
	^self systemMode
		ifTrue: [super categoryList]
		ifFalse: 
			["categories of registered classes (same order as system organization)"
			| appCategories |
			appCategories := self application categories.
			organization categories select: [:cat | appCategories includes: cat]]!

categoryMenu
	^category == nil
		ifTrue: [self class categoryMenu]
		ifFalse: [self class applicationMenu]! !

!ApplicationBrowser methodsFor: 'system switch'!

systemMode
	^systemMode!

systemMode: aBoolean
	self changeRequest ifFalse: [^self].
	systemMode := aBoolean.
	self changed: #category! !

!ApplicationBrowser methodsFor: 'class list'!

classList
	^self emphasizeForeignClasses: (self systemMode
			ifTrue: [super classList]
			ifFalse: [self onlyIncludedClasses: super classList])!

classMenu
	^className == nil ifFalse: [self class classMenu]!

className
	"Answer the receiver's currently selected class as text for 
	SelectionInListView."

	^className notNil ifTrue: [className asText]!

className: selection 
	inheritUpToClass = selection ifFalse: [inheritUpToClass := nil].
	super className: (selection notNil ifTrue: [selection asString asSymbol])!

emphasizeForeignClasses: cl 
	cl isNil ifTrue: [^nil].
	^self emphasizeList: cl criterium: [:cn | self application definesClass: cn]!

onlyIncludedClasses: cl 
	"select classes in list cl that are in my application"

	cl isNil ifTrue: [^nil].
	^cl select: [:cn | (self application includesClass: cn) or: [(self application includesClass: (cn, ' class') asSymbol)]]!

selectedClass
	"return the currently selected class object 
	(overwritten to exclude errors from 'concurrent' removal/access of classes)"

	| class |
	(className isNil or: [(class := className asClass) isNil])
		ifTrue: [^nil].
	^meta ifTrue: [class class] ifFalse: [class]! !

!ApplicationBrowser methodsFor: 'private'!

emphasizeList: cl criterium: aBlock
	^cl collect: [:cn | (aBlock value: cn)
			ifTrue: [cn asText]
			ifFalse: [cn asText emphasizeAllWith: #italic]]!

requestFileName: prompt default: initialString ifQuit: aBlock
	"return a string, requested for a file name from user;
	evaluate aBlock if the answer is empty (cancel)"

	| string |
	^(string := DialogView
			requestNewFileName: prompt
			default: (self filterFilename: initialString asString)) isEmpty
		ifTrue: [aBlock value]
		ifFalse: [string]!

sourceFilename: anInteger 
	"return the name string of the source file numbered anInteger"

	| file |
	^(file := SourceFileManager default fileAt: anInteger) isNil
		ifTrue: ['']
		ifFalse: [file name]!

spawnChanges: aChangeSequence label: aString select: aBoolean 
	"open a change sequence browser on aChangeSequence, 
	filtering on the selected method (if aBoolean is true)"

	| chgList index |
	aBoolean
		ifTrue: 
			[chgList := aChangeSequence changesForSelector: self selector asSymbol class: self classOfSelectedMethod.
			chgList isEmpty
				ifTrue: [^DialogView userInfo: 'There are no changes on selector
' , self selector asSymbol storeString].
			index := aChangeSequence indexOf: chgList last].

	chgList := (EnhancedChangeList onApplication: self application)
				listName: aString
				changes: aChangeSequence
				removed: #().
	aBoolean
		ifTrue: [chgList
			toggleListIndex: index;
			switchFilter: 5 perform: #selector;
			switchFilter: 3 perform: #className].
	ChangeSequenceView openOn: chgList!

standardProtocols
	^self meta
		ifTrue: [#('class initialization' 'instance creation' )]
		ifFalse: [#(#'initialize-release' #accessing #adding #removing #comparing #testing #copying #printing #converting )]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!ApplicationBrowser class methodsFor: 'class initialization'!

flushMenus 
	"Cause all menus to be newly created (so changes appear)" 
	"ApplicationBrowser flushMenus"

	Menus := nil!

initialize
	"ApplicationBrowser initialize"

	self initializePrompts!

initializePrompts
	"initialize the user interface prompts accessed by #promptFor:"

	self promptFor: #selectThenBrowse is: 'Select for browsing'! !

!ApplicationBrowser class methodsFor: 'accessing menus'!

applicationMenu
	"return a menu for application and category operations"

	^self menuFor: #application ifAbsent:
		[PopUpMenu
			labels: 'file in/out\spawn\add category\rename\remove\include\exclude\update\edit all\find class' withCRs
			lines: #(2 5 7 9)
			values: (Array with: self fileMenu),
				(Array with: self spawnMenu),
				#( addCategory renameCategory removeCategory ),
				(Array with: self includeMenu),
				#( excludeCategory updateCategories editCategories findClass )]!

categoryMenu
	"return a menu for application and category operations"

	^self menuFor: #category ifAbsent:
		[PopUpMenu
			labels: 'file in/out\spawn\add category\update\edit all\find class' withCRs
			lines: #(2 3 5)
			values: (Array with: self fileMenu),
				(Array with: self spawnMenu),
				#(#addCategory #updateCategories #editCategories #findClass )].!

classMenu
	"return a menu for class operations"

	^self menuFor: #class ifAbsent:
		[PopUpMenu
			labels: 'file out\print out\spawn\spawn hierarchy
hierarchy\definition\comment\protocols
inst var refs\class var refs\class refs\applications
rename\remove\include\exclude\exclude definition' withCRs
			lines: #(4 8 12 14 )
			values: #( fileOutClass printOutClass spawnClass spawnHierarchy showHierarchy editClass editComment editProtocols browseFieldReferences browseClassVariables browseClassReferences),
				(Array with: self referencesMenu),
				#( renameClass removeClass includeClass excludeClass excludeClassDefinition)]!

fileMenu
	"return a menu for application and category file in/out operations"

	^self menuFor: #file ifAbsent:
		[PopUpMenu
			labelArray: #('file in...' 'upgrade...' 'new minor version' 'new major version...' 'file out changes...' 'file out category...' 'file out application...' 'print out category' 'print out application' 'set path' 'set name for DOS')
			lines: #(1 2 4 7 9)
			values: #(#fileIn #upgradeMinor #newMinorVersion #newMajorVersion #fileOutChanges #fileOutCategory #fileOutApplication #printOutCategory #printOutApplication #setPath #setNameForDos)]!

includeMenu
	"return a menu for application and category include operations"

	^self menuFor: #include ifAbsent:
		[PopUpMenu
			labelArray: #('category' 'required application...' 'all required applications' )
			lines: #(1)
			values: #(includeCategory addRequiredApplication findRequiredApplications )]!

protocolMenu
	"return a menu for protocol operations"

	^self menuFor: #protocol ifAbsent:
		[PopUpMenu
			labels: 'file out as...\print out\spawn\add protocol...\rename as...\remove...\include\exclude\edit all\find method...' withCRs
			lines: #(3 6 8)
			values: #(fileOutProtocol printOutProtocol spawnProtocol addProtocol renameProtocol removeProtocol includeProtocol excludeProtocol editProtocols findMethodAndSelectAlphabetic )]!

referencesMenu
	"returns a menu of operations for cross referencing"

	^self menuFor: #classRefs ifAbsent:
		[PopUpMenu
			labelArray: #(owners includers senders )
			values: #(showClassOwners showClassIncluders showClassUsers )]!

selectorMenu
	"return a menu for selector operations"

	^self menuFor: #selector ifAbsent:
		[PopUpMenu
			labels: 'file out\print out\spawn\senders\implementors\messages\applications\move\remove\include\exclude' withCRs
			lines: #(3 7 9)
			values: #(fileOutMessage printOutMessage),
				(Array with: self selectorSpawnMenu),
				#( browseSenders browseImplementors browseMessages ),
				(Array with: self selectorRefsMenu),
				#( moveMethod removeMethod includeSelector excludeSelector)]!

selectorRefsMenu
	"return a menu for cross references wrt. instances of Application"

	^self menuFor: #'selector>refs' ifAbsent:
		[PopUpMenu
			labelArray: #(owners senders )
			values: #(showMethodOwners showMethodUsers )]!

selectorSpawnMenu
	"return a menu for editing implementation and changes of a selector"

	^self menuFor: #'selector>spawn' ifAbsent:
		[PopUpMenu
			labelArray: #(#method #hierarchy #changes #history #'full history' )
			lines: #(2)
			values: #(spawnMethod spawnMethodHierarchy spawnSelectorChanges spawnHistory spawnFullHistory)]!

spawnMenu
	"return a menu for application and category editing operations"

	^self menuFor: #spawn ifAbsent:
		[PopUpMenu
			labelArray: #( application category changes history 'full history' 'pre-installation' 'post-installation' required statistics inspect )
			lines: #(2 5 7)
			values: #(spawnApplication spawnCategory spawnChanges spawnHistory spawnFullHistory spawnInstallation spawnPostInstallation spawnRequired spawnStatistics inspectApplication )]! !

!ApplicationBrowser class methodsFor: 'private'!

menuFor: aSymbol ifAbsent: aBlock 
	"returns the menu associated with <aSymbol>. If it has not yet been defined, 
	evaluate <aBlock> to get and cache the menu."

	Menus isNil ifTrue: [Menus := Dictionary new].
	^Menus at: aSymbol ifAbsentPut: aBlock!

promptFor: aSymbol 
	"return a cached prompt text for a user interface situation specified by aSymbol"

	^Prompts at: aSymbol ifAbsent: ['']!

promptFor: aSymbol is: stringOrText 
	"caches a text shown to the user in a situation specified by aSymbol"

	Prompts isNil ifTrue: [Prompts := Dictionary new].
	^Prompts at: aSymbol put: stringOrText! !

!ApplicationBrowser class methodsFor: 'private-view creation'!

addDocumentationButtonViewTo: aContainer in: area on: aBrowser
	| baseModel valueModel view |
	baseModel := (PluggableAdaptor on: aBrowser)
				getSelector: #documentationMode putSelector: #documentationMode:.
	valueModel := PluggableAdaptor on: baseModel.
	(view := LabeledBooleanView model: valueModel)
		label: 'doc'; beToggle.
	view controller beToggle.
	^aContainer add: view borderedIn: area!

addInfoViewTo: aContainer in: area on: aBrowser 
	"add a view which displays the owner (application) or source file for the 
	selected item"

	| edgeDecorator view |
	view := (TextView on: aBrowser aspect: #info change: nil menu: nil)
			controller: NoController new.
	edgeDecorator := (LookPreferences edgeDecorator on: view)
			noVerticalScrollBar noMenuBar.
	^aContainer add: edgeDecorator in: area "borderedIn: area"!

addInheritedClassViewTo: aContainer in: area on: aBrowser
	| edgeDecorator view |
	(view := SelectionInListView on: aBrowser printItems: false oneItem: true
				aspect: #inheritedFromClass change: nil
				list: #inheritedFromClass menu: #superClassMenu
				initialSelection: #inheritedFromClass)
			controller: PopUpListController new.

	edgeDecorator := (LookPreferences edgeDecorator on: view) noVerticalScrollBar noMenuBar.
	^aContainer add: edgeDecorator borderedIn: area!

addSystemButtonViewTo: aContainer in: area on: aBrowser
	| baseModel valueModel view |
	baseModel := (PluggableAdaptor on: aBrowser)
				getSelector: #systemMode putSelector: #systemMode:.
	valueModel := PluggableAdaptor on: baseModel.
	(view := LabeledBooleanView model: valueModel)
		label: 'system'; beToggle.
	view controller beToggle.
	^aContainer add: view borderedIn: area!

addTextViewTo: aContainer in: area on: aBrowser initialSelection: sel initialState: anArray 

	| textView |
	textView := ApplicationTextView
				on: aBrowser
				aspect: #text
				change: #acceptText:from:
				menu: #textMenu
				initialSelection: sel.
	anArray == nil ifFalse: [self setTextView: textView fromTextState: anArray].
	^aContainer add: (LookPreferences edgeDecorator on: textView)
		in: area! !

!ApplicationBrowser class methodsFor: 'view creation'!

open
	"select an application from application management and open a browser on it"

	self openOn: (self applicationManagement selectApplication: 'Select an application to browse')
		withTextState: nil!

openOn: aBrowser withTextState: anArray
	"create and schedule a browser on an Application;
	inherit browser structure, but add a system and a documentation button
	and an inheritance switch field"

	| topView divider topWindow buttonOffset infoLineOffset |
	buttonOffset := self buttonOffset.
	infoLineOffset := (TextAttributes defaultLineGrid + 3) negated.
	divider := 0.35.
	topView := DependentComposite new.
	self
		addCategoryViewTo: topView
			in: (LayoutFrame new
				rightFraction: 0.25; bottomFraction: divider offset: buttonOffset)
			on: aBrowser readOnly: false;

		addSystemButtonViewTo: topView
			in: (LayoutFrame new
				topFraction: divider offset: buttonOffset;
				rightFraction: 0.125; bottomFraction: divider)
			on: aBrowser; 

		addDocumentationButtonViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.125; topFraction: divider offset: buttonOffset; 
				rightFraction: 0.25; bottomFraction: divider)
			on: aBrowser;

		addClassViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.25;
				rightFraction: 0.5; bottomFraction: divider offset: buttonOffset)
			on: aBrowser readOnly: false;

		addClassAndInstanceButtonViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.25; topFraction: divider offset: buttonOffset;
				rightFraction: 0.5; bottomFraction: divider)
			on: aBrowser;

		addProtocolViewTo: topView in: (0.5@0 extent: 0.25@divider) on: aBrowser readOnly: false;
		addSelectorViewTo: topView
			in: (LayoutFrame new 
				leftFraction: 0.75; rightFraction: 1.0; 
				bottomFraction: divider offset: buttonOffset)
			on: aBrowser readOnly: false;

		addInheritedClassViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.75; topFraction: divider offset: infoLineOffset;
				rightFraction: 1.0; bottomFraction: divider)
			on: aBrowser;

		addTextViewTo: topView
			in: (LayoutFrame new
				topFraction: divider;
				rightFraction: 1.0; bottomFraction: 1.0 offset: infoLineOffset)
			on: aBrowser initialSelection: nil initialState: anArray;

		addInfoViewTo: topView
			in: (LayoutFrame new
				topFraction: 1.0 offset: infoLineOffset;
				rightFraction: 1.0; bottomFraction: 1.0)
			on: aBrowser.


	(topWindow := ScheduledWindow model: aBrowser label: aBrowser application printString minimumSize: 400@250)
		component: topView;
		controller: aBrowser class defaultControllerClass new;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: ((topWindow  minimumSize * 3 + topWindow maximumSize) // 4)! !

!ApplicationBrowser class methodsFor: 'controller accessing'!

defaultControllerClass
	^ApplicationController! !



ControllerWithMenu subclass: #PopUpListController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Interface-Application'!


!PopUpListController methodsFor: 'control activity'!

menu
	^model superClassMenu!

textHasChanged
	^false!

yellowButtonActivity
	"send model 'inheritUpToClass: ' with argument selection"

	| selection |
	self menu notNil
		ifTrue: 
			[selection := self menu startUp.
			selection ~= 0 
				ifTrue: [model inheritUpToClass: selection]]
		ifFalse: [super controlActivity]! !



!Application methodsFor: 'user-interface'!

browse
	ApplicationBrowser openOn: self!

browseInitial: aClass selector: aSymbol
	"open a browser preselected on the given class and selector;
	return the browser"

	| dep browser |
	dep := self dependents.
	self browse.
	browser := self dependents detect: [:d | (dep includes: d) not] ifNone: [^nil].
	aClass isNil ifTrue: [^browser].
	^browser
		category: aClass category;
		className: aClass name;
		protocol: (aClass organization categoryOfElement: aSymbol);
		selector: aSymbol;
		changed: #category! !



Model subclass: #ApplicationManagementBrowser
	instanceVariableNames: 'management application filter '
	classVariableNames: 'ApplicationMenu '
	poolDictionaries: ''
	category: 'Interface-Application'!
ApplicationManagementBrowser comment:
'An ApplicationManagementBrowser serves as launcher for meta application functions such as creation, registration, unregistration, loading, editing, etc.
The main component is a list view displaying all registered applications. The list can be constrained by a filter.

Instance Variables:
	management	<ApplicationManagement>  the only instance of that class
	application	<String>  name with version of the currently selected application
	filter	<Symbol | nil>  aspect which must be true for an appliaction to be displayed

Class Variables:
	ApplicationMenu	<PopUpMenu>  comment'!


!ApplicationManagementBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a major/minor version release via application browser"

	anAspect == #version
		ifTrue:
			[ "this broadcast signals that an application's version changed"
			| app |
			(app := self application) notNil
				ifTrue: 
					["avoid losing selection if selected item concerns about new version"
					app := management applicationNamed: (app copyUpTo: $().
					self application: app identifier].
			^self changed: #application].
	super update: anAspect! !

!ApplicationManagementBrowser methodsFor: 'initialize-release'!

management: aManager
	management := aManager.
	aManager addDependent: self!

release
	management removeDependent: self.
	super release! !

!ApplicationManagementBrowser methodsFor: 'filter'!

filter
	^filter!

filter: value 
	filter := filter == value ifTrue: [] ifFalse: [value].
	self changed: #application! !

!ApplicationManagementBrowser methodsFor: 'application'!

application
	^application!

application: aString 
	application := aString!

applicationList
	| list |
	list := management sortedApplicationIdentifiers.
	^filter == #isModified
		ifTrue: [list select: [:item | (item findString: 'm)' startingAt: 1) ~~ 0]]
		ifFalse: [list]!

applicationMenu
	"ApplicationMenu := nil."

	| fileMenu values |
	application == nil ifTrue: [^PopUpMenu
			labels: 'new application...\upgrade...\load...\file out changes...\set directories...\update' withCRs
			lines: #(1 3 5 )
			values: #(newApplication upgradeMinor loadApplication collectAllChanges setDirectory updateList )].

	fileMenu := PopUpMenu
					labelArray: #( 'upgrade' 'new minor' 'file out changes...' 'file out all changes...' 'store for DOS' 'set directories...' 'condense changes...' )
					lines: #(1 4 6)
					values: #( upgradeMinor newMinorVersion fileOutChanges collectAllChanges storeDOS setDirectory condenseChanges).

	values := (Array with: fileMenu),
			#(openApplicationBrowser newApplication removeApplication updateList ).

	ApplicationMenu == nil ifTrue: [ApplicationMenu := PopUpMenu
					labels: 'file in/out\open browser\new application...\unregister...\update' withCRs
					lines: #(1 4)
					values: values].
	^ApplicationMenu!

collectAllChanges
	"file out all changes of all applications into one file"

	| fileName fileStream |
	(fileName := DialogView
		requestNewFileName: 'File out changes of all applications on'
		default: 'allchanges.st') isEmpty
		ifTrue: [^self].

	fileStream := (Filename named: fileName) writeStream.
	[management do: [:app | app changes fileOutOn: fileStream]]
		valueNowOrOnUnwindDo: [fileStream close]!

condenseChanges
	(DialogView confirm: 'Do you really want to condense changes?' initialAnswer: false)
		ifFalse: [^self].
	SourceFileManager default condenseChanges!

fileOutChanges
	| stream app |
	(app := management applicationIdentified: self application) changes isEmpty
		ifTrue: [^Transcript show: 'no changes to file out'; cr].
	(stream := DialogView
		requestNewFileName: 'File out application changes on'
		default: app fileString , '.cs') isEmpty
		ifTrue: [^self].
	stream := (Filename named: stream) writeStream.

	Cursor write showWhile:
		[[app changes fileOutOn: stream timeStamp]
		valueNowOrOnUnwindDo: [stream close]]!

loadApplication
	"show a collection of known applications, 
	file in the corresponding source code"

	| avail list choice |
	(avail := management availableApplications) isEmpty
		ifTrue: [^DialogView userInfo: 'No application directories found'].
	list := (avail collect: [:triple | triple first asString, ' ', (triple at: 2) asString, ' ', triple last asString]).
	choice := (PopUpMenu labelList: (Array with: list)) startUpWithHeading: 'Select an application to load'.
	choice := choice == 0 ifTrue: [^self] ifFalse: [avail at: choice].
	management load: (choice at: 2) path: choice first version: choice last!

newApplication
	"create a new empty application structure, name from user"

	| app |
	(app := self applicationManagement newApplication) == nil
		ifTrue: [^nil].
	self application: app identifier.
	self changed: #application!

newMinorVersion
	"append changes to source file, clear changes and increment minor version"

	| app |
	app := management applicationIdentified: self application ifAbsent: [^self].
	app changes isEmpty
		ifTrue: [(DialogView confirm: 'There are no changes.
Do you want a new minor version anyway?' initialAnswer: false)
			ifFalse: [^self]].

	(app := app newMinorVersion) isNil ifTrue: [^self].
	self application: app identifier.
	self changed: #application!

openApplicationBrowser
	| app |
	app := management applicationIdentified: self application
				ifAbsent: 
					[| a |
					a := management applicationNamed: (self application copyUpTo: $().
					self updateList.
					a].
	app browse!

removeApplication
	(DialogView confirm: 'really unregister ?')
		ifFalse: [^nil]. 
	(management applicationIdentified: self application) release.
	management unregister: self application.
	self changed: #application!

setDirectory
	management directories inspect!

storeDOS
	"Store the application using a DOS compatible filename.
	The DOS filename is recorded in a platform translation table in VersionedObjectAccessor and used when appropriate"

	(management applicationIdentified: self application) storeForDOS!

updateList
	self changed: #application!

upgradeMinor
	"try to upgrade anApplication to the newest available minor version"

	self application isNil
		ifTrue: [^management selectAndUpgradeApplication].
	management upgradeMinor: (management applicationIdentified: self application)! !

!ApplicationManagementBrowser methodsFor: 'private-subview creation'!

applicationView 
	^LookPreferences edgeDecorator on: (SelectionInListView
			on: self
			printItems: false
			oneItem: false
			aspect: #application
			change: #application:
			list: #applicationList
			menu: #applicationMenu
			initialSelection: #application)!

modifiedButtonView
	| baseModel valueModel view |
	baseModel := (PluggableAdaptor on: self) getSelector: #filter putSelector: #filter:.
	valueModel := (PluggableAdaptor on: baseModel) selectValue: #isModified.
	view := (LabeledBooleanView model: valueModel) label: 'modified'; beToggle.
	view controller beTriggerOnUp.
	^view! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!ApplicationManagementBrowser class methodsFor: 'view creation'!

open
	"ApplicationManagementBrowser open"

	| prompt |
	prompt := ScheduledWindow classPool at: #PromptForOpen.
	ScheduledWindow promptForOpen: false.
	self openOn: self applicationManagement.
	ScheduledWindow promptForOpen: prompt!

openOn: anApplicationManagement
	"Create and schedule a browser on ApplicationManager"

	| topView buttonOffset buttonWidth buttonCount browser |

	buttonOffset := [LabeledBooleanView defaultHeight negated].
	buttonCount := 1.
	buttonWidth := 1.0 / buttonCount.
	browser := self on: anApplicationManagement.
	(topView := DependentComposite new)
		add: browser applicationView
		in: (LayoutFrame new 
				rightFraction: 1.0;
				bottomFraction: 1 offset: buttonOffset);

		add: browser modifiedButtonView
		borderedIn: (LayoutFrame new
				topFraction: 1 offset: buttonOffset;
				rightFraction: buttonWidth;
				bottomFraction: 1).

"		add: browser updateButtonView
		borderedIn: (LayoutFrame new
				topFraction: 1 offset: buttonOffset;
				rightFraction: buttonWidth;
				bottomFraction: 1).
"
	(ScheduledWindow model: browser label: 'Application Manager' minimumSize: 100@70)
		component: topView;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: 150@200! !

!ApplicationManagementBrowser class methodsFor: 'instance creation'!

on: anApplicationManagement
	^self new management: anApplicationManagement! !

!ApplicationManagementBrowser class methodsFor: 'class initialization'!

flushMenus
	ApplicationMenu := nil! !

ApplicationBrowser initialize!




' .ApplicationSource APL-Interface(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 31 August 1993 at 7:59:52 pm'!



!ApplicationBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a minor version release via application management"

	anAspect == #version
		ifTrue: 
			[self category isNil & self documentationMode not ifTrue: [self changed: #text].
			self changed: #windowLabel with: self application identifier.
			^self applicationManagement changed: anAspect].
	super update: anAspect! !

!ApplicationBrowser methodsFor: 'private-class functions'!

findMethodAndSelectAlphabetic
	"enhances default implementation by watching the inheritance switch"

	| sel selectors cls grepBlock |
	inheritUpToClass isNil ifTrue: [^super findMethodAndSelectAlphabetic].
	self changeRequest ifFalse: [^self].
	grepBlock := self systemMode
		ifTrue: [ [:c | c asClass organization elements] ]
		ifFalse: [ [:c | self application selectorsInClass: c] ].
	selectors := (self collectListsWithSuperclasses: grepBlock) asSet asSortedCollection.
	sel := selectors select: '' fromUserIfEmpty: [^self].
	cls := self selectedClass whichClassIncludesSelector: sel.
	self newProtocolList: (cls whichCategoryIncludesSelector: sel).
	self newSelectorList: sel! !

!ApplicationBrowser methodsFor: 'private-application functions'!

spawnStatistics
	"open a workspace with statistical informations about the application"

	| s a |
	(s := TextStream on: (String new: 500))
		nextPutAll: 'classes:'; tab: 2; nextPutAll: (a := self application) classNames size printString; space; nextPut: $(; nextPutAll: (a classNames select: [:c | a definesClass: c]) size printString; nextPut: $); cr;
		nextPutAll: 'selectors:'; tab: 1; nextPutAll: a selectors size printString; cr.

	a path notNil ifTrue:
		[OSErrorHolder nonexistentSignal
			handle: [:ex | ex proceedWith: 0]
			do: [s
				nextPutAll: 'sources:'; tab: 2; nextPutAll: a sourceFile name asFilename fileSize printString; cr.
				a version isInitialVersion ifFalse: [s nextPutAll: 'binary:'; tab: 2; nextPutAll: a instBossFilename fileSize printString; cr]]].

	s := s contents.
	ComposedTextView
		open: (ValueHolder with: s)
		label: 'Statistics on ' , a identifier storeString
		icon: (Icon constantNamed: #workspace)
		extent: s asComposedText extent + 30! !

!ApplicationManagementBrowser methodsFor: 'application'!

loadApplication
	"show a collection of known applications, 
	file in the corresponding source code"

	| avail list choice |
	(avail := management availableApplications) isEmpty
		ifTrue: [^DialogView userInfo: 'No application directories found'].
	list := (avail collect: [:triple | triple first asString, ' ', (triple at: 2) asString, ' ', triple last asString]).
	choice := (PopUpMenu labelList: (Array with: list)) startUpWithHeading: 'Select an application to load'.
	choice := choice == 0 ifTrue: [^self] ifFalse: [avail at: choice].
	management load: (choice at: 2) path: choice first version: choice last binary: nil! !

!ApplicationManagementBrowser methodsFor: 'application'!

selectedApplication
	"returns the application object referred by the current selection; the selection 
	may hold an outdated identifier, so the simplest access may fail;
	update list if necessary"

	| id |
	^(id := self application) == nil ifFalse:
		[management applicationIdentified: id ifAbsent:
			[| app |
			app := management applicationNamed: (id copyUpTo: $().
			self application: (app isNil ifFalse: [app identifier]); updateList.
			app]]! !

!ApplicationManagementBrowser methodsFor: 'application'!

removeApplication
	| app |
	(app := self selectedApplication) == nil ifTrue: [^nil].
	(DialogView confirm: 'Really unregister
', app identifier storeString, ' ?')
		ifFalse: [^nil]. 
	app release.
	management unregister: app.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'application'!

newMinorVersion
	"append changes to source file, clear changes and increment minor version"

	| app |
	(app := self selectedApplication) == nil ifTrue: [^self].
	app changes isEmpty
		ifTrue: [(DialogView confirm: 'There are no changes.
Do you want a new minor version anyway?' initialAnswer: false)
			ifFalse: [^self]].

	(app := app newMinorVersion) isNil ifTrue: [^self].
	self application: app identifier.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'application'!

openApplicationBrowser
	| app |
	app := self selectedApplication.
	app browse! !

!ApplicationManagementBrowser methodsFor: 'application'!

storeDOS
	"Store the application using a DOS compatible filename.
	The DOS filename is recorded in a platform translation table in VersionedObjectAccessor and used when appropriate"

	self selectedApplication storeForDOS! !

!ApplicationManagementBrowser methodsFor: 'application'!

upgradeMinor
	"try to upgrade anApplication to the newest available minor version"

	| app |
	(app := self selectedApplication) isNil
		ifTrue: [management selectAndUpgradeApplication]
		ifFalse: [management upgradeMinor: app]! !

!ApplicationManagementBrowser methodsFor: 'application'!

fileOutChanges
	| stream app |
	(app := self selectedApplication) isNil ifTrue: [^self].
	app changes isEmpty
		ifTrue: [^Transcript show: 'no changes to file out'; cr].
	(stream := DialogView
		requestNewFileName: 'File out application changes on'
		default: app fileString , '.cs') isEmpty
		ifTrue: [^self].
	stream := (Filename named: stream) writeStream.

	Cursor write showWhile:
		[[app changes fileOutOn: stream timeStamp]
		valueNowOrOnUnwindDo: [stream close]]! !

!ApplicationManagementBrowser methodsFor: 'application'!

collectAllChanges
	"file out all changes of all applications into one file"

	| fileName fileStream |
	(fileName := DialogView
		requestNewFileName: 'File out changes of all applications on'
		default: 'allchanges.cs') isEmpty
		ifTrue: [^self].

	fileStream := fileName asFilename writeStream.
	Cursor write showWhile:
		[[management do: [:app | app changes fileOutOn: fileStream]]
			valueNowOrOnUnwindDo: [fileStream close]]! !

!ApplicationManagementBrowser methodsFor: 'application'!

newApplication
	"create a new empty application structure, name from user"

	| app |
	(app := self applicationManagement newApplication) == nil
		ifTrue: [^nil].
	self application: app identifier.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'application'!

applicationMenu
	"ApplicationMenu := nil."

	| fileMenu values |
	application == nil ifTrue: [^PopUpMenu
			labels: 'new application...\upgrade...\load...\file out changes...\set directories...\update' withCRs
			lines: #(1 3 5 )
			values: #(newApplication upgradeMinor loadApplication collectAllChanges setDirectory updateList )].

	fileMenu := PopUpMenu
					labelArray: #( 'upgrade...' 'load...' 'new minor' 'file out changes...' 'file out all changes...' 'store for DOS' 'set directories...' 'condense changes...' )
					lines: #(2 5 7)
					values: #( upgradeMinor loadApplication newMinorVersion fileOutChanges collectAllChanges storeDOS setDirectory condenseChanges).

	values := (Array with: fileMenu),
			#(openApplicationBrowser newApplication removeApplication updateList ).

	ApplicationMenu == nil ifTrue: [ApplicationMenu := PopUpMenu
					labels: 'file in/out\open browser\new application...\unregister...\update' withCRs
					lines: #(1 4)
					values: values].
	^ApplicationMenu! !

!ApplicationManagementBrowser methodsFor: 'filter'!

filter: value 
	filter := filter == value ifFalse: [value].
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a major/minor version release via application browser"

	anAspect == #version
		ifTrue:
			[ "this broadcast signals that an application's version changed"
			"avoid losing selection if selected item concerns about new version"
			^self selectedApplication; updateList].
	super update: anAspect! !

ApplicationManagementBrowser classPool at: #ApplicationMenu put: nil!


' .ApplicationSource APL-Interface(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 29 September 1993 at 2:30:37 pm'!



!ApplicationBrowser methodsFor: 'private-category functions'!

removeCategory
	"remove all classes in selected category from system (meta and soleInstance) 
	(restricted to the underlying application)"

	self systemMode ifTrue: [^super removeCategory].
	self changeRequest ifFalse: [^self].
	(DialogView confirm: 
'Are you certain that you want to remove
all classes in category ''', self category, ''' ?')
		ifFalse: [^self]. 
	self application removeCategory: self category.
	self newCategoryList: category! !

!ApplicationManagementBrowser methodsFor: 'application'!

fileOutUpgrade
	| app |
	(app := self selectedApplication) isNil ifTrue: [^self].
	management fileOutUpgrade: app! !

!ApplicationManagementBrowser methodsFor: 'application'!

applicationMenu
	"ApplicationManagementBrowser flushMenus"

	| fileMenu values |
	application == nil ifTrue: [^PopUpMenu
			labels: 'new application...\upgrade...\load...\file out changes...\set directories...\update' withCRs
			lines: #(1 3 5 )
			values: #(newApplication upgradeMinor loadApplication collectAllChanges setDirectory updateList )].

	fileMenu := PopUpMenu
					labelArray: #( 'upgrade...' 'load...' 'new minor' 'file out changes...' 'file out all changes...' 'file out upgrade...' 'store for DOS' 'set directories...' 'condense changes...' )
					lines: #(2 6 8)
					values: #( upgradeMinor loadApplication newMinorVersion fileOutChanges collectAllChanges fileOutUpgrade storeDOS setDirectory condenseChanges).

	values := (Array with: fileMenu),
			#(openApplicationBrowser newApplication removeApplication updateList ).

	ApplicationMenu == nil ifTrue: [ApplicationMenu := PopUpMenu
					labels: 'file in/out\open browser\new application...\unregister...\update' withCRs
					lines: #(1 4)
					values: values].
	^ApplicationMenu! !

ApplicationManagementBrowser flushMenus!


' .ApplicationSource APL-Interface(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 14 April 1994 at 1:08:36 pm'!



"fix
- avoid spawned browsers to inherit documentation mode
- errors in ApplicationBrowser>info during update
- improper value of inheritance field after switching class/instance
- improper result on 'application> senders' on the class list
- layout mismatch of selector and inheritance view
- cursor execute during 'remove' on the category list"!

!ApplicationBrowser methodsFor: 'copying'!

postCopy
	"browsers spawned from the receiver have no doc or system switch;
	avoid that new browsers have doc mode"

	super postCopy.
	documentationMode := false! !

!ApplicationBrowser methodsFor: 'private'!

sourceFilename: anInteger 
	"return the name string of the source file numbered anInteger"

	| file |
	^(file := Object subscriptOutOfBoundsSignal handle: [:ex | ex return]
			do: [SourceFileManager default fileAt: anInteger]) == nil
		ifTrue: ['source code not available']
		ifFalse: [file name]! !

!ApplicationBrowser methodsFor: 'text'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                               

!ApplicationBrowser methodsFor: 'inheritance pane'!

classOfSelectedMethod
	"return the class of the selected method, regarding inheritance pane"

	| c |
	^(c := self selectedClass) == nil
		ifFalse:
			[| impl |
			(impl := c whichClassIncludesSelector: selector) == nil
				ifTrue: [c	"selected method has been removed elsewhere"]
				ifFalse: [impl]]! !

!ApplicationBrowser methodsFor: 'inheritance pane'!

inheritedFromClass
	"class in which selected method is implemented"

	| c |
	^(selector == nil or: [(c := self classOfSelectedMethod) == nil])
		ifTrue: [' ']
		ifFalse: [c name]! !

!ApplicationBrowser methodsFor: 'class/inst switch'!

meta: aBoolean 
	"override implementation to update inheritance switch"

	self changeRequest ifFalse: [^self changed: #meta].
	meta := aBoolean.
	self changed: #meta.
	self inheritUpToClass: (inheritUpToClass == nil
		ifFalse: [(aBoolean
			ifTrue: [inheritUpToClass, ' class']
			ifFalse: [inheritUpToClass copyUpTo: Character space]) asSymbol]).

	(self classMode | textMode isNil) ifTrue: [self changed: #text]! !

!Browser methodsFor: 'private-class functions'!

showClassUsers
	"show a list with all applications referring to the selected class;
	if the user selects an item, we proceed as if 'class refs' has been chosen
	(note the restriction on elements of the chosen application)"

	| cls calls app |
	calls := self applicationManagement applicationsCallingSelector: nil class: (cls := self nonMetaClass) without: nil.
	app := calls keys select: (ApplicationBrowser promptFor: #selectThenBrowse) fromUserIfEmpty: [^self].
	calls := (calls at: app) collect: [:assoc | assoc key , ' ' , assoc value].

	MethodListBrowser
		openListBrowserOn: calls asSortedCollection
		label: 'Calls on ', cls name, ' in ', app identifier
		initialSelection: cls name! !

!ApplicationBrowser class methodsFor: 'view creation'!

openOn: aBrowser withTextState: anArray
	"create and schedule a browser on an Application;
	inherit browser structure, but add a system and a documentation button
	and an inheritance switch field"

	| topView divider topWindow buttonOffset infoLineOffset |
	buttonOffset := self buttonOffset.
	infoLineOffset := (TextAttributes defaultLineGrid + 3) negated.
	divider := 0.35.
	topView := DependentComposite new.
	self
		addCategoryViewTo: topView
			in: (LayoutFrame new
				rightFraction: 0.25; bottomFraction: divider offset: buttonOffset)
			on: aBrowser readOnly: false;

		addSystemButtonViewTo: topView
			in: (LayoutFrame new
				topFraction: divider offset: buttonOffset;
				rightFraction: 0.125; bottomFraction: divider)
			on: aBrowser; 

		addDocumentationButtonViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.125; topFraction: divider offset: buttonOffset; 
				rightFraction: 0.25; bottomFraction: divider)
			on: aBrowser;

		addClassViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.25;
				rightFraction: 0.5; bottomFraction: divider offset: buttonOffset)
			on: aBrowser readOnly: false;

		addClassAndInstanceButtonViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.25; topFraction: divider offset: buttonOffset;
				rightFraction: 0.5; bottomFraction: divider)
			on: aBrowser;

		addProtocolViewTo: topView in: (0.5@0 extent: 0.25@divider) on: aBrowser readOnly: false;
		addSelectorViewTo: topView
			in: (LayoutFrame new 
				leftFraction: 0.75; rightFraction: 1.0; 
				bottomFraction: divider offset: infoLineOffset)
			on: aBrowser readOnly: false;

		addInheritedClassViewTo: topView
			in: (LayoutFrame new
				leftFraction: 0.75; topFraction: divider offset: infoLineOffset;
				rightFraction: 1.0; bottomFraction: divider)
			on: aBrowser;

		addTextViewTo: topView
			in: (LayoutFrame new
				topFraction: divider;
				rightFraction: 1.0; bottomFraction: 1.0 offset: infoLineOffset)
			on: aBrowser initialSelection: nil initialState: anArray;

		addInfoViewTo: topView
			in: (LayoutFrame new
				topFraction: 1.0 offset: infoLineOffset;
				rightFraction: 1.0; bottomFraction: 1.0)
			on: aBrowser.


	(topWindow := ScheduledWindow model: aBrowser label: aBrowser application printString minimumSize: 400@250)
		component: topView;
		controller: aBrowser class defaultControllerClass new;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: ((topWindow  minimumSize * 3 + topWindow maximumSize) // 4)! !

"optimizations"!

!ApplicationBrowser methodsFor: 'class list'!

selectedClass
	"return the currently selected class object 
	(overwritten to exclude errors from 'concurrent' removal/access of classes)"

	^className == nil ifFalse: 
		[| cls |
		(cls := className asClass) == nil
			ifTrue: [className := nil]
			ifFalse: [meta ifTrue: [cls class] ifFalse: [cls]]]! !

!ApplicationBrowser methodsFor: 'class list'!

classList
	"return a list of Text (italic if specified class is not defined by the receiver)"

	| list app |
	(list := super classList) == nil ifTrue: [^nil].
	app := self application.
	self systemMode ifFalse: [list := list select: [:cn | (app includesClass: cn)
						or: [app includesClass: (cn , ' class') asSymbol]]].
	^self emphasizeList: list criterium: [:cn | app definesClass: cn]! !

!ApplicationBrowser methodsFor: 'protocol list'!

protocolList
	"list of protocols in selected class and the superclasses as specified in 
	inheritance pane (alphabetical order)"

	| cls app grepBlock list |
	(cls := self selectedClass) == nil ifTrue: [^nil].
	app := self application.
	grepBlock := self systemMode
		ifTrue: [ [:c | c asClass organization categories] ]
		ifFalse: [ [:c | (app classSubsetOf: c ifAbsent: [ClassSubset new]) categories] ].
	list := (self collectListsWithSuperclasses: grepBlock) asSortedCollection.
	^self emphasizeList: list criterium: [:p | app includesProtocol: p class: cls]! !

!ApplicationBrowser methodsFor: 'selector list'!

selectorList
	"return a list of Text: each a selector of the selected class and all superclasses 
	up to inheritUpToClass, italic for those which are not member of the receiver"

	| list cls prt app |
	protocol == nil ifTrue: [^nil].
	(cls := self selectedClass) == nil ifTrue: [^nil].
	prt := self protocol asSymbol.
	app := self application.
	list := (self collectListsWithSuperclasses: (self systemMode
					ifTrue: [[:c | c asClass organization listAtCategoryNamed: prt]]
					ifFalse: [[:c | app selectorsInProtocol: prt class: c]])) asSortedCollection.
	^self emphasizeList: list criterium: [:sel | app includesSelector: sel class: cls name]! !

ApplicationBrowser removeSelector: #emphasizeForeignClasses:!

ApplicationBrowser removeSelector: #onlyIncludedClasses:!

ApplicationBrowser removeSelector: #restrictedSuperClassList!

ApplicationBrowser removeSelector: #withControlstring:!

!ApplicationBrowser methodsFor: 'inheritance pane'!

superClassList
	"return an OrderedCollection: the names of the selected class and its superclasses in upward direction"

	^self selectedClass withAllSuperclasses collect: [:cls | cls name asSymbol]! !

!ApplicationBrowser methodsFor: 'inheritance pane'!

collectListsWithSuperclasses: aBlock 
	"collect lists obtained from evaluating aBlock in selected class and 
	superclasses up to limit as specified in inheritance field"

	| sup result |
	inheritUpToClass == nil
		ifTrue: [^aBlock value: self selectedClass name asSymbol].
	result := Set new.
	1 to: ((sup := self superClassList) indexOf: self inheritUpToClass ifAbsent: [1])
		do: [:i | result addAll: (aBlock value: (sup at: i))].
	^result! !

!ApplicationBrowser methodsFor: 'private-selector functions'!

spawnMethodHierarchy
	"open an implementors browser, restricted to superclasses of the currently 
	selected class"

	| list sel |
	sel := self selector asSymbol.
	list := self superClassList select: [:c | c asClass includesSelector: sel].
	list size == 1
		ifTrue: [^DialogView userInfo: 'No superclasses implementing this selector'].

	MethodListBrowser
		openListBrowserOn: (list reverse collect: [:c | c , ' ' , sel])
		label: self className , ': Hierarchy of ' , sel! !

!ApplicationBrowser methodsFor: 'private-category functions'!

includeCategory
	"include category with all classes and methods (class and instance)"

	(organization listAtCategoryNamed: self category)
		do: [:c | self includeClass: c asClass].
	self newClassList: className! !

!ApplicationBrowser methodsFor: 'private-category functions'!

removeCategory
	"remove all classes in selected category from system (meta and soleInstance) 
	(restricted to the underlying application)"

	self systemMode ifTrue: [^super removeCategory].
	self changeRequest ifFalse: [^self].
	(DialogView confirm: 'Are you certain that you want to remove\all classes in category ''' withCRs, self category, ''' ?')
		ifFalse: [^self]. 
	Cursor execute showWhile: [self application removeCategory: self category].
	super classList isEmpty ifTrue: [^super removeCategory].
	self newCategoryList: category! !

"file out application with major version format"!

!ApplicationBrowser methodsFor: 'private-application functions'!

fileOutApplication
	"file out the current state of all sources of the application
	(like major version without creating directories or moving source pointers)"

	| stream |
	stream := self
		requestFileName: 'File out on'
		default: self application fileString , '.st'
		ifQuit: [^nil].
	stream := SourceCodeStream on: stream asFilename writeStream.

	self application fileOutAllOn: stream! !

!ApplicationBrowser methodsFor: 'private-application functions'!

spawnStatistics
	"open a workspace with statistical informations about the application"

	| s a |
	(s := TextStream on: (String new: 500))
		nextPutAll: 'classes:'; tab: 2; nextPutAll: (a := self application) classNames size printString; space; nextPut: $(; nextPutAll: a ownClassNames size printString; nextPut: $); cr;
		nextPutAll: 'selectors:'; tab: 1; nextPutAll: a selectors size printString; cr.

	a path notNil ifTrue:
		[OSErrorHolder nonexistentSignal
			handle: [:ex | ex proceedWith: 0]
			do: [s
				nextPutAll: 'sources:'; tab: 2; nextPutAll: a sourceFile name asFilename fileSize printString; cr.
				a version isInitialVersion ifFalse: [s nextPutAll: 'binary:'; tab: 2; nextPutAll: a instBossFilename fileSize printString; cr]]].

	s := s contents.
	ComposedTextView
		open: (ValueHolder with: s)
		label: 'Statistics on ' , a identifier storeString
		icon: (Icon constantNamed: #workspace)
		extent: s asComposedText extent + 30! !

!ApplicationBrowser methodsFor: 'private-application functions'!

showSelectorIntersections
	| map app |
	map := self application selectorIntersection.
	app := (map keys asSortedCollection: [:x :y | x name < y name])
		select: (self class promptFor: #selectThenBrowse)
		fromUserIfEmpty: [^nil].
	map := (map at: app) collect: [:assoc | assoc value , ' ' , assoc key].

	MethodListBrowser
		openListBrowserOn: map asSortedCollection
		label: 'Selectors in ', self application identifier, ' and ', app identifier! !

!ApplicationBrowser methodsFor: 'private-application functions'!

setPath
	self application requestPath: nil! !

!ApplicationBrowser methodsFor: 'initialize-release'!

close
	"close this browser"

	self dependents first topComponent controller close! !

!Application methodsFor: 'user-interface'!

closeBrowsers
	"close any application browser which is open on the receiver"

	self dependents do: [:dep | (dep isKindOf: ApplicationBrowser)
			ifTrue: [dep close]]! !

!ApplicationManagement methodsFor: 'browsing'!

browser
	"return a currently open browser on the receiver (nil if none is open)"

	^self dependents detect: [:dep | dep isKindOf: ApplicationManagementBrowser]
		ifNone: []! !

"fix: remove management dependency from its browser after closing"!

!ApplicationManagementBrowser methodsFor: 'dependents access'!

removeDependent: aDependent 
	"discard the management dependency to this browser if necessary"

	super removeDependent: aDependent.
	self dependents isEmpty ifTrue: [management removeDependent: self]! !

ApplicationManagementBrowser organization renameCategory: #application to: #'application list'!

ApplicationManagementBrowser organization renameCategory: #'private-menu messages' to: #'menu messages'!

"introducing hierarchical view on applications"!

Model subclass: #ApplicationManagementBrowser
	instanceVariableNames: 'management application filter hierarchy '
	classVariableNames: 'ApplicationMenu '
	poolDictionaries: ''
	category: 'Interface-Application'!

!ApplicationManagementBrowser class methodsFor: 'view creation'!

openOn: anApplicationManagement
	"Create and schedule a browser on ApplicationManager"

	| topView buttonOffset buttonWidth buttonCount browser |

	buttonOffset := [LabeledBooleanView defaultHeight negated].
	buttonCount := 2.
	buttonWidth := 1.0 / buttonCount.
	browser := self on: anApplicationManagement.
	(topView := DependentComposite new)
		add: browser applicationView
		in: (LayoutFrame new 
				rightFraction: 1.0;
				bottomFraction: 1 offset: buttonOffset);

		add: browser modifiedButtonView
		borderedIn: (LayoutFrame new
				topFraction: 1 offset: buttonOffset;
				rightFraction: buttonWidth;
				bottomFraction: 1);

		add: browser hierarchyButtonView
		borderedIn: (LayoutFrame new
				leftFraction: buttonWidth;
				topFraction: 1 offset: buttonOffset;
				rightFraction: 1.0;
				bottomFraction: 1).

	(ScheduledWindow model: browser label: 'Application Manager' minimumSize: 100@70)
		component: topView;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: 150@200! !

!ApplicationManagementBrowser class methodsFor: 'private'!

menuFor: aSymbol ifAbsent: aBlock 
	"returns the menu associated with <aSymbol>. If it has not yet been defined, 
	evaluate <aBlock> to get and cache the menu."

	ApplicationMenu == nil ifTrue: [ApplicationMenu := Dictionary new].
	^ApplicationMenu at: aSymbol ifAbsentPut: aBlock! !

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

noSelectionMenu
	"ApplicationManagementBrowser flushMenus"

	| fileMenu |
	^self menuFor: #noSelection ifAbsent: 
		[fileMenu := PopUpMenu
					labelArray: #('upgrade...' 'load...' 'load info...' 'set directories...' 'file out all changes...' 'condense changes...' )
					lines: #(4 5 )
					values: #(#upgradeMinor #loadApplication #loadApplicationInfo #setDirectory #collectAllChanges #condenseChanges ).
		PopUpMenu
			labels: 'file in/out\new application...\update' withCRs
			lines: #(1 2 )
			values: (Array with: fileMenu), #(#newApplication #updateList )]! !

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

 !                                                                                                                                                                                                                                                                                                                                                                           

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

applicationMenu
	"ApplicationManagementBrowser flushMenus"

	^self menuFor: #application ifAbsent:
		[| regMenu |
		regMenu := PopUpMenu
			labels: 'release...\deinstall...' withCRs
			values: #(#removeApplication #deinstallApplication).
		PopUpMenu
			labels: 'file in/out\open browser\new application...\unregister...\update' withCRs
			lines: #(1 4 )
			values: (Array with: self fileMenu)
					, #(#editApplication #newApplication )
					, (Array with: regMenu)
					, #(#updateList )]! !

!ApplicationManagementBrowser methodsFor: 'filter'!

hierarchy
	^hierarchy == nil ifFalse: [hierarchy key]! !

!ApplicationManagementBrowser methodsFor: 'filter'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                          

!ApplicationManagementBrowser methodsFor: 'filter'!

expanded 
	"return applications which are shown with their sub-applications"

	^hierarchy == nil ifTrue: [#()] ifFalse: [hierarchy value]! !

!ApplicationManagementBrowser methodsFor: 'filter'!

expanded: aCollection
	"set applications which are shown with their sub-applications"

	hierarchy == nil ifFalse: [hierarchy value: aCollection]! !

!ApplicationManagementBrowser methodsFor: 'filter'!

expandSublist: anApplication 
	"add anApplication to those which are shown with their sub-applications"

	| exp |
	((exp := self expanded) includes: anApplication)
		ifFalse: [self expanded: (exp copyWith: anApplication)]! !

!ApplicationManagementBrowser methodsFor: 'filter'!

contractSublist: anApplication 
	"remove anApplication from those which are shown with their sub-applications"

	self expanded: (self expanded copyWithout: anApplication)! !

!ApplicationManagementBrowser methodsFor: 'filter'!

toggleExpandContract
	"contract selected application, if expanded; otherwise expand, if possible"

	| app |
	((app := self selectedApplication) == nil or: [app necessary isEmpty])
		ifTrue: [^self].
	self perform: ((self expanded includes: app)
			ifTrue: [#contractSublist:]
			ifFalse: [#expandSublist:])
		with: app.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'application list'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   

!ApplicationManagementBrowser methodsFor: 'application list'!

applicationList
	| list |
	list := self hierarchy == nil
				ifTrue: [management sortedApplicationIdentifiers]
				ifFalse: [self hierarchyList].
	^filter == #isModified
		ifTrue: [list select: [:item | (item findString: 'm)' startingAt: item size - 1) ~~ 0]]
		ifFalse: [list]! !

!ApplicationManagementBrowser methodsFor: 'application list'!

 !                                                                                                                                                                                                

!ApplicationManagementBrowser methodsFor: 'application list'!

selectedApplication
	"returns the application object referred by the current selection; the selection 
	may hold an outdated identifier, so the simplest access may fail;
	update list if necessary"

	| id |
	^(id := self application) == nil ifFalse:
		[id := id asString copyWithout: Character tab.
		management applicationIdentified: id ifAbsent:
			[| app |
			app := management applicationNamed: (id copyUpTo: $().
			self application: (app == nil ifFalse: [app identifier]); updateList.
			app]]! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

editApplication
	"open an ApplicationBrowser on the selected application"

	| app |
	(app := self selectedApplication) == nil ifFalse: [app browse]! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

activateSelection
	"open an ApplicationBrowser on the selected application"

	self hierarchy == nil
		ifTrue: [Cursor execute showWhile: [self editApplication]]
		ifFalse: [self toggleExpandContract]! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 

!ApplicationManagementBrowser methodsFor: 'menu messages'!

loadApplication
	"show a collection of known applications, 
	file in the corresponding source code"

	| triple |
	(triple := self selectAvailableApplication) == nil ifTrue: [^self].
	management load: (triple at: 2) path: triple first version: triple last binary: nil! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          

!ApplicationManagementBrowser methodsFor: 'menu messages'!

fileOutInstallation
	| app |
	(app := self selectedApplication) == nil ifTrue: [^self].
	management fileOutInstallation: app! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

removeApplication
	"unregister selected application, close browsers, assign new source pointers"

	| app |
	((app := self selectedApplication) == nil or: [app == management systemChanges])
		ifTrue: [^nil].
	(DialogView confirm: 'Really unregister\''' withCRs , app identifier , ''' ?' initialAnswer: false)
		ifFalse: [^nil].
	app closeBrowsers; release.
	management unregister: app.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             

!ApplicationManagementBrowser methodsFor: 'private-subview creation'!

modifiedButtonView
	| baseModel valueModel view |
	baseModel := (PluggableAdaptor on: self) getSelector: #filter putSelector: #filter:.
	valueModel := (PluggableAdaptor on: baseModel) selectValue: #isModified.
	view := (LabeledBooleanView model: valueModel) label: 'modified'; beToggle.
	view controller beToggle.
	^view! !

!ApplicationManagementBrowser methodsFor: 'private-subview creation'!

hierarchyButtonView
	| baseModel valueModel view |
	baseModel := (PluggableAdaptor on: self) getSelector: #hierarchy putSelector: #hierarchy:.
	valueModel := (PluggableAdaptor on: baseModel) selectValue: #hierarchy.
	view := (LabeledBooleanView model: valueModel) label: 'hierarchy'; beToggle.
	view controller beToggle.
	^view! !

ApplicationManagementBrowser removeSelector: #openApplicationBrowser!

ApplicationManagementBrowser removeSelector: #release!

ApplicationManagementBrowser flushMenus!

"xxxxxx xxxxxxxxxxx xxxxxxxxxx xxxxxxx

xxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxx"!

"optimize addProtocol: better standardProtocols and proposal list, factorize working part of addProtocol"!

!Browser methodsFor: 'private-protocol functions'!

addProtocol
	| aString |
	self changeRequest ifFalse: [^self].
	aString := self
			prompt: 'Enter new protocol name'
			initially: (LastProtocol == nil ifTrue: ['protocol name'] ifFalse: [LastProtocol]).
	aString isEmpty ifTrue: [^self].
	self addProtocol: aString! !

!Browser methodsFor: 'private-protocol functions'!

addProtocol: aString
	| sym cls |
	((cls := self selectedClass) organization categories includes: (sym := aString asSymbol))
		ifTrue:
			[| msg |
			msg := protocol == nil ifTrue: ['to the end?'] ifFalse: ['before ''', protocol, '''?'].
			(DialogView confirm: 'Protocol already exists.\Do you want to move it\' withCRs, msg)
				ifFalse: [^self]].
	cls organization addCategory: sym before: protocol.
	self logProtocolChange: cls name, ' organization addCategory: ', sym storeString, ' before: ', protocol storeString.
	cls reorganize.
	self newProtocolList: sym! !

!ApplicationBrowser methodsFor: 'private-protocol functions'!

addProtocol: aString 
	| sym cls |
	sym := aString asSymbol.
	(self systemMode
		or: [((cls := self selectedClass) organization categories includes: sym) not])
		ifTrue: [super addProtocol: sym]
		ifFalse: 
			["protocol exists, but is hidden by non-system mode"
			self application includeProtocol: sym class: cls.
			self newProtocolList: sym]! !

!ApplicationBrowser methodsFor: 'private-protocol functions'!

addProtocol
	"propose a list of protocols that would be new in the selected class;
	the user may choose between them or create a real new one;
	some standard protocols are included"

	| new newString |
	(self systemMode or: [(new := self newProtocols) isEmpty])
		ifTrue: [^super addProtocol].
	self changeRequest ifFalse: [^self].
	new := new asSortedCollection asOrderedCollection
		addFirst: (newString := 'NEW PROTOCOL');
		selectFromUserIfEmpty: [^self].
	new == newString ifTrue: [super addProtocol] ifFalse: [self addProtocol: new]! !

!ApplicationBrowser methodsFor: 'private-protocol functions'!

newProtocols
	"return a set of protocol names which were new to the selected class; 
	construct the list by collecting inherited protocols (without Object), 
	add most often used protocols and LastProtocol; drop own protocols"

	| cls sup new |
	new := Set new: 20.
	sup := (cls := self selectedClass) allSuperclasses.
	1 to: (sup indexOf: Object class ifAbsent: [sup indexOf: Object]) - 1
		do: [:i | new addAll: (sup at: i) organization categories].
	^new add: LastProtocol; addAll: self standardProtocols; - cls organization categories! !

!ApplicationBrowser methodsFor: 'private'!

standardProtocols
	"return a collection of protocols which occur in lots of classes"
	"the following returns the top ten of protocol occurrence:
	| b skip |
	b := Bag new.
	skip := #().
	self applicationManagement do: [:ap | (skip includes: ap name)
		ifFalse: [ap classNames do: [:c | b addAll: (ap protocolsInClass: c asClass)]]].
	(b asDictionary associations asSortedCollection: [:x :y | x value > y value]) copyFrom: 1 to: 10"

	^self meta
		ifTrue: [#(#'class initialization' #'instance creation' #constants)]
		ifFalse: [#(#accessing #private #'initialize-release' #testing #printing #'fileIn/Out' #copying)]! !

!ApplicationBrowser methodsFor: 'inheritance pane'!

superClassMenu
	"return a Menu with all superclasses of selected class. Initial selection is the 
	current inheritance class"

	| sup |
	self selectedClass == nil ifTrue: [^nil].
	^(PopUpMenu
		labelArray: (sup := self superClassList reverse asArray)
		values: (sup collect: [:sym | [:ctrl | self inheritUpToClass: sym]]))
		selection: (sup indexOf: self inheritUpToClass)! !

!ApplicationBrowser class methodsFor: 'private-view creation'!

addInheritedClassViewTo: aContainer in: area on: aBrowser
	| edgeDecorator view |
	(view := SelectionInListView on: aBrowser printItems: false oneItem: true
				aspect: #inheritedFromClass change: nil
				list: #inheritedFromClass menu: #superClassMenu
				initialSelection: #inheritedFromClass)
		controller: (ControllerWithMenu new menuHolder: (MessageSend receiver: aBrowser selector: #superClassMenu)).

	edgeDecorator := (LookPreferences edgeDecorator on: view) noVerticalScrollBar noMenuBar.
	^aContainer add: edgeDecorator borderedIn: area! !

"get rid of any PopUpListController"
PopUpListController allInstancesDo: [:ctrl |
	ctrl view controller: nil; getController.
	ctrl breakDependents]!

PopUpListController removeFromSystem!

!ApplicationBrowser methodsFor: 'private-application functions'!

fileOutInstallation
	self applicationManagement fileOutInstallation: self application! !

!ApplicationBrowser methodsFor: 'private-application functions'!

showUseOfRequired
	self application reportUseOfApplications: self application necessary! !

!ApplicationBrowser class methodsFor: 'accessing menus'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        

!ApplicationBrowser class methodsFor: 'accessing menus'!

utilitiesMenu
	"return a menu for application and category operations"

	^self menuFor: #utilities ifAbsent:
		[PopUpMenu
			labels: 'intersections\dependencies\update\edit all' withCRs
			lines: #(2)
			values: #(#showSelectorIntersections #showUseOfRequired #updateCategories #editCategories )]! !

!ApplicationBrowser class methodsFor: 'accessing menus'!

applicationMenu
	"return a menu for application and category operations"

	^self menuFor: #application ifAbsent:
		[PopUpMenu
			labels: 'file in/out\spawn\add category\rename\remove\include\exclude\utilities\find class' withCRs
			lines: #(2 5 7 8)
			values: (Array with: self fileMenu),
				(Array with: self spawnMenu),
				#( addCategory renameCategory removeCategory ),
				(Array with: self includeMenu),
				#( excludeCategory ),
				(Array with: self utilitiesMenu),
				#( findClass )]! !

ApplicationBrowser flushMenus!

"reopen Application Management browser"
self applicationManagement browser == nil
	ifFalse: [ApplicationManagementBrowser open]!


' .ApplicationSource APL-Interface(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 9 May 1994 at 3:12:33 pm'!



"introduce disabling of menu items"!

!ApplicationBrowser methodsFor: 'category list'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       

!ApplicationBrowser methodsFor: 'category list'!

validateMenu: aPopUpMenu 
	"disable menu items which are not appropriate for certain applications;
	return the menu"

	^aPopUpMenu disableValues: self disabledMenuValues! !

!ApplicationBrowser methodsFor: 'category list'!

categoryMenu
	^self validateMenu: self class applicationMenu! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

disabledMenuValues
	"return method selectors which are not proper for certain applications"

	| app |
	(app := self selectedApplication) == nil
		ifTrue: [^#(#newMinorVersion #newMajorVersion #fileOutChanges #removeApplication #deinstallApplication #editApplication )].

	app == app systemApplication
		ifTrue: [^#(#upgradeMinor #newMinorVersion #newMajorVersion #fileOutUpgrade #fileOutInstallation #removeApplication #deinstallApplication )].

	app version isInitialVersion
		ifTrue: [^#(#upgradeMinor #newMinorVersion #fileOutUpgrade #fileOutInstallation )].
	^#()! !

!ApplicationManagementBrowser methodsFor: 'application list'!

validateMenu: aPopUpMenu 
	"disable menu items which are not appropriate for certain applications; 
	return the menu"

	^aPopUpMenu disableValues: self disabledMenuValues! !

!ApplicationManagementBrowser methodsFor: 'application list'!

applicationMenu
	^self validateMenu: (application == nil
			ifTrue: [self class noSelectionMenu]
			ifFalse: [self hierarchy == nil
					ifTrue: [self class applicationMenu]
					ifFalse: [self class hierarchyMenu]])! !

"fix: - improper return value of #selectAvailableApplication
- improper reference on selector #allItalic"!

!ApplicationManagementBrowser methodsFor: 'menu messages'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      

!ApplicationManagementBrowser methodsFor: 'menu messages'!

loadApplicationInfo
	"show a collection of known applications, 
	get the corresponding info/documentation"

	| triple info id txt |
	(triple := self selectAvailableApplication) == nil ifTrue: [^self].
	info := management loadInfo: (triple at: 2) path: triple first version: triple last.
	id := info first.
	txt := info at: 2.
	info last isEmpty
		ifTrue: [txt string = ('Application ', id)
			ifTrue: [^DialogView warn: 'Application ', id storeString, '\has no documentation or required applications' withCRs]]
		ifFalse: [txt := txt, ('\\comment:\' withCRs asText emphasizeAllWith: #italic), info last].
	ComposedTextView
		open: (ValueHolder with: txt)
		label: 'Information on ', id storeString
		icon: (Icon constantNamed: #workspace)
		extent: txt asComposedText extent + 30! !

"fileMenu: enter #newMajorVersion, remove #storeForDos"!

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

fileMenu
	"ApplicationManagementBrowser flushMenus"

	^self menuFor: #file ifAbsent: [PopUpMenu
			labelArray: #('upgrade...' 'load...' 'load info...' 'set directories...' 'new minor version' 'new major version...' 'file out changes...' 'file out all changes...' 'file out upgrade...' 'file out installation...' 'condense changes...' )
			lines: #(4 6 10 )
			values: #(#upgradeMinor #loadApplication #loadApplicationInfo #setDirectory #newMinorVersion #newMajorVersion #fileOutChanges #collectAllChanges #fileOutUpgrade #fileOutInstallation #condenseChanges )]! !

!ApplicationManagementBrowser class methodsFor: 'accessing menus'!

hierarchyMenu
	"ApplicationManagementBrowser flushMenus"

	^self menuFor: #hierarchy
		ifAbsent: 
			[| regMenu |
			regMenu := PopUpMenu
				labels: 'release...\deinstall...' withCRs
				values: #(#removeApplication #deinstallApplication ).
			PopUpMenu
				labels: 'file in/out\new application...\open browser\expand/contract\unregister...\update' withCRs
				lines: #(1 5 )
				values: (Array with: self fileMenu),
					#(#newApplication #editApplication #toggleExpandContract ),
					(Array with: regMenu),
					#(#updateList )]! !

ApplicationManagementBrowser flushMenus!

!ApplicationBrowser class methodsFor: 'private'!

actionButtonOn: aValueModel label: aString action: aBinaryBlock 
	"return a button which evaluates the given block with model aValueModel"

	^Button trigger
		model: ((PluggableAdaptor on: aValueModel)
			getBlock: [:m | false]
			putBlock: aBinaryBlock
			updateBlock: [:m :a :v | false]);
		label: aString! !

!ApplicationBrowser class methodsFor: 'private'!

acceptButtonFor: aDialogView 
	"return a button which accepts the given dialog; make the button default"

	^(self actionButtonOn: aDialogView label: 'accept' action: [:m :v | m controller accept])
		beDefault! !

!ApplicationBrowser class methodsFor: 'private'!

cancelButtonFor: aDialogView 
	"return a button which cancels the given dialog;
	assume that the dialog model is set to a ValueModel on a Boolean"

	^Button trigger model: aDialogView model; label: 'cancel'! !

!ApplicationBrowser class methodsFor: 'private'!

radioButtonOn: aValueModel label: aString 
	"return a LabeledBooleanView, storing aString to aValueModel when active"

	| btn |
	(btn := PluggableAdaptor on: aValueModel) selectValue: aString.
	(btn := LabeledBooleanView model: btn) beRadioButton; label: aString.
	btn controller beToggle.
	^btn! !

!ApplicationBrowser class methodsFor: 'private'!

multipleSelectionListOn: aValueModel
	"return a SelectionSetInListView working on the list under aValueModel"

	^SelectionSetInListView
		on: aValueModel
		aspect: nil change: #value: list: #value menu: nil initialSelection: #value
		useIndex: false.
! !

!ApplicationBrowser class methodsFor: 'dialogs'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             

!ApplicationBrowser class methodsFor: 'dialogs'!

multipleSelect: aCollection prompt: aString onCancel: aBlock
	"open a dialog for selecting one or more elements in aCollection;
	return the Collection of selected elements (value of aBlock if the user cancels)"

	| listHolder listView listWrapper cancel dialog buttons ofs layout |
	listView := self multipleSelectionListOn: (listHolder := ValueHolder with: aCollection).
	listWrapper := BoundedWrapper on: (LookPreferences edgeDecorator on: listView).

	(dialog := DialogView model: (cancel := ValueHolder newBoolean))
		addVerticalSpace: 4;
		addTextLabel: aString;
		addVerticalSpace: 4;
		addWrapper: listWrapper fromX: 0.0 toX: 1.0;
		addVerticalSpace: (ofs := 4);
		addVerticalSpace: 4.
	(layout := listWrapper layout) bottomOffset: layout bottomOffset + ofs.

	buttons := Array 
		with: (self acceptButtonFor: dialog)
		with: (self actionButtonOn: listHolder label: 'all' action: [:m :v | m value: aCollection])
		with: (self actionButtonOn: listHolder label: 'none' action: [:m :v | m value: Set new])
		with: (self cancelButtonFor: dialog).

	dialog
		addRow: (1 to: buttons size) fromX: 0 toX: 1 collect: [:i | BoundedWrapper on: (buttons at: i)];
		addVerticalSpace: 4;
		open.

	^cancel value
		ifTrue: [aBlock value]
		ifFalse: [aCollection select: [:each | listHolder value includes: each]]! !

!ApplicationManagementBrowser class methodsFor: 'dialogs'!

multipleSelect: aCollection prompt: aString onCancel: aBlock
	"open a dialog for selecting one or more elements in aCollection;
	return the Collection of selected elements (value of aBlock if the user cancels)"

	| listHolder listView listWrapper cancel dialog buttons ofs layout |
	listView := ApplicationBrowser multipleSelectionListOn: (listHolder := ValueHolder with: aCollection asSortedCollection).
	listWrapper := BoundedWrapper on: (LookPreferences edgeDecorator on: listView).

	(dialog := DialogView model: (cancel := ValueHolder newBoolean))
		addVerticalSpace: 4;
		addTextLabel: aString;
		addVerticalSpace: 4;
		addWrapper: listWrapper fromX: 0.0 toX: 1.0;
		addVerticalSpace: (ofs := 4);
		addVerticalSpace: 4.
	(layout := listWrapper layout) bottomOffset: layout bottomOffset + ofs.

	(buttons := Array new: 6)
		at: 1 put: (ApplicationBrowser acceptButtonFor: dialog);
		at: buttons size put: (ApplicationBrowser cancelButtonFor: dialog).

	buttons at: 2 put: (ApplicationBrowser actionButtonOn: listHolder label: 'all'
		action: [:m :v | m value: aCollection]).
	buttons at: 3 put: (ApplicationBrowser actionButtonOn: listHolder label: 'none'
		action: [:m :v | m value: Set new]).
	buttons at: 4 put: (ApplicationBrowser actionButtonOn: listHolder label: 'add'
		action: [:m :v | | newPath |
				newPath := DialogView requestFileName: 'Enter a path' default: ParagraphEditor currentSelection version: #directory ifFail: [].
				newPath == nil ifFalse:
					[m value: (aCollection add: newPath; asSortedCollection); changed]]).
	buttons at: 5 put: (ApplicationBrowser actionButtonOn: listHolder label: 'remove'
		action: [:m :v | | elem |
				(elem := aCollection asSortedCollection select: 'Select path to be removed' fromUserIfEmpty: []) == nil
					ifFalse:
						[m value: (aCollection remove: elem; asSortedCollection); changed]]).

	dialog
		addRow: (1 to: buttons size) fromX: 0 toX: 1 collect: [:i | BoundedWrapper on: (buttons at: i)];
		addVerticalSpace: 4;
		open.

	^cancel value
		ifTrue: [aBlock value]
		ifFalse: [aCollection select: [:each | listHolder value includes: each]]! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

selectAvailableApplication
	"show a menu of known applications;
	return a triple (path, appl.name, version) e.g. (aFilename('/home/src') 'myApplic' 'v2')"

	| dirs avail choice labels values ix |
	dirs := self class multipleSelect: management directories prompt: 'Setup search path' onCancel: [^nil].
	Cursor read showWhile: 
		[avail := OrderedCollection new.
		dirs do: [:d | avail addAll: (management applicationDirectoriesIn: d asFilename)]].
	(avail ":= management availableApplications") isEmpty
		ifTrue: [DialogView userInfo: 'No application directories found'. ^nil].

	labels := (values := Array new: avail size) copy.
	ix := 0.
	avail do: [:triple | | label maj val |
		label := triple first asString, ' ', (triple at: 2).
		(maj := triple last) size = 1
			ifTrue: [maj := maj first. label := label, ' ', maj. triple at: 3 put: maj. val := triple]
			ifFalse: [val := (PopUpMenu labelArray: maj
						values: (maj collect: [:str | triple copy at: 3 put: str; yourself]))].
		labels at: (ix := ix + 1) put: label.
		values at: ix put: val].

	choice := (PopUpMenu labelArray: labels values: values)
				startUpWithHeading: 'Select an application'.
	^choice == 0 ifFalse: [choice].! !

!ApplicationBrowser methodsFor: 'text'!

info
	"return source file name for selected method or class"

	| cls owners |
	(cls := self selectedClass) == nil ifTrue: [^''].
	selector notNil 
		ifTrue: [| method index |
			method := Dictionary keyNotFoundSignal handle: [:ex | ^'']
				do: [self classOfSelectedMethod compiledMethodAt: self selector asSymbol].
			^(index := method fileIndex) isInteger
				ifTrue: [self sourceFilename: index]
				ifFalse: [index sourceFileHolderId]].

	textMode == #comment
		ifTrue: [| src | 
			(src := self nonMetaClass organization globalComment) == nil
				ifTrue: [^''].
			^src class == SourceReference
				ifTrue: [src sourceFileHolderId]
				ifFalse: [self sourceFilename: src sourceFileNumber]].

	"test if class is defined in an application"
	(owners := self applicationManagement applicationsIncludingDefinition: cls) isEmpty
		ifFalse: [^owners first identifier].

	"test if class is in system changes"
	((ChangeSet current isNew: cls) or: [ChangeSet current atClass: cls includes: #change])
		ifTrue: [^self sourceFilename: SourceFileManager default target]. 

	"class must be defined by system sources"
	^self sourceFilename: 1! !

!ApplicationBrowser methodsFor: 'private-category functions'!

addCategory
	"inherit category adding; enable system mode if necessary, else new categories are invisible"

	| cat cats |
	cat := self category.
	cats := organization categories.
	super addCategory.
	organization categories == cats | self systemMode ifTrue: [^self].
	category := (cats := organization categories) at: (cats indexOf: cat ifAbsent: [cats size +1]) -1.
	self systemMode: true; changed: #systemMode! !

!Browser methodsFor: 'private-category functions'!

addCategory
	"Prompt the user for a new category name. Insert the category into 
	the class organization before the currently selected category."

	| newCategory |
	self changeRequest ifFalse: [^self].
	((newCategory := self prompt: 'Enter new category name' initially: 'category name') isEmpty
		or: [(newCategory := newCategory asSymbol) == category])
		ifTrue: [^self].
	
	(organization categories includes: newCategory)
		ifTrue:
			[| msg |
			msg := category == nil ifTrue: ['to the end?'] ifFalse: ['before ''', category, '''?'].
			(DialogView confirm: 'Category already exists.\Do you want to move it\' withCRs, msg)
				ifFalse: [^self]].
	organization addCategory: newCategory before: category.
	ChangeSet current reorganizeSystem.
	self newCategoryList: newCategory! !

!ApplicationBrowser methodsFor: 'updating'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   

!ApplicationBrowser methodsFor: 'private'!

spawnChanges: aChangeSequence label: aString select: aBoolean 
	"open a change sequence browser on aChangeSequence, 
	filtering on the selected method (if aBoolean is true)"

	| chgList index |
	aBoolean ifTrue: 
		[| sel cname |
		sel := self selector asSymbol.
		cname := self classOfSelectedMethod asSymbol.
		index := aChangeSequence findLast: [:chg | chg selector == sel and: [chg className == cname]].
		index = 0 ifTrue:
			[^DialogView userInfo: 'There are no changes on selector\' withCRs , sel storeString]].

	chgList := (EnhancedChangeList onApplication: self application)
				listName: aString
				changes: aChangeSequence
				removed: #().
	aBoolean
		ifTrue: [chgList
			toggleListIndex: index;
			switchFilter: 5 perform: #selector;
			switchFilter: 3 perform: #className].
	ChangeSequenceView openOn: chgList! !

!ApplicationBrowser methodsFor: 'private-application functions'!

newMinorVersion
	"append changes to source file, clear changes and increment minor version"

	self application newMinorVersion! !

!ApplicationBrowser methodsFor: 'private-application functions'!

newMajorVersion
	self application newMajorVersion! !

"fix: mnu during #hierarchyList when a required application is not loaded
- try to keep current selection when switching hierarchy mode"!

!ApplicationManagementBrowser methodsFor: 'application list'!

hierarchyList
	| man exp list strings recursive |
	exp := self expanded.
	list := OrderedCollection new.
	(man := self applicationManagement) do: [:app | | depend |
		"collect all applications, reject those supporting another one" 
		depend := nil.
		man do: [:ap | (ap requiresApplication: app) ifTrue: [depend := ap]].
		(depend == nil
			or: [app requiresApplication: depend	"direct vice versa dependent"])
			ifTrue: [list add: app]].

	strings := OrderedCollection new.
	recursive := [:app :depth | | txt |
		txt := (((String new: depth) atAllPut: Character tab), app identifier) asText.
		strings add: txt.
		app necessary isEmpty ifFalse:
			[(exp includes: app)
				ifTrue: [app necessary do: [:req |
					recursive value: (man applicationNamed: req ifAbsent: [Application name: '[', req, ']']) value: depth+1]]
				ifFalse: [txt allBold]]].

	list do: [:app | recursive value: app value: 0].
	^strings! !

!ApplicationManagementBrowser methodsFor: 'filter'!

hierarchy: mode 
	"set the hierarchy mode indicator to 
	- nil : hierarchy mode is off, no currently expanded applications 
	- nil -> expanded : hierarchy mode is off, remember currently expanded applications 
	- mode -> expanded : hierarchy mode is on, with currently expanded applications 
	(mode argument should be #hierarchy or nil)"

	| exp selected |
	hierarchy := mode == nil & (exp := self expanded) isEmpty ifFalse: [mode -> exp].
	(selected := self application) == nil ifFalse: [selected := mode == nil
			ifTrue: [selected asString copyWithout: Character tab]
			ifFalse: [self hierarchyList
				detect: [:txt | (txt findString: selected startingAt: 1) > 0] ifNone: []]].
	self application: selected; updateList! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

newMinorVersion
	"append changes to source file, clear changes and increment minor version"

	self selectedApplication newMinorVersion.
	self updateList! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

newMajorVersion
	self selectedApplication newMajorVersion.
	self updateList! !

ApplicationManagementBrowser removeSelector: #storeDOS!


' .ApplicationSource APL-Interface(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 18 May 1994 at 11:29:38 am'!



!ApplicationManagementBrowser methodsFor: 'menu messages'!

deinstallApplication
	"remove all classes/selectors of the selected application from system 
	and unregister (see also Application>deinstall)"

	| app msg |
	msg := '''', (app := self selectedApplication) identifier , ''' ?'.
	(DialogView confirm: 'Really remove all classes and selectors in\' withCRs , msg initialAnswer: false)
		ifFalse: [^nil].
	(DialogView confirm: 'Do you really want to deinstall ', msg)
		ifFalse: [^nil].
	app closeBrowsers; deinstall.
	self updateList! !

!ApplicationBrowser class methodsFor: 'private'!

title: aString forDialog: aDialogView 
	"add aString as label for the given dialog; return the new y-position"

	^aDialogView addVerticalSpace: 4; addTextLabel: aString asText allBold asComposedText centered; addVerticalSpace: 8; yPosition! !

!ApplicationBrowser class methodsFor: 'private'!

addButtons: anArray for: aDialogView 
	"add buttons as specified by anArray to the given dialog
	(yet supporting #accept and #cancel only)"

	| yp i btnspace |
	yp := aDialogView yPosition + 16.
	i := 1.
	btnspace := 0.5 / anArray size.
	anArray do: [:symbol | | btn |
		btn := #(#acceptButtonFor: #cancelButtonFor:) at: (#(accept cancel) indexOf: symbol).
		btn := self perform: btn with: aDialogView.
		aDialogView yPosition: yp; addWrapper: (BoundedWrapper on: btn) atX: btnspace * i.
		i := i+2].
	aDialogView addVerticalSpace: 4! !

!ApplicationBrowser class methodsFor: 'dialogs'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     

!ApplicationBrowser class methodsFor: 'dialogs'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     

!ApplicationBrowser class methodsFor: 'accessing menus'!

fileMenu
	"return a menu for application and category file in/out operations"

	^self menuFor: #file ifAbsent:
		[PopUpMenu
			labelArray: #('file in...' 'upgrade...' 'new minor version' 'new major version...' 'file out changes...' 'file out category...' 'file out application...' 'file out installation...' 'print out category' 'print out application' 'set path' 'set platform names')
			lines: #(1 2 4 8 10)
			values: #(#fileIn #upgradeMinor #newMinorVersion #newMajorVersion #fileOutChanges #fileOutCategory #fileOutApplication #fileOutInstallation #printOutCategory #printOutApplication #setPath #setNamesForPlatforms)]! !

ApplicationBrowser flushMenus!

!ApplicationBrowser methodsFor: 'category list'!

disabledMenuValues
	"return method selectors which are not proper for system applications"

	| noCat app |
	noCat := self category == nil
		ifTrue: [#(spawnCategory printOutCategory fileOutCategory renameCategory removeCategory includeCategory excludeCategory )]
		ifFalse: [#()].

	(app := self application) == app systemApplication
		ifTrue: [^#(#upgradeMinor #newMinorVersion #newMajorVersion #fileOutInstallation #setPath #setNamesForPlatforms #spawnHistory #spawnFullHistory #spawnInstallation #spawnPostInstallation #spawnRequired #spawnStatistics #addRequiredApplication #findRequiredApplications #showUseOfRequired ), noCat].

	app version isInitialVersion
		ifTrue: [^#(#upgradeMinor #newMinorVersion #fileOutInstallation #spawnHistory #spawnFullHistory ), noCat].

	app necessary isEmpty
		ifTrue: [^#(#showUseOfRequired ), noCat].
	^noCat! !

!ApplicationBrowser methodsFor: 'updating'!

newApplicationInfo
	"the application information has changed; force an update if necessary"

	self category == nil & self documentationMode not ifTrue: [self changed: #text].! !

!ApplicationBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a minor version release via application management"

	| apm ap |
	anAspect == #version ifFalse: [^super update: anAspect].
	apm := (ap := application) applicationManagement.
	(application := apm applicationNamed: ap name ifAbsent: [ap]) == ap
		ifFalse: 
			["my application unregistered, attach to the new one"
			ap removeDependent: self.
			application addDependent: self].
	self newApplicationInfo.
	self changed: #windowLabel with: application identifier.
	apm changed: #version! !

!ApplicationBrowser methodsFor: 'private-application functions'!

addRequiredApplication
	"prompt the user for an application which should be regarded as necessary for 
	running the underlying application and add it to the set of required applications"

	| app |
	(app := self applicationManagement selectApplication: 'Select a necessary application') == nil ifTrue: [^self].
	self application includeApplication: app.
	self newApplicationInfo! !

!ApplicationBrowser methodsFor: 'private-application functions'!

findRequiredApplications
	self application findRequiredApplications.
	self newApplicationInfo! !

!ApplicationBrowser methodsFor: 'private-application functions'!

setNamesForPlatforms
	"open a dialog to define application names compatible to supported platforms; 
	return the name map (nil on cancel)"

	self application requestPlatformNamesOnCancel: []! !

ApplicationBrowser removeSelector: #setNameForDos!


' .ApplicationSource APL-Interface(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 May 1994 at 12:42:33 pm'!



!ApplicationBrowser class methodsFor: 'dialogs'!

request: messageString platformNames: aDictionary default: aString onCancel: aBlock
	"open a dialog for entering compatible filenames; return the new dictionary"

	| platforms currentValues dialog cancel y1 newValues |
	currentValues := (platforms := StorableObjectAccessor supportedPlatforms)
		collect: [:platf | ValueHolder with: (aDictionary at: platf
			ifAbsent:
				["suggest a filename, indicated by a ?"
				| cls try |
				try := (cls := Filename defaultClassForPlatform: platf)
					suggest: (cls name == #DosFilename | (cls name == #OS2Filename)
						ifTrue: [aString copyFrom: 1 to: (aString size min: 8)]
						ifFalse: [aString]).
				try copyWith: $?])].

	dialog := DialogView model: (cancel := ValueHolder newBoolean).
	y1 := self title: messageString forDialog: (dialog width: 250).

	currentValues with: platforms do: [:vmodel :platform |
		y1 := dialog 
			yPosition: y1 + 10;
			addVisual: platform asComposedText fromX: 0 toX: 0.2;
			yPosition: y1 + 8;
			addTextFieldOn: vmodel initially: vmodel value fromX: 0.25 toX: 1;
			yPosition].
	self addButtons: #(accept cancel) for: dialog.
	dialog open.

	cancel value ifTrue: [^aBlock value].

	newValues := Dictionary new.
	currentValues with: platforms do: [:vmodel :platform | | fname cls guess |
		"discard the ? if any"
		((fname := vmodel value) isEmpty not and: [fname last == $?])
			ifTrue: [fname := fname copyFrom: 1 to: fname size -1].
		"check validity"
		cls := Filename defaultClassForPlatform: platform.
		[(guess := cls suggest: fname) sameAs: fname]
			whileFalse:
				[fname := DialogView
					request: ('''', fname, ''' is not compatible to ', cls name, '.\Retry...' withCRs)
					initialAnswer: guess
					ifEmpty: [^aBlock value].
				((fname findString: cls defaultDirectoryName startingAt: 1) > 1
					or: [fname includes: cls separator])
					ifTrue: [fname := guess]].
		newValues at: platform put: fname].
	^newValues! !

ApplicationBrowser class removeSelector: #request:platformNames:onCancel:!

!ApplicationBrowser class methodsFor: 'dialogs'!

request: messageString path: initialPath filename: initialFilename onCancel: aBlock
	"open a dialog for entering a Filename for a directory path and a file and a platform specification;
	return an Array(String, String, String|nil) (value of aBlock if the user cancels)"

	| pathHolder fileHolder cancel dialog platforms platform y0 y1 check y2 |
	platforms := StorableObjectAccessor exportPlatforms.
	platform := ValueHolder new.

	dialog := DialogView model: (cancel := ValueHolder newBoolean).
	y0 := self title: messageString forDialog: (dialog width: 250).

	y1 := dialog
		addTextLabel: 'Target directory' atX: 0;
		addVerticalSpace: 4;
		addTextFieldOn: (pathHolder := ValueHolder newString) initially: initialPath asString fromX: 0 toX: 0.7;
		addVerticalSpace: 8;
		addTextLabel: 'Filename' atX: 0;
		addVerticalSpace: 4;
		addTextFieldOn: (fileHolder := ValueHolder newString) initially: initialFilename asString fromX: 0 toX: 0.7;
		yPosition.

	y2 := dialog
		yPosition: y0;
		addColumn: (1 to: platforms size) fromX: 0.75 toX: 1 collect:
			[:i | BoundedWrapper on: (self radioButtonOn: platform label: (platforms at: i))];
		yPosition.

	dialog yPosition: (y1 max: y2).
	self addButtons: #(#accept #cancel) for: dialog.
	dialog open.

	cancel value ifTrue: [^aBlock value].

	check := [:fname | Filename named: fname ifFail: [:ignore |
		DialogView warn: 'Illegal filename: ' , fname. ^aBlock value]].
	(pathHolder := pathHolder value) isEmpty
		ifTrue: [pathHolder := Filename currentDirectory asString]
		ifFalse: [check value: pathHolder].
	(fileHolder := fileHolder value) isEmpty
		ifTrue: [fileHolder := nil]
		ifFalse: [check value: fileHolder].

	^Array with: pathHolder with: fileHolder with: platform value! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

collectAllChanges
	"file out all changes of all applications into one file"

	| fileName fileStream modified |
	modified := OrderedCollection new.
	management do: [:app | app isModified ifTrue: [modified add: app name]].
	modified isEmpty ifTrue: [DialogView userInfo: 'There are no changes to file out.'].
	modified := ApplicationBrowser multipleSelect: modified prompt: 'Select applications' onCancel: [^self].

	(fileName := DialogView
		requestNewFileName: 'File out changes of all applications on'
		default: 'allchanges.cs') isEmpty
		ifTrue: [^self].

	fileStream := fileName asFilename writeStream.
	Cursor write showWhile:
		[[management do: [:app | (modified includes: app name) ifTrue: [app changes fileOutOn: fileStream]]]
			valueNowOrOnUnwindDo: [fileStream close]]! !

"fix: initial selection for application browsers failed when passing a metaclass"!

!Application methodsFor: 'user-interface'!

browseInitial: aClass selector: aSymbol
	"open a browser preselected on the given class and selector;
	return the browser"

	| dep browser cls meta |
	dep := self dependents.
	self browse.
	browser := self dependents detect: [:d | (dep includes: d) not] ifNone: [^nil].
	aClass isNil ifTrue: [^browser].
	cls := (meta := aClass isMeta) ifTrue: [aClass soleInstance] ifFalse: [aClass].
	^browser
		category: cls category;
		className: cls name;
		meta: meta;
		protocol: (aClass organization categoryOfElement: aSymbol);
		selector: aSymbol;
		changed: #category! !


' .ApplicationSource APL-Interface(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 September 1994 at 8:44:59 pm'!



"fixes:
- omitting change request during several application browser functions
- AppMan browser buttons were toggle (instead of toggle)
- class owner query failed if instance switch was false

add ons:
- confirm closing the AppMan browser
- enhanced functionality on menu item change directory
- required by...: query to find out applications using the edited one
- implementors/messages operator restricted to applications
"!

!ApplicationManagementBrowser methodsFor: 'updating'!

changeRequest
	^thisContext sender selector ~~ #close
		or: [DialogView confirm: 'Do you really want to close\the Application Management Browser?' withCRs]! !

!ApplicationBrowser class methodsFor: 'private'!

addButtons: anArray for: aDialogView 
	"add buttons as specified by anArray to the given dialog 
	(yet supporting #accept and #cancel only)"

	aDialogView addButtons: (anArray collect: 
		[:symbol | | btn |
		btn := #(#acceptButtonFor: #cancelButtonFor: ) at: (#(#accept #cancel ) indexOf: symbol).
		self perform: btn with: aDialogView])! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

setDirectory
	management changePaths! !

!ApplicationBrowser class methodsFor: 'accessing menus'!

utilitiesMenu
	"return a menu for application and category operations"

	^self menuFor: #utilities ifAbsent:
		[PopUpMenu
			labels: 'intersections\dependencies\required by...\update\edit all' withCRs
			lines: #(3)
			values: #(#showSelectorIntersections #showUseOfRequired #showApplicationUsers #updateCategories #editCategories )]! !

!ApplicationBrowser class methodsFor: 'accessing menus'!

selectorRefsMenu
	"return a menu for cross references wrt. instances of Application"

	^self menuFor: #'selector>refs' ifAbsent:
		[PopUpMenu
			labelArray: #(owners senders implementors messages)
			values: #(showMethodOwners showMethodUsers browseImplementorsInApps browseMessagesInApps )]! !

ApplicationBrowser flushMenus!

!Browser class methodsFor: 'browsing'!

select: promptString from: symbolCollection onCancel: aBlock
	"show a menu of the given symbols, abbreviated to 20 characters;
	return the selected symbol (result of aBlock if the user cancels)
	(will not work on sets)"

	| shortened choice |
	(shortened := symbolCollection copy asOrderedCollection) isEmpty
		ifTrue: [^aBlock value].
	1 to: shortened size do: [:i | shortened at: i put: ((shortened at: i) contractTo: 25)].
	choice := shortened select: promptString fromUserIfEmpty: [^aBlock value].
	^symbolCollection at: (shortened findFirst: [:str | str == choice])! !

!Browser class methodsFor: 'browsing'!

selectAndBrowseWithApps: selectorCollection
	"show a menu of the given selectors, abbreviated to a maximum size;
	show a menu of application defining the chose selector;
	browse implementors restricted to selected application(s)"

	| aSymbol |
	selectorCollection isEmpty ifTrue: [^DialogView warn: 'No messages'].
	aSymbol := self select: '' from: selectorCollection onCancel: [^self].
	self browseImplementorsInApps: aSymbol! !

!Browser class methodsFor: 'browsing'!

browseImplementorsInApps: aSymbol
	"show a menu of applications defining the selector given by aSymbol;
	browse implementors restricted to those defined in selected application(s)"

	| classes map app all label sys |
	classes := SortedCollection new.
	Cursor execute showWhile:
		[Smalltalk allBehaviorsDo: [:cls | (cls includesSelector: aSymbol) ifTrue: [classes add: cls name]]].
	classes isEmpty ifTrue: [^DialogView warn: 'Nobody'].

	(map := OrderedCollection new: 50) add: (all := '*** all ***') -> nil.
	self applicationManagement do: [:ap | | classes2 |
		classes2 := classes select: [:clsname | ap includesSelector: aSymbol class: clsname].
		classes2 isEmpty ifFalse: [map add: ap name -> classes2]].

	label := map size == 1
		ifTrue:
			[map at: 1 put: (sys := '*** system ***') -> nil.
			'No application implements that.']
		ifFalse:
			[map size == 2 ifTrue: [map removeFirst].
			'in which application?'].

	app := (map collect: [:assoc | assoc key]) select: label fromUserIfEmpty: [^self].
	classes := app == all
		ifTrue:
			[| classes2 |
			classes2 := Set new.
			map removeFirst; do: [:assoc | classes2 addAll: assoc value].
			label := ' in any application'.
			classes2 asSortedCollection]
		ifFalse: [app == sys
			ifTrue: [label := ''. classes]
			ifFalse:
				[label := ' in ', app.
				(map at: (map findFirst: [:assoc | assoc key == app])) value]].

	MethodListBrowser
		openListBrowserOn: (classes collect: [:clsname | clsname, ' ', aSymbol])
		label: 'Implementors of #' , aSymbol, label! !

!ApplicationBrowser methodsFor: 'private-application functions'!

showApplicationUsers
	"show a list with all applications requiring this application immediately; 
	selecting an item opens a browser on the selected application"

	| apps app |
	apps := self applicationManagement applicationsRequiring: self application.
	app := apps select: (self class promptFor: #selectThenBrowse)
				fromUserIfEmpty: [^nil].
	app browse! !

!ApplicationBrowser methodsFor: 'private-class functions'!

includeClass
	"include definition and all selectors of selected class (class and metaclass)"

	self changeRequest ifFalse: [^self].
	self includeClass: self nonMetaClass.
	self newClassList: className! !

!Browser methodsFor: 'private-selector functions'!

browseMessagesInApps
	self class selectAndBrowseWithApps:
		(self selectedClass compiledMethodAt: self selector asSymbol)
			allSymbolLiterals asSortedCollection! !

!Browser methodsFor: 'private-selector functions'!

browseImplementorsInApps
	"show a menu of applications defining the selected selector;
	browse implementors restricted to those defined in selected application(s)"

	self class browseImplementorsInApps: self selector asSymbol! !

!ApplicationBrowser class methodsFor: 'private-view creation'!

addSystemButtonViewTo: aContainer in: area on: aBrowser
	| btn |
	btn := (PluggableAdaptor on: aBrowser)
				getSelector: #systemMode putSelector: #systemMode:.
	btn := Button switch model: btn; label: 'system'.
	^aContainer add: btn borderedIn: area! !

!ApplicationBrowser class methodsFor: 'private-view creation'!

addDocumentationButtonViewTo: aContainer in: area on: aBrowser
	| btn |
	btn := (PluggableAdaptor on: aBrowser)
				getSelector: #documentationMode putSelector: #documentationMode:.
	btn := Button switch model: btn; label: 'doc'.
	^aContainer add: btn borderedIn: area! !

!ApplicationBrowser class methodsFor: 'private-view creation'!

addInheritedClassViewTo: aContainer in: area on: aBrowser
	| edgeDecorator view |
	(view := SelectionInListView on: aBrowser printItems: false oneItem: true
				aspect: #inheritedFromClass change: nil
				list: #inheritedFromClass menu: #superClassMenu
				initialSelection: #inheritedFromClass)
		controller: (ControllerWithMenu new menuHolder: (MessageSend receiver: aBrowser selector: #superClassMenu)).

	edgeDecorator := (LookPreferences edgeDecorator on: view) noVerticalScrollBar noMenuBar.
	^aContainer add: edgeDecorator in: area! !

!ApplicationManagementBrowser methodsFor: 'private-subview creation'!

hierarchyButtonView
	| btn |
	btn := (PluggableAdaptor on: self) getSelector: #hierarchy putSelector: #hierarchy:.
	btn := (PluggableAdaptor on: btn) selectValue: #hierarchy.
	^(Button switch model: btn) label: 'hierarchy'! !

!ApplicationManagementBrowser methodsFor: 'private-subview creation'!

modifiedButtonView
	| btn |
	btn := (PluggableAdaptor on: self) getSelector: #filter putSelector: #filter:.
	btn := (PluggableAdaptor on: btn) selectValue: #isModified.
	^(Button switch model: btn) label: 'modified'! !

!ApplicationBrowser class methodsFor: 'dialogs'!

request: messageString platformNames: aDictionary default: aString onCancel: aBlock
	"open a dialog for entering compatible filenames; return the new dictionary"

	| platforms currentValues dialog cancel y1 newValues |
	currentValues := (platforms := StorableObjectAccessor supportedPlatforms)
		collect: [:platf | ValueHolder with: (aDictionary at: platf
			ifAbsent:
				["suggest a filename, indicated by a ?"
				| try |
				try := (Filename defaultClassForPlatform: platf)
					suggest: (platf == #dos | (platf == #os2)
						ifTrue: [aString copyFrom: 1 to: (aString size min: 8)]
						ifFalse: [aString]).
				try copyWith: $?])].

	dialog := DialogView model: (cancel := ValueHolder newBoolean).
	y1 := self title: messageString forDialog: (dialog width: 250).

	currentValues with: platforms do: [:vmodel :platform |
		y1 := dialog 
			yPosition: y1 + 10;
			addVisual: platform asComposedText fromX: 0 toX: 0.2;
			yPosition: y1 + 8;
			addTextFieldOn: vmodel initially: vmodel value fromX: 0.25 toX: 1;
			yPosition].
	self addButtons: #(accept cancel) for: dialog.
	dialog open.

	cancel value ifTrue: [^aBlock value].

	newValues := Dictionary new.
	currentValues with: platforms do: [:vmodel :platform | | fname cls guess |
		"discard the ? if any"
		((fname := vmodel value) isEmpty not and: [fname last == $?])
			ifTrue: [fname := fname copyFrom: 1 to: fname size -1].
		"check validity"
		cls := Filename defaultClassForPlatform: platform.
		[(guess := cls suggest: fname) sameAs: fname]
			whileFalse:
				[fname := DialogView
					request: ('''', fname, ''' is not compatible to ', cls name, '.\Retry...' withCRs)
					initialAnswer: guess
					ifEmpty: [^aBlock value].
				((fname findString: cls defaultDirectoryName startingAt: 1) > 1
					or: [fname includes: cls separator])
					ifTrue: [fname := guess]].
		newValues at: platform put: fname].
	^newValues! !

!ApplicationManagementBrowser class methodsFor: 'view creation'!

openOn: anApplicationManagement
	"Create and schedule a browser on ApplicationManager"

	| topView buttonOffset buttonWidth buttonCount browser |

	buttonOffset := [LabeledBooleanView defaultHeight negated].
	buttonCount := 2.
	buttonWidth := 1.0 / buttonCount.
	browser := self on: anApplicationManagement.
	(topView := DependentComposite new)
		add: browser applicationView
		in: (LayoutFrame new 
				rightFraction: 1.0;
				bottomFraction: 1 offset: buttonOffset);

		add: browser modifiedButtonView
		in: (LayoutFrame new
				topFraction: 1 offset: buttonOffset;
				rightFraction: buttonWidth;
				bottomFraction: 1);

		add: browser hierarchyButtonView
		in: (LayoutFrame new
				leftFraction: buttonWidth;
				topFraction: 1 offset: buttonOffset;
				rightFraction: 1.0;
				bottomFraction: 1).

	(ScheduledWindow model: browser label: 'Application Manager' minimumSize: 100@70)
		component: topView;
		icon: (Icon constantNamed: #systemBrowser);
		openWithExtent: 160@200;
		label: 'Application Manager' iconLabel: 'AppMan'! !

ApplicationManagementBrowser open!

!Browser methodsFor: 'private-class functions'!

showClassOwners
	"show a list of applications having the definition of this class;
	if the user selects one of them open a browser on it"

	| app |
	app := (self applicationManagement applicationsIncludingDefinition: self nonMetaClass)
		select: (ApplicationBrowser promptFor: #selectThenBrowse)
		fromUserIfEmpty: [^self].
	app browseInitial: self selectedClass selector: nil! !

!ApplicationBrowser methodsFor: 'private-class functions'!

excludeClass
	"exclude all selectors and definition of selected class (class and metaclass)"

	self changeRequest ifFalse: [^self].
	Cursor execute showWhile: 
		[ | c |
		self application excludeClass: (c := self nonMetaClass); excludeClass: c class].
	self newClassList: className! !

!ApplicationBrowser methodsFor: 'private-class functions'!

excludeClassDefinition
	"move selected class' definition to system application"

	| cls |
	self changeRequest ifFalse: [^self].
	self application
		excludeClassDefinition: (cls := self nonMetaClass);
		excludeClassDefinition: cls class.
	self newClassList: className! !

!ApplicationBrowser methodsFor: 'private-category functions'!

addCategory
	"inherit category adding; enable system mode if necessary, else new categories are invisible"

	| cat cats |
	self changeRequest ifFalse: [^self].
	cat := self category.
	cats := organization categories.
	super addCategory.
	organization categories == cats | self systemMode ifTrue: [^self].
	category := (cats := organization categories) at: (cats indexOf: cat ifAbsent: [cats size +1]) -1.
	self systemMode: true; changed: #systemMode! !

!ApplicationBrowser methodsFor: 'private-category functions'!

excludeCategory
	"exclude all class info organized by currently selected system category"

	self changeRequest ifFalse: [^self].
	self application excludeCategory: self category.
	self newCategoryList: category! !

!ApplicationBrowser methodsFor: 'private-category functions'!

includeCategory
	"include category with all classes and methods (class and instance)"

	self changeRequest ifFalse: [^self].
	(organization listAtCategoryNamed: self category)
		do: [:c | self includeClass: c asClass].
	self newClassList: className! !


' .ApplicationSource APL-Interface(1.8)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 27 September 1994 at 12:05:11 pm'!



"fix: file i/o error during statistics function"!

!ApplicationBrowser methodsFor: 'private-application functions'!

spawnStatistics
	"open a workspace with statistical informations about the application"

	| s a |
	(s := TextStream on: (String new: 500))
		nextPutAll: 'classes:'; tab: 2; print: (a := self application) classNames size; space; nextPut: $(; print: a ownClassNames size; nextPut: $); cr;
		nextPutAll: 'selectors:'; tab: 1; print: a selectors size; cr.

	a path notNil ifTrue:
		[s nextPutAll: 'sources:'; tab: 2; print: (OSErrorHolder nonexistentSignal
				handle: [:ex | ex returnWith: 0]
				do: [a sourceFile name asFilename fileSize]); cr.
		a version isInitialVersion ifFalse:
			[s nextPutAll: 'binary:'; tab: 2; print: (OSErrorHolder nonexistentSignal
				handle: [:ex | ex returnWith: 0]
				do: [a instBossFilename fileSize]); cr]].

	s := s contents.
	ComposedTextView
		open: (ValueHolder with: s)
		label: 'Statistics on ' , a identifier storeString
		icon: (Icon constantNamed: #workspace)
		extent: s asComposedText extent + 30! !


' .ApplicationSource APL-Interface(1.9)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 15 February 1995 at 9:19:10 pm'!



"fix: (see also APL-Framework(1.15) and higher)
- mnu #isEmpty during major version with rename
- invalid application browser models after major version with rename
- update the browser when accepting code
- switch to system mode after 'find class' if necessary
- error when entering an empty string as platform compatible name
- errors in ApplicationBrowser class>openOn:/open
- no update on AppMan browser list if selected entry is outdated
	(avoid irritation or loss of selection)"!

!ApplicationBrowser methodsFor: 'updating'!

newApplicationVersion
	"the application version has changed; update window label and info text"

	self newApplicationInfo; changed: #windowLabel with: self application identifier! !

!ApplicationBrowser methodsFor: 'initialize-release'!

application: anApplication 
	"set or change the application of the browser; 
	install the browser as dependent, release old dependency if any; 
	update output concerning the application version"

	| initial |
	application == anApplication ifTrue: [^self].
	(initial := application == nil) ifFalse: [application removeDependent: self].
	(application := anApplication) addDependent: self.
	initial ifFalse: [self newApplicationVersion].
	self changed: #category! !

!ApplicationBrowser methodsFor: 'initialize-release'!

on: anApplication 
	"initialize the receiver to browse on anApplication"

	super on: Smalltalk organization.
	systemMode := documentationMode := false.
	self application: anApplication! !

!ApplicationBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by an application modification or minor version release"

	anAspect == #version ifTrue: [^self newApplicationVersion].
	super update: anAspect! !

!ApplicationBrowser methodsFor: 'updating'!

update: anAspect with: anApplication 
	"usually invoked by an application major version release"

	(anAspect == #application and: [anApplication isKindOf: Application])
		ifTrue: ["my application got a new major version, attach to the new object"
			^self application: anApplication].
	super update: anAspect with: anApplication! !

!ApplicationBrowser methodsFor: 'doIt/accept/explain'!

acceptText: aText from: aController 
	| bool |
	bool := self documentationMode
				ifTrue: [self acceptDocumentation: aText from: aController]
				ifFalse: [super acceptText: aText from: aController].
	^bool
		and: 
			[self newApplicationVersion.
			self applicationManagement updateApplicationList.
			true]! !

!ApplicationBrowser methodsFor: 'private-category functions'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   

!ApplicationBrowser class methodsFor: 'dialogs'!

request: messageString platformNames: aDictionary default: aString onCancel: aBlock
	"open a dialog for entering compatible filenames; return the new dictionary"

	| platforms currentValues dialog cancel y1 newValues |
	currentValues := (platforms := StorableObjectAccessor supportedPlatforms)
		collect: [:platf | ValueHolder with: (aDictionary at: platf
			ifAbsent:
				["suggest a filename, indicated by a ?"
				| try |
				try := (Filename defaultClassForPlatform: platf)
					suggest: (platf == #dos | (platf == #os2)
						ifTrue: [aString copyFrom: 1 to: (aString size min: 8)]
						ifFalse: [aString]).
				try copyWith: $?])].

	dialog := DialogView model: (cancel := ValueHolder newBoolean).
	y1 := self title: messageString forDialog: (dialog width: 250).

	currentValues with: platforms do: [:vmodel :platform |
		y1 := dialog 
			yPosition: y1 + 10;
			addVisual: platform asComposedText fromX: 0 toX: 0.2;
			yPosition: y1 + 8;
			addTextFieldOn: vmodel initially: vmodel value fromX: 0.25 toX: 1;
			yPosition].
	self addButtons: #(accept cancel) for: dialog.
	dialog open.

	cancel value ifTrue: [^aBlock value].

	newValues := Dictionary new.
	currentValues with: platforms do: [:vmodel :platform | | fname cls guess |
		"discard the ? if any"
		((fname := vmodel value) isEmpty not and: [fname last == $?])
			ifTrue: [fname := fname copyFrom: 1 to: fname size -1].
		fname isEmpty ifFalse:
			["check validity"
			cls := Filename defaultClassForPlatform: platform.
			[(guess := cls suggest: fname) sameAs: fname]
				whileFalse:
					[fname := DialogView
						request: ('''', fname, ''' is not compatible to ', cls name, '.\Retry...' withCRs)
						initialAnswer: guess
						ifEmpty: [^aBlock value].
					(fname first = cls currentDirectoryString first or: [fname includes: cls separator])
						ifTrue: [fname := guess]].
			newValues at: platform put: fname]].
	^newValues! !

!ApplicationBrowser class methodsFor: 'view creation'!

openOn: anApplication 
	self
		openOn: (self new on: Smalltalk organization; application: anApplication)
		withTextState: nil! !

!ApplicationBrowser class methodsFor: 'view creation'!

open
	"select an application from Application Management and open a browser on it"

	| app |
	app := self applicationManagement selectApplication: 'Select an application to browse'.
	^app == nil ifFalse: [self openOn: app]! !

!ApplicationManagementBrowser methodsFor: 'application list'!

selectedApplication
	"returns the application object referred by the current selection"

	^application == nil
		ifFalse: 
			[| id |
			id := application string copyWithout: Character tab.
			management applicationNamed: (id copyUpTo: $()]! !

!ApplicationManagementBrowser methodsFor: 'application list'!

updateList
	"some applications changed their version; redisplay newest identifiers; 
	avoid losing the current selection if it is outdated"

	| app |
	(app := self selectedApplication) == nil
		ifFalse: 
			[app := app identifier.
			self hierarchy == nil
				ifFalse: 
					[| idx |
					idx := application string lastIndexOf: Character tab.
					app := ((application string copyFrom: 1 to: idx) , app) asText].
			self application: app].
	self changed: #application! !

!ApplicationManagementBrowser methodsFor: 'updating'!

update: anAspect 
	"usually invoked by a major/minor version release via application browser"

	anAspect == #version
		ifTrue:
			[ "this broadcast signals that an application's version changed"
			"avoid losing selection if selected item concerns about new version"
			^self updateList].
	super update: anAspect! !

"code maintenance"!

!ApplicationManagementBrowser methodsFor: 'menu messages'!

upgradeMinor
	"try to upgrade anApplication to the newest available minor version"

	management upgradeMinor: self selectedApplication! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

activateSelection
	"open an ApplicationBrowser on the selected application"

	Cursor execute showWhile: [self editApplication]! !

!ApplicationManagementBrowser methodsFor: 'menu messages'!

productInfo
	Cursor execute showWhile: [management class productInfo]! !

!ApplicationBrowser methodsFor: 'text'!

info
	"return source file name for selected method or class"

	| cls owners |
	(cls := self classOfSelectedMethod) == nil ifTrue: [^''].
	selector == nil 
		ifFalse:
			[| mth idx str |
			mth := cls compiledMethodAt: selector ifAbsent: [^''].
			(idx := mth fileIndex) isInteger ifFalse: [^idx sourceFileHolderId].
			str := self sourceFilename: idx.
			idx == 2 ifFalse: [^str].
			"provide a note if selector does not belong to edited application"
			(self application includesSelector: selector class: cls) ifTrue: [^str].
			owners := self applicationManagement applicationsIncludingSelector: selector class: cls.
			owners := owners isEmpty ifTrue: ['unregistered'] ifFalse: [owners first identifier].
			^str, ' (', owners, ')'].

	textMode == #comment
		ifTrue: [| src | 
			(src := self nonMetaClass organization globalComment) == nil
				ifTrue: [^''].
			^src class == SourceReference
				ifTrue: [src sourceFileHolderId]
				ifFalse: [self sourceFilename: src sourceFileNumber]].

	"test if class is defined in an application"
	(owners := self applicationManagement applicationsIncludingDefinition: cls) isEmpty
		ifFalse: [^owners first identifier].

	"test if class is in system changes"
	((ChangeSet current isNew: cls) or: [ChangeSet current atClass: cls includes: #change])
		ifTrue: [^self sourceFilename: SourceFileManager default target]. 

	"class must be defined by system sources"
	^self sourceFilename: 1! !

"enhancements: emphasize method comments"!

!Scanner methodsFor: 'public access'!

scanComments: textOrString do: aBlock
	"scan textOrString for comments, evaluate aBlock with start and end positions of each comment found"

	self on: textOrString readStream.
	[hereChar ~~ nil]
		whileTrue:
			["Skip delimiters fast, there almost always is one."
			[(tokenType := typeTable at: hereChar asInteger) == #xDelimiter]
				whileTrue:
					[(hereChar := source next) == nil ifTrue: [^self	"end of input"]].

			tokenType == #xDoubleQuote
				ifTrue:
					[| cmtStart char |
					cmtStart := source position.
					[(char := source next) == $"]
						whileFalse:
							[char == nil
								ifTrue: [^self offEnd: 'Unmatched comment quote']].
					aBlock value: cmtStart value: source position.
					hereChar := source next]
				ifFalse: [tokenType == #xLetter		"most tokens are words or keywords"
					ifTrue: [self xLetter]
					ifFalse: [(tokenType at: 1) = $x
						ifTrue: [self perform: tokenType]
						ifFalse: [hereChar := source next]]]]! !

!ApplicationBrowser methodsFor: 'text'!

emphasisFor: aSymbol 
	"return text emphasis to add to text classified by aSymbol"

	^aSymbol == #comment ifTrue: [#italic]! !

!ApplicationBrowser methodsFor: 'text'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           


' .ApplicationSource APL-Interface(1.10)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 7 April 1995 at 6:36:19 pm'!



!ApplicationBrowser methodsFor: 'text'!

text
	"it is possible to add inherited selectors to the selector list, 
	so the selected class may be different from the implementing class;
	if there is no text, return a short info about the application"

	| text |
	self documentationMode ifTrue: [^self application documentation asText].
	textMode == #methodDefinition & selector notNil
		ifTrue:
			[| emph scn |
			text := self classOfSelectedMethod sourceMethodAt: selector.
			(emph := self emphasisFor: #comment) == nil ifFalse:
				["emphasize method comments"
				self errorSignal handle: [:ex | ex return] from: (scn := Scanner new) do:
					[scn scanComments: text string do:
						[:cmtStart :cmtEnd | text emphasizeFrom: cmtStart to: cmtEnd with: emph]]].
			^text].
	^(text := super text) isEmpty
		ifTrue: [self applicationInformation]
		ifFalse: [text]! !

!ApplicationBrowser methodsFor: 'private-category functions'!

findClass
	"Prompt for a class and position myself there; set system mode if necessary"

	| cls |
	self changeRequest ifFalse: [^self].
	(cls := self pickAClass: 'Find class:') = '' ifTrue: [^self].
	cls == nil ifTrue: [^DialogView warn: 'No matching class'].
	cls := cls isBehavior
			ifTrue: [cls isMeta ifTrue: [cls soleInstance] ifFalse: [cls]]
			ifFalse: [cls class].
	(self application includesClass: cls name)
		ifFalse: [self systemMode: true; changed: #systemMode].
	self newCategoryList: cls category; newClassList: cls name! !