
' .ApplicationSource AccessControl(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 3:39:29 pm'!



'- required applications for AccessControl(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('StorableObject' ) ))!

'- sources of AccessControl(1.0) -'!

StorableObjectAccessor subclass: #AccessControlledObjectAccessor
	instanceVariableNames: 'accessControl '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Security'!


!AccessControlledObjectAccessor methodsFor: 'initialize-release'!

defaultAccess
	accessControl := AccessControlList default!

initialize
	super initialize.
	self defaultAccess! !

!AccessControlledObjectAccessor methodsFor: 'permission check'!

deletePermission
	^accessControl deletePermission!

modifyPermission
	^accessControl modifyPermission!

readPermission
	^accessControl readPermission!

writePermission
	^accessControl writePermission! !



Object subclass: #AccessControlList
	instanceVariableNames: 'modify read write delete '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Security'!
AccessControlList comment:
'AccessControlList records permissions granted to users or groups concerning
	- modification	permission to modify the access control list itself
	- read			permission to read whatever is controlled
	- write			permission to write whatever is controlled
	- delete			permission to delete whatever is controlled.

Every permission holds
	- the symbol #everybody
	- a dictionary of gids -> (Set of uids) or -> #everybody.
The entry #everybody grants permission to every user and group.

Limitations:
You can not add or remove single uids or groups for entries holding #everybody,
as we would have to compute the complement of the entry to be removed.

Instance variables:
modify	<Dictionary of: Symbol>	permission to modify the access control list itself
read	<Dictionary of: Symbol>	permission to read whatever is controlled
write	<Dictionary of: Symbol>	permission to write whatever is controlled
delete	<Dictionary of: Symbol>	permission to delete whatever is controlled
'!


!AccessControlList methodsFor: 'initialize-release'!

initialize
	read := #everybody.
	self class noPermissionSignal
		handle: [:ex | ex proceed]
		do: [self uid == nil
		ifTrue: 
			[modify := #everybody.
			write := #everybody.
			delete := #everybody]
		ifFalse: 
			[modify := Dictionary new.
			self addPermission: #modify forUser: self uid group: self gid.
			write := Dictionary new.
			self addPermission: #write forUser: self uid group: self gid.
			delete := Dictionary new.
			self addPermission: #delete forUser: self uid group: self gid.
			]]! !

!AccessControlList methodsFor: 'accessing'!

addPermission: permission forUser: uid group: gid 
	"For user uid in group gid add permission (modify read write delete)"

	| users groupsPermitted |
	self modifyPermission.
	groupsPermitted := self permission: permission.
	groupsPermitted == #everybody ifTrue: [^self].
	users := groupsPermitted at: gid ifAbsentPut: [Set new].
	users == #everybody ifTrue: [^self].
	users add: uid!

permissions
	"Answer the legal permission identifiers"

	^#( #modify #read #write #delete )!

removePermission: permission forUser: uid group: gid 
	"For user uid in group gid remove permission (modfiy read write delete)"

	| users groupsPermitted |
	self modifyPermission.
self halt: 'next instruction not consistent with addPermission:...'.
	gid == #everybody ifTrue: [^self].
	groupsPermitted := self permission: permission.
	groupsPermitted == #everybody ifTrue: [^self setUserPermissionForUser: uid group: gid].
self halt: 'why adding a new set?'.
	users := groupsPermitted at: gid ifAbsentPut: [Set new].
self halt: 'next instruction does NOT remove anything'.
	users == #everybody ifTrue: [^self].
	users remove: uid ifAbsent: []!

setUserPermissionForUser: uid group: gid 
	read := Dictionary new.
	self
		addPermission: #read
		forUser: self uid
		group: self gid! !

!AccessControlList methodsFor: 'testing'!

isContextACL
	^false! !

!AccessControlList methodsFor: 'private'!

check: permission forUser: uid group: gid 
	| users groups |
	uid == nil ifTrue: [^false].
	(groups := self permission: permission) == #everybody ifTrue: [^true].
	users := groups at: gid ifAbsent: [^false].
	^users == #everybody or: [users includes: uid]!

checkPermission: permission forUser: uid group: gid 
	"raise a signal if the specified access is not allowed for user 'uid' in group 'gid'"

	(self check: permission forUser: uid group: gid)
		ifFalse: [self class noPermissionSignal raiseRequestWith: permission]!

gid
	^(Smalltalk at: #UserRegistry ifAbsent: [^nil]) currentGID!

permission: access 
	"return info about user groups with permission for the specified access"

	access == #modify ifTrue: [^modify].
	access == #read ifTrue: [^read].
	access == #write ifTrue: [^write].
	access == #delete ifTrue: [^delete].
	DialogView internalError: 'Illegal access mode symbol ' , access printString!

uid
	^(Smalltalk at: #UserRegistry ifAbsent: [^nil]) currentUID! !

!AccessControlList methodsFor: 'permission check'!

deletePermission 
	"Check delete permission for the current user.
	Signal noPermission on denial"
	self checkPermission: #delete forUser: self uid group: self gid!

modifyPermission 
	"Check modify permission for the current user.
	Signal noPermission on denial"
	self checkPermission: #modify forUser: self uid group: self gid!

readPermission
	"Check read permission for the current user.
	Signal noPermission on denial"
	self checkPermission: #read forUser: self uid group: self gid!

writePermission 
	"Check write permission for the current user.
	Signal noPermission on denial"
	self checkPermission: #write forUser: self uid group: self gid! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!AccessControlList class methodsFor: 'instance creation'!

default
	"Answer a default ACL with read access for everybody,
	modiy, write, delete access for current user"
	^self new initialize! !

!AccessControlList class methodsFor: 'Signal constants'!

noPermissionSignal
	"Answer the signal used for signalling no permission"
	^OSErrorHolder noPermissionsSignal! !

