
' .ApplicationSource APL-ChangeSequence(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 3:53:11 pm'!



'- required applications for APL-ChangeSequence(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('X_Changes' ) ))!

'- sources of APL-ChangeSequence(1.0) -'!

MethodDefinitionChange subclass: #MethodDefinitionBackup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
MethodDefinitionBackup comment:
'MethodDefinitionBackup restricts possible source references to saved states, i.e. the source has been stored in some external source file other than the changes file. The use of MethodDefinitionBackup makes sense in an environment of change sequences, when you want to have the possibility to restore a previous state of a method definition.'!


!MethodDefinitionBackup methodsFor: 'accessing'!

name
	"returns the line which should appear in the list view"

	^'(*)' , super name!

type
	^#backup! !

!MethodDefinitionBackup methodsFor: 'fileIn/Out'!

fileOutOn: aSourceStream 
	"backups are for information only; don't store them"!

fileOutOn: aStream previous: prevChange next: nextChange
	"backups are for information only; don't store them"

	| prev next |
	prev := className = prevChange className
		and: [category = prevChange category
			and: [prevChange class == MethodDefinitionChange]].
	next := className = nextChange className
		and: [category = nextChange category
			and: [nextChange class == MethodDefinitionChange]].
	
	prev = next ifTrue: [^self].
	prev ifFalse: [self fileOutHeaderOn: aStream].
	next ifFalse: [aStream space; nextPut: $!!; cr; cr]!

representBinaryOn: binWriter 
	"special representation if inst var 'file' refers to an external stream"

	(file isKindOf: ExternalStream)
		ifFalse: [^super representBinaryOn: binWriter].

	^MessageSend
		receiver: self class
		selector: #selector:class:
		arguments: (Array with: self selector with: self className)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!MethodDefinitionBackup class methodsFor: 'instance creation'!

selector: aSymbol class: className 
	"create a backup, i.e. a pointer to the current source code of the specified 
	selector in class named 'className'"

	^self new
		className: className;
		selector: aSymbol;
		category: (className asClass organization categoryOfElement: aSymbol);
		getSource! !



ChangeListController subclass: #ChangeSequenceController
	instanceVariableNames: ''
	classVariableNames: 'ChangeSequenceYellowButtonMenu PasteBuffer '
	poolDictionaries: ''
	category: 'Tools-Changes'!
ChangeSequenceController comment:
'A ChangeSequenceController is able to manipulate ChangeSequences.
Special features are copy/cut/paste operations handling elements of a ChangeSequence.
See instance protocol ''menu messages'' for available functions.

Class Variables:
	ChangeSequenceYellowButtonMenu	<PopUpMenu>  normal menu
	NoSelectionMenu	<PopUpMenu>  menu if nothing selected
	PasteBuffer	<Change>  buffer for copy/cut/paste operations'!


!ChangeSequenceController methodsFor: 'menu messages'!

acceptList
	"propagate list to application"

	model acceptList!

addDoIt
	| expression |
	expression := DialogView
				request: 'Enter expression for do it'
				initialAnswer: ''
				ifEmpty: [^self].
	model addDoIt: expression!

copySelection
	"copy current selection"

	| chg |
	(chg := model selectedChange) notNil
		ifTrue: [PasteBuffer := chg]!

cutSelection
	"cut current selection"

	| chg |
	(chg := model forgetCurrent) notNil
		ifTrue: [PasteBuffer := chg]!

fileOut
	| fileName fileStream |
	(fileName := DialogView
			requestNewFileName: 'File out on'
			default: model listName , '.cs') isEmpty
		ifTrue: [^nil].
	fileStream := fileName asFilename writeStream.
	
	[model fileOutOn: fileStream timeStamp]
		valueNowOrOnUnwindDo: [fileStream close]!

pasteSelection
	"paste current selection"

	PasteBuffer notNil ifTrue: [model addAtSelection: PasteBuffer copy]!

restoreSelection
	model restoreSelection.
	view displayRemovedOn: view graphicsContext!

updateChanges
	"update list of changes"

	model updateChanges; updateList! !

!ChangeSequenceController methodsFor: 'initialize-release'!

initializeMenu
	ChangeSequenceYellowButtonMenu isNil ifTrue: [self class initializeMenus].
	self menuHolder: (ValueHolder with: ChangeSequenceYellowButtonMenu)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!ChangeSequenceController class methodsFor: 'class initialization'!

flushMenus
	ChangeSequenceYellowButtonMenu := nil!

initializeMenus
	"build the operate button menu and cache it in the class variable"

	| editMenu specialMenu | 
	editMenu := PopUpMenu
			labelArray: #(copy cut paste 'add do it' )
			lines: #(3)
			values: #(#copySelection #cutSelection #pasteSelection #addDoIt ).

	specialMenu := PopUpMenu
			labelArray: #('spawn compare' 'spawn system' 'spawn change list' 'check conflicts, save as...' 'check with system, save as...')
			lines: #(3)
			values: #(#spawnCompare #browse #copyView #check #checkWithSystem ).

	ChangeSequenceYellowButtonMenu := PopUpMenu
			labels: 'file in/out\replay all\remove\restore all\forget\replay selection\remove selection\restore selection\reinstall selection\edit\accept\update\special' withCRs
			lines: #(1 4 5 9 10 12)
			values: (Array with: self fileMenu), 
				#(doAll),
				(Array with: self removeMenu),
				#( restoreAll forget doThis removeItem restoreItem restoreSelection ),
				(Array with: editMenu),
				#( #acceptList #updateChanges ),
				(Array with: specialMenu)! !



ChangeList variableSubclass: #EnhancedChangeList
	instanceVariableNames: 'application '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!


!EnhancedChangeList methodsFor: 'initialize-release'!

application: anApplication
	application := anApplication!

initialize
	"work on ChangeSequences instead of OrderedCollections"

	super initialize.
	changes := ChangeSequence new.
	list := ChangeSequence new! !

!EnhancedChangeList methodsFor: 'accessing'!

addAtSelection: entry 
	"insert entry before selected item, add if none selected"

	selectionIndex == 0
		ifTrue: [changes addLast: entry]
		ifFalse: [changes add: entry beforeIndex: selectionIndex].
	self updateList!

addDoIt: smalltalkExpression 
	self addAtSelection: (OtherChange new text: smalltalkExpression)!

value: aString
	"sent by the controller on 'accept':
	compile the string as code for a do it, add it as do it to change list"

	| ctrl stream |
	(DialogView confirm: 'Do you want the code to be inserted as a do it?')
		ifFalse: [^self].
	stream := ReadStream on: aString.
	ctrl := thisContext sender receiver.
	Object compilerClass new
		from: stream class: UndefinedObject context: nil notifying: ctrl;
		translate: stream noPattern: true ifFail: [^self].
	self addDoIt: aString asString.! !

!EnhancedChangeList methodsFor: 'private'!

mode
	"return a symbol specifying the kind of change sequence"

	| s |
	s := (self listName copyUpTo: Character space).
	s = 'Changes' ifTrue: [^#changes].
	s = 'History' ifTrue: [^#history].
	s = 'Full' ifTrue: [^#fullHistory].
	s = 'Pre' ifTrue: [^#preInstallation].
	s = 'Post' ifTrue: [^#postInstallation].
	^nil! !

!EnhancedChangeList methodsFor: 'private-menu messages'!

acceptList
	"propagate list to application"

	| mode |
	self forget.
	mode := #(#changes #history #fullHistory #preInstallation #postInstallation)
		indexOf: self mode.
	application perform: (#(#setChanges: #setHistory: #setFullHistory: #setPreInstallation: #setPostInstallation:) at: mode)
		with: changes copy!

forgetCurrent
	"remove occurrence of selected change from changes, list and removed; 
	return the removed item, nil if none selected"

	^selectionIndex > 0
		ifTrue: 
			[| chg |
			chg := list removeAtIndex: selectionIndex.
			changes remove: chg.
			removed remove: chg ifAbsent: [].
			selectionIndex := selectionIndex min: list size.
			self updateSelection: selectionIndex.
			chg]!

restoreSelection
	"compile the selected change;
	the source reference is restored to its original value,
	all changes on the same item occurring after the selected change are dropped"

	| restoreChg |
	(restoreChg := self selectedChange) isNil ifTrue: [^self].
	restoreChg restore.		"recompile; restore source reference if necessary"

	"forget all following changes concerning the object underlying aChange"
	self listIndex + 1 to: list size do: [:i | | chg |
		(restoreChg overwrites: (chg := list at: i))
			ifTrue: [removed add: chg]].

	self updateChanges.
	changes last getSource.	"update last change (inserted during restore)"
	self updateList.

	"the change which entered the underlying change sequence during restore stays
	in the list (unremoved); this is necessary for those cases where the restored change
	is followed by other saved changes, thus guaranteeing that saved changes
	will be 'overwritten' during next file out."!

updateChanges
	"replace changes with those defined by underlying application"

	changes := (application perform: self mode) copy! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!EnhancedChangeList class methodsFor: 'instance creation'!

onApplication: app
	^self new application: app! !



!ChangeSet methodsFor: 'converting'!

asChangeSequence 
	"return a ChangeSequence (collection of Change objects)
	for all the changes the receiver knows about."

	| sequence |
	sequence := ChangeSequence new.
	(self class superclassOrder: self changedClasses asOrderedCollection)
		do: [:class | sequence addAll: (self changeSequenceFor: class)].

	classRemoves do: [:className | sequence removeClass: className].

	reorganizeSystem ifTrue:
		[sequence addDoIt: 'Smalltalk organization changeFromString: ', Smalltalk organization printString storeString].

	specialDoIts do: [:expression | sequence addDoIt: expression].
	^sequence!

changeSequenceFor: aClass
	"returns a ChangeSequence (collection of Change objects)
	for all the changes the receiver knows about aClass"

	| sequence changes removes cname |
	sequence := ChangeSequence new.
	removes := OrderedCollection new.
	changes := OrderedCollection new.

	"treat class changes.  i.e.  new class, definition, comment, renaming.
	If the class is to be initialized on fileIn, add the class to aClassCollection"

	cname := aClass name.
	(self atClass: aClass includes: #add)
		ifTrue:
		[sequence addClass: cname; commentClass: cname. 
		changes addAll: aClass selectors].

	(self atClass: aClass includes: #rename)
		ifTrue: [sequence renameClass: (self oldNameFor: aClass) to: cname].

	(self atClass: aClass includes: #change)
		ifTrue: [sequence changeClass: cname].

	(self atClass: aClass includes: #comment)
		ifTrue: [sequence commentClass: cname].

					"next file out changed methods"
	(methodChanges at: cname ifAbsent: [Dictionary new])
		keysAndValuesDo: [:selector :type | type = #remove
			ifTrue: [removes add: selector]
			ifFalse: [changes add: selector]].

	changes do: [:selector | | chg |
			chg := sequence
				addSelector: selector
				protocol: (aClass organization categoryOfElement: selector)
				class: cname.
			chg getSource].
	removes do: [:selector | sequence removeSelector: selector class: cname].

	(self atClass: aClass includes: #reorganize)
		ifTrue: [sequence addDoIt: cname , ' organization changeFromString: ' , aClass organization printString storeString].

	"reinitialize metaclass if necessary"
	(aClass isMeta and: [changes includes: #initialize])
		ifTrue: [sequence add: (ClassOtherChange new className: aClass soleInstance name; type: #initialize)].

	^sequence! !



ChangeListView subclass: #ChangeSequenceView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!


!ChangeSequenceView methodsFor: 'controller access'!

defaultControllerClass
	^ChangeSequenceController! !



OrderedCollection variableSubclass: #ChangeSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
ChangeSequence comment:
'ChangeSequence extends the change recording mechanisms by a holder for sequences of changes (opposed to ChangeSet). For every change generate a new change description and enter it in the receiver. See Change class hierarchy for possible object classes.

Note the following problem:
Each method compilation comprises the following process:
	(1) basic compilation with Compiler>compile:in:notifying:ifFail:
	(2)	(a) ChangeSet current propagates a change due to the result (add/changeSelector:...).
		(b) Application management propagates the change to all applications.
		(c) Each application concerning about the change creates an appropriate change object
			and records it in its change sequence.
	(3) The new compiled method is installed in the method dictionary.
	(4) SourceFileManager default stores the code and returns a new source pointer
	(5) Reclassification of the compiled selector (if necessary)

Each change object created in step (2)(c) has now an invalid source pointer (usually it is nil
anyway) and should be updated by sending #updateChanges.
'!


!ChangeSequence methodsFor: 'accessing'!

changesForClass: aClass 
	"collect the changes for aClass"

	^self select: [:chg | chg className == aClass name]!

changesForSelector: selector class: aClass 
	"collect the changes for selector"

	^self select: [:chg | chg selector == selector and: [chg className == aClass asSymbol]]! !

!ChangeSequence methodsFor: 'change management'!

addClass: className 
	^self changeClass: className!

addDoIt: smalltalkExpression 
	^self add: (OtherChange new text: smalltalkExpression)!

addSelector: selector protocol: protocol class: className 
	"add and return a Change object for the specified parameters;
	position of the source is left open, it is set on execution of #classifySelector:class:,
	called from ClassDescription>compile:classified:notifying:"

	^self add: ((MethodDefinitionChange new)
		className: className;
		selector: selector;
		category: protocol)!

backupSelector: selector protocol: protocol class: className 
	"create (and return) a change object which saves the source code info for the 
	specified method"

	| chg |
	chg := MethodDefinitionBackup new
		className: className;
		selector: selector;
		category: protocol;
		getSource.

	"backup stored versions only"
	^chg sourceFileNumber ~= 2 ifTrue: [self add: chg]!

changeClass: className 
	"assume access to class definition returns the new state"

	| class chg |
	(class := className asClass) isMeta
		ifTrue: [chg := ClassOtherChange new type: #'inst vars for']
		ifFalse: [chg := ClassDefinitionChange new].
	^self add: (chg className: class; text: class definition)!

changeSelector: selector class: aClass 
	"because receiver records full history, a change is like an add"

	self
		addSelector: selector
		protocol: (aClass organization categoryOfElement: selector)
		class: aClass!

classifySelector: selector under: protocol class: className 
	"set protocol for last entry"

	| last |
	(self isEmpty not and: [(last := self last) className = className asSymbol & (last selector = selector)])
		ifTrue: 
			[last category: protocol]!

commentClass: className 
	| cm |
	(cm := className asClass organization globalComment) isNil
		ifTrue: [^nil].
	self add: ((ClassCommentChange
			file: (SourceFileManager default fileAt: cm sourceFileNumber)
			position: cm position)
			className: className;
			type: #comment)!

removeClass: className 
	self add: (ClassOtherChange new className: className; type: #remove)!

removeClassChanges: aClass 
	"remove all change objects on class associated with 'aClass' "

	^self removeAllSuchThat: [:chg | chg className == aClass name]!

removePureClassChanges: aClass 
	"remove all change objects on class associated with 'aClass'
	(not selector changes, only definition, comment, etc.)"

	^self removeAllSuchThat: [:chg | chg className = aClass name and: [chg selector isEmpty]]!

removeSelector: selector class: className 
	self add: (MethodOtherChange new className: className; selector: selector; type: #remove)!

removeSelectorChanges: selector class: aClass 
	"remove all change objects associated with 'selector' in 'aClass'"

	^self removeAllSuchThat: [:chg | chg selector == selector and: [chg className == aClass asSymbol]]!

renameClass: className to: newName 
	self add: (ClassOtherChange new className: className; type: #rename; text: className, ' ', #rename:, ' ', newName).

	"ChangeScanner inserts a second change, but we don't "
"	self add: (ClassOtherChange new className: newName; type: 'rename to' asSymbol)"!

renameProtocol: oldName to: newName class: className 
	self add: (OtherChange new text: className , ' ' , #organization , ' renameCategory: ' , oldName storeString , ' to: ' , newName storeString)!

requestToUpdate
	"if last entry is an add entry for this selector set source reference"

	self isEmpty ifFalse:
		[ | chg |
		((chg := self last) file isNil and: [chg respondsTo: #getSource])
			ifTrue: [chg getSource]]!

updateChanges
	"if last entry is an add entry for this selector set source reference"

	self isEmpty ifFalse:
		[ | chg |
		((chg := self last) file isNil and: [chg respondsTo: #getSource])
			ifTrue: [chg getSource]]! !

!ChangeSequence methodsFor: 'printing'!

printOn: stream 
	self isEmpty
		ifTrue: [stream nextPutAll: 'No Changes recorded']
		ifFalse: [self do: 
				[:element | 
				element printOn: stream.
				stream cr]]! !

!ChangeSequence methodsFor: 'file out'!

fileOutOn: stream 
	self do: [:e | e fileOutOn: stream]! !

!ChangeSequence methodsFor: 'user interface'!

browse
	ChangeListView openOn: (ChangeList new
			listName: 'a ChangeSequence'
			changes: self
			removed: #())! !



ClassDefinitionChange subclass: #ClassDefinitionBackup
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
ClassDefinitionBackup comment:
'Class definitions do not necessarily have a source text stored on a file. To backup a class definition this class provides an own text holder. Backups do not write the source code during file out. The use of ClassDefinitionBackup makes sense in an environment of change sequences, when you want to have the possibility to restore a previous state of a method definition.
'!


!ClassDefinitionBackup methodsFor: 'fileIn/Out'!

fileOutOn: aSourceStream 
	"backups are for information only; don't store them"! !

!ClassDefinitionBackup methodsFor: 'accessing'!

name
	"returns the line which should appear in the list view"

	^'(*)' , super name!

text: aString
	file := ReadStream on: aString.
	position := 0!

type
	^#backup! !




' .ApplicationSource APL-ChangeSequence(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 31 August 1993 at 5:18:32 pm'!



!ChangeSequence methodsFor: 'change management'!

changeSelector: selector class: aClass 
	"because receiver records full history, a change is like an add"

	self
		addSelector: selector
		protocol: (aClass asClass organization categoryOfElement: selector)
		class: aClass! !


' .ApplicationSource APL-ChangeSequence(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 1 September 1993 at 3:26:53 pm'!



!ChangeSequence methodsFor: 'change management'!

changeSelector: selector class: aClass backup: aBoolean 
	"because receiver records full history, a change is like an add; 
	if aBoolean is true a backup (if reasonable) will be inserted before;
	protocol assignment for both changes is the current value for selector"

	| p |
	p := aClass asClass organization categoryOfElement: selector.
	aBoolean ifTrue: [self backupSelector: selector protocol: p class: aClass].
	self addSelector: selector protocol: p class: aClass! !

!ChangeSequence methodsFor: 'change management'!

changeSelector: selector class: aClass 
	"record the change, without a backup"

	self changeSelector: selector class: aClass backup: false! !


' .ApplicationSource APL-ChangeSequence(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 29 September 1993 at 2:30:00 pm'!



"fix for: improper file out text for class renames (newName not Symbol)
enhanced: inspect selection on ChangeSequenceController menu"!

!ChangeSequence methodsFor: 'change management'!

renameClass: className to: newName 
	self add: (ClassOtherChange new className: className; type: #rename; text: className, ' ', #rename:, ' ', newName storeString).

	"ChangeScanner inserts a second change, but we don't "
"	self add: (ClassOtherChange new className: newName; type: 'rename to' asSymbol)"! !

!ChangeSequenceController methodsFor: 'menu messages'!

inspectSelection
	"inspect current selection"

	| chg |
	(chg := model selectedChange) notNil
		ifTrue: [chg inspect]! !

!ChangeSequenceController class methodsFor: 'class initialization'!

initializeMenus
	"build the operate button menu and cache it in the class variable"

	| editMenu specialMenu | 
	editMenu := PopUpMenu
			labelArray: #(copy cut paste 'add do it' inspect)
			lines: #(3 4)
			values: #(#copySelection #cutSelection #pasteSelection #addDoIt inspectSelection).

	specialMenu := PopUpMenu
			labelArray: #('spawn compare' 'spawn system' 'spawn change list' 'check conflicts, save as...' 'check with system, save as...')
			lines: #(3)
			values: #(#spawnCompare #browse #copyView #check #checkWithSystem ).

	ChangeSequenceYellowButtonMenu := PopUpMenu
			labels: 'file in/out\replay all\remove\restore all\forget\replay selection\remove selection\restore selection\reinstall selection\edit\accept\update\special' withCRs
			lines: #(1 4 5 9 10 12)
			values: (Array with: self fileMenu), 
				#(doAll),
				(Array with: self removeMenu),
				#( restoreAll forget doThis removeItem restoreItem restoreSelection ),
				(Array with: editMenu),
				#( #acceptList #updateChanges ),
				(Array with: specialMenu)! !

ChangeSequenceController flushMenus!


' .ApplicationSource APL-ChangeSequence(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 4 February 1994 at 3:28:56 pm'!



!MethodDefinitionBackup methodsFor: 'fileIn/Out'!

fileOutOn: aStream previous: prevChange next: nextChange
	"backups are for information only; don't store them"

	| prev next |
	prev := prevChange class == MethodDefinitionChange
		and: [className = prevChange className
			and: [category = prevChange category]].
	next := nextChange class == MethodDefinitionChange
		and: [className = nextChange className
			and: [category = nextChange category]].
	
	prev = next ifTrue: [^self].
	prev ifFalse: [self fileOutHeaderOn: aStream].
	next ifFalse: [aStream space; nextPut: $!!; cr; cr]! !


' .ApplicationSource APL-ChangeSequence(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 12 April 1994 at 4:00:27 pm'!



"fix: ClassDefinitionBackup>mnu: #getSource during reinstall"!

!Change methodsFor: 'source code'!

getSource
	"set source pointers for source code access from current system; 
	do nothing by default (see MethodDefinitionChange>getSource for example)"
! !

!ClassCommentChange methodsFor: 'accessing'!

getSource
	"flush file/position for system code access"

	self file: nil position: nil! !

!Change methodsFor: 'testing'!

isBackup
	^false! !

!ClassDefinitionBackup methodsFor: 'testing'!

isBackup
	^true! !

!MethodDefinitionBackup methodsFor: 'testing'!

isBackup
	^true! !

!MethodDefinitionBackup methodsFor: 'fileIn/Out'!

fileOutOn: aStream previous: prevChange next: nextChange
	"backups are for information only; don't store them"

	| match prev next |
	match := [:chg | (chg isKindOf: MethodDefinitionChange)
			and: [chg isBackup not
			and: [className = chg className and: [category = chg category]]]].

	(prev := match value: prevChange) == (next := match value: nextChange)
		ifTrue: [^self].
	prev ifFalse: [self fileOutHeaderOn: aStream].
	next ifFalse: [aStream space; nextPut: $!!; cr; cr]! !

"improvements: find proper place for inserting (#addAtSelection:)
convert do-it expressions to special changes (if possible)"!

!EnhancedChangeList methodsFor: 'accessing'!

addDoIt: smalltalkExpression 
	"convert argument to an appropriate change object; add it to the current list 
	(special case for periods, which terminate generated change chunks)"

	(smalltalkExpression includes: $.)
		ifTrue: [self addAtSelection: (OtherChange new text: smalltalkExpression)]
		ifFalse: [ChangeScanner new scanFile: smalltalkExpression readStream
					do: [:change | self addAtSelection: change]]! !

!EnhancedChangeList methodsFor: 'accessing'!

addAtSelection: entry 
	"insert entry before selected item, add if none selected"

	| chg |
	(chg := self selectedChange) == nil
		ifTrue: [list isEmpty
			ifTrue: [changes add: entry]
			ifFalse: [changes add: entry after: list last]]
		ifFalse: [changes add: entry before: chg].
	self updateList! !

!ChangeSequence methodsFor: 'change management'!

updateLastChange
	"if last entry is an add entry for this selector set source reference"

	self isEmpty ifFalse: 
		[| chg |
		(chg := self last) file == nil ifTrue: [chg getSource]]! !

!ChangeSequence methodsFor: 'change management'!

updateChanges
	self updateLastChange! !

!ChangeSequence methodsFor: 'change management'!

 !                                                                                                                                                                                                                                                                                                                                     

!ChangeSequence methodsFor: 'change management'!

updateCondensed
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	self do: [:chg | (chg isKindOf: MethodDefinitionChange)
			ifTrue: [chg sourceFileNumber = 2 ifTrue: [chg getSource]]
			ifFalse: [chg getSource	"especially comment changes"]]! !

ChangeSequence removeSelector: #requestToUpdate!

!ChangeSequenceController class methodsFor: 'class initialization'!

editMenu
	"return a menu for operators editing the change list (copy/cut/paste...)"

	^PopUpMenu
		labelArray: #(#copy #cut #paste #inspect )
		lines: #(3 )
		values: #(#copySelection #cutSelection #pasteSelection #inspectSelection )! !

!ChangeSequenceController class methodsFor: 'class initialization'!

initializeMenus
	"build the operate button menu and cache it in the class variable"
	"ChangeSequenceController initializeMenus"

	ChangeSequenceYellowButtonMenu := PopUpMenu
			labels: 'file in/out\replay all\remove\restore all\forget\replay selection\remove selection\restore selection\reinstall selection\edit\accept\update\utilities' withCRs
			lines: #(1 4 5 9 10 12)
			values: (Array with: self fileMenu), 
				#(doAll),
				(Array with: self removeMenu),
				#( restoreAll forget doThis removeItem restoreItem restoreSelection ),
				(Array with: self editMenu),
				#( #acceptList #updateChanges ),
				(Array with: self utilitiesMenu)! !

ChangeSequenceController flushMenus!


' .ApplicationSource APL-ChangeSequence(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 4:08:16 pm'!



"fix: improper equality test for metaclass names"!

!ChangeSequence methodsFor: 'change management'!

removeClassChanges: aClass 
	"remove all change objects on class associated with 'aClass' "

	| sym |
	sym := aClass asSymbol.
	^self removeAllSuchThat: [:chg | chg className == sym]! !

!ChangeSequence methodsFor: 'change management'!

removePureClassChanges: aClass 
	"remove all change objects on class associated with 'aClass'
	(not selector changes, only definition, comment, etc.)"

	| sym |
	sym := aClass asSymbol.
	^self removeAllSuchThat: [:chg | chg selector isEmpty and: [chg className == sym]]! !

!ChangeSequence methodsFor: 'change management'!

removeSelectorChanges: selector class: aClass 
	"remove all change objects associated with 'selector' in 'aClass'"

	| sym |
	sym := aClass asSymbol.
	^self removeAllSuchThat: [:chg | chg selector == selector and: [chg className == sym]]! !

!ChangeSequence methodsFor: 'change management'!

updateSelectors: selectorSet class: className
	"update source references for the given selectors;
	treat the last occurrence of each selector only"

	| set sym |
	sym := className asSymbol.
	set := selectorSet copy.
	self reverseDo: [:chg | chg className == sym 
		ifTrue: [(set remove: chg selector ifAbsent: []) == nil
			ifFalse: [chg getSource]]]! !

ChangeSequence removeSelector: #changesForSelector:class:!

ChangeSequence removeSelector: #changesForClass:!


' .ApplicationSource APL-ChangeSequence(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 13 January 1995 at 3:56:17 pm'!



"fix: mnu #asSymbol when trying to backup changed code for methods with nil protocol"!

!ChangeSequence methodsFor: 'change management'!

 !                                                                                                                                                                                                                                                                                                                                                                            

"fix: Loss of new elements in a Changes editor was possible"!

!EnhancedChangeList methodsFor: 'changing'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                   

"code maintenance"!

ChangeSequenceController removeSelector: #addDoIt!

ChangeSequence removeSelector: #updateChanges!


' .ApplicationSource APL-ChangeSequence(1.8)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 13 January 1995 at 4:42:40 pm'!



!EnhancedChangeList methodsFor: 'changing'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 


' .ApplicationSource APL-ChangeSequence(1.9)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 January 1995 at 12:02:33 pm'!



"adapt to new handling of source/changes file references in Change objects since version 1.10 of ExternalSource (version 1.12 for VisualWorks 2.0)"!

!ChangeSequence methodsFor: 'change management'!

commentClass: className 
	"assume that the current comment source resides in the changes file"

	| cm |
	((cm := className asClass organization globalComment) isKindOf: RemoteString)
		ifFalse: [self error: 'RemoteString expected'].
	self add: ((ClassCommentChange file: cm sourceFileNumber position: cm sourceKey)
			className: className;
			type: #comment)! !

!ChangeSequence methodsFor: 'change management'!

backupSelector: selector protocol: protocol class: className 
	"create (and return) a change object which saves the source code info for the 
	specified method"

	| chg |
	chg := MethodDefinitionBackup new className: className; selector: selector; getSource.
	^(chg file == nil or: [chg sourceFileNumber == 2])
		ifFalse: 
			["backup stored versions only"
			protocol == nil ifFalse: [chg category: protocol].
			self add: chg]! !


' .ApplicationSource APL-ChangeSequence(1.10)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 January 1995 at 1:22:36 pm'!



!MethodDefinitionBackup methodsFor: 'fileIn/Out'!

representBinaryOn: binWriter 
	"special representation if inst var 'file' refers to an external stream"

	^(file isInteger or: [file isKindOf: ExternalStream])
		ifTrue: [MessageSend
				receiver: self class
				selector: #selector:class:
				arguments: (Array with: self selector with: self className)]
		ifFalse: [super representBinaryOn: binWriter]! !


' .ApplicationSource APL-ChangeSequence(1.11)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 January 1995 at 2:09:16 pm'!



!EnhancedChangeList methodsFor: 'changing'!

changeRequest
	"check if the current list holds new elements"

	| mode src |
	(mode := self mode) == nil ifTrue: [^true].
	src := application perform: mode.
	changes detect: [:chg | (src includes: chg) not and: [chg text ~= '"- end of history -"']]
		ifNone: [^true].
	(self listName findString: 'Changes conflicts in upgrading ' startingAt: 1) == 1
		ifTrue: [^true].	
	
	mode := DialogView
				choose: 'The list seems to have new elements'
				labels: #('Forget and close' "'Save and close'" 'Do not close' )
				values: #(true "0" false )
				default: false.
	mode == 0 ifTrue: [self acceptList. mode := true].
	^mode! !


' .ApplicationSource APL-ChangeSequence(1.12)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 7 April 1995 at 5:37:30 pm'!



"reduced output when filing out changes (avoid superfluous protocol headers)"!

!ChangeSequence methodsFor: 'file out'!

fileOutOn: aStream 
	| prev suc |
	prev := nil. suc := 1.
	self do: 
		[:chg | 
		chg
			fileOutOn: aStream
			previous: prev
			next: (suc = self size ifFalse: [self at: (suc := suc + 1)]).
		prev := chg]! !

"bug fix: 
- changes referring to discarded source code after condense changes
	(some change objects cannot be resolved to source code,
	especially method changes followed by a method remove,
	but also method changes referring to renamed classes)
- class renames have to be propagated to change sequence elements
	(method changes might refer to old names of renamed classes until 
	newly compiled)"!

!ChangeSequence methodsFor: 'change management'!

updateCondensed
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	self removeAllSuchThat: [:chg | (chg isKindOf: MethodDefinitionChange)
			ifTrue: [chg sourceFileNumber = 2 and: [chg methodInSystem not
						or: 
							[chg getSource.
							false]]]
			ifFalse: 
				["especially comment changes"
				chg getSource.
				false]]! !

!ClassRelatedChange methodsFor: 'accessing'!

renameClass: aSymbol to: newName 
	"if the receiver refers to class name aSymbol then change to newName; 
	return true/false to indicate if a change was performed"

	^self className == aSymbol asSymbol
		and: [self className: newName. true]! !

!ClassDefinitionChange methodsFor: 'accessing'!

superclassName
	superclassName == nil ifTrue: [superclassName := (file reset; upTo: Character space) asSymbol].
	^superclassName! !

!ClassDefinitionChange methodsFor: 'accessing'!

renameClass: oldName to: newName 
	"exchange class name references properly as well as the text 
	(uses knowledge of class declaration format); 
	return true/false to indicate if a change was performed"

	| oldsym |
	className == (oldsym := oldName asSymbol)
		ifTrue: 
			[self text: ((WriteStream on: (String new: 200))
				nextPutAll: (file reset; through: $#);
				nextPutAll: newName; crtab;
				nextPutAll: (file throughAll: className; nextChunk);
				contents).
			self className: newName.
			^true].

	"check superclass"
	^self superclassName == oldsym
		and: 
			[self text: ((WriteStream on: (String new: 200))
				nextPutAll: newName; space;
				nextPutAll: (file reset; upTo: Character space; nextChunk);
				contents).
			superclassName := newName asSymbol.
			true]! !

!ClassDefinitionBackup methodsFor: 'accessing'!

renameClass: aSymbol to: newName 
	"ignore any rename requests"

	^false! !

!ClassOtherChange methodsFor: 'accessing'!

renameClass: oldName to: newName 
	^self type == #rename
		ifTrue: 
			["check if the new rename overwrites the receiver"
			(file reset; next) == $" ifTrue: [^false].
			file skipThrough: Character space; skipThrough: Character space.
			(String readFrom: file) = oldName asString
				and: 
					["since class renames are propagated immediately this 
					change should be obsolete; discard its text"
					self text: ('"' , self text copyWith: $").
					false]]
		ifFalse: [super renameClass: oldName to: newName]! !

!OtherChange methodsFor: 'accessing'!

renameClass: className to: newName 
	"exchange text occurrences of className with newName"

	| txt start |
	^(start := (txt := self text) findString: className startingAt: 1) > 0
		and: 
			[| prs stop |
			prs := self class parserClass new.
			(start = 1 or: [(prs isTokenish: (txt at: start - 1)) not])
				and: [((stop := start + className size) > txt size or: [(prs isTokenish: (txt at: stop)) not])
					and: 
						[self text: (txt copyReplaceFrom: start to: stop with: newName).
						self renameClass: className to: newName.
						true]]]! !

!ChangeSequence methodsFor: 'change management'!

renameAllClass: className to: newName 
	"a class has been renamed; change name references of change elements; 
	return the index for proper insertion of a rename change (nil if no such 
	index exists)"

	| index oldMeta newMeta |
	index := nil.
	1 to: self size do:
		[:idx | | chg |
		(((chg := self at: idx) renameClass: className to: newName)
				and: [index == nil])
			ifTrue:
				[(chg isKindOf: ClassDefinitionChange)
					ifTrue:
						["leave this change unrenamed; a rename should be inserted behind"
						chg renameClass: newName to: className.
						index := idx + 1]
					ifFalse: [index := 0]]].

	oldMeta := (className, ' class') asSymbol.
	newMeta := (newName, ' class') asSymbol.
	self do: [:chg | chg renameClass: oldMeta to: newMeta].
	^index = 0 ifFalse: [index]! !

!ChangeSequence methodsFor: 'change management'!

renameClass: className to: newName 
	"add a class rename change to the receiver;
	propagate the rename to each element immediately"

	| chg index |
	chg := ClassOtherChange new
			className: className;
			type: #rename;
			text: className, ' ', #rename:, ' ', newName storeString.
	(index := self renameAllClass: className to: newName) == nil
		ifTrue: [self add: chg]
		ifFalse: [self insert: chg before: index]

	"ChangeScanner inserts a second change, but we don't "
"	self add: (ClassOtherChange new className: newName; type: 'rename to' asSymbol)"! !