
' .ApplicationSource BASE(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 4:03:49 pm'!



'- sources of BASE(1.0) -'!

!String methodsFor: 'converting'!

/ anotherString 
	"returns a string for a file name with the receiver as a directory,
	and 'anotherString' as a file name within the directory."

	^self asFilename constructString: anotherString!

asClass
	^self asSymbol asClass! !



!Symbol methodsFor: 'converting'!

asClass
	"return the class (soleInstance or meta) with name equal to the receiver
	(special handling for meta class names)"

	^Smalltalk at: self ifAbsent:
		[| class index start |
		"try to resolve metaclass, scan for a trailing ' class' "
		(start := self size - 5) > 0 ifFalse: [^nil].
		index := self findString: ' class' startingAt: start ifAbsent: [^nil].
		class := Smalltalk at: (self copyFrom: 1 to: index-1) asSymbol ifAbsent: [^nil].
		class isBehavior ifTrue: [class class]]! !



!Character methodsFor: 'converting'!

asString
	^String with: self! !



!Object methodsFor: 'error handling'!

debug
	InputState default ctrlDown ifTrue: [self halt: 'halt at debug point']!

notYetImplemented	
	DialogView userError: 'Not yet implemented.'! !



!ClassOrganizer methodsFor: 'accessing'!

changeFromString: aString 
	"Parse the string and make this be the receiver's structure.  Categories or elements not found are not affected.  New elements are ignored."

	| scanner oldElements newElements newCategories newStops currentStop anArray |
	scanner := Compiler preferredParserClass new scanTokens: aString.
	"If nothing was scanned and I had no elements before, then default me"
	(scanner size = 0 and: [elementArray size = 0])
		ifTrue: [^self setDefaultList].

	oldElements := elementArray asSet.
	newCategories := Array new: scanner size.
	newStops := Array new: scanner size.
	currentStop := 0.
	newElements := WriteStream on: (Array new: 16).
	1 to: scanner size do: 
		[:i | 
		anArray := scanner at: i.
		newCategories at: i put: anArray first asSymbol.
		"we want full control of the ordering of selectors, so do not sort"
		(anArray copyFrom: 2 to: anArray size) "asSortedCollection" do:
			[:elem |
			(oldElements remove: elem ifAbsent: [nil]) notNil ifTrue:
				[newElements nextPut: elem.
				currentStop := currentStop+1]].
		newStops at: i put: currentStop].

	"Ignore extra elements but don't lose any existing elements!!"
	oldElements := oldElements collect:
		[:elem | Array with: (self categoryOfElement: elem) with: elem].
	newElements := newElements contents.
	categoryArray := newCategories.
	categoryStops := newStops.
	elementArray := newElements.
	oldElements do: [:pair | self classify: pair last under: pair first].! !



!Filename methodsFor: 'instance creation'!

/ stringOrFilename 
	"well readable operator for building pathnames (with polymorphic arguments):
	returns a string for a file name with the receiver as a directory, 
	and 'stringOrFilename' as a file name within the directory."

	^self constructString: stringOrFilename asString! !

!Filename methodsFor: 'file utilities'!

copyRecursiveTo: newName 
	"copies the file or, if it is a directory, recursively duplicates the file tree"

	| destination |
	destination := newName asFilename.
	self isDirectory
		ifTrue: 
			[destination makeDirectory.
			self directoryContents do: [:each | 
				(self / each) asFilename copyRecursiveTo: destination / each]]
		ifFalse: [self copyTo: destination]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!Filename class methodsFor: 'instance creation'!

name: nameString extension: ext 
	"create a filename with nameString and the extension ext, they are 
	separeted by the extensionSeparator"

	| stream |
	stream := WriteStream on: (String new: 64).
	stream nextPutAll: nameString; nextPut: self extensionSeparator; nextPutAll: ext.
	^self named: stream contents!

path: path name: nameString
	"create a filename with path and nameString"

	| stream |
	stream := WriteStream on: (String new: 64).
	stream 
		nextPutAll: path; 
		nextPut: self separator; 
		nextPutAll: nameString.
	^self named: stream contents!

path: path name: nameString extension: ext 
	"create a filename with path and nameString and the extension ext"

	| stream |
	stream := WriteStream on: (String new: 64).
	stream 
		nextPutAll: path; 
		nextPut: self separator; 
		nextPutAll: nameString; 
		nextPut: self extensionSeparator; 
		nextPutAll: ext.
	^self named: stream contents!

stringFromArray: anArray 
	"create a filename using the components of anArray separated by 'Filename 
	separator'. If the path is absolute, the first element of the Array must be the 
	Filename separator"

	| stream comp baseLocation |
	stream := ReadWriteStream on: (String new: 64).
	comp := ReadStream on: anArray.
	baseLocation := comp next asFilestring.
	stream nextPutAll: baseLocation.
	[comp atEnd]
		whileFalse: 
			[baseLocation last = Filename separator ifFalse: [stream nextPut: Filename separator].
			stream nextPutAll: (baseLocation := comp next asFilestring)].
	^stream contents asFilestring! !

!Filename class methodsFor: 'constants'!

extensionSeparator
	"return the platform's filename extension separator"

	self concreteClass == Filename ifTrue: [^self subclassResponsibility].
	^$.! !



!Behavior methodsFor: 'converting'!

asClass
	^self!

asSymbol
	^self name asSymbol! !



!ClassDescription methodsFor: 'accessing'!

commentTemplateString
	"Answer a default comment string for the receiver."

	| advanced st |
	(advanced := Smalltalk at: #ClassDeclarations ifAbsent: []) isBehavior
		ifTrue: ["the system analysis utilities from the APOK give most sophisticated templates"
			^Cursor execute showWhile:
				[(advanced on: self) commentTemplateString]].

	st := WriteStream on: (String new: 1000).
	st nextPutAll:
'This class has not yet been commented.  The comment should state the purpose of the class, what messages are subclassResponsibility, and the type and purpose of each instance and class variable.  The comment should also explain any unobvious aspects of the implementation.'.

	#(instVarNames classVarNames)
		with: #('Instance Variables:' 'Class Variables:') 
		do: [:get :title | | list |
			list := self perform: get.
			get == #classVarNames ifTrue: [list := list asSortedCollection].
			list isEmpty
				ifFalse: [st cr; cr; nextPutAll: title].
			list do: [:varName | st crtab; nextPutAll: varName; tab; nextPutAll: '<Object>	comment']].

	^st contents! !



Object subclass: #DummyObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Objects'!
DummyObject comment:
'DummyObject ignores all messages besides ''doesNotUnderstand:''.'!


!DummyObject methodsFor: 'error handling'!

doesNotUnderstand: aMessage 
	"ignore if I do not understand a message. To prevent infinite recursion do 
	not ignore 'doesNotUnderstand:'"

	^aMessage selector == #doesNotUnderstand:
		ifTrue: [super doesNotUnderstand: aMessage]
		ifFalse: [self]! !



!Dictionary methodsFor: 'accessing'!

at: key ifAbsentPut: generatingBlock 
	" If key is not found, create a new    
	entry for key and set its value to value of    
	generatingBlock. Answer the object at key."

	| index assoc |
	index := self findKeyOrNil: key.
	assoc := self basicAt: index.
	assoc == nil
		ifTrue: 
			[assoc := key -> generatingBlock value.
			self atNewIndex: index put: assoc].
	^assoc value! !




' .ApplicationSource BASE(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 8 October 1993 at 4:37:39 pm'!



!CompiledMethod methodsFor: 'source code management'!

comment
	"return the comment string entered between method header and first instructions;
	return nil if comment not available or empty"

	| src comment |
	(src := self getSource) == nil ifTrue: [^nil].
	comment := self mclass parserClass new parseMethodComment: src setPattern: [:val | ].
	^comment isEmpty
		ifFalse: [(comment := comment first) isEmpty ifFalse: [comment]]! !


' .ApplicationSource BASE(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 25 April 1994 at 7:15:46 pm'!



"move functionality from DialogView to Filename"!

!Filename methodsFor: 'file utilities'!

checkExistMode: aSymbol 
	"aSymbol is one of #(#any #new #old #mustBeNew #mustBeOld #directory); 
	return 
	- true, if refers to a file matching mode aSymbol 
	- true if aSymbol is #new (#old), file exists (not), user confirms to use anyway 
	- false, otherwise"

	^aSymbol == #any or: 
		[| idx |
		idx := #(#new #old #mustBeNew #mustBeOld #directory ) indexOf: aSymbol.
		(#(false true false true true ) at: idx) == self exists
			ifTrue: [idx < 5 or: [self isDirectory]]
			ifFalse: [idx < 3 and: [DialogView confirm: nameString, (#(' already exists' ' does not exist' ) at: idx), '. Use anyway?']]]! !

!Filename class methodsFor: 'instance creation'!

named: aString ifFail: aBlock 
	"return a new Filename with the given name string; evaluate aBlock with 
	- nil, if aString is invalid as filename 
	- the filename created so far, but modified due to canonicalization"

	| filename |
	filename := OSErrorHolder invalidArgumentsSignal handle: [:ex | ^aBlock value: nil]
				do: [self named: aString].

	"check conflicts due to canonicalization; handle the case ./aString"
	^(aString = filename asString 
			or: [aString = (self currentDirectory constructString: filename asString)])
		ifTrue: [filename]
		ifFalse: [aBlock value: filename]! !

!Filename class methodsFor: 'instance creation'!

named: aString mode: aSymbol ifFail: aBlock 
	"return 
	- value of aBlock with argument nil, if aString is not a valid filename 
	- value of aBlock with the filename created, but modified by canonicalization 
	- the filename, if aString is a valid filename matching mode aSymbol 
	- value of aBlock with argument aSymbol, otherwise"

	| filename |
	filename := self named: aString ifFail: [:fname | ^aBlock value: fname].
	^(filename checkExistMode: aSymbol)
		ifTrue: [filename]
		ifFalse: [aBlock value: aSymbol]! !

Filename class removeSelector: #name:extension:!

(Message selector: #excludeSelector:class: arguments: #(#path:name: #'Filename class' ))!

(Message selector: #excludeSelector:class: arguments: #(#path:name:extension: #'Filename class' ))!


' .ApplicationSource BASE(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 3:40:26 pm'!



!Filename methodsFor: 'file utilities'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  


' .ApplicationSource BASE(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 May 1994 at 7:29:50 pm'!



"prevent OS/2 from copying the parent directory '..' "!

!Filename methodsFor: 'file utilities'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          


' .ApplicationSource BASE(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 19 May 1994 at 2:31:23 pm'!



!Filename methodsFor: 'file utilities'!

copyRecursiveTo: newName 
	"copies the file or, if it is a directory, recursively duplicates the file tree
	(minimum protection against infinite loops, via user interaction)"

	| dest files tail |
	dest := newName asFilename.
	self isDirectory ifFalse: [^self copyTo: dest].
	tail := dest tail.
	(files := self directoryContents) do:
		[:fname | ((tail match: fname)
			and: [DialogView confirm: ('Copying directory ''' , tail , '''\to ''' , newName asString , '''\might lead into an infinite loop.\\Do you want to abort?') withCRs])
			ifTrue: [^self]].

	OSErrorHolder existingReferentSignal handle: [:ignore] do: [dest makeDirectory].
	files do: [:fname | fname = '..'
			ifFalse: [(self / fname) asFilename copyRecursiveTo: dest / fname]]! !


' .ApplicationSource BASE(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 May 1994 at 12:02:11 pm'!



!Filename class methodsFor: 'instance creation'!

named: aString ifFail: aBlock 
	"return a new Filename with the given name string; evaluate aBlock with 
	- nil, if aString is invalid as filename 
	- the filename created so far, but modified due to canonicalization"

	| filename |
	filename := OSErrorHolder invalidArgumentsSignal handle: [:ex | ^aBlock value: nil]
				do: [self named: aString].

	"check conflicts due to canonicalization; handle the case ./aString"
	^((aString sameAs: filename asString) 
			or: [aString sameAs: (self currentDirectory constructString: filename asString)])
		ifTrue: [filename]
		ifFalse: [aBlock value: filename]! !


' .ApplicationSource BASE(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 10 April 1995 at 1:43:31 pm'!



!CharacterArray methodsFor: 'converting'!

withArgs: collectionOrString andCRs: aBoolean
	"mode 1: replace nth % by nth element in collectionOrString 
	mode 2: replace %n (n from [1..9]) by nth element in collectionOrString
	mode 1 and 2 must not be mixed up; suppress replacement by specifying %%;
	replace occurrences of $\ with CR (if aBoolean is true)
	(convenience: if collectionOrString is string treat it as array with the string)"

	| args msg tmp ch printArg nxtArg |
	args := collectionOrString isString ifTrue: [Array with: collectionOrString] ifFalse: [collectionOrString].
	msg := WriteStream on: (String new: 200).
	tmp := self readStream.
	printArg := 
		[:idx | | arg |
		arg := args at: idx.
		Object messageNotUnderstoodSignal handle: [:ex | ex restartDo: [msg print: arg]] from: arg
			do: [msg nextPutAll: arg asString]].
	nxtArg := 0.
	[(ch := tmp next) == nil]
		whileFalse: [ch == $%
				ifTrue: 
					[| idx |
					(idx := '123456789%' indexOf: (ch := tmp next) ifAbsent: [11]) < 10
						ifTrue: [printArg value: idx		"mode 2"]
						ifFalse: 
							[ch == $% ifFalse: [printArg value: (nxtArg := nxtArg + 1)	"mode 1"].
							ch == $\ & aBoolean
								ifTrue: [msg cr]
								ifFalse: [ch == nil ifFalse: [msg nextPut: ch]]]]
				ifFalse: [ch == $\ & aBoolean
						ifTrue: [msg cr]
						ifFalse: [msg nextPut: ch]]].
	^msg contents! !

!CharacterArray methodsFor: 'converting'!

withArgs: collectionOrString 
	"replace % with arguments (see comment in #withArgs:andCRs:)"

	^self withArgs: collectionOrString andCRs: true! !

"get rid of an unnecessary polymorphism selector
(the only send was in Application> #sourceIdentifierForVersion:)"!

Character removeSelector: #asString!

Filename class removeSelector: #stringFromArray:!