!ClassDescription methodsFor: 'compiling'!

compile: code classified: heading notifying: requestor 
	"Compile the argument, code, as source code in the context of the receiver 
	and install the result in the receiver's method dictionary under the 
	classification indicated by the second argument, heading.  The third 
	argument, requestor, is to be notified if an error occurs.  The argument code 
	is either a string or an object that converts to a string or a 
	PositionableStream on an object that converts to a string."

	"Changes for RemoteObjects: insertion of a handler for Stream endOfStream signal 
	to catch this exception as soon as possible and avoid a remote exception propagation"

	| selector |
	Stream endOfStreamSignal
		handle: [:ex | ex proceedWith: nil]
		do: [selector := self compile: code copyLocal notifying: requestor ifFail: [^nil]].

	"repeat the code access (copyLocal), cause the 'correct it...' option may change its contents"
	(methodDict at: selector)
		sourcePointer: (SourceFileManager default
				storeMethodSource: code copyLocal asString
				class: self
				category: heading
				safely: true).

	"1st assignment induces a change to system;
	application management records the change in change objects;
	the 2nd assignment creates a new source pointer which must be
	propagated to those change objects:"
	self applicationManagement updateChanges.

	"proceed as usual"
	self organization classify: selector under: heading.
	^selector! !

