
' .ApplicationSource ExternalSource(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 8:52:49 pm'!



'- required applications for ExternalSource(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('X_Changes' ) ))!

'- sources of ExternalSource(1.0) -'!

!ClassDescription methodsFor: 'accessing'!

comment: aString 
	"Set the receiver's comment to be the argument, aString.
	modified by application management"

	| aStream |
	aString size = 0
		ifTrue: 
			[self organization classComment: aString]
		ifFalse: 
			["double internal quotes of the comment string"
			(self commentExternal: aString)
				ifTrue: [^self].
			aStream := WriteStream on: (String new: aString size).
			aStream nextPutAll: self name , ' comment:'; cr.
			aString storeOn: aStream.
			self organization classComment: aStream contents.
			ChangeSet current commentClass: self]!

commentExternal: aString 
	"if an external source reference is available for aString, store it;
	return true for success of operation (false otherwise)"

	^Signal noHandlerSignal
		handle:
			[:ex | "no external source reference available"
			ex returnWith: false]
		do: 
			["try to get an external source reference (see handle block for failure)"
			| src |
			src := self applicationManagement class sourceSignal raiseRequest.
			src class == SourceReference
				ifFalse: [self error: 'invalid source reference'. ^false].
			self organization setComment: src.
			true]!

setComment: aSourceReference 
	"assignment primitive for application management;
	does not inform change management, does not log on changes file"

	self organization setComment: aSourceReference! !

!ClassDescription methodsFor: 'compiling'!

compile: code classified: heading source: aSourceReference logging: aChangeManager ifFail: failBlock 
	"compile code, the result is a CompiledMethod; 
	classify resulting method under protocol 'heading', 
	set its source pointer to aSourceReference; 
	inform aChangeManager about the change 
	('ChangeSet current' represents the usual system change mechanism, 
	ApplicationManager records the change, but skips the system mechanism)"

	^Cursor execute showWhile: 
		[ | methodNode selector |
		methodNode := self compilerClass new
				compile: code
				in: self
				notifying: nil
				ifFail: failBlock.
		(methodDict includesKey: (selector := methodNode selector))
			ifTrue: [aChangeManager changeSelector: selector class: self]
			ifFalse: [aChangeManager addSelector: selector class: self].
		self addSelector: selector withMethod: (methodNode generate sourcePointer: aSourceReference).

		"application management records the change in change objects;
		the new source pointer must be propagated to those change objects:"
		aChangeManager updateChanges.

		"proceed as usual"
		self organization classify: selector under: heading.
		selector].! !



!CompiledMethod methodsFor: 'source code management'!

fileIndex
	"1 if the source code of the receiver is on the *.sources file, 
	2 if it is on the *.changes file,
	0 if the receiver has no stored source code (source pointer == nil), 
	the source pointer itself for external source references"

	| i src |
	^((i := SourceFileManager default fileIndexFor: (src := self sourcePointer)) == 0
		and: [src ~~ nil])
		ifTrue: [src	"this is the case for external source references"]
		ifFalse: [i]!

filePosition
	"Answer the position of this method's source code 
	on its file; handles external source references"

	"no check for nil source pointers, obviously not necessary"
	| src |
	^(src := self sourcePointer) sourceCodeManager filePositionFor: src!

getSource
	"Answer the source code for the receiver. Answer nil if this method 
	has no stored source."

	| src |
	^(src := self sourcePointer) == nil
		ifFalse: [Cursor read showWhile: [src sourceCodeManager stringAt: src ifAbsent: []]]! !



!SmallInteger methodsFor: 'source code management'!

sourceCodeManager
	"return the object which may decode the receiver as source code reference"

	^SourceFileManager default! !



"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!ChangeListView class methodsFor: 'private'!

switchTemplates
	"changes with external source reference return a SourceReference on #file
	(which is always a unique object); filtering with #fileName works well"

	^#(	(1 file hasFilter: switchFilter:perform: fileName)
		(2 type hasFilter: switchFilter:perform: class)
		(3 class hasFilter: switchFilter:perform: className)
		(4 category hasFilter: switchFilter:perform: category)
		(5 selector hasFilter: switchFilter:perform: selector)
		(6 same hasFilter: switchFilter:perform: values) )! !



!ClassDefinitionChange methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream 
	"file out aString (should be source code attached to receiver); 
	redirect source pointer to new position;
	reset change entries in current change set corresponding to the receiver"

	| c |
	ChangeSet current
		atClass: (c := self classObject) remove: #add;
		atClass: c remove: #change.
	^super fileOutExternal: aString on: aSourceStream! !



!SourceFileManager methodsFor: 'accessing'!

store: aString safely: shouldCommit 
	"Save a string onto the current target file. Commit the changes to disk if 
	shouldCommit is true"

	| filePosition file |
	file := self targetFile.
	filePosition := file writePosition.
	file nextChunkPut: aString.
	shouldCommit ifTrue: [file commit].
	^self asSourceReference: filePosition!

storeComment: aString
	"Save a string onto the current target file.
	Return the new source reference (a RemoteString)"

	^RemoteString
		newString: aString
		onFileNumber: self target
		toFile: self targetFile!

storeMethodSource: aString class: aClass category: catName safely: shouldCommit
	"Store the source code for a method onto the target file"

	| key |
	self targetFile == nil ifTrue: [^nil].
	self printCategoryChunk: catName for: aClass; cr; cr.
	key := self store: aString safely: false.
	self store: ' ' safely: shouldCommit.
	^key! !

!SourceFileManager methodsFor: 'private'!

asSourceReference: filePosition 
	^filePosition * 4 + currentTarget - 1! !



!SourceCodeStream methodsFor: 'file out support'!

removeInitializations: aChangeSequence
	"remove all class initializations which occur explicitly in the text of the given changes"

	aChangeSequence do: [:chg | | tokens |
		tokens := Scanner new scanTokens: chg text.
		2 to: tokens size do: [:i | | class | 
			((tokens at: i) == #initialize and: [(class := tokens at: i-1) asClass notNil])
				ifTrue: [classesToInitialize remove: class ifAbsent: []]]]! !



SourceCodeStream subclass: #ExternalSourceCodeStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Support'!


!ExternalSourceCodeStream methodsFor: 'accessing'!

storeComment: aString
	"Save a string onto the current target file.
	Return the source reference (appropriate for the receiver)"

	^self store: aString!

willUpdateSourcePointers
	"storing onto the receiver will change the source references"

	^true! !

!ExternalSourceCodeStream methodsFor: 'private'!

asSourceReference: filePosition 
	^SourceReference for: self sourceFileHolder position: filePosition!

sourceFileHolder
	^self subclassResponsibility! !



ExternalSourceCodeStream subclass: #ApplicationSourceStream
	instanceVariableNames: 'application '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Support'!


!ApplicationSourceStream methodsFor: 'initialize-release'!

on: anApplication 
	application := anApplication.
	super on: anApplication instSourceFilename appendStream! !

!ApplicationSourceStream methodsFor: 'private'!

sourceFileHolder
	^application! !



!Change methodsFor: 'accessing'!

text
	"Answer the text displayed by the receiver."

	| theText |
	
	file class == SourceReference ifTrue: [^file string].
	[file position: position.
	theText := file nextChunk]
		valueNowOrOnUnwindDo: [file close].
	^theText! !

!Change methodsFor: 'file accessing'!

fileName
	(file isKindOf: BufferedExternalStream)
		ifTrue: [^file name].
	file class == SourceReference
		ifTrue: [^file sourceFileHolder printString]
		ifFalse: [^"'some local stream ' ," file printString]! !

!Change methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream 
	"file out aString and return the location where it was written to"

	^aSourceStream cr; cr; store: aString!

fileOutExternalOn: aSourceStream 
	"file out source code and redirect source reference of the receiver"

	| srcPtr |
	srcPtr := self fileOutExternal: self text on: aSourceStream.
	self file: srcPtr position: nil!

restore
	"restore the sources' state due to the contents of the receiver"

	self fileIn! !



!ClassCategoryReader methodsFor: 'fileIn/Out'!

scanFrom: aStream sourceHolder: fileHolder 
	"rewritten from self>scanFrom: (executed if fileHolder is nil):
	file in methods from aStream, print name/category of found methods to transcript;
	do NOT append source code to the changes file,
	set source pointers to the original source file"

	| string p |
	fileHolder == nil ifTrue: [^self scanFrom: aStream].

	Transcript nextPutAll: class name; nextPut: $<; nextPutAll: category; cr; endEntry.
	
	[p:= aStream position.
	(string := aStream nextChunk) size > 0]
		whileTrue: ["done when double terminators"
			class
				compile: string
				classified: category
				source: (SourceReference for: fileHolder position: p)
				logging: self applicationManagement
				ifFail: []
			]! !



!ClassOrganizer methodsFor: 'comment'!

                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                           

setComment: aSourceReference 
	"assignment used in application management routines; valid arguments have 
	class UndefinedObject, SourceReference or RemoteString"

	globalComment := aSourceReference!

storeCommentOn: aSourceStream 
	"Store the comment about the class on aSourceStream"

	| srcRef saveEmphasis |
	globalComment isNil ifTrue: [^nil].
	saveEmphasis := aSourceStream emphasis.
	srcRef := aSourceStream emphasis: #italic; cr; storeComment: globalComment string.
	aSourceStream willUpdateSourcePointers
		ifTrue: [globalComment := srcRef].
	aSourceStream emphasis: saveEmphasis.
	^srcRef! !



!ChangeSequence methodsFor: 'file out'!

fileOutExternalOn: aSourceStream 
	"forward to all registered changes"

	self do: [:change | change fileOutExternalOn: aSourceStream]! !



Object subclass: #SourceReference
	instanceVariableNames: 'sourceHolder filePosition '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Support'!


!SourceReference methodsFor: 'accessing'!

file: anObject position: aNumber 
	"anObject is the holder of a source file, it must understand #sourceFile; 
	aNumber is an offset to the beginning of the source file"

	sourceHolder := anObject.
	filePosition := aNumber!

filePosition
	"return the location of the string on a file"

	^filePosition!

filePositionFor: ignore
	"return the location of the string on its source file (compatibility with default)"

	^filePosition!

sourceCodeManager
	"get the object which organizes the source code access"

	^self!

sourceFileHolder
	^sourceHolder!

sourceFileNumber
	"compatibility with RemoteString"

	^0!

stringAt: ignore ifAbsent: aBlock 
	"retrieve the string stored in the receiver's source file at receiver's position;
	if the source file is not accessible an empty ReadStream will serve as substitute"

	^Stream positionOutOfBoundsSignal
		handle: [:ex | ex returnWith: aBlock value]
		do: [self sourceFile position: filePosition; nextChunk]! !

!SourceReference methodsFor: 'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: ' on '.
	self sourceFileHolder printOn: aStream! !

!SourceReference methodsFor: 'private'!

sourceFile
	"get the source file stream from sourceFileHolder"

	^self sourceFileHolder sourceFile!

string
	"return the stored string (compatibility for class comment access)"

	^self stringAt: self ifAbsent: ['']! !

!SourceReference methodsFor: 'fileIn/Out'!

representBinaryOn: binWriter 
	"don't store the entire source file holder (an application) but just a symbol"

	^MessageSend
		receiver: self class
		selector: #for:position:
		arguments: (Array with: self sourceFileHolder name asSymbol with: self filePosition)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!SourceReference class methodsFor: 'instance creation'!

for: aSourceHolder position: aNumber 
	"aSourceHolder holds the file stream which includes the source string; 
	aNumber is the position of the source in that file; 
	get the file by sending 'aSourceHolder>sourceFile' "

	^self new
		file: (aSourceHolder isString
			ifTrue: [self applicationManagement applicationNamed: aSourceHolder asString ifAbsent: [aSourceHolder]]
			ifFalse: [aSourceHolder])
		position: aNumber! !



!ClassCommentChange methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream 
	"file out aString (should be comment attached to receiver); 
	redirect class' comment pointer to new position"

	| c srcPtr |
	((c := self classObject) == nil or: [c organization classComment ~= aString])
		ifTrue: [srcPtr := super fileOutExternal: aString on: aSourceStream]
		ifFalse:
			["redirect comment source pointer (current version only)"
			srcPtr := c organization storeCommentOn: aSourceStream.
			"now that the comment has a been saved, reset the change flag"
			ChangeSet current atClass: c remove: #comment].
	aSourceStream cr.
	^srcPtr! !



!MethodDefinitionChange methodsFor: 'source code'!

eraseSource
	"patch the source file with a blank string to prevent source scanners generate 
	a method. See also file in procedure ClassCategoryReader>scanFrom:"

	| fileStream start stop chunk |
	self sourceFileNumber = 1
		ifTrue: [^DialogView warn: 'Overwriting sources in the system source file
is not allowed.'].

	file class == SourceReference
		ifTrue: [fileStream := file sourceFile. start := file filePosition]
		ifFalse: [fileStream := file. start := position].

	"read the source, construct properly overwriting string, and write it to file"
	(chunk := fileStream position: start; nextChunk) atAllPut: Character space.
	stop := fileStream nextChunk isEmpty
		ifTrue: [' !!'	"end of category"]
		ifFalse: [''	"method definition follows"].
	fileStream close.
	fileStream name asFilename readWriteStream
		position: start;
		nextPutAll: stop; "end of category"
		nextPutAll: chunk; space; "overwrite code and terminator"
		close!

restore
	"restore a method definition state due to the contents of the receiver;
	redirect source pointer to the position stored by the receiver"

	| srcPtr manager c |
	"recompile"
	self fileIn.

	"redirect method source pointer"
	(srcPtr := file) class == SourceReference
		ifFalse: [((manager := SourceFileManager default) fileAt: 1) = file
			ifTrue: [manager target: 1 while: [srcPtr := manager asSourceReference: position]]
			ifFalse: ["must be the changes file, source pointer doesn't need a restore"
				^self]].

	((c := self classObject) compiledMethodAt: self selector) sourcePointer: srcPtr.
	self resetInChangeSet: ChangeSet current ownClass: c!

sourceFileAndPosition: aBlock 
	"get the source code reference (file and position) for this method
	from information currently in the system."

	| class method srcFile srcPosition |
	((class := self classObject) isNil or: [(class includesSelector: selector) not])
		ifTrue: [^nil].

	method := class compiledMethodAt: selector.
	(method sourcePointer isManifest)
		ifTrue: 
			["system source reference: Integer or nil"
			 | i |
			(i := method fileIndex) == 0
				ifTrue: [DialogView internalError: 'Unexpected fileIndex for method'. ^nil].
			(srcFile := SourceFileManager default fileAt: i) == nil
				ifTrue: [^nil].
			srcPosition := method filePosition]
		ifFalse:
			["external source reference"
			srcFile := method sourcePointer. srcPosition := nil].

	^aBlock value: srcFile value: srcPosition!

sourceFileNumber
	"return a number for the file where the source code resides 
	(compatible to SourceFileManager and SourceReference)"

	file = (SourceFileManager default fileAt: 1) ifTrue: [^1].
	file class == SourceReference ifTrue: [^file sourceFileNumber].
	^2! !

!MethodDefinitionChange methodsFor: 'change management'!

resetInChangeSet: aChangeSet ownClass: aClass
	"reset the change mark in the current change set corresponding to the receiver;
	'aClass' must be the class object associated to the receiver
	(we are passing it here for efficiency)"

	aChangeSet removeSelectorChanges: self selector class: aClass! !

!MethodDefinitionChange methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream
	"file out aString (should be source code attached to receiver);
	redirect source pointer to new position"

	| c srcPtr method|
	(c := self classObject) == nil
		ifTrue: ["create and pass a dummy class (just a holder for the name)"
			c := Class new setName: self className].
	srcPtr := aSourceStream
				storeMethodSource: aString
				class: c
				category: self category
				safely: aSourceStream willUpdateSourcePointers.

	"redirect method source pointer (current version only)"
	(method := c compiledMethodAt: self selector ifAbsent: [^srcPtr]) getSource = aString
		ifTrue:
			[method sourcePointer: srcPtr.
			"now that the code has a been saved, mark the selector as unchanged"
			self resetInChangeSet: ChangeSet current ownClass: c].
	^srcPtr! !



!MethodDefinitionBackup methodsFor: 'fileIn/Out'!

fileOutExternalOn: aSourceStream 
	"assume that a backup refers to a saved state, don't store again"! !



!ChangeSet methodsFor: 'private'!

atClass: class remove: changeType 
	"saving code to file (e.g. definition, comment, ...) should reset the change mark"

	| set |
	(set := classChanges at: class name ifAbsent: [^nil])
		remove: changeType ifAbsent: [^nil].
	set isEmpty
		ifTrue: [classChanges removeKey: class name].
	^changeType! !




' .ApplicationSource ExternalSource(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 31 August 1993 at 8:18:39 pm'!



!ClassDefinitionChange methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream 
	"file out aString (should be source code attached to receiver); 
	redirect source pointer to new position;
	reset change entries in current change set corresponding to the receiver"

	| c |
	(c := self classObject) isNil
		ifFalse: [ChangeSet current atClass: c remove: #add; atClass: c remove: #change].
	^super fileOutExternal: aString on: aSourceStream! !

!MethodDefinitionChange methodsFor: 'fileIn/Out'!

fileOutExternal: aString on: aSourceStream
	"file out aString (should be source code attached to receiver);
	redirect source pointer to new position"

	| c srcPtr method invalid |
	(invalid := (c := self classObject) == nil)
		ifTrue: ["create and pass a dummy class (just a holder for the name)"
			c := Class new setName: self className].
	srcPtr := aSourceStream
				storeMethodSource: aString
				class: c
				category: self category
				safely: aSourceStream willUpdateSourcePointers.

	"redirect method source pointer (current version only)"
	invalid ifFalse:
		[(method := c compiledMethodAt: self selector ifAbsent: [^srcPtr]) getSource = aString
			ifTrue:
				[method sourcePointer: srcPtr.
				"now that the code has a been saved, mark the selector as unchanged"
				self resetInChangeSet: ChangeSet current ownClass: c]].
	^srcPtr! !

!SourceCodeStream methodsFor: 'file out support'!

removeInitializations: collectionOfClasses 
	"remove all elements of collectionOfClasses from classes to initialize"

	classesToInitialize removeAll: collectionOfClasses! !


' .ApplicationSource ExternalSource(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 1 September 1993 at 3:27:26 pm'!



(Message selector: #excludeSelector:class: arguments: #(#scanFrom:sourceHolder: #ClassCategoryReader ))!

(Message selector: #excludeSelector:class: arguments: #(#compile:classified:source:logging:ifFail: #ClassDescription ))!


' .ApplicationSource ExternalSource(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 September 1993 at 1:35:49 pm'!



!MethodDefinitionChange methodsFor: 'source code'!

eraseSource
	"patch the source file with a blank string to prevent source scanners generate 
	a method. See also file in procedure ClassCategoryReader>scanFrom:"

	| fileStream start stop chunk access |
	self sourceFileNumber = 1
		ifTrue: [^DialogView warn: 'Overwriting sources in the system source file
is not allowed.'].

	file class == SourceReference
		ifTrue: [fileStream := file sourceFile. start := file filePosition]
		ifFalse: [fileStream := file. start := position].

	"read the source, construct properly overwriting string, and write it to file"
	(chunk := fileStream position: start; nextChunk) atAllPut: Character space.
	stop := fileStream nextChunk isEmpty
		ifTrue: [' !!'	"end of category"]
		ifFalse: [''	"method definition follows"].
	fileStream close.
	fileStream := fileStream name asFilename.
	(access := fileStream isWritable) ifFalse: [fileStream makeWritable].
	fileStream readWriteStream
		position: start;
		nextPutAll: stop; "end of category"
		nextPutAll: chunk; space; "overwrite code and terminator"
		close.
	access ifFalse: [fileStream makeUnwritable]! !


' .ApplicationSource ExternalSource(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 4 November 1993 at 2:53:13 pm'!



"fix: post installation inits may include inits which are not part of auto init section; #removeAll: is not appropriate"!

!SourceCodeStream methodsFor: 'file out support'!

removeInitializations: collectionOfClasses 
	"remove all elements of collectionOfClasses from classes to initialize"

	collectionOfClasses do: [:cls | classesToInitialize remove: cls ifAbsent: []]! !


' .ApplicationSource ExternalSource(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 11 April 1994 at 7:30:02 pm'!



!SourceReference methodsFor: 'accessing'!

stringAt: ignore ifAbsent: aBlock 
	"return the string stored in the receiver's source file at receiver's position; 
	return the result of aBlock if the source file is not accessible"

	^sourceHolder stringAt: filePosition ifAbsent: aBlock! !

"adapt to patch for PeekableStream in APL-Framework(1.4)-(1.5)"!

!ClassDescription methodsFor: 'accessing'!

commentExternal: aString 
	"if an external source reference is available for aString, store it;
	return true for success of operation (false otherwise)"

	^Signal noHandlerSignal
		handle:
			[:ex | "no external source reference available"
			ex returnWith: false]
		do: 
			["try to get an external source reference (see handle block for failure)"
			| src |
			src := self applicationManagement class sourceSignal raiseRequest.
			src class == SourceReference ifFalse: [^false].
			self organization setComment: src.
			true]! !

"avoid the dialog during access on methods without source"!

!MethodDefinitionChange methodsFor: 'source code'!

sourceFileAndPosition: aBlock 
	"get the source code reference (file and position) for this method
	from information currently in the system."

	| class method fileRef filePos |
	(class := self classObject) == nil ifTrue: [^nil].
	method := class compiledMethodAt: selector ifAbsent: [^nil].
	(fileRef := method fileIndex) isInteger
		ifTrue: 
			["system source reference"
			fileRef == 0 ifTrue: [^nil	"source pointer is nil"].
			(fileRef := SourceFileManager default fileAt: fileRef) == nil
				ifTrue: [^nil].
			filePos := method filePosition]
		ifFalse: ["fileRef is a SourceReference, filePos remains nil"].

	^aBlock value: fileRef value: filePos! !

(Message selector: #excludeSelector:class: arguments: #(#switchTemplates #'ChangeListView class' ))!


' .ApplicationSource ExternalSource(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 4:13:32 pm'!



"handle source file holder references which could not be resolved properly"!

!SourceReference methodsFor: 'private'!

sourceFileHolderIfAbsent: aBlock 
	"return the source file holder (result of aBlock if not resolvable)"

	^sourceHolder isString
		ifTrue: [sourceHolder := self applicationManagement applicationNamed: sourceHolder ifAbsent: [^aBlock value]]
		ifFalse: [sourceHolder]! !

!SourceReference methodsFor: 'accessing'!

sourceFileHolderId 
	"return the identifier of the source file holder"

	^(self sourceFileHolderIfAbsent: [^'unresolved source holder ', sourceHolder]) identifier! !

!SourceReference methodsFor: 'accessing'!

sourceFileHolder
	^self sourceFileHolderIfAbsent: [sourceHolder]! !

!SourceReference methodsFor: 'accessing'!

stringAt: ignore ifAbsent: aBlock 
	"return the string stored in the receiver's source file at receiver's position; 
	return the result of aBlock if the source file is not accessible"

	^(self sourceFileHolderIfAbsent: [^aBlock value])
		stringAt: filePosition ifAbsent: aBlock! !

!SourceReference methodsFor: 'printing'!

printOn: aStream 
	super printOn: aStream.
	aStream nextPutAll: ' on '; nextPutAll: self sourceFileHolderId! !

!Change methodsFor: 'file accessing'!

fileName
	^(file isKindOf: BufferedExternalStream)
		ifTrue: [file name]
		ifFalse: [file class == SourceReference
				ifTrue: [file sourceFileHolderId]
				ifFalse: [file printString]]! !


' .ApplicationSource ExternalSource(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 May 1994 at 12:07:28 pm'!



!MethodDefinitionBackup methodsFor: 'accessing'!

text
	"backups can be persistent and refer to a non-application file;
	therefore we have to take care for access faults"

	^OSErrorHolder inaccessibleSignal
		handle: [:ex | ex returnWith: 'Backup text not available.\Cannot access source file\''' withCRs, file name asString, '''']
		do: [super text]! !


' .ApplicationSource ExternalSource(1.8)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 September 1994 at 8:45:27 pm'!



(Message selector: #excludeSelector:class: arguments: #(#sourceFileHolder #ApplicationSourceStream ))!

(Message selector: #excludeSelector:class: arguments: #(#on: #ApplicationSourceStream ))!

(Message selector: #excludeClassDefinition: arguments: #(#ApplicationSourceStream ))!

(Message selector: #excludeClassDefinition: arguments: #(#'ApplicationSourceStream class' ))!


' .ApplicationSource ExternalSource(1.9)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 22 September 1994 at 12:01:30 pm'!



"fix: do not file out external if the destination stream does not modify source pointers"!

!ChangeSequence methodsFor: 'file out'!

fileOutExternalOn: aSourceStream 
	"forward to all registered changes; force normal file-out (instead of external)
	if aSourceStream does not change source pointers"

	aSourceStream willUpdateSourcePointers ifFalse: [^self fileOutOn: aSourceStream].
	self do: [:change | change fileOutExternalOn: aSourceStream]! !


' .ApplicationSource ExternalSource(1.10)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 January 1995 at 11:51:00 am'!



"fix: improper source/changes file reference after #invalidSourceFileAction:with:
(source file access failure imposes discarding the file entry without Change objects (used as persistent source code pointers in Application Management) becoming aware; redefining a proper value did not update persistent Change objects; to overcome dependency on system source file entries the inst var 'file' may store Integer values referring to the source file of the SourceFileManager respectively"!

!PositionableStream methodsFor: 'fileIn'!

chunkAt: anInteger
	^self position: anInteger; nextChunk! !

!SourceReference methodsFor: 'fileIn/Out'!

chunkAt: anInteger
	^self stringAt: anInteger ifAbsent: ['']! !

!SourceFileManager methodsFor: 'accessing'!

chunkAt: aKey 
	^self stringAt: aKey ifAbsent: ['']! !

!Change methodsFor: 'file accessing'!

fileName
	^file isInteger
		ifTrue: 
			[| filename |
			(filename := SourceFileManager default nameAt: file) == nil
				ifTrue: ['undefined changes file']
				ifFalse: [filename asString]]
		ifFalse: [(file isKindOf: BufferedExternalStream)
				ifTrue: [file name]
				ifFalse: [(file isKindOf: SourceReference)
						ifTrue: [file sourceFileHolderId]
						ifFalse: [file printString]]]! !

!Change methodsFor: 'accessing'!

text
	"return the text the receiver is pointing to"

	^(file isInteger ifTrue: [SourceFileManager default] ifFalse: [file])
		chunkAt: position! !

!MethodDefinitionChange methodsFor: 'source code'!

text
	"return the text the receiver is pointing to"

	| cls mth |
	file == nil ifFalse: [^super text].
	((cls := self classObject) == nil or: [(mth := cls compiledMethodAt: self selector ifAbsent: []) == nil])
		ifTrue: [self errorSignal raiseWith: self errorString: 'Source code not available for ' , self className , '>' , self selector]
		ifFalse: [mth getSource]! !

!MethodDefinitionChange methodsFor: 'source code'!

eraseSource
	"patch the source file with a blank string to prevent source scanners generate 
	a method. See also file in procedure ClassCategoryReader>scanFrom:"

	| fileStream start stop chunk access |
	self sourceFileNumber = 1
		ifTrue: [^DialogView warn: 'Overwriting sources in the system source file
is not allowed.'].

	start := position.
	(fileStream := file) isInteger
		ifTrue:
			[| sfm |
			fileStream := (sfm := SourceFileManager default) fileAt: file.
			start := sfm filePositionFor: position]
		ifFalse: [(file isKindOf: SourceReference)
			ifTrue: [fileStream := file sourceFile. start := file filePosition]
			ifFalse: [start := position]].

	"read the source, construct properly overwriting string, and write it to file"
	(chunk := fileStream chunkAt: start) atAllPut: Character space.
	stop := fileStream nextChunk isEmpty
		ifTrue: [' !!'	"end of category"]
		ifFalse: [''	"method definition follows"].
	fileStream close.
	fileStream := fileStream name asFilename.
	(access := fileStream isWritable) ifFalse: [fileStream makeWritable].
	fileStream readWriteStream
		position: start;
		nextPutAll: stop; "end of category"
		nextPutAll: chunk; space; "overwrite code and terminator"
		close.
	access ifFalse: [fileStream makeUnwritable]! !

!MethodDefinitionChange methodsFor: 'source code'!

sourceFileNumber
	"return a number for the file where the source code resides 
	(compatible to SourceFileManager and SourceReference)"

	^file isInteger
		ifTrue: [file]
		ifFalse: [(file isKindOf: SourceReference)
				ifTrue: [file sourceFileNumber]
				ifFalse: [SourceFileManager default allFiles indexOf: file ifAbsent: []]]! !

!MethodDefinitionChange methodsFor: 'source code'!

sourceFileAndPosition: aBlock 
	"get the source code reference (file and position) for this method
	from information currently in the system."

	| class method fileRef filePos |
	(class := self classObject) == nil ifTrue: [^nil].
	method := class compiledMethodAt: selector ifAbsent: [^nil].
	(fileRef := method fileIndex) isInteger
		ifTrue: 
			["system source reference"
			fileRef == 0 ifTrue: [^nil	"source pointer is nil"].
			filePos := method sourcePointer]
		ifFalse: ["fileRef is a SourceReference, filePos remains nil"].

	^aBlock value: fileRef value: filePos! !

"change file references to system files to appropriate integer values"
| sfm sysfiles files |
sysfiles := (sfm := SourceFileManager default) allFiles.
files := IdentityDictionary new.
self applicationManagement do: [:app | (app history, app changes) do:
	[:chg | | f |
	((f := chg file) == nil | f isInteger or: [(f := sysfiles indexOf: f
		ifAbsent: [(f isKindOf: BufferedExternalStream)
			ifTrue:
				[| target |
				target := files at: f ifAbsent:
					[DialogView
						choose: 'One of the changes refers to file\' withCRs, f name asString, '\Tell me how to map it' withCRs
						labels: #('sources' 'changes' 'debug')
						values: #(1 2 0)
						default: 1].
				target = 0 ifTrue: [self halt].
				files at: f put: target]
			ifFalse: [0]]) == 0])
		ifFalse: [sfm target: f while:
			[chg file: f position: (sfm asSourceReference: (chg instVarAt: 2))]]]]!

!MethodDefinitionChange methodsFor: 'source code'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          

"modified source file access error handling: avoid discarding source files if errors occur, improve error messages"!

!SourceFileManager methodsFor: 'private'!

invalidSourceFile: fileIndex with: errorString
	"Take some action based on the fact that the file at fileIndex either can't be 
	opened, or has no source at the requested position"

	| label str flag |
	(str := TextStream on: (String new: 1000))
		cr; nextPut: $"; nextPutAll: (label := 'Source file ', fileIndex printString, ' is invalid');
		crtab; nextPut: $(; nextPutAll: errorString; nextPut: $);
		cr; cr; nextPutAll: 'Edit the following expression, and evaluate:'; nextPut: $";
		cr; crtab; emphasis: #large;
		nextPutAll: 'SourceFileManager default ';
		crtab: 2; nextPutAll: 'file: '; print: fileIndex;
		crtab: 2; nextPutAll: 'name: ';
		emphasis: #(#large #bold); store: (self nameAt: fileIndex) asString; emphasis: #large;
		crtab: 2; nextPutAll: 'writable: '; print: ((self fileAt: fileIndex) isKindOf: ExternalReadAppendStream).

	flag := ScheduledWindow classPool at: #PromptForOpen.
	ScheduledWindow promptForOpen: false.
	[ComposedTextView
		open: (ValueHolder with: (str := str contents))
		label: label
		icon: (Icon constantNamed: #workspace)
		extent: (str asComposedText extent + (30@40) min: 480@200)]
		valueNowOrOnUnwindDo: [ScheduledWindow promptForOpen: flag]! !

!SourceFileManager methodsFor: 'accessing'!

stringAt: aKey ifAbsent: aBlock 
	"Retrieve the string stored in the receiver's files at aKey. aKey is an Integer 
	which encodes both the file index and position"

	| fileIndex filePosition file |
	(aKey == nil or: [(file := files at: (fileIndex := self fileIndexFor: aKey)) == nil])
		ifTrue: [^aBlock value].
	filePosition := self filePositionFor: aKey.
	^OSErrorHolder errorSignal
		handle: 
			[:ex | 
			(OSErrorHolder noResourcesSignal accepts: ex signal)
				ifTrue: [Transcript cr; show: 'Out of files during source access']
				ifFalse: 
					[self invalidSourceFile: fileIndex with: ex errorString.
					files at: fileIndex put: nil].
			ex returnWith: aBlock value]
		do: 
			[| str mth |
			(str := file chunkAt: filePosition) isEmpty
				ifTrue: 
					[str := 'Source access at position ' , filePosition printString.
					str := file size < filePosition
								ifTrue: [str , ' beyond file size']
								ifFalse: [str , ' returns an empty string'].
					mth := CompiledMethod allInstances detect: [:m | m sourcePointer == aKey]
								ifNone: [].
					mth == nil ifFalse: [str := str , '. Check ' , mth printString].
					fileIndex == 2 ifTrue: [str := str , '. Maybe image/changes file mismatch'].
					self invalidSourceFile: fileIndex with: str.
					aBlock value]
				ifFalse: [str]]! !


' .ApplicationSource ExternalSource(1.11)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 January 1995 at 1:39:12 pm'!



!ChangeSequence methodsFor: 'file out'!

fileOutExternalOn: aSourceStream 
	"forward to all registered changes; force normal file-out (instead of external) 
	if aSourceStream does not change source pointers"

	aSourceStream willUpdateSourcePointers
		ifFalse: [^self fileOutOn: aSourceStream targetFile].
	self do: [:change | change fileOutExternalOn: aSourceStream]! !


' .ApplicationSource ExternalSource(1.12)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 23 January 1995 at 5:07:11 pm'!



"bug fix: bad source code pointer assignment during reinstall for backups on the system sources"!

!MethodDefinitionChange methodsFor: 'source code'!

restore
	"restore a method definition state due to the contents of the receiver; 
	redirect source pointer to the position stored by the receiver"

	| fileRef cls |
	self fileIn.	"recompile"
	"redirect method source pointer"
	(fileRef := self sourceFileNumber) = 2
		ifTrue: [^self	"source pointer does not need a restore"].
	((cls := self classObject) compiledMethodAt: self selector)
		sourcePointer: (fileRef = 0
				ifTrue: [file	"SourceReference"]
				ifFalse: [position	"SourceFileManager key"]).
	self resetInChangeSet: ChangeSet current ownClass: cls! !