' .ApplicationSource ExternalSource(1.11)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 28 December 1994 at 3:49:30 pm'!



"fix: mnu #nextChunkPut: during file-out of some OtherChange"!

!ChangeSequence methodsFor: 'file out'!

fileOutExternalOn: aSourceStream 
	"forward to all registered changes; force normal file-out (instead of external) 
	if aSourceStream does not change source pointers"

	aSourceStream willUpdateSourcePointers
		ifFalse: [^self fileOutOn: aSourceStream targetFile].
	self do: [:change | change fileOutExternalOn: aSourceStream]! !


' .ApplicationSource ExternalSource(1.12)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 16 January 1995 at 5:06:05 pm'!



"fix: improper source/changes file reference after #invalidSourceFileAction:with:
(source file access failure imposes discarding the file entry without Change objects (used as persistent source code pointers in Application Management) becoming aware; redefining a proper value did not update persistent Change objects; to overcome dependency on system source file entries the inst var 'file' may store Integer values referring to the source file of the SourceFileManager respectively"!

!PositionableStream methodsFor: 'fileIn'!

chunkAt: anInteger
	^self position: anInteger; nextChunk! !

!SourceReference methodsFor: 'fileIn/Out'!

chunkAt: anInteger
	^self stringAt: anInteger ifAbsent: ['']! !

!SourceFileManager methodsFor: 'accessing'!

chunkAt: aKey 
	^self stringAt: aKey ifAbsent: ['']! !

!Change methodsFor: 'file accessing'!

fileName
	^file isInteger
		ifTrue: [(SourceFileManager default nameAt: file) asString]
		ifFalse: [(file isKindOf: BufferedExternalStream)
				ifTrue: [file name]
				ifFalse: [(file isKindOf: SourceReference)
						ifTrue: [file sourceFileHolderId]
						ifFalse: [file printString]]]! !

!Change methodsFor: 'accessing'!

text
	"return the text the receiver is pointing to"

	^(file isInteger ifTrue: [SourceFileManager default] ifFalse: [file])
		chunkAt: position! !

!MethodDefinitionChange methodsFor: 'source code'!

text
	"return the text the receiver is pointing to"

	| cls mth |
	file == nil ifFalse: [^super text].
	((cls := self classObject) == nil or: [(mth := cls compiledMethodAt: self selector ifAbsent: []) == nil])
		ifTrue: [self errorSignal raiseWith: self errorString: 'Source code not available for ' , self className , '>' , self selector]
		ifFalse: [mth getSource]! !

!MethodDefinitionChange methodsFor: 'source code'!

eraseSource
	"patch the source file with a blank string to prevent source scanners generate 
	a method. See also file in procedure ClassCategoryReader>scanFrom:"

	| idx fileStream start stop chunk access |
	(idx := self sourceFileNumber) = 1
		ifTrue: [^DialogView warn: 'Overwriting sources in the system source file
is not allowed.'].

	start := position.
	(fileStream := file) isInteger
		ifTrue:
			[| sfm |
			fileStream := (sfm := SourceFileManager default) fileAt: idx.
			start := sfm filePositionFor: position]
		ifFalse: [(file isKindOf: SourceReference)
			ifTrue: [fileStream := file sourceFile. start := file filePosition]].

	"read the source, construct properly overwriting string, and write it to file"
	(chunk := fileStream chunkAt: start) atAllPut: Character space.
	stop := fileStream nextChunk isEmpty
		ifTrue: [' !!'	"end of category"]
		ifFalse: [''	"method definition follows"].
	fileStream close.
	fileStream := fileStream name asFilename.
	(access := fileStream isWritable) ifFalse: [fileStream makeWritable].
	fileStream readWriteStream
		position: start;
		nextPutAll: stop; "end of category"
		nextPutAll: chunk; space; "overwrite code and terminator"
		close.
	access ifFalse: [fileStream makeUnwritable]! !

!MethodDefinitionChange methodsFor: 'source code'!

sourceFileNumber
	"return a number for the file where the source code resides 
	(compatible to SourceFileManager and SourceReference)"

	^file isInteger
		ifTrue: [file]
		ifFalse: [(file isKindOf: SourceReference)
				ifTrue: [file sourceFileNumber]
				ifFalse: [SourceFileManager default allFiles indexOf: file ifAbsent: []]]! !

!MethodDefinitionChange methodsFor: 'source code'!

sourceFileAndPosition: aBlock 
	"get the source code reference (file and position) for this method
	from information currently in the system."

	| class method fileRef filePos |
	(class := self classObject) == nil ifTrue: [^nil].
	method := class compiledMethodAt: selector ifAbsent: [^nil].
	(fileRef := method fileIndex) isInteger
		ifTrue: 
			["system source reference"
			fileRef == 0 ifTrue: [^nil	"source pointer is nil"].
			filePos := method sourcePointer]
		ifFalse: ["fileRef is a SourceReference, filePos remains nil"].

	^aBlock value: fileRef value: filePos! !

!MethodDefinitionChange methodsFor: 'source code'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              

"change file references to system files to appropriate integer values"
| sfm sysfiles files |
sysfiles := (sfm := SourceFileManager default) allFiles.
files := IdentityDictionary new.
self applicationManagement do: [:app | (app history, app changes) do:
	[:chg | | f |
	((f := chg file) == nil | f isInteger or: [(f := sysfiles indexOf: f
		ifAbsent: [(f isKindOf: BufferedExternalStream)
			ifTrue:
				[| target |
				target := files at: f ifAbsent:
					[DialogView
						choose: 'One of the changes refers to file\' withCRs, f name asString, '\Tell me how to map it' withCRs
						labels: #('sources' 'changes' 'debug')
						values: #(1 2 0)
						default: 1].
				target = 0 ifTrue: [self halt].
				files at: f put: target]
			ifFalse: [0]]) == 0])
		ifFalse: [chg file: f position: (sfm keyForFile: f position: (chg instVarAt: 2))]]]!

"modified source file access error handling: avoid discarding source files if errors occur, improve error messages"!

!SourceFileManager methodsFor: 'private'!

invalidSourceFile: fileIndex with: errorString
	"Take some action based on the fact that the file at fileIndex either can't be 
	opened, or has no source at the requested position"

	| writable label str longForm flag |
	writable := (self fileAt: fileIndex) isWritable.
	(str := TextStream on: (String new: 1000))
		cr; nextPut: $"; nextPutAll: (label := 'Source file ', fileIndex printString, ' is invalid');
		crtab; nextPut: $(; nextPutAll: errorString; nextPut: $);
		cr; cr; nextPutAll: 'Edit the following expression, and evaluate:'; nextPut: $";
		cr; crtab; emphasis: #large;
		nextPutAll: 'SourceFileManager default '.
	(fileIndex == self changesIndex & writable)
		ifTrue: [str nextPutAll: 'changesFileName: '; crtab: 2. longForm := false]
		ifFalse: [(fileIndex == self sourcesIndex & writable not)
			ifTrue: [str nextPutAll: 'sourcesFileName: '; crtab: 2. longForm := false]
			ifFalse:
				[str crtab: 2; nextPutAll: 'file: ', fileIndex printString; crtab: 2; nextPutAll: 'name: '.
				longForm := true]].
	str emphasis: #(#large #bold); store: (self nameAt: fileIndex) asString; emphasis: #large.
	longForm ifTrue: [str crtab: 2; nextPutAll: 'writable: '; print: writable].

	flag := ScheduledWindow promptForOpen.
	ScheduledWindow promptForOpen: false.
	[ComposedTextView
		open: (ValueHolder with: (str := str contents))
		label: label
		icon: (Icon constantNamed: #workspace)
		extent: (str asComposedText extent + (30@40) min: 480@200)]
		valueNowOrOnUnwindDo: [ScheduledWindow promptForOpen: flag]! !

!SourceFileManager methodsFor: 'accessing'!

stringAt: aKey ifAbsent: aBlock 
	"Retrieve the string stored in the receiver's files at aKey. aKey is an Integer 
	which encodes both the file index and position"

	| fileIndex filePosition file |
	(aKey == nil or: [(file := files at: (fileIndex := self fileIndexFor: aKey)) == nil])
		ifTrue: [^aBlock value].
	filePosition := self filePositionFor: aKey.
	^OSErrorHolder errorSignal
		handle: 
			[:ex | 
			(OSErrorHolder noResourcesSignal accepts: ex signal)
				ifTrue: [Transcript cr; show: 'Out of files during source access']
				ifFalse: 
					[self invalidSourceFile: fileIndex with: ex errorString.
					files at: fileIndex put: nil].
			ex returnWith: aBlock value]
		do: 
			[| str mth |
			(str := file chunkAt: filePosition) isEmpty
				ifTrue: 
					[str := 'Source access at position ' , filePosition printString.
					str := file size < filePosition
								ifTrue: [str , ' beyond file size']
								ifFalse: [str , ' returns an empty string'].
					mth := CompiledMethod allInstances detect: [:m | m sourcePointer == aKey]
								ifNone: [].
					mth == nil ifFalse: [str := str , '. Check ' , mth printString].
					fileIndex == self changesIndex ifTrue: [str := str , '. Maybe image/changes file mismatch'].
					self invalidSourceFile: fileIndex with: str.
					aBlock value]
				ifFalse: [str]]! !


' .ApplicationSource ExternalSource(1.13)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 17 January 1995 at 2:24:12 pm'!



!MethodDefinitionChange methodsFor: 'source code'!

eraseSource
	"patch the source file with a blank string to prevent source scanners generate 
	a method. See also file in procedure ClassCategoryReader>scanFrom:"

	| fileStream start stop chunk access |
	self sourceFileNumber = 1
		ifTrue: [^DialogView warn: 'Overwriting sources in the system source file
is not allowed.'].

	start := position.
	(fileStream := file) isInteger
		ifTrue:
			[| sfm |
			fileStream := (sfm := SourceFileManager default) fileAt: file.
			start := sfm filePositionFor: position]
		ifFalse: [(file isKindOf: SourceReference)
			ifTrue: [fileStream := file sourceFile. start := file filePosition]
			ifFalse: [start := position]].

	"read the source, construct properly overwriting string, and write it to file"
	(chunk := fileStream chunkAt: start) atAllPut: Character space.
	stop := fileStream nextChunk isEmpty
		ifTrue: [' !!'	"end of category"]
		ifFalse: [''	"method definition follows"].
	fileStream close.
	fileStream := fileStream name asFilename.
	(access := fileStream isWritable) ifFalse: [fileStream makeWritable].
	fileStream readWriteStream
		position: start;
		nextPutAll: stop; "end of category"
		nextPutAll: chunk; space; "overwrite code and terminator"
		close.
	access ifFalse: [fileStream makeUnwritable]! !


' .ApplicationSource ExternalSource(1.14)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 23 January 1995 at 4:22:52 pm'!



"bug fix: bad source code pointer assignment during reinstall for backups on the system sources"!

!MethodDefinitionChange methodsFor: 'source code'!

restore
	"restore a method definition state due to the contents of the receiver; 
	redirect source pointer to the position stored by the receiver"

	| fileRef cls |
	self fileIn.	"recompile"
	"redirect method source pointer"
	(fileRef := self sourceFileNumber) = SourceFileManager default changesIndex
		ifTrue: [^self	"source pointer does not need a restore"].
	((cls := self classObject) compiledMethodAt: self selector)
		sourcePointer: (fileRef = 0
				ifTrue: [file	"SourceReference"]
				ifFalse: [position	"SourceFileManager key"]).
	self resetInChangeSet: ChangeSet current ownClass: cls! !