"Upgrade utility script for application sources
(C) Georg Heeg Objektorientierte Systeme, 1993-1995

The script executes application upgrade just as invoked with 
	ApplicationManagementBrowser> file in/out...> upgrade...
for all applications denoted in 'upgrade.dat'.

NOTE: 
	Application Management must be present in the image executing this script.
	The script applies to installed applications only.

20 April 1995, hg"


| apm upgradeAppExact upgradeApp upgradeOne upgData ctx currentPath basepath |
apm := self applicationManagement.

"--- find out the path of this file ---"
ctx := thisContext.
[ctx == nil or: [ctx receiver isKindOf: ExternalStream]] whileFalse: [ctx := ctx sender].
currentPath := ctx receiver name asFilename directory.
basepath := currentPath directory.

"--- use fat-filesystem compatible application directory names ---"
#('AccessControl' 'APL-ChangeSequence' 'APL-Framework' 'APL-Interface' 'APL-SystemLink' 'BASE' 'ExternalSource' 'StorableObject' 'Version' 'X_Changes' 'X_Interface' 'X_OS-Support') do:
	[:apname | | ap fname |
	(ap := apm applicationNamed: apname) == nil ifFalse:
		[ap releaseSourceFile.
		fname := ap label filenameForPlatform: #dos.
		#(#unix #mac #win32 #os2) do: [:pf | ap label setFilename: fname forPlatform: pf].
		"attach to new path if the distribution disk has been copied to another directory"
		ap setPath: (ap path tail = 'vw20'
				ifTrue: [basepath construct: 'vw20']
				ifFalse: [basepath])]].


"--- upgrade a single application, watch start and stop minor version ---"
upgradeAppExact := [:app :start :stop | | shouldstart start2 msg |
	"upgrade application app from its current source file;
	begin including minor version start, stop after including minor version stop"

	shouldstart := app minorVersion + 1.
	(start2 := start == nil ifTrue: [shouldstart] ifFalse: [start]) > shouldstart
		ifTrue:
			[msg := '*** upgrade mismatch: ***\You must upgrade ''', app identifier, '''\to minor version ', (start2 - 1) printString, '\before executing this upgrade.'.
			DialogView warn: msg withCRs asComposedText centered.
			^self error: '*** upgrade mismatch: ***']
		ifFalse: [start2 < shouldstart
			ifTrue: 
				[msg := '*** upgrade mismatch: ***\The script starts upgrading with minor version ', start2 printString, ',\but ''', app identifier, ''' already has minor version ', app minorVersion printString, '.\Proceed upgrading from minor version ', shouldstart printString, ' ?'.
				(DialogView confirm: msg withCRs asComposedText centered)
					ifFalse: [^self error: '*** upgrade mismatch: ***']]].
	app upgradeMinorUpTo: (VersionDescriptor major: app majorVersion minor: stop)].


"--- upgrade a single application ---"
upgradeApp := [:app :maj :start :stop |
	"upgrade application app from its current source file;
	begin including minor version start, stop after including minor version stop"
	stop == nil
		ifTrue: [app upgradeMinor	"upgrade all"]
		ifFalse: [stop = app minorVersion
			ifTrue: ["done"]
			ifFalse: [upgradeAppExact value: app value: start value: stop]]].


"--- upgrade a single application as specified in data file input line ---"
upgradeOne := [:spec | | args delta start stop |
	"scan for input <application name/v1>/<minor1>-<minor2>"
	args := UnixFilename components: spec.
	args at: 1 put: (apm applicationNamed: args first).
	(delta := args removeLast) = '*'
		ifTrue: [start := stop := nil]
		ifFalse:
			[start := (Integer readFrom: (delta := delta readStream)).
			stop := (Integer readFrom: delta) negated].
	upgradeApp valueWithArguments: (args add: start; add: stop; asArray)].
		

"--- upgrade all applications as specified in data file 'upgrade.dat' ---"
upgData := (currentPath construct: 'upgrade.dat') readStream detectLineEndConvention.
[[upgData skipSeparators atEnd]
	whileFalse: [upgradeOne value: (upgData upTo: Character cr)]]
	valueNowOrOnUnwindDo: [upgData close].
