
' .ApplicationSource Version(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 4:09:17 pm'!



'- required applications for Version(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('AccessControl' ) ))!

'- sources of Version(1.0) -'!

Object subclass: #NoVersion
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Version'!
NoVersion comment:
'This is a dummy version descriptor, answering the null string to all messages.'!


!NoVersion methodsFor: 'error handling'!

doesNotUnderstand: aMessage 
	^aMessage selector == #doesNotUnderstand:
		ifTrue: [super doesNotUnderstand: aMessage]
		ifFalse: ['']! !



AccessControlledObjectAccessor subclass: #VersionedObjectAccessor
	instanceVariableNames: 'version '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Version'!
VersionedObjectAccessor comment:
'Versionsnummer fuer Objekte mit Namen (major und minor Versionen). Der Dateiname beinhaltet die Versionsnummer. Zusaetzlich kann festgehalten werden, ob sich das Objekt von der Version unterscheidet.

Instance Variables:
	version	<VersionDescriptor>  Versionsnummer

'!


!VersionedObjectAccessor methodsFor: 'accessing'!

identifier
	^name , $( asString, self version printString, $) asString!

incrementMajor
	version := version newMajor!

incrementMinor 
	version := version newMinor!

modified
	^self version modified!

touch
	^self version touch!

version
	^version!

version: aVersionDescriptor 
	version := aVersionDescriptor! !

!VersionedObjectAccessor methodsFor: 'initialize release'!

initialize
	super initialize.
	version := NoVersion new! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!VersionedObjectAccessor class methodsFor: 'instance creation'!

initialVersion 
	^self new version: VersionDescriptor initialVersion! !



Magnitude subclass: #VersionDescriptor
	instanceVariableNames: 'major minor changed '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Version'!
VersionDescriptor comment:
'VersionDescriptor supports version management with version numbers.
A version consists of a major and a minor version number and a change flag signalling a modification. The change flag is cleared when installing the next release.

Instance variables:
major	<Integer>	major (version) number
minor	<Integer>	minor (release) number
changed	<Boolean>	flag indicated a modifcation to the current version'!


!VersionDescriptor methodsFor: 'accessing'!

major
	"Answer the major version number"

	^major!

minor
	"Answer the minor version number"

	^minor!

modified
	"Answer if the version has been changed since the last release"

	^changed!

touch
	"Mark the version descriptor as changed"

	changed := true!

untouch
	"Mark the version descriptor as unchanged"

	changed := false! !

!VersionDescriptor methodsFor: 'testing'!

isInitialVersion
	^major == 0 and: [minor == 0]! !

!VersionDescriptor methodsFor: 'printing'!

fileString
	"Answer a string which may be used in filenames"
	^self major printString , self class separator asString , self minor printString!

printOn: stream 
	self major printOn: stream.
	stream nextPut: self class separator.
	self minor printOn: stream.
	self modified ifTrue: [stream nextPut: $m]! !

!VersionDescriptor methodsFor: 'initialize-release'!

initialize
	major := 0.
	minor := 0.
	changed := false!

major: majorNumber minor: minorNumber 
	minor := minorNumber.
	major := majorNumber! !

!VersionDescriptor methodsFor: 'comparing'!

< aVersionDescriptor 
	"less than aVersionDescripor if my major number is less, 
	or (if majors are equal) if my minor number is less,
	or (if both majors and minors are equal) if receiver is unmodified but the other is"

	^self major < aVersionDescriptor major
		or: [self major = aVersionDescriptor major 
			and: [self minor < aVersionDescriptor minor
				or: [self minor = aVersionDescriptor minor
					and: [self modified not & aVersionDescriptor modified]]]]!

= anObject
	^(minor == anObject minor
		and: [major == anObject major])
		and: [self modified == anObject modified]!

hash
	^minor + major! !

!VersionDescriptor methodsFor: 'release managing'!

newMajor
	"return a new (not just modified !!) version descriptor with an incremented 
	major version number"

	^self class major: self major + 1 minor: 0!

newMinor
	"return a new (not just modified !!) version descriptor with an incremented 
	minor version number"

	^self class major: self major minor: self minor + 1! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!VersionDescriptor class methodsFor: 'instance creation'!

initialVersion
	"Answer the initial version (the smallest possible one)"

	^self new touch!

major: majorNumber minor: minorNumber 
	"create a new version descriptor with major and minor version numbers"

	^self new major: majorNumber minor: minorNumber!

new
	^super new initialize!

readFromString: s
	"scan a VersionDescriptor from the string s;
	check format correctness specified by the expression <Integer>.<Integer>"

	| dot maj min |
	(dot := s indexOf: $.) isZero
		ifTrue: [DialogView userError: 'Invalid version number format.
Please use the pattern ''<Integer>.<Integer>'''. ^nil].
	((maj := Integer readFromString: s) negative
		or: [(min := Integer readFromString: (s copyFrom: dot +1 to: s size)) negative])
		ifTrue: [DialogView userError: 'Use non-negative numbers only'. ^nil].
		
	^self major: maj minor: min! !

!VersionDescriptor class methodsFor: 'constants'!

separator
	^$.! !




' .ApplicationSource Version(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 September 1993 at 1:24:46 pm'!



!VersionedObjectAccessor methodsFor: 'initialize release'!

initialize
	super initialize.
	version := VersionDescriptor new! !

NoVersion removeFromSystem!


' .ApplicationSource Version(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 23 December 1994 at 1:15:07 pm'!



"code maintainance"!

!VersionDescriptor class methodsFor: 'constants'!

modifier
	"return the character to mark modification"

	^$m! !

!VersionDescriptor class methodsFor: 'instance creation'!

major: majorNumber minor: minorNumber 
	"create a new version descriptor with major and minor version numbers"

	^super new major: majorNumber minor: minorNumber! !

!VersionDescriptor class methodsFor: 'instance creation'!

new
	^self major: 0 minor: 0! !

!VersionDescriptor class methodsFor: 'instance creation'!

readFrom: aStream 
	"scan a VersionDescriptor from aStream; 
	format: <white space><PositiveInteger>.<PositiveInteger>"

	| maj min |
	maj := aStream skipSeparators peek isDigit
			ifTrue: [Integer readFrom: aStream]
			ifFalse: ['(non-negative major version number expected)'].
	min := maj isInteger
			ifTrue: [aStream next == self separator
				ifTrue: [aStream peek isDigit
					ifTrue: [Integer readFrom: aStream]
					ifFalse: ['(non-negative minor version number expected)']]
				ifFalse: ['(''.'' expected)']]
			ifFalse: [maj].
	^min isInteger
		ifTrue: [(self major: maj minor: min)
				perform: (aStream peek == self modifier
					ifTrue: [aStream next. #touch]
					ifFalse: [#untouch])]
		ifFalse: [self error: 'Invalid version number ' , min]! !

VersionDescriptor class removeSelector: #readFromString:!

!VersionDescriptor methodsFor: 'printing'!

printOn: stream 
	stream print: self major; nextPut: self class separator; print: self minor.
	self modified ifTrue: [stream nextPut: self class modifier]! !

!VersionDescriptor methodsFor: 'initialize-release'!

major: majorNumber minor: minorNumber 
	"set major/minor numbers and reset the modification flag"

	minor := minorNumber.
	major := majorNumber.
	changed := false! !

VersionDescriptor removeSelector: #initialize!

VersionDescriptor removeSelector: #fileString!


' .ApplicationSource Version(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 23 January 1995 at 5:19:42 pm'!



"code maintenance"!

!VersionedObjectAccessor methodsFor: 'accessing'!

identifier
	^(WriteStream on: (String new: 32))
		nextPutAll: name;
		nextPut: $(;
		print: self version;
		nextPut: $); contents! !