
' .ApplicationSource APL-SystemLink(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 9:33:05 pm'!



'- sources of APL-SystemLink(1.0) -'!

!ClassDescription methodsFor: 'compiling'!

compile: code classified: heading notifying: requestor 
	"Compile the argument, code, as source code in the context of the receiver 
	and install the result in the receiver's method dictionary under the 
	classification indicated by the second argument, heading.  The third 
	argument, requestor, is to be notified if an error occurs.  The argument code 
	is either a string or an object that converts to a string or a 
	PositionableStream on an object that converts to a string."

	| selector |
	selector := self
				compile: code
				notifying: requestor
				ifFail: [^nil].
	(methodDict at: selector)
		sourcePointer: (SourceFileManager default
				storeMethodSource: code asString
				class: self
				category: heading
				safely: true).

	"1st assignment induces a change to system;
	application management records the change in change objects;
	the 2nd assignment creates a new source pointer which must be
	propagated to those change objects:"
	self applicationManagement updateChanges.

	"proceed as usual"
	self organization classify: selector under: heading.
	^selector! !



!BOSSReader methodsFor: 'private'!

commitSources
	"If we loaded any compiled methods, 
	commit the changes file and update the source pointers now"

	| stored app |
	compiledCode == nil ifTrue: [^self].

	"enter source code to changes file and update source pointers (if necessary)"
	stored := false.
	compiledCode do: [:info | info storeSource ifTrue: [stored := true]].
	stored ifTrue: [SourceFileManager default commitSources].

	app := self applicationManagement.
	compiledCode
		do: 
			[:info | | code |
			code := info code.
			info become: code.	"identities are now swapped"
			code commit: info logging: app]! !



!ClassOrganizer methodsFor: 'categories'!

addCategory: heading before: nextHeading
	"Add a new category named heading.  If nextHeading is specified (not nil)
	and can be found, then INSERT before that entry.  Otherwise ADD it at the end.
	If heading already exists, then MOVE it where it would have gone otherwise."

	| nextIndex elements | 

	(categoryArray indexOf: heading) > 0
		ifTrue:  "already there - maybe move"
			["slow but sure;  remove all, then insert all"
			elements := self listAtCategoryNamed: heading.	"Save elements"
			elements do: [:elt | self removeElement: elt].
			self removeCategory: heading.		"Remove old entry"
			self addCategory: heading before: nextHeading.	"Now insert anew"
			self classifyAll: elements under: heading.			"And restore elements"
			^self].
	nextIndex := categoryArray indexOf: nextHeading
							ifAbsent: [categoryArray size+1].
	categoryArray := categoryArray  "insert before nextIndex"
			copyReplaceFrom: nextIndex
			to: nextIndex-1
			with: (Array with: heading).
	categoryStops := categoryStops
			copyReplaceFrom: nextIndex
			to: nextIndex-1
			with: (Array with: (nextIndex=1
					ifTrue: [0]
					ifFalse: [categoryStops at: nextIndex-1])).
	self class == ClassOrganizer
		ifTrue: [self applicationManagement includeProtocol: heading class: self myClass]
		ifFalse: [self applicationManagement includeCategory: heading]!

removeCategory: cat 
	"Remove the category named, cat.  Create an error if the category has
	any elements in it.
	linked to application management"

	| index lastStop |
	index := categoryArray indexOf: cat ifAbsent: [^self].
	lastStop := 
		index = 1
			ifTrue: [0]
			ifFalse: [categoryStops at: index - 1].
	(categoryStops at: index) - lastStop > 0 
		ifTrue: [^self error: 'cannot remove non-empty category'].

	self class == ClassOrganizer
		ifTrue: [self applicationManagement removeProtocol: cat class: self myClass]
		ifFalse: [self applicationManagement removeCategory: cat].

	categoryArray := 
		(categoryArray copyFrom: 1 to: index - 1)
			, (categoryArray copyFrom: index + 1 to: categoryArray size).
	categoryStops := 
		(categoryStops copyFrom: 1 to: index - 1)
			, (categoryStops copyFrom: index + 1 to: categoryStops size).!

renameCategory: oldName to: newName
	"Answer the array of elements associated with the name, categoryName."

	| i symbol |
	i := categoryArray indexOf: oldName
		ifAbsent: [^ false].
	symbol := newName asSymbol.
	categoryArray indexOf: symbol
		ifAbsent: [categoryArray at: i put: symbol.
				self class == ClassOrganizer
					ifTrue: [self applicationManagement
								renameProtocol: oldName
								to: newName
								class: self myClass asSymbol]
					ifFalse: [self applicationManagement
								renameCategory: oldName
								to: newName].
				  ^ true].
	^ false  "newName was already there"! !

!ClassOrganizer methodsFor: 'elements'!

classify: element under: heading 
	"Store the argument, element, in the category named heading."

	| catName catIndex elemIndex realHeading |

	realHeading := heading asSymbol.
	(catName := self categoryOfElement: element) notNil
		ifTrue:  "Element already there"
			[realHeading = Default ifTrue: [^self].	"Default causes no change"
			realHeading = catName ifTrue: [^self].	"heading didnt change"
			self removeElement: element].	"remove from old heading if did change"

	(categoryArray indexOf: realHeading) = 0 ifTrue: [self addCategory: realHeading].
	"add realHeading if not there already"

	catIndex := categoryArray indexOf: realHeading.
	elemIndex := 
		catIndex > 1
			ifTrue: [categoryStops at: catIndex - 1]
			ifFalse: [0].
	[(elemIndex := elemIndex + 1) <= (categoryStops at: catIndex) 
		and: [element >= (elementArray at: elemIndex)]] whileTrue.

	"elemIndex is now the index for inserting the element. Do the insertion before it."
	elementArray := 
		(elementArray copyFrom: 1 to: elemIndex - 1)
			, (Array with: element) 
			, (elementArray copyFrom: elemIndex to: elementArray size).	"insertion"

	"add one to stops for this and later categories"
	catIndex to: categoryArray size do: 
		[:i | categoryStops at: i put: (categoryStops at: i) + 1].

	"notify application management via system change set (not nice but safe)"
	self class == ClassOrganizer
		ifTrue: [self applicationManagement
					classifySelector: element under: heading class: self myClass]
		ifFalse: [self applicationManagement
					classifyClass: element under: heading].

	"remove empty default category if any"
	categoryArray indexOf: Default ifAbsent: [^self].
	(self listAtCategoryNamed: Default) size = 0
		ifTrue: [self removeCategory: Default].! !

!ClassOrganizer methodsFor: 'comment'!

globalComment
	"return the RemoteString of the comment"

	^globalComment! !

!ClassOrganizer methodsFor: 'private'!

myClass
	"Find out the class with receiver as its organization"

	Smalltalk
		allClassesDo: 
			[:class | 
			class organization == self ifTrue: [^class].
			class class organization == self ifTrue: [^class class]].
	self error: 'Organization could not identify its class'! !



!BOSSCompiledCodeHolder methodsFor: 'installing'!

commit: code logging: aChangeManager 
	"broadcast change events to application management"

	| adding |
	class notNil & aChangeManager notNil
		ifTrue: [(class includesSelector: selector)
			ifTrue: [aChangeManager changeSelector: selector class: class]
			ifFalse: ["wait until the selector has been installed" adding := true]].

	self commit: code.

	adding notNil ifTrue: [aChangeManager addSelector: selector class: class].

	"1st instruction induces a change to system;
	application management records the change in change objects;
	the 2nd instruction creates a new source pointer which must be
	propagated to those change objects:"

	self applicationManagement updateChanges! !



!ChangeSet methodsFor: 'change management'!

addClass: class 
	"Include indication that a new class was created."

	classRemoves remove: class name ifAbsent: [].
	self atClass: class add: #add.
	self applicationManagement addClass: class!

addDoIt: smalltalkExpression 
	"When filing out changes, append this string to the file."

	specialDoIts add: smalltalkExpression.
	self applicationManagement addDoIt: smalltalkExpression!

changeClass: class 
	"Include indication that a class definition has been changed."

	self atClass: class add: #change.
	self applicationManagement changeClass: class!

commentClass: class 
	"Include indication that a class comment has been changed."

	self atClass: class add: #comment.
	self applicationManagement commentClass: class!

removeClass: class 
	"Include indication that a class has been forgotten."

	| name |

	self applicationManagement removeClass: class.
	"Do this first instead of doing it twice (because of ^)"

	(self isNew: class) ifTrue:
		[^self removeClassChanges: class]. 	"only remember old classes"
	(self atClass: class includes: #rename) 	"remember as old name"
		ifTrue: [name := self oldNameFor: class]
		ifFalse: [name := class name].
	self removeClassChanges: class.
	classRemoves add: name!

renameClass: class as: newName 
	"Include indication that a class has been renamed."

	| value |
	(self atClass: class includes: #rename) ifFalse:
		[self atClass: class add: 'oldName: ', class name. 	"only original name matters"
		self atClass: class add: #rename].
 	"copy changes using new name (metaclass too)"
	(Array with: classChanges with: methodChanges) do:
		[:changes |
		(value := changes at: class name ifAbsent: [nil]) == nil ifFalse:
			[changes at: newName put: value.
			changes removeKey: class name].
		(value := changes at: class class name ifAbsent: [nil]) == nil ifFalse:
			[changes at: (newName, ' class') put: value.
			changes removeKey: class class name]].
	self applicationManagement renameClass: class as: newName! !

!ChangeSet methodsFor: 'method changes'!

addSelector: selector class: class 
	"Include indication that a method has been added."

	self
		atSelector: selector
		class: class
		put: #add.
	self applicationManagement addSelector: selector class: class!

changeSelector: selector class: class 
	"Include indication that a method has been edited."

	(self atSelector: selector class: class) = #add 
		ifFalse: [self atSelector: selector class: class put: #change]
			"Don't forget a method is new just because it's been changed".
	self applicationManagement changeSelector: selector class: class!

removeSelector: selector class: class 
	"Include indication that a method has been forgotten."

	(self atSelector: selector class: class) = #add
		ifTrue: [self removeSelectorChanges: selector 
					class: class]					"Forgot a new method, no-op"
		ifFalse: [self atSelector: selector
					class: class
					put: #remove].
	self applicationManagement removeSelector: selector class: class! !



!SourceFileManager methodsFor: 'initialize-release'!

file: index name: aFilename writable: isWritable
	"Set the file at index to be a file stream on the file named aFilename.  If isWritable
	is true, use a readAppendStream, otherwise use a readStream.  Since this method
	uses lineEndTransparent, it really isn't useful except for the system default manager."
	"modifications due to application management"

	| name file oldFile |
	(index isInteger and: [index between: 1 and: files size])
		ifFalse:	[^self error: 'Illegal file index'].
	(files at: index) == nil
		ifFalse:	[(oldFile := files at: index) close].
	name := aFilename asFilename.
	isWritable
		ifTrue:	[file := name readAppendStream]
		ifFalse:	[file := name readStream].
	file lineEndTransparent.
	oldFile isNil
		ifTrue: [files at: index put: file]
		ifFalse:
			["during work of application management lots of change objects are created;
			ensure that all the file references use the new file"
			file close; become: oldFile]! !

!SourceFileManager methodsFor: 'utilities'!

condenseChanges
	"Move all the changes onto a compacted sources file.
	Because this gets the changes file out of synch with any stored image
	file, the user is asked whether to snapshot now."
	"update source pointers in change objects created by Application Manangement"
	
	"SourceFileManager default condenseChanges."

	| new changesFile newFile oldFile home |
	(changesFile := files at: currentTarget) == nil
		ifTrue: [^self].

	new := self class new.
	home := self targetFile name asFilename head asFilename.
	newFile := home construct:
			(Time millisecondClockValue printString, Filename changeExtension).
	new file: currentTarget name: newFile writable: true.
	new target: currentTarget.
	new timeStamp.
	new copyAllChangedMessages.
	changesFile close.
	(oldFile := changesFile name asFilename) delete.
	newFile renameTo: oldFile.
	self file: currentTarget name: oldFile writable: true.

	self applicationManagement updateCondensedChanges.

	(DialogView confirm: 
	'You must snapshot to be sure your image file 
	is in sync with the new changes file.
		Do you want to snapshot now?')
		ifTrue: 
			[ObjectMemory saveDialogThenQuit: false]!

condenseChangesOntoSources
	"Move all the changes onto the end of the sources file."
	"modifications due to application management"
	
	"SourceFileManager default condenseChangesOntoSources."

	| file output |
	(files at: 1) close.
	output := self class new.
	output file: 1 name: (self nameAt: 1) writable: true.
	output target: 1.
	output copyAllChangedMessages.
	file := self nameAt: currentTarget.
	(files at: currentTarget) close.
	file delete.
	self file: currentTarget name: file writable: true.

	self applicationManagement updateCondensedChanges.

	(DialogView confirm: 
	'You must snapshot to be sure your image file 
	is in sync with the new changes file.
		Do you want to snapshot now?')
		ifTrue: 
			[ObjectMemory saveDialogThenQuit: false]! !



!PeekableStream methodsFor: 'fileIn'!

fileIn
	"This is special for reading expressions from text that has been formatted
	with exclamation delimitors.  The expressions are read and passed to the
	default system compiler.  Answer the result of compilation."
	"include a link to application management"

	| val |
	val := self applicationManagement fileInFrom: self.
	"if application management processed the file-in, the receiver is at end"

	Cursor read showWhile:
		[[[self atEnd]
			whileFalse: 
				[self skipSeparators.
				val := (self peekFor: $!!)
							ifTrue: [(Object evaluatorClass evaluate: self nextChunk logged: false)
									scanFrom: self]
							ifFalse: [Object evaluatorClass evaluate: self nextChunk logged: true]]
			] valueNowOrOnUnwindDo: [self close]].
	^val! !




' .ApplicationSource APL-SystemLink(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 31 August 1993 at 8:13:28 pm'!



"optimization of #myClass and #classify:under:"
!

!ClassOrganizer methodsFor: 'private'!

myClass
	"Find out the class with receiver as its organization"

	| cls |
	"try to shortcut if this is a call from myClass
	during compilation/installation of a method"
	((cls := thisContext sender sender receiver) isBehavior
		and: [cls organization == self])
		ifTrue: [^cls].

	Smalltalk do: [:each |
		each isBehavior
			ifTrue: 
				[each organization == self ifTrue: [^each].
				each class organization == self ifTrue: [^each class]]].
	self error: 'Organization could not identify its class'! !

!ClassOrganizer methodsFor: 'categories'!

addCategory: aSymbol beforeIndex: anInteger
	"insert a really new category (no check) named aSymbol before anInteger;
	anInteger = nil forces appending the new category"

	| next |
	next := anInteger == nil ifTrue: [categoryArray size+1] ifFalse: [anInteger].
	categoryArray := categoryArray
			copyReplaceFrom: next
			to: next-1
			with: (Array with: aSymbol).
	categoryStops := categoryStops
			copyReplaceFrom: next
			to: next-1
			with: (Array with: (next=1 ifTrue: [0] ifFalse: [categoryStops at: next-1]))! !

!ClassOrganizer methodsFor: 'categories'!

addCategory: heading before: nextHeading
	"Add a new category named heading.  If nextHeading is specified (not nil)
	and can be found, then INSERT before that entry.  Otherwise ADD it at the end.
	If heading already exists, then MOVE it where it would have gone otherwise."

	(categoryArray indexOf: heading) > 0
		ifTrue:
			[ "already there - maybe move (slow but sure; remove all, then insert all)"
			| elements | 
			elements := self listAtCategoryNamed: heading.	"Save elements"
			elements do: [:elt | self removeElement: elt].
			self removeCategory: heading.		"Remove old entry"
			self addCategory: heading before: nextHeading.	"Now insert anew"
			self classifyAll: elements under: heading.			"And restore elements"
			^self].
	self
		addCategory: heading
		beforeIndex: (categoryArray indexOf: nextHeading ifAbsent: []).

	self class == ClassOrganizer
		ifTrue: [self applicationManagement includeProtocol: heading class: self myClass]! !

!ClassOrganizer methodsFor: 'categories'!

removeCategory: cat 
	"Remove the category named, cat.  Create an error if the category has
	any elements in it.
	linked to application management"

	| index lastStop |
	index := categoryArray indexOf: cat ifAbsent: [^self].
	lastStop := index = 1
			ifTrue: [0]
			ifFalse: [categoryStops at: index - 1].
	(categoryStops at: index) - lastStop > 0 
		ifTrue: [^self error: 'cannot remove non-empty category'].

	self class == ClassOrganizer
		ifTrue: [self applicationManagement removeProtocol: cat class: self myClass].

	categoryArray := 
		(categoryArray copyFrom: 1 to: index - 1)
			, (categoryArray copyFrom: index + 1 to: categoryArray size).
	categoryStops := 
		(categoryStops copyFrom: 1 to: index - 1)
			, (categoryStops copyFrom: index + 1 to: categoryStops size).! !

!ClassOrganizer methodsFor: 'elements'!

classify: element under: heading 
	"Store the argument, element, in the category named heading."

	| catName catIndex elemIndex realHeading |

	realHeading := heading asSymbol.
	(catName := self categoryOfElement: element) notNil
		ifTrue:  "Element already there"
			[realHeading = Default ifTrue: [^self].	"Default causes no change"
			realHeading = catName ifTrue: [^self].	"heading didnt change"
			self removeElement: element].	"remove from old heading if did change"

	"add realHeading if not there already"
	(catIndex := categoryArray indexOf: realHeading) = 0
		ifTrue: [self addCategory: realHeading beforeIndex: nil. catIndex := categoryArray size].

	elemIndex := catIndex > 1
			ifTrue: [categoryStops at: catIndex - 1]
			ifFalse: [0].
	[(elemIndex := elemIndex + 1) <= (categoryStops at: catIndex) 
		and: [element >= (elementArray at: elemIndex)]] whileTrue.

	"elemIndex is now the index for inserting the element. Do the insertion before it."
	elementArray := 
		(elementArray copyFrom: 1 to: elemIndex - 1)
			, (Array with: element) 
			, (elementArray copyFrom: elemIndex to: elementArray size).	"insertion"

	"add one to stops for this and later categories"
	catIndex to: categoryArray size do: 
		[:i | categoryStops at: i put: (categoryStops at: i) + 1].

	"notify application management via system change set"
	self class == ClassOrganizer
		ifTrue: [self applicationManagement classifySelector: element under: heading class: self myClass].

	"remove empty default category if any"
	catIndex := categoryArray indexOf: Default ifAbsent: [^self].
	(self listAtCategoryNumber: catIndex) size = 0
		ifTrue: [self removeCategory: Default].! !

"enable class definition backups"!

!ClassBuilder methodsFor: 'evaluate'!

modifyExistingClass
	"Enumerate the legal types of changes and perform them.  If a new
	class was created, the final check for which kind of mutation is
	skipped, since a new class needs no mutation"

	| old |
	old := self currentClass definition.
	self runValidationChecks.
	self needsMutation
		ifTrue:	[self doMutationInPlace
			ifTrue:	[self mutateCurrentClass]
			ifFalse:	[self mutateToNewClass]].
	self changeMicroState.
	^ApplicationManagement sourceSignal
		handle: [:ex | 
			"someone asks for the old definition text"
			ex proceedWith: old]
		do: [self logChanged: self currentClass]! !


' .ApplicationSource APL-SystemLink(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 11 April 1994 at 5:37:17 pm'!



"fix: applications lost categories when a category was moved within its organization"!

!ClassOrganizer methodsFor: 'private'!

removeCategoryNumber: anInteger 
	"remove the category numbered anInteger; return elements of that category"

	| elements head stop |
	elements := self listAtCategoryNumber: anInteger.
	head := anInteger = 1 ifTrue: [0] ifFalse: [categoryStops at: anInteger - 1].
	stop := categoryStops at: anInteger.
	elementArray := (elementArray copyFrom: 1 to: head)
				, (elementArray copyFrom: stop + 1 to: elementArray size).
	categoryArray := categoryArray copyWithout: (categoryArray at: anInteger).
	categoryStops := (categoryStops copyFrom: 1 to: anInteger - 1)
				, ((categoryStops copyFrom: anInteger + 1 to: categoryStops size)
						collect: [:int | int - elements size]).
	^elements! !

!ClassOrganizer methodsFor: 'private'!

moveCategory: aSymbol number: srcInteger beforeIndex: anInteger
	"change category order: move category aSymbol from its current index
	srcInteger to appear just before index anInteger"

	| next elements head |
	next := anInteger == nil ifTrue: [categoryArray size+1] ifFalse: [anInteger].
	srcInteger + 1 = next ifTrue: [^self].

	elements := self removeCategoryNumber: srcInteger.
	srcInteger < next ifTrue: [next := next - 1].

	head := next = 1 ifTrue: [0] ifFalse: [categoryStops at: next - 1].
	elementArray := elementArray
			copyReplaceFrom: head+1
			to: head
			with: elements.
	categoryArray := categoryArray
			copyReplaceFrom: next
			to: next-1
			with: (Array with: aSymbol).
	categoryStops := (categoryStops copyFrom: 1 to: next - 1)
				"next - 1 is inserted twice (once for source, second for new)"
				, ((categoryStops copyFrom: next - 1 to: categoryStops size)
						collect: [:int | int + elements size])! !

!ClassOrganizer methodsFor: 'categories'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     

"optimization"!

!ClassOrganizer methodsFor: 'elements'!

classify: element under: heading 
	"Store the argument, element, in the category named heading."

	| catName catIndex elemIndex realHeading |

	realHeading := heading asSymbol.
	(catName := self categoryOfElement: element) notNil
		ifTrue:  "Element already there"
			[realHeading = Default ifTrue: [^self].	"Default causes no change"
			realHeading = catName ifTrue: [^self].	"heading didnt change"
			self removeElement: element].	"remove from old heading if did change"

	"add realHeading if not there already"
	catIndex := categoryArray indexOf: realHeading ifAbsent: [self addCategory: realHeading beforeIndex: nil. categoryArray size].

	elemIndex := catIndex > 1
			ifTrue: [categoryStops at: catIndex - 1]
			ifFalse: [0].
	[(elemIndex := elemIndex + 1) <= (categoryStops at: catIndex) 
		and: [element >= (elementArray at: elemIndex)]] whileTrue.

	"elemIndex is now the index for inserting the element. Do the insertion before it."
	elementArray := elementArray
			copyReplaceFrom: elemIndex
			to: elemIndex - 1
			with: (Array with: element).

	"add one to stops for this and later categories"
	catIndex to: categoryArray size do: 
		[:i | categoryStops at: i put: (categoryStops at: i) + 1].

	"notify application management via system change set"
	self class == ClassOrganizer
		ifTrue: [self applicationManagement classifySelector: element under: heading class: self myClass].

	"remove empty default category if any"
	catIndex := categoryArray indexOf: Default ifAbsent: [^self].
	(self listAtCategoryNumber: catIndex) size = 0
		ifTrue: [self removeCategory: Default].! !


' .ApplicationSource APL-SystemLink(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 3:42:58 pm'!



"fix: passed the wrong heading to move in #addCategory:before:"!

!ClassOrganizer methodsFor: 'categories'!

addCategory: heading before: nextHeading 
	"Add a new category named heading. If nextHeading is specified (not nil) 
	and can be found, then INSERT before that entry. Otherwise ADD it at the end. 
	If heading already exists, then MOVE it where it would have gone otherwise."

	| i next |
	next := categoryArray indexOf: nextHeading ifAbsent: [].
	(i := categoryArray indexOf: heading) > 0 ifTrue: [^self
			moveCategory: heading
			number: i
			beforeIndex: next].
	self addCategory: heading beforeIndex: next.
	self class == ClassOrganizer
		ifTrue: [self applicationManagement includeProtocol: heading class: self myClass]! !


' .ApplicationSource APL-SystemLink(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 July 1994 at 5:47:43 pm'!



"modifications for VisualWorks 2.0"!

!ChangeSet methodsFor: 'change management'!

addClass: class 
	"Include indication that a new class was created."

	classRemoves remove: class fullName ifAbsent: [].
	self atClass: class add: #add.
	self applicationManagement addClass: class! !

!ChangeSet methodsFor: 'change management'!

removeClass: class 
	"Include indication that a class has been forgotten."

	| name |
	self applicationManagement removeClass: class.
	(self isNew: class) ifTrue:
		[^self removeClassChanges: class]. 	"only remember old classes"
	(self atClass: class includes: #rename) 	"remember as old name"
		ifTrue: [name := self oldNameFor: class]
		ifFalse: [name := class fullName].
	self removeClassChanges: class.
	classRemoves add: name! !

!ChangeSet methodsFor: 'change management'!

renameClass: class as: newName 
	"Include indication that a class has been renamed."

	| value |
	(self atClass: class includes: #rename) ifFalse:
		[self atClass: class add: 'oldName: ', class name. 	"only original name matters"
		self atClass: class add: #rename].
 	"copy changes using new name (metaclass too)"
	(Array with: classChanges with: methodChanges) do:
		[:changes |
		(value := changes removeKey: class fullName ifAbsent: []) == nil ifFalse:
			[changes at: newName asString put: value].
		(value := changes removeKey: class class fullName ifAbsent: []) == nil ifFalse:
			[changes at: (newName, ' class') put: value]].
	self applicationManagement renameClass: class as: newName! !

!SourceFileManager methodsFor: 'initialize-release'!

file: index name: aFilename writable: isWritable
	"Set the file at index to be a file stream on the file named aFilename.  If isWritable
	is true, use a readAppendStream, otherwise use a readStream.  Since this method
	uses lineEndTransparent, it really isn't useful except for the system default manager."

	| name file oldFile |
	(index isInteger and: [index between: 1 and: files size])
		ifFalse:	[^self error: 'Illegal file index'].
	(files at: index) == nil
		ifFalse:	[(oldFile := files at: index) close].
	name := aFilename asFilename.
	isWritable
		ifTrue:	[file := name readAppendStream]
		ifFalse:	[file := name readStream].
	file lineEndTransparent.
	oldFile isNil
		ifTrue: [files at: index put: file]
		ifFalse:
			["during work of application management lots of change objects are created;
			ensure that all the file references use the new file"
			file close; become: oldFile].
	names at: index put: aFilename asString.
	readOnlyFlags at: index put: isWritable not.
	self changed: #filename with: index! !

!SourceFileManager methodsFor: 'utilities'!

condenseChanges
	"Move all the changes onto a compacted sources file.
	Because this gets the changes file out of synch with any stored image
	file, the user is asked whether to snapshot now."
	"update source pointers in change objects created by Application Management"
	
	"SourceFileManager default condenseChanges."

	self condenseFile: currentTarget.
	self applicationManagement updateCondensedChanges.
	self confirmSnapshot! !

!SourceFileManager methodsFor: 'utilities'!

condenseChangesOntoSources
	"Move all the changes onto the end of the sources file."
	
	"SourceFileManager default condenseChangesOntoSources."

	self condenseFile: currentTarget onto: self sourcesIndex.
	self applicationManagement updateCondensedChanges.
	self confirmSnapshot! !

!PeekableStream methodsFor: 'fileIn'!

fileIn
	"This is special for reading expressions from text that has been formatted
	with exclamation delimitors.  The expressions are read and passed to the
	default system compiler.  Answer the result of compilation."

	| val |
	val := self applicationManagement fileInFrom: self.
	"if application management processed the file-in, the receiver is at end"

	Cursor read showWhile:
		[[[self atEnd]
			whileFalse: 
				[self skipSeparators.
				val := (self peekFor: $!!)
							ifTrue: [(Object evaluatorClass evaluate: self nextChunk logged: false)
									scanFrom: self]
							ifFalse: [Cursor execute showWhile:
								[Object evaluatorClass evaluate: self nextChunk logged: true]]]
			] valueNowOrOnUnwindDo: [self close]].
	^val! !


' .ApplicationSource APL-SystemLink(1.5)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 3 March 1995 at 5:35:03 pm'!



"fix: release files if running short of handles"!

!IOAccessor class methodsFor: 'utilities'!

releaseFiles
	"request file consuming applications/objects to release files they do not need;
	return true/false to indicate success"
	"IOAccessor releaseFiles"

	^self applicationManagement releaseSourceFiles! !

!IOAccessor class methodsFor: 'instance creation'!

openFileNamed: nm direction: rwMode creation: creationRule 
	"Return an accessor on the file named (a String or Filename) with 
	the appropriate read/write mode. 
	try to release files if we are running short of handles"

	^OSErrorHolder noResourcesSignal handle: [:ex | self releaseFiles
			ifTrue: [ex restart]
			ifFalse: [ex reject]]
		do: [self defaultForFiles handleValue: [nm asFilename openHandleForDirection: rwMode creation: creationRule]]! !