
' .ApplicationSource X_Changes(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 4:11:49 pm'!



'- required applications for X_Changes(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('BASE' ) ))!

'- sources of X_Changes(1.0) -'!

!ClassDefinitionChange methodsFor: 'accessing'!

systemSourceText
	^(Smalltalk at: className ifAbsent: [^Text new]) definition asText! !

!ClassDefinitionChange methodsFor: 'checking'!

isInSystem
	^Smalltalk includesKey: className!

overwrites: aChange 
	"class definitions overwrite removals and other definitions of the same class"

	^self className = aChange className
		and: [self class == aChange class
			or: [aChange class == ClassOtherChange and: [aChange type == #remove]]]! !



!ChangeListController methodsFor: 'menu messages'!

purgeMethods 
	model purgeMethods.
	view displayRemovedOn: view graphicsContext!

removeSame
	model removeSame.
	view displayRemovedOn: view graphicsContext!

spawnCompare
	ChangeCompareView
		open: view model
		label: 'System version of ' , model listName! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!ChangeListController class methodsFor: 'class initialization'!

fileMenu
	^PopUpMenu
		labelArray: #('read file/directory...' 'write file...' 'recover last changes' 'display system changes' )
		values: #( #fileIn #fileOut #recover #getChanges )!

initialize
	"ChangeListController initialize"

	| conflictMenu mainValues |
	conflictMenu := PopUpMenu
			labelArray: #('check conflicts, save as...'
							'check with system, save as...'
							'compare with system')
			values: #(check checkWithSystem spawnCompare).

	mainValues := (Array with: self fileMenu), 
			#(doAll),
			(Array with: self removeMenu),
			#( restoreAll copyView forget doThis removeItem restoreItem browse), 
			(Array with: conflictMenu).

	ChangeListYellowButtonMenu :=
		PopUpMenu
			labels:
'file in/out
replay all
remove...
restore all
spawn all...
forget
replay selection
remove selection
restore selection
spawn selection...
conflicts'
			lines: #( 1 5 6 10 )
			values: mainValues!

removeMenu
	^PopUpMenu
		labelArray: #('remove all' 'remove same' 'purge methods' )
		values: #(#removeAll #removeSame #purgeMethods )! !



ComposedTextView subclass: #ChangeCompareView
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tools-Changes'!


!ChangeCompareView methodsFor: 'private'!

getContents
	"return the system source code for the currently selected change in 
	the underlying ChangeList (return '- same -' for textual identity);
	if advanced string comparison is present display the diff sections"

	| aChange sysText chgText comp diffs |
	(aChange := model selectedChange) == nil ifTrue: [^Text new].
	(sysText := aChange systemSourceText) isEmpty ifTrue: [^sysText].
	sysText = (chgText := model value asText)
		ifTrue: [^' - same -' asText].

	(comp := #MethodComparison asClass) isNil
		ifTrue: ["there is no string comparison utility; give as much info as possible"
			(model equalWithoutComments: sysText and: chgText)
				ifTrue: 
					[| insert |
					insert := '- equal without comment -\\' withCRs.
					^(insert , sysText string) asText emphasizeFrom: 1 to: insert size with: #italic]
				ifFalse: [^sysText]].

	"advanced string comparison"
	diffs := (comp source: sysText string target: chgText string) minimalSet.
	diffs first do: [:range | (sysText := sysText asText)
			emphasizeFrom: range first
			to: range last
			with: #(bold underline)].
	diffs last do: [:range | chgText
			emphasizeFrom: range first
			to: range last
			with: #(bold underline)].

	model setValue: chgText.
	"make the change list's code view redraw"
	"avoid infinite recursion, don't use: model changed: #listIndex."
	diffs := model dependents detect: [:dep | dep class == ComposedTextView] ifNone: [].
	diffs editText: chgText; invalidate.
	^sysText! !



!ChangeList methodsFor: 'selecting'!

selectedChange
	"Answer the item currently selected."

	^selectionIndex > 0 ifTrue: [list at: selectionIndex]! !

!ChangeList methodsFor: 'private-menu messages'!

purgeMethods 
	"assume some methods have more than one occurrence;
	'remove' all but the last ones"

	Cursor execute showWhile: 
		[ | tmp |
		tmp := Set new.
		list reverseDo: [:change | | sel |
			(sel := change selector) isEmpty
				ifFalse: [(tmp includes: (sel := change className, '>', sel)) 
					ifTrue: [removed add: change]
					ifFalse: [tmp add: sel]]]]!

removeSame
	Cursor execute showWhile: 
		[list do: [:change | (removed includes: change)
				ifFalse: [(change isInSystem and: [self equalWithoutComments: change text and: change systemSourceText])
						ifTrue: [removed add: change]]]]! !



!Change methodsFor: 'accessing'!

systemSourceText
	^Text new! !

!Change methodsFor: 'checking'!

isInSystem
	^false!

overwrites: aChange 
	"return a Boolean specifying if the effect of the receiver covers the effect of 
	aChange (subclasses should perform more sophisticated checks)"

	^false! !



!MethodDefinitionChange methodsFor: 'accessing'!

systemSourceText
	| systemVersion |
	(systemVersion := self copy)
		sourceFileAndPosition: [:aFile :aPosition | 
			systemVersion file: aFile position: aPosition.
			^systemVersion text asText].
	^Text new! !

!MethodDefinitionChange methodsFor: 'checking'!

checkWith: aChecker
	| systemVersion oldChanges |
	aChecker changesAt: self name add: self.
	aChecker checkSystem ifTrue:
		[self isSystemChange ifFalse:
			[systemVersion := self class new className: className; category: category; selector: selector.
			systemVersion sourceFileAndPosition:
				[:aFile :aPosition |
				systemVersion file: aFile position: aPosition.
				(aChecker equalWithoutComments: self text and: systemVersion text) ifFalse:
					[oldChanges := aChecker changesAt: self name.
					oldChanges size > 1 ifTrue:
						[oldChanges do: [:c | c isInSystem ifTrue: [^self]]].
					aChecker changesAt: self name add: systemVersion]]]]!

isInSystem
	self sourceFileAndPosition: [:aFile :aPosition | ^true].
	^false!

isSystemChange
	self sourceFileAndPosition:
		[:aFile :aPosition |
		(file = aFile and: [position = aPosition])
			ifTrue: [^true]].
	^false!

overwrites: aChange 
	"a method change overwrites every other method change on the same 
	selector/class combination"

	^self selector = aChange selector and: [self className = aChange className]! !



!MethodOtherChange methodsFor: 'accessing'!

text
	"return the file out text which can replay the method state decribed by the receiver"

	self type == #remove ifTrue: [^self className , ' removeSelector: ' , self selector storeString].
	^super text! !



!ClassOtherChange methodsFor: 'accessing'!

systemSourceText
	^type == 'inst vars for' asSymbol
		ifTrue: [ | cls | (cls := self classObject) isNil
			ifTrue: [Text new]
			ifFalse: [cls definition asText]]
		ifFalse: [Text new]!

text
	self type == #remove
		ifTrue: [^self className, ' removeFromSystem'].
	self type == #initialize
		ifTrue: [^self className, ' initialize'].
	^super text! !

!ClassOtherChange methodsFor: 'checking'!

isInSystem
	^type == 'inst vars for' asSymbol and: [self classObject ~~ nil]!

overwrites: aChange 
	"a class removal certainly overwrites any other change on the same class; 
	changes of same type do so (this works for comments also)"

	^className == aChange className
		and: [type == #remove
			or: [(self class == aChange class) and: [type == aChange type]]]! !



!ClassCommentChange methodsFor: 'accessing'!

systemSourceText
	| c |
	^(c := self classObject) isNil
		ifTrue: [Text new	"class does not exist"]
		ifFalse: [(c name , ' comment:
' , c comment printString) asText]! !

!ClassCommentChange methodsFor: 'checking'!

isInSystem
	^self classObject ~~ nil
		and: [(ChangeSet current classNamed: className) comment size ~= 0]! !

ChangeListController initialize!




' .ApplicationSource X_Changes(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 11 April 1994 at 5:26:28 pm'!



"new menu items for removing and utilities; see appropriate menus"!

!MethodOtherChange methodsFor: 'accessing'!

systemSourceText
	"the system text of a removal is the current method definition (if present)"

	| txt |
	self type == #remove ifFalse: [^super systemSourceText].
	txt := (MethodDefinitionChange new className: self className; selector: self selector)
		systemSourceText.
	^txt isEmpty ifTrue: ['- does not exist -' asText] ifFalse: [txt]! !

!ChangeList methodsFor: 'private-menu messages'!

findClass
	"show a menu of all class names occurring in the current list; 
	if the user selects, then select the first change with that class name"

	| choice |
	choice := Set new.
	list do: [:chg | choice add: chg className].
	(choice := choice asSortedCollection) isEmpty
		ifFalse: [choice first isEmpty ifTrue: [choice removeFirst]].
	choice := choice select: 'Select a class name' fromUserIfEmpty: [^self].
	self updateSelection: (selectionIndex := list findFirst: [:chg | chg className == choice])! !

!ChangeList methodsFor: 'private-menu messages'!

removeHalf: anInteger 
	"selectionIndex divides the list in two halves; mark all elements of half 
	anInteger for removal"

	| start stop |
	(stop := start := selectionIndex) == 0 ifTrue: [^self].
	anInteger == 1 ifTrue: [start := 1] ifFalse: [stop := list size].
	start to: stop do: [:i | removed add: (list at: i)]! !

!ChangeList methodsFor: 'private-menu messages'!

invert 
	"invert the marked-for-removal state of each element of the list"

	list do: [:chg | removed remove: chg ifAbsent: [removed add: chg]]! !

!ChangeListController methodsFor: 'menu messages'!

removeHalf1 
	"remove all changes from top of list to selectionIndex"

	model removeHalf: 1.
	view displayRemovedOn: view graphicsContext! !

!ChangeListController methodsFor: 'menu messages'!

removeHalf2 
	"remove all changes from selectionIndex to the end"

	model removeHalf: 2.
	view displayRemovedOn: view graphicsContext! !

!ChangeListController methodsFor: 'menu messages'!

 !                                                                                                                                        

!ChangeListController methodsFor: 'menu messages'!

findClass
	"see comment at ChangeList>findClass"

	model findClass! !

!ChangeListController methodsFor: 'menu messages'!

browseOwners
	"show applications which own the selected MethodChange selector"

	| chg cls |
	((chg := model selectedChange) == nil or: [chg selector isEmpty]) ifTrue: [^self].
	(cls := chg classObject) == nil ifTrue: [DialogView warn: 'The class does not exist'. ^self].
	Browser new onClass: cls; selector: chg selector; showMethodOwners! !

!ChangeListController methodsFor: 'menu messages'!

browseImplementors
	| chg |
	(chg := model selectedChange) == nil ifTrue: [^self].
	Browser browseAllImplementorsOf: chg selector! !

!ChangeListController methodsFor: 'menu messages'!

browseSenders
	| chg |
	((chg := model selectedChange) == nil or: [chg selector isEmpty]) ifTrue: [^self].
	Browser browseAllCallsOn: chg selector! !

!ChangeListController methodsFor: 'menu messages'!

browseMessages 
	| chg cls mth |
	((chg := model selectedChange) == nil or: [chg selector isEmpty])
		ifTrue: [^self].
	(cls := chg classObject) == nil
		ifTrue: [DialogView warn: 'The class does not exist'. ^self].
	mth := cls compilerClass new compile: chg text in: cls notifying: nil ifFail: [^self].
	Browser showMenuThenBrowse: mth generate allSymbolLiterals asSortedCollection! !

!ChangeListController class methodsFor: 'class initialization'!

removeMenu
	^PopUpMenu
		labelArray: #('remove all' 'all to end' 'all upper' 'invert' 'remove same' 'purge methods' )
		lines: #(3 4 )
		values: #(#removeAll #removeHalf2 #removeHalf1 #invert #removeSame #purgeMethods )! !

!ChangeListController class methodsFor: 'class initialization'!

utilitiesMenu
	^PopUpMenu
		labelArray: #('find class...' 'spawn compare' 'spawn system' 'spawn change list' 'owners' 'implementors' 'senders' 'messages' 'check conflicts, save as...' 'check with system, save as...')
		lines: #(1 4 8 )
		values: #(#findClass #spawnCompare #browse #copyView #browseOwners #browseImplementors #browseSenders #browseMessages #check #checkWithSystem )! !

!ChangeListController class methodsFor: 'class initialization'!

initialize
	"ChangeListController initialize"

	| mainValues |
	mainValues := (Array with: self fileMenu), 
			#(doAll),
			(Array with: self removeMenu),
			#( restoreAll forget doThis removeItem restoreItem ), 
			(Array with: self utilitiesMenu).

	ChangeListYellowButtonMenu := PopUpMenu
			labels: 'file in/out\replay all\remove...\restore all\forget\replay selection\remove selection\restore selection\utilities' withCRs
			lines: #( 1 4 5 8 )
			values: mainValues! !

ChangeListController initialize!

"fix: compare view text did not change if only emphasis was different"!

!ChangeCompareView methodsFor: 'private'!

getContents
	"return the system source code for the currently selected change in 
	the underlying ChangeList (return '- same -' for textual identity);
	if advanced string comparison is present display the diff sections"

	| aChange sysText chgText comp diffs |
	(aChange := model selectedChange) == nil ifTrue: [^Text new].
	(sysText := aChange systemSourceText) isEmpty ifTrue: [^sysText].
	sysText = (chgText := model value asText)
		ifTrue: [^' - same -' asText].

	(comp := #MethodComparison asClass) isNil
		ifTrue: ["there is no string comparison utility; give as much info as possible"
			(model equalWithoutComments: sysText and: chgText)
				ifTrue: 
					[| insert |
					insert := '- equal without comment -\\' withCRs.
					^(insert , sysText string) asText emphasizeFrom: 1 to: insert size with: #italic]
				ifFalse: [^sysText]].

	"advanced string comparison"
	diffs := (comp source: sysText string target: chgText string) minimalSet.
	diffs first do: [:range | (sysText := sysText asText)
			emphasizeFrom: range first
			to: range last
			with: #(bold underline)].
	diffs last do: [:range | chgText
			emphasizeFrom: range first
			to: range last
			with: #(bold underline)].

	model setValue: chgText.
	"make the change list's code view redraw"
	"avoid infinite recursion, don't use: model changed: #listIndex."
	diffs := model dependents detect: [:dep | dep class == ComposedTextView] ifNone: [].
	diffs editText: chgText; invalidate.

	"force sysText to be different from current displayContents"
	self editText: ''.
	^sysText! !

"change filter 'class' to satisfy on class and metaclass"!

!Change methodsFor: 'accessing'!

nonMetaClassName
	"provide a common property for class and metaclass related changes"

	^(self className copyUpTo: Character space) asSymbol! !

!ChangeListView class methodsFor: 'private'!

switchTemplates
	"changes with external source reference return a SourceReference on #file
	(which is always a unique object); filtering with #fileName works well"

	^#(	(1 file hasFilter: switchFilter:perform: fileName)
		(2 type hasFilter: switchFilter:perform: class)
"		(3 class hasFilter: switchFilter:perform: className)
"		(3 'class/metaclass' hasFilter: switchFilter:perform: nonMetaClassName)
		(4 category hasFilter: switchFilter:perform: category)
		(5 selector hasFilter: switchFilter:perform: selector)
		(6 same hasFilter: switchFilter:perform: values) )! !


' .ApplicationSource X_Changes(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 3:16:17 pm'!



!ChangeListController methodsFor: 'menu messages'!

invert 
	"invert the marked-for-removal state of each element of the list"

	model invert.
	view invalidate! !


' .ApplicationSource X_Changes(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 15 February 1995 at 7:40:31 pm'!



"optimization"!

!ChangeCompareView methodsFor: 'private'!

getContents
	"return the system source code for the currently selected change in 
	the underlying ChangeList (return '- same -' for textual identity);
	if advanced string comparison is present display the diff sections"

	| aChange sysText chgText comp |
	(aChange := model selectedChange) == nil ifTrue: [^Text new].
	(sysText := aChange systemSourceText) isEmpty ifTrue: [^sysText].
	sysText = (chgText := model value asText)
		ifTrue: [^' - same -' asText].

	comp := Smalltalk at: #MethodComparison ifAbsent:
			["there is no string comparison utility; give as much info as possible"
			^(model equalWithoutComments: sysText and: chgText)
				ifTrue: 
					[| insert |
					insert := '- equal without comment -\\' withCRs.
					(insert , sysText string) asText emphasizeFrom: 1 to: insert size with: #italic]
				ifFalse: [sysText]].

	"advanced string comparison"
	Cursor read showWhile:
		[comp emphasizeDiffsFor: (sysText := sysText asText) and: chgText].

	model setValue: chgText.
	"make the change list's code view redraw"
	"avoid infinite recursion, don't use: model changed: #listIndex."
	(model dependents detect: [:dep | dep class == ComposedTextView])
		editText: chgText; invalidate.

	"force sysText to be different from current displayContents"
	self editText: ''.
	^sysText! !

!ChangeCompareView methodsFor: 'updating'!

update: aSymbol 
	"react on text change of the model (aChangeList) announced by #value"

	aSymbol == #value ifTrue: [self updateDisplayContents]! !