
' .ApplicationSource X_Interface(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 4:46:01 pm'!



'- sources of X_Interface(1.0) -'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!DialogView class methodsFor: 'FillInTheBlank dialogs'!

request: messageString initialAnswer: aString 
	"Create an instance of the receiver whose question is messageString. 
	Display it centered around the cursor. Supply aString as an initial 
	answer. Return what the user accepts.

	For backward compatibility, replace the 'onCancel'-block by 'nil'
	to avoid creation of accept/cancel buttons."

	^self request: messageString initialAnswer: aString onCancel: ['']!

request: messageString initialAnswer: aString ifEmpty: aBlock 
	"return a string requested from user; 
	evaluate aBlock if the answer is empty or user cancels the request"

	| string |
	^(string := self
				request: messageString
				initialAnswer: aString
				onCancel: ['']) isEmpty
		ifTrue: [aBlock value]
		ifFalse: [string]! !

!DialogView class methodsFor: 'file name dialogs'!

requestFileName: message default: default version: versionType ifFail: failBlock
	"Prompt for a file name.  Evaluate failBlock if none obtained.
	If the user supplies a pattern (string containing $* or $#, then match against it.

	If versionType is #any, accept any legal file name.
	If versionType is #new, and the file exists, ask for confirmation to use it anyway.
	If version Type is #old, and the file does not exist, ask for confirmation to use its name.
	If versionType is #mustBeNew, and the file exists, evaluate failBlock.
	If versionType is #mustBeOld, and the file does not exist, evaluate failBlock."

	"DialogView requestFileName: 'file:' default: '*.st' version: #mustBeNew ifFail: ['']"

	| fileName menu currentDir haveOne |
	(#(#new #old #mustBeNew #mustBeOld #any) includes: versionType)
		ifFalse:	[self error: 'Illegal value for file version: ', versionType printString].
	fileName := default.
	currentDir := Filename defaultDirectoryName copyWith: Filename separator.
	[true] whileTrue: 
		[fileName := self request: message initialAnswer: fileName ifEmpty: [^failBlock value].
		haveOne := true.
		((fileName includes: $*) or: [fileName includes: $#])

			ifTrue:	"Is a pattern, set up a menu"
				[| list list2 choice |
				list := (Filename filesMatching: fileName) asSortedCollection.
				list2 := list collect:
					[:fname |
					(fname findString: currentDir startingAt: 1) = 1
						ifTrue: [fname copyFrom: currentDir size + 1 to: fname size]
						ifFalse: [fname asString]].
				menu := PopUpMenu
							labelList: (Array
											with: list2
											with: #('new pattern' 'abort'))
							values: list2, #(#newPattern #abort).
				choice := menu startUpWithHeading:
					(list isEmpty
						ifTrue: [' No Matches ']
						ifFalse: ['Choose a file']) asText.
				(choice == 0 or: [choice == #abort])
					ifTrue: [^failBlock value].
				choice == #newPattern
					ifFalse: [fileName := choice]
					ifTrue: [haveOne := false] "try again"].

			"got a single file name or match code said 'try again'"
		haveOne
			ifTrue:
				[| err |
				versionType = #any
					ifTrue:
						[^fileName].
				(versionType = #new or: [versionType = #mustBeNew])
					ifTrue:
						[fileName asFilename exists
							ifFalse: [^fileName].
						(versionType = #new and:
								[self confirm: fileName, ' already exists. Use anyway?'])
							ifTrue: [^fileName].
						err := ' File already exists \  ']
					ifFalse:
						["versionType must be either #old or #mustBeOld"
						fileName asFilename exists
							ifTrue:	[^fileName].
						(versionType = #old and:
								[self confirm: fileName, ' does not exist. Use anyway?'])
							ifTrue: [^fileName].
						fileName = fileName asFilename asString
							ifTrue: [err := ' File not found: \  ']
							ifFalse: [err := ' Illegal file name: \  ']].
				menu := PopUpMenu labels: 'try again\abort' withCRs.
				(menu
					startUpWithHeading: err withCRs , fileName , ' ') = 2
						ifTrue: [^failBlock value]]]! !

!DialogView class methodsFor: 'message boxes'!

internalError: messageText 
	"Display a message box with debug and cancel button"
	"self internalError: 'Internal Error'"

	| result |
	result := DialogView
				choose: messageText
				labels: #('cancel' 'debug' )
				values: #(false true )
				default: false.
	result ifTrue: [self halt]!

userError: messageText
	"inform the user that he called a procedure which is not appropriate;
	display a message box with ok button, wait until the user acknowledges"
	"self userError: 'User Error'" 

	self warn: messageText asComposedText centered.
	self debug!

userInfo: messageText
	"inform the user about some state in the system;
	display a message box with ok button, wait until the user acknowledges" 
	"self userInfo: 'Information'"

	self warn: messageText asComposedText centered!

userWarning: messageText 
	"Display a message box with message and ok and cancel button"
	"self userWarning: 'Warning'"

	| result |
	result := self
				choose: messageText
				labels: #('OK' 'cancel' )
				values: #(false true )
				default: false.
	result ifTrue: [self debug]! !



!Collection methodsFor: 'user interface'!

select: prompt fromUserIfEmpty: aBlock
	"start a popup menu with all elements of the receiver as items;
	return the element the user selects, evaluate aBlock if nothing is selected"

	| collection selection choices |
	self isEmpty ifTrue: [DialogView warn: 'Nobody'. ^aBlock value].
	collection := self isSequenceable ifTrue: [self] ifFalse: [self asOrderedCollection].
	choices := collection first isString
		ifTrue: [collection]
		ifFalse: [collection collect: [:each | each printString]].
	selection := PopUpMenu labelList: (Array with: choices).
	selection := prompt isEmpty
		ifTrue: [selection startUp]
		ifFalse: [selection startUpWithHeading: prompt].
	^selection == 0 
		ifTrue: [aBlock value]
		ifFalse: [collection at: selection]!

selectFromUser
	^self selectFromUserIfEmpty: []!

selectFromUserIfEmpty: aBlock
	^ self select: 'Select a choice' fromUserIfEmpty: aBlock! !



!PopUpMenu methodsFor: 'controlling'!

startUpIfQuit: aBlock
	"evaluate aBlock if choice is empty"
	
	| choice |
	(choice:= self startUpAt: WindowSensor cursorPoint centerX: true centerY: true) == 0
		ifTrue: [^aBlock value].
	^choice! !




' .ApplicationSource X_Interface(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 September 1993 at 1:21:10 pm'!



!DialogView methodsFor: 'interaction'!

 !                                                                                                                                                                                                                                                                                                                       

!DialogView methodsFor: 'private'!

 !                                                                                                                                                                                                                                                                                                                                                                                  

!DialogView methodsFor: 'interaction'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          


' .ApplicationSource X_Interface(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 11 April 1994 at 7:14:59 pm'!



"add class variables to InputSensor to hold mouse event timeouts"

(InputSensor fullScopeHas: #ClickTimeOut ifTrue: [:assoc | ])
	ifFalse: [InputSensor addClassVarName: #ClickTimeOut]. 
 
(InputSensor fullScopeHas: #DoubleClickTimeOut ifTrue: [:assoc | ])
	ifFalse: [InputSensor addClassVarName: #DoubleClickTimeOut]!

!InputSensor class methodsFor: 'constants'!

clickTimeOut: milliseconds
	ClickTimeOut := milliseconds! !

!InputSensor class methodsFor: 'constants'!

doubleClickTimeOut: milliseconds
	DoubleClickTimeOut := milliseconds! !

!InputSensor class methodsFor: 'class initialization'!

initialize
	"initialize mouse click tresholds"

	self clickTimeOut: 150; doubleClickTimeOut: 150! !

InputSensor initialize!

!InputSensor methodsFor: 'mouse'!

waitButtonForMilliSeconds: duration 
	"wait the specified time (milliseconds) for the user to press any mouse button; 
	abort waiting and return true as soon as a button is pressed (false otherwise)"

	| until |
	until := Time millisecondClockValue + duration.
	
	[self anyButtonPressed ifTrue: [^true].
	Time millisecondClockValue > until]
		whileFalse: [self pollForActivity].
	^false! !

!InputSensor methodsFor: 'mouse'!

waitNoButtonForMilliSeconds: duration 
	"wait the specified time (milliseconds) for the user to release all mouse buttons; 
	abort waiting and return true as soon as all buttons are released (false otherwise)"

	| until |
	until := Time millisecondClockValue + duration.
	
	[self anyButtonPressed ifFalse: [^true].
	Time millisecondClockValue < until]
		whileTrue: [self pollForActivity].
	^false! !

!InputSensor methodsFor: 'mouse'!

redButtonActivity 
	"return kind of activity as one of #(nil #click
	#doubleClick #shiftClick #ctrlClick #ctrlShiftClick #stillPressed)"

	self redButtonPressed ifFalse: [^nil].

	(self waitNoButtonForMilliSeconds: ClickTimeOut)  "look for a button hold"
		ifFalse: [^#stillPressed].

	(self waitButtonForMilliSeconds: DoubleClickTimeOut)
		ifTrue: [self waitNoButton. ^#doubleClick].

	^self shiftDown
		ifTrue: [self ctrlDown ifTrue: [#ctrlShiftClick] ifFalse: [#shiftClick]]
		ifFalse: [self ctrlDown ifTrue: [#ctrlClick] ifFalse: [#click]]! !

"handle invalidArgumentsSignal and ensure validity of input"!

!DialogView class methodsFor: 'file name dialogs'!

requestFileName: message default: default version: versionType ifFail: failBlock
	"Prompt for a file name.  Evaluate failBlock if none obtained.
	If the user supplies a pattern (string containing $* or $#, then match against it.

	If versionType is #any, accept any legal file name.
	If versionType is #new, and the file exists, ask for confirmation to use it anyway.
	If version Type is #old, and the file does not exist, ask for confirmation to use its name.
	If versionType is #mustBeNew, and the file exists, evaluate failBlock.
	If versionType is #mustBeOld, and the file does not exist, evaluate failBlock."

	"DialogView requestFileName: 'file:' default: '*.st' version: #mustBeNew ifFail: ['']"

	| fileName menu currentDir haveOne |
	(#(#new #old #mustBeNew #mustBeOld #any) includes: versionType)
		ifFalse:	[self error: 'Illegal value for file version: ', versionType printString].
	fileName := default.
	currentDir := Filename defaultDirectoryName copyWith: Filename separator.
	[true] whileTrue: 
		[fileName := self request: message initialAnswer: fileName ifEmpty: [^failBlock value].
		haveOne := true.
		((fileName includes: $*) or: [fileName includes: $#])

			ifTrue:	"Is a pattern, set up a menu"
				[| list list2 choice |
				list := (Filename filesMatching: fileName) asSortedCollection.
				list2 := list collect:
					[:fname |
					(fname findString: currentDir startingAt: 1) = 1
						ifTrue: [fname copyFrom: currentDir size + 1 to: fname size]
						ifFalse: [fname asString]].
				menu := PopUpMenu
							labelList: (Array with: list2 with: #('new pattern' 'abort'))
							values: list2, #(#newPattern #abort).
				choice := menu startUpWithHeading:
					(list isEmpty
						ifTrue: [' No Matches ']
						ifFalse: ['Choose a file']) asText.
				(choice == 0 or: [choice == #abort])
					ifTrue: [^failBlock value].
				choice == #newPattern
					ifFalse: [fileName := choice]
					ifTrue: [haveOne := false] "try again"].

			"got a single file name or match code said 'try again'"
		haveOne
			ifTrue:
				[| err exists |
				"test existence, thus ensuring correct syntax"
				exists := OSErrorHolder invalidArgumentsSignal
					handle: [:ex | err := ' Illegal file name: \  '. ex return with: false]
					do: [err := nil. fileName asFilename exists].
				err == nil ifTrue:
					[versionType = #any ifTrue: [^fileName].
					(versionType = #new or: [versionType = #mustBeNew])
						ifTrue:
							[exists ifFalse: [^fileName].
							(versionType = #new and:
									[self confirm: fileName, ' already exists. Use anyway?'])
								ifTrue: [^fileName].
							err := ' File already exists \  ']
						ifFalse:
							["versionType must be either #old or #mustBeOld"
							exists ifTrue:	[^fileName].
							(versionType = #old and:
									[self confirm: fileName, ' does not exist. Use anyway?'])
								ifTrue: [^fileName].
							err := fileName = fileName asFilename asString
								ifTrue: [' File not found: \  ']
								ifFalse: [' Illegal file name: \  ']]].
				((PopUpMenu labels: 'try again\abort' withCRs)
					startUpWithHeading: err withCRs , fileName , ' ') = 2
						ifTrue: [^failBlock value]]]! !

!DialogView methodsFor: 'interaction'!

 !                                                                                                                                                                                                                                                                                                                                             

"do not keep the dialogs in front as in former versions"!

!DialogView methodsFor: 'interaction'!

openType: windowType 
	"Interact with the user"

	| wrapper topView box |
	(wrapper := BorderedWrapper on: self) border: (Border width: 2).
	(topView := ScheduledWindow new)
		component: wrapper;
		controller: DialogController new;
		model: self closeChannel.
	box := wrapper preferredBounds.
	box := box align: box center with: WindowSensor cursorPoint.
	topView openTransientIn: box type: windowType! !


' .ApplicationSource X_Interface(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 3 May 1994 at 4:12:13 pm'!



"introduce disabling of menu items"!

!PopUpMenu methodsFor: 'private'!

backupFromDisabled: labelText
	"disabled labels store a backup of the original state; return it (nil if none exists)"

	| bak |
	^((bak := labelText emphasisAt: 1) class == Array
		and: [bak size == 2
		and: [(bak := bak at: 2) class == Association
		and: [bak key == #disabledMenuItem]]])
		ifTrue: bak! !

!PopUpMenu methodsFor: 'private'!

labelAt: anInteger disable: aBoolean 
	"if aBoolean is true disable label, otherwise enable; 
	disable: create a new label text with emphasis #italic + old label backup; 
	enable: restore the label backup from current label emphasis; 
	this is strictly private: a literal array providing the labels might be corrupted"

	| label bak |
	bak := self backupFromDisabled: (label := labels at: anInteger).
	bak == nil == aBoolean
		ifFalse: [^self	"enable, no backup, or disable, but backup exist"].
	labels at: anInteger put: (aBoolean
		ifTrue: [Text string: label string emphasis: (Array with: #italic with: #disabledMenuItem -> label)]
		ifFalse: [bak])! !

!PopUpMenu methodsFor: 'private'!

disabledAt: anIndex 
	"return true iff label at anIndex has been disabled"

	^(self backupFromDisabled: (labels at: anIndex)) notNil! !

!PopUpMenu methodsFor: 'accessing'!

disableValues: aCollection 
	"disable all values in this menu which occur in aCollection;
	make sure that all other values are enabled"

	labels := labels shallowCopy.	"avoid corrupting literal arrays providing labels"
	1 to: values size do:
		[:i | | disable val |
		self labelAt: i disable: (disable := aCollection includes: (val := values at: i)).
		(disable not and: [val isKindOf: PopUpMenu])
			ifTrue: [val disableValues: aCollection]]! !

!PopUpMenu methodsFor: 'private'!

valueAtSelection
	"Answer the item current selected."

	(selection = 0 or: [selection > values size or: [self disabledAt: selection]])
		ifTrue: [^0].
	^values at: selection! !

"factorizing and improving filename requests"!

!DialogView class methodsFor: 'file name dialogs'!

resolveFilePattern: aString directory: directoryString 
	"show a menu of all files matching aString; return
	- aString, if the argument does not include wild cards
	- #newPattern, if the user selects this item in the menu
	- nil, if the user selects nothing or the label #abort
	- aString, otherwise"

	| match menu |
	((aString includes: $*) or: [aString includes: $#])
		ifFalse: [^aString].
	match := (Filename filesMatching: aString) asSortedCollection
		collect: [:fname | (fname findString: directoryString startingAt: 1) = 1
					ifTrue: [fname copyFrom: directoryString size + 1 to: fname size]
					ifFalse: [fname asString]].
	menu := PopUpMenu labelList: (Array with: match with: #('new pattern' 'abort' ))
				values: match , #(#newPattern #abort ).
	match := menu startUpWithHeading: (match isEmpty
					ifTrue: [' No Matches ']
					ifFalse: ['Choose a file']) asText.
	^(match == 0 or: [match == #abort]) ifFalse: [match]! !

!DialogView class methodsFor: 'file name dialogs'!

requestFileName: message default: default version: versionType ifFail: failBlock
	"Prompt for a file name, evaluate failBlock if none obtained.
	If the user supplies a pattern (string containing $* or $#), then match against it.
	Case versionType of
	- #any, accept any legal file name.
	- #new, accept existing files' names, if the user confirms.
	- #old, accept not existing files' names, if the user confirms.
	- #mustBeNew, if the file exists, evaluate failBlock.
	- #mustBeOld, if the file does not exist, evaluate failBlock.
	- #directory, if the file is not a directory, evaluate failBlock."

	"DialogView requestFileName: 'file:' default: '*.st' version: #directory ifFail: ['']"

	| fileName currentDir choice |
	(#(#new #old #mustBeNew #mustBeOld #directory #any) includes: versionType)
		ifFalse: [self error: 'Illegal value for file version: ', versionType printString].
	fileName := default.
	currentDir := Filename defaultDirectoryName copyWith: Filename separator.

	[fileName := self request: message initialAnswer: fileName ifEmpty: [^failBlock value].
	(choice := self resolveFilePattern: fileName directory: currentDir) == nil
		ifTrue: [^failBlock value].
	choice == #newPattern ifFalse:
		[choice := Filename named: (fileName := choice) mode: versionType ifFail:
			[:conflict | | msg |
			(msg := #(nil #new #old #mustBeNew #mustBeOld #directory )
						indexOf: conflict ifAbsent: [1]) > 3
				ifTrue: [((msg := msg - 2) = 4 and: [fileName asFilename exists not])
					ifTrue: [msg := 3]].
			msg := #(' Illegal file name: \  ' ' File already exists \  ' ' File not found: \  ' ' File is not a directory \  ') at: msg. 

			((PopUpMenu labels: 'try again\abort' withCRs)
				startUpWithHeading: msg withCRs , fileName , ' ') = 2
					ifTrue: [^failBlock value].
			nil].
		choice == nil ifFalse: [^fileName]]] repeat! !


' .ApplicationSource X_Interface(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 5 September 1994 at 8:30:59 pm'!



!DialogView methodsFor: 'construction-adding'!

addButtons: anArray 
	"add buttons as given in anArray to the receiver; 
	begin at yPosition, distribute them evenly spaced and horizontally"

	| yp i btnspace |
	yp := self addVerticalSpace: 16; yPosition.
	btnspace := 0.5 / anArray size.
	i := 1.
	anArray do: [:btn | 
		self yPosition: yp; addWrapper: (BoundedWrapper on: btn) atX: btnspace * i.
		i := i + 2].
	self addVerticalSpace: 4! !

!ListController methodsFor: 'menu messages'!

activateSelection
	"invoked by double click; do whatever you want to the model (e.g. editing, etc.); 
	return true if activation could be called (else false)"

	^(self model respondsTo: #activateSelectionFrom:)
		ifTrue: [(self model activateSelectionFrom: self) notNil]
		ifFalse: [(self model respondsTo: #activateSelection)
				ifTrue: [self model activateSelection notNil]]! !

!ListController methodsFor: 'menu messages'!

doubleClickAt: aPoint 
	"select the element at aPoint if necessary, then perform doubleClick 
	activity"

	| index oldSelection |
	view isEmpty ifTrue: [^nil]. 
	(index := self view findSelection: aPoint) == nil ifTrue: [^nil].
	(oldSelection := view selection) = index
		ifFalse: 
			[view moveSelectionBox: index.
			self changeSelectionFrom: oldSelection to: index.
			view selection = oldSelection ifTrue: [^self	"change request failed"]].
	self activateSelection! !

!ListController methodsFor: 'menu messages'!

redButtonActivity
	| selectionMovement oldSelection nextSelection cPoint click |
	selectionMovement := false.
	oldSelection := view selection.
	(click := self sensor redButtonActivity) == #doubleClick
		ifTrue: [^self doubleClickAt: self sensor cursorPoint].
	[cPoint := self cursorPointWithScrolling.
	self sensor redButtonPressed | click notNil]
		whileTrue: 
			[| trialSelection |
			self poll.
			(trialSelection := view findSelection: cPoint) == nil
				ifFalse: 
					[view moveSelectionBox: (nextSelection := trialSelection).
					trialSelection ~= oldSelection ifTrue: [selectionMovement := true]].
			click := nil].
	(view bounds containsPoint: cPoint)
		ifFalse: 
			[oldSelection = nextSelection
				ifFalse: [view moveSelectionBox: oldSelection; makeSelectionVisible; invalidate].
			^self].
	nextSelection == nil ifTrue: [^self].
	(selectionMovement and: [oldSelection = nextSelection])
		ifTrue: [^self].
	self changeSelectionFrom: oldSelection to: nextSelection! !

DialogView removeSelector: #putInFront:!

DialogView removeSelector: #defaultButtonCenterIn:!


' .ApplicationSource X_Interface(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 7 April 1995 at 3:49:27 pm'!



"return the result of activateSelection"!

!ListController methodsFor: 'private'!

doubleClickAt: aPoint 
	"select the element at aPoint if necessary, then perform doubleClick 
	activity"

	| index oldSelection |
	view isEmpty ifTrue: [^nil]. 
	(index := self view findSelection: aPoint) == nil ifTrue: [^nil].
	(oldSelection := view selection) = index
		ifFalse: 
			[view moveSelectionBox: index.
			self changeSelectionFrom: oldSelection to: index.
			view selection = oldSelection ifTrue: [^self	"change request failed"]].
	^self activateSelection! !

"bug fix: Collections with texts were not properly handled
	(sorting is provided automatically now)"!

!Collection methodsFor: 'user interface'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     


' .ApplicationSource X_Interface(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 12 April 1995 at 1:07:51 pm'!



Collection removeSelector: #selectFromUser!

!Collection methodsFor: 'user interface'!

selectFromUser: prompt sort: aBoolean ifEmpty: aBlock 
	"open a menu with receivers' elements as items (sort them if aBoolean is true); 
	return the element the user selects, evaluate aBlock if nothing is selected"

	| sequence strings isSorted sorted selection |
	self isEmpty
		ifTrue: 
			[DialogView warn: 'Nobody'.
			^aBlock value].
	sequence := self isSequenceable
				ifTrue: [self]
				ifFalse: [self asOrderedCollection].
	strings := (sequence first isString or: [sequence first class == Text])
				ifTrue: [sequence]
				ifFalse: [sequence collect: [:each | each printString]].
	sorted := (isSorted := aBoolean not or: [strings isKindOf: SortedCollection])
				ifTrue: [strings]
				ifFalse: [strings asSortedCollection].
	selection := PopUpMenu labelList: (Array with: sorted).
	selection := prompt isEmpty
				ifTrue: [selection startUp]
				ifFalse: [selection startUpWithHeading: prompt].
	^selection == 0
		ifTrue: [aBlock value]
		ifFalse: 
			[isSorted ifFalse: [selection := strings indexOf: (sorted at: selection)].
			sequence at: selection]! !

!Collection methodsFor: 'user interface'!

selectFromUser: prompt ifEmpty: aBlock 
	^self
		selectFromUser: prompt
		sort: self isSequenceable not
		ifEmpty: aBlock! !

!Collection methodsFor: 'user interface'!

select: prompt fromUserIfEmpty: aBlock 
	"backward compatibility"

	^self selectFromUser: prompt ifEmpty: aBlock! !