
	"Application Management installation script;
	(assumed to be placed in the same directory where all application subdirectories reside)
	select 'file in' from a file browser menu to execute the contents of this file
	to install Application Management "


| targetSystem ctx path vwpath names pathFor applications recover apm |

"--- this script may only be applied once ---"
(apm := Smalltalk at: #ApplicationManager ifAbsent: []) == nil
	ifFalse:
		[DialogView warn: 'Application Management is already installed.'.
		^nil].

"--- find out kind of Smalltalk environment ---"
targetSystem := #('Objectworks\Smalltalk(R), Release 4.1*' 
		'VisualWorks(TM), Release 1.0*' 
		'VisualWorks(R) Release 2.0*' 
		'*')
	findFirst: [:str | str match: Smalltalk version].
targetSystem := #(ow41 vw10 vw20 unknown) at: targetSystem.

"--- find out the path of this file ---"
ctx := thisContext.
[ctx == nil or: [ctx receiver isKindOf: ExternalStream]] whileFalse: [ctx := ctx sender].
path := ctx receiver name asFilename head asFilename.
vwpath := path construct: 'vw20'.

pathFor := [:appName |
	"dependent on target system and alternative applications we read from different paths"
	(targetSystem = #vw20 and: [(vwpath construct: appName) exists])
		ifTrue: [vwpath]
		ifFalse: [path]].

"--- prepare import: define classes which are part of an application structure ---"
(path construct: 'appman.st0') asFilename fileIn.

"--- names of base applications which construct Application Management ---"
names := #('base' 'x_change' 'apl-chan' 
			"version management and remote source"
		'storable' 'accessco' 'version' 'external' 
			"application management and change management"
		'apl-fram' 'apl-syst').

"--- execute source/structure loader block for each application name ---"
applications := names collect: [:string |
	[:appName | | filename bos app |
		filename := (((pathFor value: appName)
			construct: appName) construct: 'v1') construct: 'applic.bos'.
		bos := BinaryObjectStorage onOld: filename readStream.
		InputState default ctrlDown ifTrue: [self halt: 'debug point reached'].
		"application structure"
		Transcript cr; show: 'loading ', appName, ' (structure)... '.
		app := bos next.
		"application sources"
		Transcript show: '(source)... '.
		bos nextClasses; close.
		Transcript show: 'done '.
		app] 
	value: string].


"-----------------------------------
	ApplicationManagement begin...
------------------------------------"

Behavior flushVMmethodCache.	"vw20 binary import does not flush"
#ApplicationManagement asClass install.
apm := self applicationManagement.

"--- complete loading/installation of first applications ---"
applications do: [:app | 
	app label path: (pathFor value: app name).
	app preInstallation do: [:chg | Compiler evaluate: chg text logged: true].
	apm register: app].
applications do: [:app | 
	app postInstallation do: [:chg | Compiler evaluate: chg text logged: true].
	"reset change set entries"
	app ownClassNames do: [:cn | ChangeSet current removeClassChanges: cn asClass]].

"--- load interface applications with AppMan utilities ---"
#('x_interf' 'apl-inte' ) do:
	[:appName | | app |
	app := apm load: appName path: (pathFor value: appName) version: 'v1' binary: true.
	app ownClassNames do: [:cn | ChangeSet current removeClassChanges: cn asClass].
	applications := applications copyWith: app].

"--- patches for ObjectWorks R4.1 / VisualWorks 2.0 ---"
path := targetSystem == #vw20 ifTrue: [vwpath] ifFalse: [path].
((path construct: 'patches') filesMatching: '*.st') do:
	[:filename | | file |
	apm
		fileInFrom: (file := filename asFilename readStream)
		to: apm systemChanges external: false.
	file close].

"--- update source pointers of methods from bootstrap file 'appman.st0' ---"
recover := [:appname :selectors | | list |
	"get source pointers for selectors in application in form of a change sequence"
	list := (self applicationManagement applicationNamed: appname) recoverOwn.
	selectors do: [:assoc | | chg mth |
		chg := list detect: [:mchg | mchg selector == assoc value and: [mchg className == assoc key asSymbol]] ifNone: [].
		mth := assoc key asClass compiledMethodAt: assoc value ifAbsent: [].
		mth isNil | chg isNil ifFalse:
			[mth sourcePointer: (#SourceReference asClass for: appname position: (chg instVarAt: 2))]]].

Cursor wait showWhile: [recover
	value: 'ExternalSource' value: (Array
		with: ClassOrganizer->#setComment:
		with: ClassDescription->#setComment:);
	value: 'APL-ChangeSequence' value: (Array
		with: #MethodDefinitionBackup asClass class -> #selector:class:);
	value: 'APL-Framework' value: (Array
		with: #ClassSubset->#classRef:
		with: #ClassSubset->#definition:)].

"--- open the Application Management launcher ---"
#ApplicationManagementBrowser asClass open.

Screen default ringBell.
DialogView warn: '\Installation is done.' withCRs.
