
' .ApplicationSource StorableObject(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 16 August 1993 at 4:37:11 pm'!



'- required applications for StorableObject(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('BASE' ) ))!

'- sources of StorableObject(1.0) -'!

Object subclass: #StorableObjectAccessor
	instanceVariableNames: 'name path extension convertingTable '
	classVariableNames: 'FilenameClasses SupportedPlatforms '
	poolDictionaries: ''
	category: 'Framework-Version'!
StorableObjectAccessor comment:
'Bezeichner und Pfad fuer ein Objekt. Der zugeordnete Dateiname setzt sich zusammen aus dem Pfad und dem Bezeichner, er ist entweder ein Directory (z.B. das Basisdirectory von Applikationen) oder eine tatsaechliche Datei. Der Pfad ist symbolisch oder absolut. Er sollte wegen der Plattformunabhaengigkeit symbolisch anfangen.
Wenn der Dateiname nicht den der aktuellen Plattform genuegenden Einschrankungen (z.B. 8 Zeichen bei msdos) genuegt, wird der Benutzer nach einem dementsprechend veraenderten Namen gefragt. Dazu wird eine plattformabhaengige Methode in der konkreten Klasse fuer Filename benutzt (#suggest). In einer Instanzvariablen wird eine Tabelle gehalten, die einem Objektbezeichner seinen Dateinamen zuordnet. Wenn der wie oben zusammengesetzte Dateiname nicht den Konvenionen der Plattform entspricht, wird der Eintrag in der Tabelle geprueft. Ist kein Eintrag vorhanden oder der Eintrag fuehrt auch nicht zu einer existenten Datei, wird der Benutzer nach dem richtigen Namen gefragt und dieser in die Tabelle eingetragen.
Die Tabelle wird bei einem sogenannten "Downgrade" z.B. von DEC nach MSDOS aufgebaut, indem fuer jedes Objekt mit Label ein erlaubter Dateiname vom Benutzer abgefragt wird. Entweder wird die so entstandene Tabelle in das MSDOS image aufgenommen, oder der Benutzer muss bei jeder Datei, die nicht so schon passt den richtigen Namen angeben. 
Beim Downgrade wird ein Script erzeut, dass das Kopieren und Umbenennen der Dateien auf Betriebssystemebene (z.B. per ftp) erleichtert.

name		Name des Objektes
identifier		Bezeichner des Objektes, hier identisch mit name
fullFilename	Dateiname incl. Pfad
filename		Dateiname ohne Pfad
convert		konvertiere Dateinamen auf der aktuellen Plattform 

Instance variables:
name	<String> name of the object
path	<String | SequenceableCollection>	path holding the file
extension	<String>	use this a the filename extension
convertingTable	<Dictionary key: Symbol value: String>	dictionary of os specific legal filenames
'!


!StorableObjectAccessor methodsFor: 'accessing'!

dirnameForDOS
	"the fullFilename as dirname (without extension) on DOS"

	^self path / (self filenameForPlatform: 'dos')!

filename
	"the filename on the current platform"

	^self filenameForPlatform: OSHandle currentOS!

identifier
	^name!

name
	^name!

path
	^path! !

!StorableObjectAccessor methodsFor: 'initialize-release'!

extension: e 
	extension := e!

initialize
	convertingTable := Dictionary new: 3!

name: n
	name := n!

path: p 
	"path is an ordered collection of filename components (directories). You may 
	also use an instance if Filename (this will be converted if the platform 
	changes). Do not use pure strings including the Filename separator."

	path := p asFilename! !

!StorableObjectAccessor methodsFor: 'private'!

filenameForPlatform: aString 
	"return a file name string compatible with platform called 'aString'"

	^self filenameForPlatform: aString
		ifNone: [self findAndSetClosestMatchForPlatform: aString]!

filenameForPlatform: aString ifNone: aBlock 
	"return a file name string compatible with platform called 'aString'; 
	evaluate aBlock if none installed"

	^convertingTable at: aString ifAbsent: aBlock!

                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    

setFilename: f forPlatform: os 
	^convertingTable at: os put: f! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!StorableObjectAccessor class methodsFor: 'class initialization'!

initialize
	"StorableObjectAccessor initialize"
	SupportedPlatforms := #( 'unix' 'mac' 'dos').
	FilenameClasses := Dictionary new.
	SupportedPlatforms with: #( #UnixFilename #MacFilename  #DosFilename)
		do: [:platform :fClass | FilenameClasses at: platform put: fClass]! !

!StorableObjectAccessor class methodsFor: 'instance creation'!

path: path name: n extension: extension 
	^(self new) path: path; name: n; extension: extension! !

!StorableObjectAccessor class methodsFor: 'private instance creation'!

new
	^super new initialize! !



"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!


!OSHandle class methodsFor: 'class accessing'!

 !                                                      

StorableObjectAccessor initialize!




' .ApplicationSource StorableObject(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 September 1993 at 1:22:38 pm'!



!StorableObjectAccessor methodsFor: 'initialize-release'!

name: aString 
	"assign a new name; 
	reset the filename converting info if aString differs from the old name value"

	name = aString ifFalse: [convertingTable := Dictionary new: 3].
	name := aString! !


' .ApplicationSource StorableObject(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 4 November 1993 at 2:49:01 pm'!



"1) include os2 support
2) fix an error due to empty input in compatible filename inquiry"!

!StorableObjectAccessor class methodsFor: 'class initialization'!

initialize
	"StorableObjectAccessor initialize"

	SupportedPlatforms := #('unix' 'mac' 'dos' 'os2' ).
	(FilenameClasses := Dictionary new)
		at: 'unix' put: #UnixFilename;
		at: 'mac' put: #MacFilename;
		at: 'dos' put: #DosFilename;
		at: 'os2' put: #OS2Filename! !

!StorableObjectAccessor methodsFor: 'private'!

findAndSetClosestMatchForPlatform: aString 
	"the converting table lookup failed;
	search for filenames defined for unix, mac, dos (in this order); 
	define a new file name string legal for platform called 'aString'
	(via Filename>suggest:  and user interaction);
	enter it in the converting table"

	| fName sName class |
	fName := name.
	SupportedPlatforms do: [:pf |
		(convertingTable includesKey: pf) ifTrue: [fName := convertingTable at: pf]].

	class := (FilenameClasses at: aString) asClass.
	[(sName := class suggest: fName) = fName]
		whileFalse:
			[fName := DialogView
				request: ('Compatible filename for "' , name, '" on platform "',  aString, '"')
				initialAnswer: sName
				onCancel: nil.
			(fName isEmpty or: [fName = class defaultDirectoryName or: [fName includes: class separator]])
				ifTrue: [fName := sName]].

	^self setFilename: fName forPlatform: aString! !

StorableObjectAccessor initialize!


' .ApplicationSource StorableObject(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 11 April 1994 at 5:50:46 pm'!



"Windows NT compatibility patch plus code maintenance"!

!StorableObjectAccessor methodsFor: 'private'!

findClosestMatchForClass: aFilenameClass 
	"return a file name string legal wrt. aFilenameClass (via #suggest:  and user interaction)"

	| n guess |
	n := name.
	[(guess := aFilenameClass suggest: n) sameAs: n]
		whileFalse:
			[n := DialogView
				request: ('Enter ', aFilenameClass name, ' compatible string for\''' withCRs, name, '''')
				initialAnswer: guess
				onCancel: nil.
			(n isEmpty or: [n = aFilenameClass defaultDirectoryName or: [n includes: aFilenameClass separator]])
				ifTrue: [n := guess]].
	^n! !

!StorableObjectAccessor methodsFor: 'private'!

filenameForPlatform: aString 
	"return a file name string compatible with platform called aString 
	(see class variable SupportedPlatforms for possible values); 
	if necessary create it and enter it in the converting table"

	^self filenameForPlatform: aString
		ifNone: 
			[| n |
			n := self findClosestMatchForClass: (FilenameClasses at: aString) asClass.
			self setFilename: n forPlatform: aString]! !

!StorableObjectAccessor methodsFor: 'private'!

setFilename: aString forPlatform: platform 
	"install aString as filename for the receiver, compatible to platform specified by 
	2nd argument (String or FilenameClass)"

	| key |
	key := platform isBehavior
			ifTrue: [FilenameClasses keyAtValue: platform name asSymbol]
			ifFalse: [platform].
	^convertingTable at: key put: aString! !

!StorableObjectAccessor class methodsFor: 'class initialization'!

initialize
	"StorableObjectAccessor initialize"

	SupportedPlatforms := #('unix' 'mac' 'dos' 'os2' 'win32').
	(FilenameClasses := Dictionary new)
		at: 'unix' put: #UnixFilename;
		at: 'mac' put: #MacFilename;
		at: 'dos' put: #DosFilename;
		at: 'os2' put: #OS2Filename;
		at: 'win32' put: #NTFSFilename! !

StorableObjectAccessor initialize!

StorableObjectAccessor removeSelector: #findAndSetClosestMatchForPlatform:!

StorableObjectAccessor class removeSelector: #path:name:extension:!


' .ApplicationSource StorableObject(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 6 May 1994 at 3:26:34 pm'!



StorableObjectAccessor removeSelector: #dirnameForDOS!


' .ApplicationSource StorableObject(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 May 1994 at 7:34:37 pm'!



!Filename class methodsFor: 'defaults'!

defaultClassForPlatform: aString
	"return the filename class which is default for platform aString
	(nil if the class is not installed in this image)"

	| cls |
	cls := #(#UnixFilename #MacFilename #DosFilename #OS2Filename #NTFSFilename)
			at: (#('unix' 'mac' 'dos' 'os2' 'win32' ) indexOf: aString).
	^cls asClass! !

!StorableObjectAccessor class methodsFor: 'constants'!

 !                                                                                                                                                                                                                                

!StorableObjectAccessor methodsFor: 'accessing'!

platformNameMap
	"return a copy of the converting table"

	^convertingTable copy! !


' .ApplicationSource StorableObject(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 19 May 1994 at 1:18:59 pm'!



!StorableObjectAccessor class methodsFor: 'constants'!

supportedPlatforms
	"return an Array of ids of platforms which can be accessed (ordered by power)"

	^#('unix' 'mac' 'win32' 'os2' 'dos' )
		select: [:key | (FilenameClasses at: key) asClass isKindOf: Filename class]! !

!StorableObjectAccessor class methodsFor: 'constants'!

exportPlatforms
	"return an array of ids of platforms whose filenames can be created"

	| platforms |
	^(platforms := self supportedPlatforms) copyFrom: (platforms indexOf: OSHandle currentOS)
		to: platforms size! !


' .ApplicationSource StorableObject(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 September 1994 at 8:45:44 pm'!



"turn to Symbols as platform reference"!

!Filename class methodsFor: 'defaults'!

defaultClassForPlatform: aSymbol 
	"return the filename class which is default for platform aSymbol 
	(nil if the class is not installed in this image)"

	| cls |
	cls := #(UnixFilename MacFilename DosFilename OS2Filename NTFSFilename)
		at: (#(unix mac dos os2 win32) indexOf: aSymbol asSymbol).
	^Smalltalk at: cls
		ifAbsent: 
			[cls := #(FATFilename HPFSFilename)
					at: (#(dos os2) indexOf: aSymbol asSymbol).
			Smalltalk at: cls]! !

!StorableObjectAccessor methodsFor: 'private'!

filenameForPlatform: aSymbol ifNone: aBlock 
	"return a file name string compatible with platform called 'aSymbol'; 
	evaluate aBlock if none installed"

	^convertingTable at: aSymbol
		ifAbsent: 
			["former mapping was string->string; check and update"
			| n |
			n := convertingTable removeKey: aSymbol asString ifAbsent: [^aBlock value].
			self setFilename: n forPlatform: aSymbol]! !

!StorableObjectAccessor methodsFor: 'private'!

setFilename: aString forPlatform: platform 
	"install aString as filename for the receiver, compatible to platform specified by 
	2nd argument (String or FilenameClass)"

	| key |
	key := platform isBehavior
			ifTrue: [FilenameClasses keyAtValue: platform name asSymbol]
			ifFalse: [platform].
	^convertingTable at: key asSymbol put: aString! !

!StorableObjectAccessor methodsFor: 'private'!

filenameForPlatform: aSymbol 
	"return a file name string compatible with platform called aSymbol 
	(see class variable SupportedPlatforms for possible values); 
	if necessary create it and enter it in the converting table"

	^self filenameForPlatform: aSymbol
		ifNone: 
			[| n |
			n := self findClosestMatchForClass: (FilenameClasses at: aSymbol asSymbol) asClass.
			self setFilename: n forPlatform: aSymbol]! !

!StorableObjectAccessor class methodsFor: 'constants'!

supportedPlatforms
	"return an Array of ids of platforms which can be accessed (ordered by power)"

	^#(#unix #mac #win32 #os2 #dos)
		select: [:key | (FilenameClasses at: key) asClass isKindOf: Filename class]! !

!StorableObjectAccessor class methodsFor: 'class initialization'!

initialize
	"StorableObjectAccessor initialize"

	| vw20 |
	vw20 := (Smalltalk includesKey: #DosFilename) not.
	(FilenameClasses := Dictionary new)
		at: #unix put: #UnixFilename;
		at: #mac put: #MacFilename;
		at: #dos put: (vw20 ifTrue: [#FATFilename] ifFalse: [#DosFilename]);
		at: #os2 put: (vw20 ifTrue: [#HPFSFilename] ifFalse: [#OS2Filename]);
		at: #win32 put: #NTFSFilename! !

StorableObjectAccessor initialize!

!OSHandle class methodsFor: 'class accessing'!

 !                                                                