
' .ApplicationSource APL-Framework(1.0)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 17 August 1993 at 2:18:36 pm'!



'- comment for APL-Framework(1.0) -'!

(Message selector: #documentation: arguments: ((Array new: 1) at: 1 put: (Text string: 'APL-Framework contains the basic class definitions for Application Management' runs: (RunArray runs: #(55 22 ) values: #(nil #italic ))); yourself))!

'- required applications for APL-Framework(1.0) -'!

(Message selector: #checkRequired: arguments: #(#('BASE' 'Categorizer' 'APL-ChangeSequence' 'ExternalSource' 'Version' ) ))!

'- sources of APL-Framework(1.0) -'!

Dictionary variableSubclass: #ClassSubset
	instanceVariableNames: 'definition classRef '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Application'!


!ClassSubset methodsFor: 'accessing'!

categories
	^self keys!

categoryOf: anElement ifNone: aBlock 
	"return the category set that includes anElement, 
	evaluate aBlock if the element is not registered"

	^self detect: [:cat | cat includes: anElement] ifNone: aBlock!

classRef: aClassName 
	classRef == nil
		ifTrue: [classRef := aClassName]
		ifFalse: ["self error: 'Don''t change the class reference in a' , self class name"]!

definition: aBoolean
	definition := aBoolean!

mappedClass
	^classRef asClass!

renameCategory: cat to: newName 
	(self includesKey: newName)
		ifTrue: [self error: 'category ' , newName , ' already exists']
		ifFalse: [self at: newName put: (self removeKey: cat)]!

renameTo: aClassName 
	classRef := aClassName!

selectors
	| set |
	set := Set new.
	self do: [:protocol | set addAll: protocol].
	^set!

selectorsInProtocol: p 
	"return the selectors in protocol p, ordered as defined by the underlying class' 
	organization"

	| set |
	set := self at: p ifAbsent: [^#()].
	^(self mappedClass organization listAtCategoryNamed: p)
		select: [:sel | set includes: sel]! !

!ClassSubset methodsFor: 'testing'!

includesCategory: aSymbol 
	^self includesKey: aSymbol!

includesDefinition
	^definition == true!

includesSelector: aSymbol 
	self categoryOf: aSymbol ifNone: [^false].
	^true!

isEmpty
	^self includesDefinition not and: [super isEmpty]! !

!ClassSubset methodsFor: 'adding'!

add: anElement category: aSymbol 
	"returns anElement"
	^(self addCategory: aSymbol) add: anElement!

addCategory: aSymbol
	"returns contents of category 'aSymbol' (a Set)" 
	^aSymbol == nil
		ifTrue: [self error: 'Categorizer rejects category nil']
		ifFalse: [self at: aSymbol ifAbsentPut: [Set new]]!

addDefinition
	^definition := true!

addSelector: s protocol: p 
	^self add: s category: p asSymbol!

classify: entry under: categoryName 
	"move entry from its current category to a new one (don't add);
	return entry (nil, if it could not be found)"

	(self categoryOf: entry ifNone: [^nil])
		remove: entry ifAbsent: [^nil].
	^self add: entry category: categoryName! !

!ClassSubset methodsFor: 'removing'!

removeCategory: aSymbol 
	"returns contents of category 'aSymbol' (a Set)"

	^self removeKey: aSymbol ifAbsent: []!

removeDefinition
	"release definition of the underlying class, return previous definition state"

	^self includesDefinition and: [definition := false. true]!

removeEmptyCategories
	"remove all categories with no elements;
	return false, if no empty category was found, true otherwise"

	| removed |
	removed := false.
	self copy associationsDo: [:each | each value isEmpty
			ifTrue: 
				[self removeKey: each key. removed := true]].
	^removed!

removeSelector: aSymbol
	"removes aSymbol from its category and returns it (nil if not found)"

	^(self categoryOf: aSymbol ifNone: [^nil])
		remove: aSymbol ifAbsent: []! !

!ClassSubset methodsFor: 'printing'!

printOn: aStream 
	classRef printOn: aStream.
	self includesDefinition ifTrue: [aStream nextPutAll: ' with definition'].
	aStream cr.
	aStream nextPut: $(.
	self
		associationsDo: 
			[:assoc | 
			assoc key asString printOn: aStream.
			assoc value do: 
				[:elemInCat | 
				aStream space.
				elemInCat printOn: aStream].
			aStream cr].
	aStream nextPut: $)! !

!ClassSubset methodsFor: 'file out'!

fileOutProtocol: p on: aStream
	"file out the source code of all the selectors in protocol p"

	aStream fileOutMessages: (self at: p ifAbsent: [^self]) for: self mappedClass!

fileOutSourceOn: aSourceStream 
	"copied and modified from ClassDescription"

	| class |
	class := self mappedClass.
	self includesDefinition
		ifTrue: 
			[(class isMeta and: [class instVarNames isEmpty])
				ifFalse: [aSourceStream cr; cr; emphasis: #bold; store: class definition].
			class organization storeCommentOn: aSourceStream.
			aSourceStream cr].

	class organization categories do: [:p | self fileOutProtocol: p on: aSourceStream].

	(class isMeta and: [self includesSelector: #initialize])
		ifTrue: [aSourceStream initializeThisClass: class soleInstance]!

printOutOn: aStream 
	self includesDefinition ifTrue: 
		[| class emph |
		emph := aStream emphasis.
		aStream
			emphasis: #bold; nextPutAll: (class := self mappedClass) definition; cr; cr;
			emphasis: #italic; nextPutAll: class comment; cr; cr;
			emphasis: emph].
	self categories do:
		[:p |
		self printOutProtocol: p on: aStream.
		aStream cr]!

printOutProtocol: p on: aStream 
	| class emph |
	class := self mappedClass.
	emph := aStream emphasis.
	aStream cr; emphasis: #bold; nextPutAll: class name , ' methodsFor: ' , p; emphasis: emph.
	(self selectorsInProtocol: p)
		do: [:sel | class printMethod: sel on: aStream].
	aStream cr!

representBinaryOn: binWriter 
	"take care of the instance variables 'definition' and 'classRef'"

	^MessageSend
		receiver: self class
		selector: #of:definition:selectors:
		arguments: (Array
				with: classRef
				with: self includesDefinition
				with: self associations asArray)! !

!ClassSubset methodsFor: 'copying'!

copyEmpty: aNumber 
	^(super copyEmpty: aNumber)
		definition: self includesDefinition;
		classRef: classRef! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!ClassSubset class methodsFor: 'instance creation'!

of: aClass 
	"return a new class subset with no definition or selectors"

	^self of: aClass name definition: false selectors: #()!

of: aClassname definition: aBoolean selectors: anArrayOfAssociations 
	"return a new class subset for a class named 'aClassname'; 
	it will contain the method selectors organized by 'anArrayOfAssociations' 
	and the class' definition (if 'aBoolean' is true)"

	^(self withAll: anArrayOfAssociations) definition: aBoolean; classRef: aClassname! !



!Object methodsFor: 'fileIn/Out'!

binaryStoredDependents
	"(extracted from BOSSWriter>trace:body:class:) 
	return the dependents of the receiver which should be stored when storing the 
	receiver"

	^DependentsFields at: self ifAbsent: []! !

!Object methodsFor: 'application support'!

applicationManagement
	"return the application management"

	^Smalltalk at: #ApplicationManager ifAbsent: [DummyObject new]! !



Object subclass: #Application
	instanceVariableNames: 'label classes poolDefinitions necessary documentation responsible changes history doits sourceFile '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Application'!
Application comment:
'Application is our term for a software package. It basically comprises class and method definitions, which represent the computational solution of a specific need. 

All definitions or changes (also include/exclude) with effect on classes, methods and organizations, which are part of an Application, are registered in a change sequence (remark: the system has a change SET).

Each new version (major or minor) moves the contents of "changes" to "history". Thus "changes" has a temporary character, keeping only changes between one version and another, while "history" keeps all the changes until next condense changes happens.
Each major version stores the current source code of all registered methods and classes to a major version source file and redirects the source pointers of the methods to this file. Do not delete it, if you are not sure, that nobody needs it !!
Each minor version appends the source code for changes registered in "changes" to the major version source file, also redirecting method source pointers !!

All data of an application (sources, poolDictionaries, browser descriptions, etc.) is written to subdirectories of
	Path ::= <any path>/<application name>

All data of a major version is written to a directory named
	MajorPath ::= Path/v<major version number>

All the source code of a major version is written to a file named
	SourceFile ::= MajorPath/s.st


Instance Variables:
label			<VersionedObjectAccessor>	holder for name, version, path, access restrictions
responsible		<Object>	representative of a software engineer or supporter
necessary		<Collection of: Application>	applications, which should be installed
classes			<Dictionary key: Symbol value: ClassSubset>	registered classes/methods
poolDefinitions	<Collection of: Symbol>	global variables
documentation	<Text> comment on the application itself (summary, purpose etc.)
changes		<ChangeSequence>	changes to registered classes/methods since last minor version
history			<ChangeSequence>	changes since last condense changes
doits			<ChangeSequence> expressions to execute before class and method installation
sourceFile		<ExternalReadStream>	contains the smalltalk code
'!


!Application methodsFor: 'classes'!

addClass: aClass 
	"register aClass including its definition"

	(self addClassSubset: aClass) definition: true.
	self changes addClass: aClass.
	self touch!

changeClass: aClass 
	"aClass' definition has changed."

	(self definesClass: aClass)
		ifFalse: [^self].
	self changes backupClass: aClass; changeClass: aClass.
	self touch!

changeOrIncludeClass: aClass 
	"aClasses definition has changed. It is associated with me, so if I do not own 
	aClass I have to include it's definition"

	(self includesClass: aClass)
		ifFalse: [self includeClass: aClass].
	self changes backupClass: aClass; changeClass: aClass!

classifyClass: className under: category 
	"management informs that class is in category"!

commentClass: className 
	"record class' new comment"

	(self definesClass: className)
		ifTrue: [self changes commentClass: className. self touch]!

excludeClass: aClass 
	"exclude aClass, its definition and all its methods from application"

	| subset sys |
	(subset := self removeClassSubset: aClass) isNil
		ifTrue: [^nil].

	(sys := self systemApplication)
		addChanges: (self changes removeClassChanges: aClass).

	"enter exclude changes after removing all others"
	subset selectors do: [:sel |
		sys changeOrIncludeSelector: sel class: aClass.
		self changes excludeSelector: sel class: aClass name].
	subset includesDefinition
		ifTrue:
			[sys includeClass: aClass.
			self changes excludeClass: aClass name].
	self touch!

excludeClassDefinition: aClass 
	(self classSubsetOf: aClass ifAbsent: [^nil])
		removeDefinition ifFalse: [^nil].
	self systemApplication
		changeOrIncludeClass: aClass;
		addChanges: (self changes removePureClassChanges: aClass).
	self touch!

includeClass: aClass 
	"aClass is included (structure and definition);
	includes all changes on aClass which are not managed by a selector change"

	| someChanges |
	(self definesClass: aClass) ifTrue: [^aClass].

	(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass]) addDefinition.
	((someChanges := self systemApplication excludeClassDefinition: aClass) isNil
		or: [someChanges isEmpty])
		ifTrue:
			["assume metaclasses will never be included without its soleInstance;
			so we don't need a change entry"
			aClass isMeta ifFalse: [self changes includeClass: aClass]]
		ifFalse:
			[self changes addAll: someChanges].
	self touch.
	^aClass!

removeClass: aClass 
	"management informs that aClass is removed from system"

	(self removeClassSubset: aClass) isNil 
	& (self removeClassSubset: aClass class) isNil
		ifTrue: [^self].
	self changes removeClass: aClass name.
	self touch!

removeFromSystem: aClass 
	"removes aClass from system (meta and soleInstance)
	(restricted to the corresponding class subset of the receiver);
	fails for metaclasses and classes with subclasses;
	return aClass (or nil if fail)"

	| selectors metaSelectors |
	selectors := self selectorsInClass: aClass.
	metaSelectors := self selectorsInClass: aClass class.
	(((self definesClass: aClass)
		and: [selectors size = aClass selectors size])
		and: [metaSelectors size = aClass class selectors size])
		ifTrue: 
			["the receiver owns aClass with definition and all the selectors;
			remove aClass in a single step"
			aClass subclasses isEmpty
				ifFalse: [Transcript show: aClass name , ' has subclasses - not removed'; cr. ^nil].
			"not necessary (but better if application management is idle)"
			self removeClass: aClass.	
			aClass removeFromSystem.
			^aClass].

	"remove registered selectors one after another"
	selectors do: [:sel | aClass removeSelector: sel].
	metaSelectors do: [:sel | aClass class removeSelector: sel].

	"remove registered protocols (soleInstance and meta) which are empty now"
	(Array with: aClass with: aClass class) do: 
		[:class | 
		(self protocolsInClass: class) do:
			[:p | (class organization listAtCategoryNamed: p) isEmpty
				ifTrue: 
					[class organization removeCategory: p.
					SourceFileManager default logChange: class name , ' organization removeCategory: ' , p storeString.
					class reorganize]]].

	"unregister the class' definition"
	self excludeClassDefinition: aClass; excludeClassDefinition: aClass class.
	"unregister the class subset"
	(self removeClassSubset: aClass) isNil
	& (self removeClassSubset: aClass class) isNil
		ifTrue: [^nil].
	^aClass!

renameClass: aClass as: newName 
	"rename aClass' subset and reenter it under the new key; 
	add a rename change if the receiver defines aClass"

	| rename subset metaName |
	rename := false.
	(subset := self removeClassSubset: aClass) isNil
		ifFalse: 
			[subset renameTo: newName.
			classes at: newName asSymbol put: subset.
			rename := subset includesDefinition].
	(subset := self removeClassSubset: aClass class) isNil
		ifFalse: 
			[metaName := (newName , ' class') asSymbol.
			subset renameTo: metaName.
			classes at: metaName put: subset.
			rename := rename | subset includesDefinition].
	rename
		ifTrue: 
			[self changes renameClass: aClass name to: newName.
			self touch]! !

!Application methodsFor: 'protocols'!

excludeProtocol: protocol class: aClass 
	"exclude protocol and all selectors in that protocol known to the receiver"

	(self selectorsInProtocol: protocol class: aClass)
		do: [:sel | self excludeSelector: sel class: aClass].
	(self classSubsetOf: aClass ifAbsent: [^nil])
		removeCategory: protocol!

fileOutProtocol: protocol class: className on: aStream  
	(self classSubsetOf: className ifAbsent: [^nil])
		fileOutProtocol: protocol on: aStream!

includeProtocol: protocolName class: aClass 
	"includeProtocol;
	 if class is not in structure, register it (without definition)"

	self touch.
	^(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass])
		addCategory: protocolName asSymbol!

removeProtocol: protocol class: aClass 
	"discard protocol with all its selectors from info structure"

	^(self classSubsetOf: aClass ifAbsent: [^nil])
		removeCategory: protocol!

renameProtocol: oldName to: newName class: aClass 
	| subset |
	((subset := self classSubsetOf: aClass ifAbsent: [^self])
		includesCategory: oldName) ifFalse: [^self].
	subset renameCategory: oldName to: newName.
	self changes renameProtocol: oldName to: newName class: aClass.
	self touch! !

!Application methodsFor: 'comparing'!

= anApplication 
	"applications should be equal if their identifiers are equal"

	^self identifier = anApplication identifier!

hash
	^self identifier hash! !

!Application methodsFor: 'copying'!

postCopy
	"ensure ownership of label"

	super postCopy.
	self releaseSourceFile.
	label := label copy! !

!Application methodsFor: 'print out'!

printOutCategory: aSymbol on: aStream
	self printOutClasses: (self allClassNamesIn: (self classNamesInCategory: aSymbol))
		on: aStream!

printOutClass: aClass on: aStream
	(self classSubsetOf: aClass ifAbsent: [^self])
		printOutOn: aStream!

printOutClasses: classNameList on: aStream 
	| realClasses last |
	realClasses := classNameList collect: [:cName | cName asClass].
	realClasses := realClasses select: [:class | class isMeta not].
	realClasses := ChangeSet superclassOrder: realClasses.
	last := realClasses last.
	realClasses do: 
		[:class | 
		self printOutClassWithMeta: class on: aStream.
		class == last ifFalse: [aStream cr; nextPut: Character newPage]].
	aStream cr; cr.
	realClasses do: [:class | self printOutClassInit: class on: aStream]!

printOutClassInit: aClass on: aStream 
	((self classSubsetOf: aClass ifAbsent: [^self])
		includesSelector: #initialize)
		ifTrue: [aStream nextPutAll: aClass asSymbol , ' initialize'; cr]!

printOutClassWithMeta: aClass on: aStream 
	self printOutClass: aClass on: aStream.
	self printOutClass: aClass asClass class on: aStream!

printOutProtocol: protocol class: aClass on: aStream 
	(self classSubsetOf: aClass ifAbsent: [^nil])
		printOutProtocol: protocol on: aStream!

printOutSourcesOn: aStream
	self preInstallation isEmpty
		ifFalse:
			[aStream
				emphasis: #italic; nextPutAll: '- installation of ', self identifier, ' -';
				emphasis: nil; cr; cr.
			self preInstallation do: [:chg | aStream nextPutAll: chg text; cr; cr]].
	aStream
		emphasis: #italic; nextPutAll: '- sources of ', self identifier, ' -';
		emphasis: nil; cr; cr.
	self printOutClasses: self classNames on: aStream

"postInstallation not yet printed"! !

!Application methodsFor: 'selectors'!

addSelector: selector class: aClass 
	"add new selector to application, protocol as in system"

	| protocol |
	protocol := self protocolFor: selector class: aClass.
	self
		addSelector: selector
		protocol: protocol
		inSubsetOf: aClass.
	self changes addSelector: selector protocol: protocol class: aClass.
	self touch!

changeOrIncludeSelector: selector class: aClass 
	"a method change has happened, and the receiver has been determined to 
	record the change; if the class/selector combination is not yet registered, do it 
	right now; a new registration is coupled to the creation of a backup version for 
	the original definition of the method"

	(self includesSelector: selector class: aClass)
		ifTrue:
			["this branch forces a backup, remove if you dont want it"
			self changes
				backupSelector: selector
				protocol: (self protocolFor: selector class: aClass)
				class: aClass]
		ifFalse: 
			[self includeSelector: selector class: aClass.
			self systemApplication excludeSelector: selector class: aClass].
	self touch.
	self changes changeSelector: selector class: aClass!

changeSelector: selector class: aClass 
	"Include indication that a method has been edited."

	(self includesSelector: selector class: aClass)
		ifTrue: [self changes changeSelector: selector class: aClass. self touch]!

classifySelector: selector under: protocol class: aClass 
	"change the protocol of the specified selector"

	((self classSubsetOf: aClass ifAbsent: [^nil])
		classify: selector under: protocol asSymbol) == nil ifTrue: [^nil].
	self changes
		classifySelector: selector
		under: protocol
		class: aClass.
	self cleanUpProtocols touch!

excludeSelector: selector class: aClass 
	"selector is excluded from application, included into system application; 
	all recorded changes move to system application"

	(self removeSelector: selector inSubsetOf: aClass) isNil
		ifTrue: [^nil]. 

	self systemApplication
		changeOrIncludeSelector: selector class: aClass;
		addChanges: (self changes removeSelectorChanges: selector class: aClass).

	"enter the exclude change after removing all others"
	self changes excludeSelector: selector class: aClass.
	self touch!

includeSelector: selector class: aClass 
	"include class/selector combination to the application structure; 
	create a backup version in the change sequence to remember the original 
	state of the source code"

	| protocol |
	protocol := self protocolFor: selector class: aClass.
	self
		addSelector: selector
		protocol: protocol
		inSubsetOf: aClass.
	self changes
		backupSelector: selector
		protocol: protocol
		class: aClass!

includeSelector: selector protocol: protocol class: aClass 
	"include selector to application, exclude from system changes;
	move changes for selector from system application to receiver"

	| someChanges |
	(self includesSelector: selector class: aClass)
		ifTrue: [^self].
	self addSelector: selector protocol: protocol inSubsetOf: aClass.
	((someChanges := self systemApplication excludeSelector: selector class: aClass) isNil
		or: [someChanges isEmpty])
		ifTrue: [self changes includeSelector: selector protocol: protocol class: aClass]
		ifFalse: [self changes addAll: someChanges].
	self touch!

removeSelector: selector class: aClass 
	"selector has been removed; discard it from its class subset, forward the event 
	to the receiver's changes"

	(self removeSelector: selector inSubsetOf: aClass) isNil ifTrue: [^self].
	self changes removeSelector: selector class: aClass.
	self touch! !

!Application methodsFor: 'prerequesites'!

excludeApplication: applicationName
	^necessary remove: applicationName!

findRequiredApplications
	"enter all applications, which prop the receiver, to necessary variable"

	self applicationManagement do: [:app | (self testUseOfApplication: app)
			ifTrue: [self includeApplication: app]]!

includeApplication: anApplication
	"register the use of anApplication"

	necessary isNil ifTrue: [necessary := OrderedCollection new].
	^(necessary includes: anApplication name)
		ifFalse: [necessary add: anApplication name]!

testUseOfApplication: anApplication 
	"check if there are references to classes defined in anApplication"

	anApplication == self ifTrue: [^false].
	^Cursor execute showWhile: 
		[| appClasses |
		appClasses := anApplication classNames select: [:name | anApplication definesClass: name].

		"check if some own class or own class' superclass is defined by 'anApplication' "
		self classNames do: [:cname | | requiredClass |
			requiredClass := (self definesClass: cname)
				ifTrue: [cname asClass superclass name]
				ifFalse: [cname].
			(appClasses includes: requiredClass) ifTrue: [^true]].

		"check if there are code references to a class defined by 'anApplication' "
		appClasses detect: [:name | (self allCallsOn: (Smalltalk associationAt: name ifAbsent: [])) isEmpty not]
				ifNone: [^false].
		true]! !

!Application methodsFor: 'printing'!

printOn: aStream 
	aStream nextPutAll: self identifier! !

!Application methodsFor: 'inquiry'!

classNames
	"returns soleInstance class names of registered classes"

	^self allClassNamesIn: classes keys!

classNamesInCategory: category 
	"the names of classes I contain in category, a collection of strings. 
	Metaclasses are keyed with 'class'"

	| names |
	names := self classNames select: [:cn | (self systemCategoryOfClass: cn)
					= category].
	^self allClassNamesIn: names!

protocolsInClass: aClass 
	"return protocols contained in the class subset for aClass"

	aClass isNil ifTrue: [^Array new].
	^(self classSubsetOf: aClass ifAbsent: [^Array new]) categories!

selectors
	"return a set of assoc (selector -> class) of all registered selectors of all 
	registered classes (soleInstance and meta)"

	| allSelectors |
	allSelectors := Set new.
	classes keys do: [:classname |
		allSelectors addAll: ((self selectorsInClass: classname)
			collect: [:sel | sel -> classname])].
	^allSelectors!

selectorsInClass: aClass 
	"the selectors I contain in class"

	aClass isNil ifTrue: [^Array new].
	^(self classSubsetOf: aClass ifAbsent: [^Array new]) selectors!

selectorsInProtocol: p class: aClass 
	"the selectors I contain in class and protocol"

	^(self classSubsetOf: aClass ifAbsent: [^Array new])
		selectorsInProtocol: p!

sentSelectorsInClass: aClassOrSymbol own: aSetOfSymbol 
	"return a set of message selectors sent by all registered selectors of aClass"

	| sent own |
	sent := Set new.
	(own := self selectorsInClass: aClassOrSymbol) do: [:sel |
		sent addAll: (aClassOrSymbol asClass compiledMethodAt: sel) allSymbolLiterals].
	aSetOfSymbol == nil ifFalse: [aSetOfSymbol addAll: own].
	^sent!

sentSelectorsOwn: aSetOfSymbol 
	"return a set of message selectors sent by all registered selectors of all 
	registered classes (soleInstance and meta)"

	| sent |
	sent := Set new.
	classes keys do: [:class |
		sent addAll: (self sentSelectorsInClass: class own: aSetOfSymbol)].
	^sent! !

!Application methodsFor: 'fileIn/Out'!

binaryStoredDependents
	"avoid storing any dependents on BOSSWriters"

	^nil!

fileInFromStream: aStream startAt: startVersion stopAt: stopVersion external: flag 
	"file in sources from a stream: 
	start after the first occurence of startVersion (current position if nil), 
	stop at occurence of stopVersion (or at end of stream). 
	All changes are logged by the receiver; 
	if 'flag' is false, all the sources are appended to the changes file; 
	otherwise keep source pointers to aStream (holder for the file is the receiver, 
	so take care that aStream is a stream on the receivers source file!!)"

	flag ifTrue: [self path isNil
			ifTrue: [self label path: aStream name asFilename]
			ifFalse: [self instSourceFilename asString = aStream name asString
				ifFalse: [^self error: 'Attempt to assign a second source file']]].

	^self applicationManagement logTo: self
		while: [aStream fileInFrom: startVersion to: stopVersion sources: (flag ifTrue: [self])]!

fileOutCategory: aSymbol on: aSourceStream 
	self
		fileOutClasses: (self allClassNamesIn: (self classNamesInCategory: aSymbol))
		on: aSourceStream!

fileOutChanges
	| fileStream |
	self changes isEmpty
		ifTrue: [Transcript show: 'no changes to file out']
		ifFalse: 
			[fileStream := self changesFilename asFilename writeStream.
			fileStream timeStamp.
			self changes fileOutOn: fileStream.
			fileStream close]!

fileOutClass: aClass on: aStream
	"file out sources of class and metaclass"

	| subset |
	(subset := self classSubsetOf: aClass ifAbsent: []) notNil 
		ifTrue: [subset fileOutSourceOn: aStream].
	aClass class nonTrivial ifFalse: [^self].
	subset := self classSubsetOf: aClass class ifAbsent: [^self].
	aStream cr; cr; store: '"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "'; cr.
	subset fileOutSourceOn: aStream!

fileOutClasses: classNameList on: aStream 
	"file out the current source code for all registered elements
	(definition, comment, methods, etc.) of the classes in classNameList"

	| classList first |
	first := true.
	aStream deferInitializations.
	classList := classNameList collect: [:name | | class |
		(class := name asClass) isMeta
			ifTrue: [class soleInstance]
			ifFalse: [class]].
	(classList := ChangeSet superclassOrder: classList) do: [:class | 
		first ifTrue: [first := false] ifFalse: [aStream cr; newPage; cr].
		self fileOutClass: class on: aStream].
	aStream removeInitializations: self postInstallation.
	aStream cr; finishInitializations; cr!

fileOutDocumentationOn: aSourceStream 
	"write a message which can be sent to application management"

	documentation isNil ifTrue: [^self].
	aSourceStream
		cr; cr; store: ('- comment for ' , self identifier , ' -') storeString;
		cr; cr; store: (Message selector: #documentation: arguments: (Array with: self documentation)) storeString!

fileOutOrder: nonMetaClasses
	"find out the best order for filing out the given classes:
	- superclass order has highest priority
	- account class references within the code"

	| table result size |
	table := self referencesOn: nonMetaClasses.
	"filter out candidates for critical class dependency"
	table associationsDo: [:assoc | assoc value: (assoc value select: [:call |
		call value == #new or: ['initialize*' match: call value]])].
	result := OrderedCollection new.
	[(size := table size) = 0]
		whileFalse:
		[table copy keysAndValuesDo: [:cls :calls |
			(table includesKey: cls superclass)
				ifTrue: ["wait until superclass has been ordered"]
				ifFalse: [calls isEmpty
					ifTrue: [table removeKey: cls. result add: cls]
					ifFalse: [| j |
						j := result size + 1.
						calls detect: [:assoc | "check if a dependent has not been ordered;
											find the minimum dependent index"
								(j := j min: (result indexOf: assoc key asClass)) == 0]
							ifNone: [table removeKey: cls. result insert: cls before: j]]]].
			size = table size ifTrue: [self halt: 'not decidable'].
		].
	^result!

fileOutPools
	| store filename |
	self hasPoolDefinitions ifFalse: [^self].
	filename := DialogView
		request: 'filename for pool'
		initialAnswer: self fileString , '.pol'
		ifEmpty: [^self].

	[store := BinaryObjectStorage onNew: filename asFilename writeStream.
	self fileOutPoolsOn: store]
		valueNowOrOnUnwindDo: [store close]!

fileOutPoolsOn: aStore
	| externalPools |
	externalPools := Dictionary new.
	self poolDefinitions do: [:pool | externalPools at: pool put: (Smalltalk at: pool)].
	aStore nextPut: externalPools.!

fileOutPostInstallationOn: aSourceStream 
	"write the list of expressions which finalize the installation of the receiver"

	| list |
	(list := self postInstallation) isEmpty ifTrue: [^self].
	aSourceStream cr; cr.
	list fileOutExternalOn: aSourceStream!

fileOutPreInstallationOn: aSourceStream 
	"write the list of expressions which prepare the installation of the receiver"

	| list |
	(list := self preInstallation) isEmpty ifTrue: [^self].
	aSourceStream cr; cr; store: ('- installation of ' , self identifier , ' -') storeString.
	list fileOutExternalOn: aSourceStream!

fileOutRequiredOn: aSourceStream 
	"avoid doits which cannot be evaluated for users without application 
	management; return a message which can be sent to application 
	management"

	| required |
	(required := self necessary asArray) isEmpty
		ifTrue: [^self].
	aSourceStream
		cr; cr; store: ('- required applications for ' , self identifier , ' -') storeString;
		cr; cr; store: (Message selector: #checkRequired: arguments: (Array with: required)) storeString!

fileOutSourceIdentifierOn: aSourceStream 
	aSourceStream cr; store: self sourceIdentifier storeString; cr!

fileOutSourcesOn: fileStream 
	fileStream cr; cr; store: ('- sources of ' , self identifier , ' -') storeString.
	self fileOutClasses: self classNames on: fileStream!

storeForDOS
	"installs a copy of the application's directory tree under a directory name 
	compatible to MS-DOS"

	self directory copyRecursiveTo: self label dirnameForDOS! !

!Application methodsFor: 'private'!

allClassNamesIn: aCollection 
	"Only non-empty class subsets are part of the class info structure. 
	This method accounts for the case that there is an entry for a metaclass but not 
	for the non-metaclass and returns the names of all classes part of the info 
	structure in any way (meta or not), mapped to the names of the non-metaclasses"

	^aCollection collect: [:className | | class |
			(class := className asClass) isMeta
				ifTrue: [class soleInstance name]
				ifFalse: [className]]!

incrementMajorVersion
	"increment major version, create corresponding source file directories"

	label := label incrementMajor.
	self directory exists ifFalse: [self directory makeDirectory].
	self minorDirectory makeDirectory!

incrementMinorVersion
	label incrementMinor!

moveMethodsInSourcefileToChanges
	"move all methods in my source file to system changes"

	classes do: [:subset | SourceFileManager default
			fileOutMessages: subset selectors
			for: subset mappedClass
			logging: false]!

protocolFor: selector class: aClass 
	"system protocol for selector, if none return default"

	| protocol |
	protocol := aClass asClass organization categoryOfElement: selector.
	protocol isNil ifTrue: [^ClassOrganizer defaultProtocol].
	^protocol!

sourceFile
	"return (and cache) the file stream containing the sources (open it if necessary)"

	| path |
	sourceFile == nil ifFalse: [^sourceFile].

	^OSErrorHolder nonexistentSignal
		handle: [:ex |
			path := path isNil ifTrue: [''] ifFalse: [path asString].
			path := DialogView requestFileName: 'The path/file for sources of ', self identifier storeString, '
(' , path storeString , ')
is invalid.
Enter the new path:' default: path version: #mustBeOld.
			path isEmpty 
				ifTrue:
					["return a dummy, so that continuing without source file is possible"
					sourceFile := '' readStream]
				ifFalse:
					["save the correct path and use it to repeat the source file access"
					self setPath: path.
					ex restart]]
		do:
			[path := self version isInitialVersion ifTrue: [self path] ifFalse: [self instSourceFilename].
			sourceFile := path asFilename readStream]!

sourceIdentifier
	"return a String which identifies the receiver with its version in its source file"

	^self class sourceHeader , self identifier!

storeFile
	"returns a SourceFileManager for the current source file, 
	which redirects the source code pointers during file out procedure"

	^ApplicationSourceStream on: self!

systemApplication
	"return the system application from application management"

	^self applicationManagement systemChanges!

systemCategoryOfClass: aClass 
	| c |
	c := aClass asClass.
	c isNil ifTrue: [^nil].
	^(c isMeta
		ifTrue: [c soleInstance]
		ifFalse: [c]) category! !

!Application methodsFor: 'accessing'!

categories
	"search categories with at least one method or class definition in application"

	| categories |
	categories := Set new.
	self classNames do: [:name |
		categories add: (self systemCategoryOfClass: name)].
	^categories!

changes
	"returns the temporary changes (changes since last version release)"

	^changes!

documentation
	"if documentation is nil, initialize it with my documentationFilename. 
	documentation is either a Filename, then read the first element on 
	this file to be my documentation; or the documentation cached."

"	documentation isNil
		ifTrue: [documentation := self fullDocFilename].
	self documentationIsCached ifFalse: 
		[documentation := self documentationClass fromOrOnFile: documentation].
"
	^documentation isNil ifTrue: [Text new] ifFalse: [documentation]!

documentation: aText
	documentation := aText!

fullHistory
	"return a copy a of all saved and current changes"

	^(ChangeSequence new: (self history size + self changes size + 1))
		addAll: self history;
		addDoIt: '"- end of history -"'; 
		addAll: self changes;
		yourself!

history
	"return changes made before release of last minor version"

	^history isNil
		ifTrue: [ChangeSequence new]
		ifFalse: [history]!

identifier
	"return name of receiver with version"

	^self label identifier!

label
	"either name of application or a VersionedObjectAccessor"

	^label!

majorVersion
	^self version major!

minorVersion
	^self version minor!

name
	"the name of an application is only the pure name string without 
	version numbers"

	^self label name!

necessary
	"collection of application this application requires"

	^necessary!

path
	"the directory for installed source code"

	^self label path!

version
	^self label version!

version: aVersionDescription 
	self label version: aVersionDescription.
	self changed: #version! !

!Application methodsFor: 'do its'!

addDoIt: smalltalkExpression 
	self changes addDoIt: smalltalkExpression.
	self touch!

doits
	^doits isNil ifTrue: [ChangeSequence new] ifFalse: [doits]!

postInstallation
	"return expressions to execute after installation of the receicer's sources
	(without separator)"

	| i |
	i := 0.
	self doits detect: [:chg | i := i+1. chg text = '"post installation"'] ifNone: [^ChangeSequence new].
	^doits copyFrom: i+1 to: doits size!

preInstallation
	"return expressions to execute before installation of the receicer's sources"

	| i |
	^(i := self postInstallation size) = 0
		ifTrue: ["no post installation"
			self doits]
		ifFalse: ["drop the trailing post installation (and the separator)"
			doits copyFrom: 1 to: doits size - i - 1]!

setDoits: aChangeSequence 
	"assign a collection of doits which are filed out before any other source"

	doits := aChangeSequence isEmpty ifFalse: [aChangeSequence]!

setPostInstallation: aChangeSequence 
	"install expressions to execute after installation of the receicer's sources; 
	they are appended to the end of the pre installation doits"

	| pre |
	pre := self preInstallation.
	aChangeSequence isEmpty
		ifFalse: [pre addDoIt: '"post installation"'; addAll: aChangeSequence].
	self setDoits: pre!

setPreInstallation: aChangeSequence 
	"install expressions to execute before installation of the receicer's sources; 
	they are placed before any post installation doits"

	| post |
	(post := self postInstallation) isEmpty
		ifFalse: [aChangeSequence addDoIt: '"post installation"'; addAll: post].
	self setDoits: aChangeSequence! !

!Application methodsFor: 'testing'!

checkRequired: anArray 
	"called during file in, just forward to application management"

	self applicationManagement checkRequired: (anArray isString
			ifTrue: [Array with: anArray]
			ifFalse: [anArray])!

definesClass: aClass 
	"returns true if the class' definition is included"

	^(self classSubsetOf: aClass ifAbsent: [^false]) includesDefinition!

hasPoolDefinitions
	^poolDefinitions notNil!

hasUpgrade
	"return true if there are new sources since the last file in"

	| latest |
	^self version isInitialVersion not
		and: [(latest := self latestMinor) notNil and: [latest > self version]]!

includesCategory: categoryName 
	"true if at least one method or class definition is in category"

	^self classNames detect: [:cn | (self systemCategoryOfClass: cn) = categoryName]!

includesClass: className 
	"true if any selector in class is included"

	^(self classSubsetOf: className ifAbsent: [^false]) isEmpty not.!

includesProtocol: protocol class: aClass
	^(self classSubsetOf: aClass ifAbsent: [^false])
		includesCategory: protocol!

includesSelector: selector class: aClass 
	^(self classSubsetOf: aClass ifAbsent: [^false])
		includesSelector: selector!

installationAllowed
	"you are only allowed to install me if you are allowed to modify me"

	^self modificationAllowed!

isEmpty
	"true if no class definitions or methods are includes"

	^classes isEmpty!

isModified
	^self changes isEmpty not!

isNecessaryApplication: applicationName 
	"true if application is necessary for me"

	^necessary includes: applicationName!

modificationAllowed
	"only my guru may modify me. 
	smalltalkSuperUser (should be st80) is everybody's guru"

	| currentUser |
	^(Smalltalk includesKey: #AccessControlList)
		ifTrue: 
			[currentUser := #AccessControlList asClass user login.
			currentUser = self version guru or: [currentUser = #AccessControlList asClass smalltalkSuperUser]]
		ifFalse: [true]! !

!Application methodsFor: 'poolDefinitions'!

addPoolDefinition: poolName
	self poolDefinitions add: poolName!

definePoolDictionaries
	| bos externalPools |
	self hasPoolDefinitions ifFalse: [^self].
	bos := BinaryObjectStorage onOld: self instPoolFilename readStream.
	externalPools := bos next.
	bos close.
	externalPools associationsDo: [:pool | (Smalltalk includesKey: pool key)
			ifTrue: [(Smalltalk at: pool key) addAll: pool value associations]
			ifFalse: [Smalltalk at: pool key put: pool value]]!

poolDefinitions
	^self hasPoolDefinitions
		ifTrue: [poolDefinitions]	
		ifFalse: [poolDefinitions := Set new]!

removePoolDefinition: poolName
	self poolDefinitions remove: poolName! !

!Application methodsFor: 'version management'!

getMinorVersionPosition: aVersion 
	"return a file index for the beginning of sources for aVersion"

	InputState default shiftDown
		ifFalse: 
			[| p key |
			key := self class sourceHeader , self name , '(' , aVersion printString , ')'.
			(p := ChangeList new findLast: key in: self sourceFile) isNil
				ifTrue: [^nil	"file does not include minor version descriptors"].
			^p - key size - 1]
		ifTrue: 
			[| stream p |
			stream := self versionFileName readStream.
			[stream atEnd & p isNil] whileFalse: 
				[| minor |
				minor := Integer readFrom: stream skipSeparators.
				p := Integer readFrom: stream skipSeparators.
				minor = aVersion minor ifFalse: [p := nil]].
			stream close.
			^p]!

latestMinor
	"return a VersionDescriptor for the latest version string in the source file"

	| p string |
	p := ChangeList new findLast: self class sourceHeader , self name , '(' in: self sourceFile.
	p isNil ifTrue: [^nil].	"file does not include minor version descriptors"

	string := self sourceFile position: p; nextChunk.
	string := string copyFrom: 1 to: string size -1.	"strip off trailing ) "
	^VersionDescriptor readFromString: string!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp newMajor major |
	self version isInitialVersion ifFalse: [self sourceFile]. "believe me, it is necessary"

	(newApp := self cleanUp copy setPath) isNil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) isZero
		ifTrue: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor <= major
		ifTrue: [newApp path asString = self path asString
			ifTrue: [DialogView userError: 'You must specify a new path
if an old major number is used'. ^self]
			ifFalse: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]]
		ifFalse: [major + 1 = newMajor
			ifFalse: [(DialogView confirm: 'The major number increment is greater than 1.
Do you want to proceed?' initialAnswer: false)
				ifTrue: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]
				ifFalse: [^self]]].

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	newApp incrementMajorVersion; storeSource; storePools; noChanges; storeBinary.
	self applicationManagement unregister: self identifier; register: newApp.
	self asBackup.
	^newApp!

newMinorVersion
	"increment the minor version number; 
	save current changes to source file and history, redirect source code pointers;
	broadcast the change of version to dependent browsers"

	| latest |
	self version isInitialVersion
		ifTrue: [DialogView userError: 'Minor versions are not allowed
before the first major version release'. ^nil].

	(latest := self latestMinor) > self version
		ifTrue: [DialogView userError:
'This application has already been released to version
', latest printString, '
Try to upgrade and make your release afterwards'. ^nil].

	latest < self version copy untouch
		ifTrue: [(DialogView confirm:
'This application has not been released since version
', latest printString, '
Do you want to proceed?' initialAnswer: false)
			ifFalse: [^nil]].

	self history addDoIt: '"', self sourceIdentifier, '"'.		"insert a separator"
	self
		incrementMinorVersion;
		fixChanges: true;
		changed: #version!

touch
	"mark that a change occurred"

	label isString ifTrue: [^self].
	label touch.
"	self changed: #windowLabel"	"include this if a browser depends on receiver"!

upgradeMinor
	"file in all the (new) sources in the receiver's source file:
	start with the receiver's following minor version, file in up to end of file"

	| latest |
	((latest := self latestMinor) isNil or: [latest > self version])
		ifFalse: [^self].	"nothing to upgrade"

	self upgradeMinorUpTo: nil; version: latest!

upgradeMinorUpTo: aVersion
	"file in all the (new) sources in the receiver's source file:
	- start with the receiver's following minor version
	- stop at the beginning of aVersion (file in all if argument is nil)"

	| oldChanges newChanges stream chgList |
	oldChanges := self changes.
	self noChanges.
	"use a private copy of the source file accessor"
	(stream := self instSourceFilename readStream)
		position: (self getMinorVersionPosition: self version newMinor).
	self
		fileInFromStream: stream
		startAt: self copy incrementMinorVersion sourceIdentifier storeString
		stopAt: (aVersion notNil ifTrue: [(self copy version: aVersion) sourceIdentifier storeString])
		external: true.

	oldChanges isEmpty ifFalse:
		[oldChanges
			addDoIt: '"the following changes result from upgrade"';
			addAll: (newChanges := self changes)].
	self clearChanges.	"move upgrade changes to history"
	aVersion notNil
		ifTrue: [self version: aVersion].

	"user interaction for unsaved temporary changes"
	oldChanges isEmpty ifTrue: [^self].
	chgList := (EnhancedChangeList onApplication: self)
			listName: 'Changes conflicts in upgrading ' , self identifier storeString 
			changes: oldChanges
			removed: newChanges.

	DialogView userInfo: 
'The temporary changes have been removed from application.
Select "remove selection" for those you want to forget,
select "reinstall" for those which have been overwritten,
select "accept" to assign the contents of the list to the application.'.
 
	ChangeSequenceView openOn: chgList! !

!Application methodsFor: 'filenames'!

changesFilename
	^Filename
		name: self fileString
		extension: 'cs'!

directory
	"return a Filename for the directory for all major version subdirectories
	('<path>/<application name>') "

	^(label path / label filename) asFilename!

fileString
	^self label filename!

instBossFilename
	"return the source file name"

	^Filename
		path: self minorDirectory asString
		name: 'applic'
		extension: 'bos'!

instDocFilename
	"return the documentation file name"

	^Filename
		path: self minorDirectory
		name: 'd'
		extension: 'od'!

instPoolFilename
	"return the pool file name"

	^Filename
		path: self minorDirectory
		name: 'p' , self minorVersion printString
		extension: 'pol'!

instSourceFilename
	"return the source file name"

	^Filename
		path: self minorDirectory asString
		name: 's'
		extension: 'st'!

minorDirectory
	"return a Filename for the directory keeping minor versions 
	('<path>/<application name>/<v_>' , where _ is the major version number)"

	^Filename path: self directory asString name: 'v' , self version major printString!

setNameForDos
	"install a dos-compatible name for the application directory"

	| n |
	n := self name.
	n := self label filenameForPlatform: 'dos'
		ifNone: [DosFilename suggest: (n copyFrom: 1 to: (8 min: n size))].
	n := DialogView
				request: 'Enter dos-compatible name for application'
				initialAnswer: n
				ifEmpty: [^nil].
	n = (DosFilename suggest: n)
		ifFalse: [DialogView userError: n storeString, '
is not compatible with DOS filename conventions.
Try again'. ^nil].

	self label setFilename: n forPlatform: 'dos'!

setPath
	"set the path (directory name) where the receiver installs its root directory; 
	return nil if the user cancels operation or answers an empty string"

	| path |
	(path := label path) isNil
		ifTrue: [path := Filename defaultDirectory]
		ifFalse: [(path exists and: [path isDirectory not]) 
			ifTrue: [path := path head]].
	path := DialogView
				requestFileName: 'enter path for application'
				default: path asString
				version: #mustBeOld
				ifFail: [^nil].
	(path := path asFilename) isDirectory
		ifFalse: [DialogView userError: path asString storeString, '
is not a directory. Try again'. ^nil].

	self setPath: path!

setPath: stringOrFilename
	"set the path (directory name) where the receiver installs its root directory;
	careful: no checks performed, release of source file"

	self releaseSourceFile label path: stringOrFilename!

versionFileName 
	^Filename
		path: self minorDirectory asString
		name: 'versions'! !

!Application methodsFor: 'checking conflicts'!

checkConflicts
	"return a change sequence with removal of classes and selectors in an 
	installed version of the receiver which are not present in the receiver;
	applying this method to a new application will find obsolete classes and methods"

	| app |
	app := self applicationManagement applicationNamed: self name ifAbsent: [^ChangeSequence new].
	^self checkConflictsWith: app!

checkConflictsWith: anApplication 
	"return a change sequence with removal of classes and selectors in anApplication
	not present in the receiver; applying this method to anApplication
	which is older than the receiver you will find obsolete classes and methods"

	| conflicts ownClasses |
	conflicts := ChangeSequence new.
	ownClasses := self classNames.
	(anApplication classNames reject: [:name | ownClasses includes: name])
		do: [:name | 
			conflicts addDoIt: name, ' subclasses isEmpty
	ifFalse: [self notify: ''', name , ' has subclasses'']'.
			conflicts removeClass: name asClass].

	anApplication classNames do: [:name | | classSelectors meta |
		classSelectors := self selectorsInClass: name.
		((anApplication selectorsInClass: name) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: name asClass].
		classSelectors := self selectorsInClass: (meta := name asClass class name).
		((anApplication selectorsInClass: meta) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: meta asClass]].
	^conflicts! !

!Application methodsFor: 'initialize-release'!

asBackup
	"a new version has been created, the receiver is now necessary only for 
	source code references of some history changes; 
	so forget any information but source code accessing"

	self releaseSourceFile.
	classes := poolDefinitions := necessary := documentation := responsible := changes := history := doits := nil!

cleanUp
	"remove empty class and protocol structures"

	self cleanUpClasses.
	self cleanUpProtocols!

cleanUpClasses
	"remove empty class subsets"

	classes copy do: [:c | c isEmpty ifTrue: [self removeClassSubset: c mappedClass]]!

cleanUpProtocols
	"remove empty class protocols"

	| removed |
	removed := false.
	classes keysDo: [:key | removed := removed | (self classSubsetOf: key ifAbsent: []) removeEmptyCategories].
	removed ifTrue: [self cleanUpClasses]!

clearChanges
	"move current changes to history"

	self setHistory: (self history addAll: self changes; yourself).
	self noChanges!

initialize
	classes := Dictionary new.
	label := VersionedObjectAccessor initialVersion.
	self noChanges.
	necessary := OrderedCollection new!

name: aString 
	self label name: aString!

noChanges 
	"forget changes"

	self setChanges: ChangeSequence new!

release
	"move all methods in my source file to system changes and release external 
	references"

	self moveMethodsInSourcefileToChanges.
	self releaseExternalReferences.
	super release!

releaseExternalReferences
	self releaseSourceFile!

releaseSourceFile
	sourceFile notNil
		ifTrue: 
			[sourceFile close.
			sourceFile := nil]!

setChanges: changeSequence 
	"absolute assignment to changes, keeping version consistent"

	changes := changeSequence.
	self version perform: (changes isEmpty
			ifTrue: [#untouch]
			ifFalse: [#touch]).
	self changed: #version!

setFullHistory: aChangeSequence 
	"assume aChangeSequence is a subset of the receiver's full history: 
	separate aChangeSequence, replacing history with all those elements 
	occurring both in history and aChangeSequence (regard others as deleted),
	and replacing the current changes with the rest"

	| hist rest |
	hist := self history asSet.
	hist := (rest := aChangeSequence copy) removeAllSuchThat: [:chg | hist includes: chg].
	rest removeAllSuchThat: [:chg | chg class = OtherChange and: [chg text = '"- end of history -"']].
	self setHistory: hist.
	self setChanges: rest!

setHistory: aChangeSequence 
	history := aChangeSequence isEmpty ifFalse: [aChangeSequence]! !

!Application methodsFor: 'categories'!

excludeCategory: categoryName 
	"exclude all class and meta class info in the specified system category, 
	which the receiver knows about"

	(self classNamesInCategory: categoryName)
		do: 
			[:c | | class |
			self excludeClass: (class := c asClass); excludeClass: class class]!

includeCategory: categoryName!

removeCategory: categoryName 
	"remove all classes in specified category from system (meta and soleInstance) 
	(restricted to the corresponding class subsets of the receiver)"

	| classList |
	classList := (self classNamesInCategory: categoryName)
				collect: [:name | name asClass].
	(ChangeSet superclassOrder: classList)
		reverseDo: [:c | self removeFromSystem: c]!

renameCategory: oldName to: newName 
	self changes add: (CategoryRenameChange new rename: oldName to: newName)! !

!Application methodsFor: 'class subset access'!

addClassSubset: aClass 
	"put a ClassSubset for aClass into my set of classes"

	^classes at: aClass asSymbol ifAbsentPut: [ClassSubset of: aClass asClass]!

addSelector: selector protocol: protocol inSubsetOf: aClass 
	"add selector to class subset of aClass"

	(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass])
		addSelector: selector protocol: protocol!

classSubsetOf: aClassOrSymbol ifAbsent: aBlock 
	"returns the class subset of a class (specified by a class object or its name 
	symbol) registered by the receiver"

	^classes at: aClassOrSymbol asSymbol ifAbsent: aBlock!

removeClassSubset: aClass 
	"remove subset of aClass"

	^classes removeKey: aClass asSymbol ifAbsent: [].!

removeSelector: selector inSubsetOf: aClass 
	"exclude selector from class subset"

	^(self classSubsetOf: aClass ifAbsent: [^nil])
		removeSelector: selector! !

!Application methodsFor: 'support'!

messageToManager: message 
	| subject file |
	(Smalltalk includesKey: #UnixProcess)
		ifFalse: [	^DialogView userInfo: 'OS command interface is not installed!!'].

	responsible isNil
		ifTrue: [^DialogView userWarning: 'It seems no one to be responsible for this application'].

	subject := DialogView
		request: 'Please enter a string describing the mail subject'
		initialAnswer: ''
		ifEmpty: [^DialogView userError: 'Mail without subject description won''t be sent!!'].

	file := ('' / 'tmp' / 'appman.mail') asFilename writeStream.
	file nextPutAll:
'Subject: ' , subject , '
To: ' , responsible , '
' , message.
	file cr close.

	#UnixProcess asClass
		command: '/usr/lib/sendmail'
		arguments: (Array with: responsible with: file name)
		inputBlock: [message]! !

!Application methodsFor: 'retrieving'!

allCallsOn: aLiteral 
	"return a collection of (class -> selector) for all selectors that call on
	aLiteral (restricted to elements registered by the receiver)"

	| lit |
	lit := aLiteral isBehavior
		ifTrue: [Smalltalk associationAt: aLiteral name ifAbsent: [aLiteral]]
		ifFalse: [aLiteral].
	^Cursor execute showWhile: 
		[| calls |
		calls := OrderedCollection new.
		classes keysDo: [:name | (name asClass whichSelectorsReferTo: lit)
			do: [:sel | (self includesSelector: sel class: name)
					ifTrue: [calls add: name -> sel]]].
		calls]!

referencesOn: aCollection
	"return a <Dictionary key: Class value: <Array of: <Symbol1 -> Symbol2>>>
	where each entry means: 
	Class is an element of aCollection, referred to by class named Symbol1 in selector Symbol2;
	only foreign references are interesting, so drop references Class->Class"

	| table |
	table := aCollection collect: [:classOrName | | c x |
		(c := classOrName asClass) isMeta
			ifTrue: [c -> #()]
			ifFalse:
				[x := self allCallsOn: c.
				x := x select: [:assoc | assoc key ~= c name].
				c -> x asSet asArray]].
	^Dictionary withAll: table asSet! !

!Application methodsFor: 'private-storing'!

fixChanges: newVersion 
	"append source code for changes to source file, move changes to history; 
	redirect source code pointers to source file (usually from image changes file); 
	if 'newVersion' is true, an application header and a time stamp will be written, 
	and the start position for the new minor version sources is stored in a special file"

	| stream startPos |
	stream := self storeFile.
	Cursor write showWhile:
		[
		[stream cr; cr.
		newVersion ifTrue: 
			[startPos := stream targetFile writePosition.
			self fileOutSourceIdentifierOn: stream.
			stream timeStamp].
		self changes fileOutExternalOn: stream] valueNowOrOnUnwindDo: [stream close].

		self clearChanges.
		startPos == nil ifFalse: [self storeMinorVersionPosition: startPos].
		].
	self storeBinary!

storeBinary
	"store the application structure and the current state of all sources of the 
	application on a BinaryObjectStorage 
	format: <application structure> <class definitions and sources>"

	self changes isEmpty ifFalse:
		["avoid inconsistency arising from unsaved changes"
		DialogView userError: 'Binary storage must be rejected
if the application has temporary changes'.
		^self].

	Transcript cr; show: 'storing binary'.
	Cursor write showWhile:
		[(BinaryObjectStorage onNew: self instBossFilename writeStream)
			writerClass: ApplicationBOSSWriter;
			nextPut: self releaseSourceFile;
			sourceMode: #reference;
			nextPutClassSubsets: classes values;
			close]!

storeMinorVersionPosition: anInteger 
	"a new minor version has been released: assume anInteger to be the start 
	position of the minor version sources in the source file; write the current minor 
	version number followed by anInteger to the version map file"

	| stream |
	stream := self versionFileName appendStream.
	[stream nextPutAll: self minorVersion printString; tab; nextPutAll: anInteger printString; cr]
		valueNowOrOnUnwindDo: [stream close]!

storePools
	"file out the pool contents"

	| store |
	self hasPoolDefinitions ifFalse: [^self].
	
	[store := BinaryObjectStorage onNew: self instPoolFilename writeStream.
	self fileOutPoolsOn: store]
		valueNowOrOnUnwindDo: [store close]!

storeSource
	"file out the current state of all sources of the application,
	move source code references to the output file;
	format: <application identifier> <time stamp> <application comment> 
		<pre installation> <required applications> <source code> <post installation>"

	| stream |
	Cursor write showWhile:
	[
		[self
			fileOutSourceIdentifierOn: (stream := self storeFile);
			fileOutDocumentationOn: stream timeStamp;
			fileOutPreInstallationOn: stream;
			fileOutRequiredOn: stream;
			fileOutSourcesOn: stream;
			fileOutPostInstallationOn: stream
		]
			valueNowOrOnUnwindDo: [stream close. self releaseSourceFile]
	]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!Application class methodsFor: 'constants'!

sourceHeader
	"keyword to identify special expressions in application source files"

	^' .ApplicationSource '! !

!Application class methodsFor: 'instance creation'!

fromFileNamed: aString 
	| obj | self shouldNotImplement.
	Cursor read showWhile: [obj := #BinaryStorage fromFileNamed: aString].
	^obj!

name: applicationName 
	^self new name: applicationName!

new
	^super new initialize! !



Application subclass: #SystemApplication
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Framework-Application'!


!SystemApplication methodsFor: 'classes'!

changeOrIncludeClass: aClass 
	"include only if not in another application"

	(self applicationManagement applicationsIncludingClass: aClass) isEmpty
		ifTrue: 
			[Transcript show: aClass printString , ' included in System Application'; cr.
			self includeClass: aClass]!

excludeClass: aClass 
	"forget about aClass (inclusive definition) and any changes on it;
	return the changes which are dropped"

	(self removeClassSubset: aClass) isNil ifTrue: [^nil].
	^self changes removeClassChanges: aClass!

excludeClassDefinition: aClass 
	"stop holding the class' definition, discard it from structure and remove 
	corresponding changes; 
	return all the changes on the class' definition"

	(self classSubsetOf: aClass ifAbsent: [^nil])
		removeDefinition ifFalse: [^nil].
	self cleanUpClasses.
	^self changes removePureClassChanges: aClass!

includeClass: aClass 
	"aClass is included (structure and definition)"

	(self definesClass: aClass)
		ifFalse: 
			[(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass]) addDefinition].
	^aClass! !

!SystemApplication methodsFor: 'accessing'!

label
	"My label is a string, so generate a temporary VersionedObjectAccessor"

	^VersionedObjectAccessor initialVersion name: label!

name: aString
	label := aString!

version 
	^VersionDescriptor initialVersion! !

!SystemApplication methodsFor: 'changes'!

addChangedObjectsFor: changeSequence 
	"add those objects underlying the elements in changeSequence to the 
	application structure of the receiver; don't extend the change sequence 
	(backup or include makes no sense here)"

	changeSequence do: [:chg | | c | 
		(self includesSelector: chg selector class: (c := chg classObject))
			ifFalse: [self
					addSelector: chg selector
					protocol: chg category
					class: c]]!

addChanges: changeSequence 
	"add changeSequence to my changes"

	self changes addAll: changeSequence!

addSystemChanges
	"construct representation for all changes in the system, which are not managed 
	by an application"

	| seq |
	seq := self systemChangeSequence.
	self addChangedObjectsFor: seq; addChanges: seq!

systemChangeSequence
	"returns a ChangeSequence with all changes in the system change set, which 
	are not managed by an application"

	| man seq |
	man := self applicationManagement.
	seq := ChangeSet current asChangeSequence.

	"drop class initialize's and doit's"
	seq := seq select: [:chg | chg class ~~ OtherChange
			and: [chg class ~~ ClassOtherChange or: [chg type ~~ #initialize]]].

	"drop changes which have externally stored source"
	seq := seq select: [:chg | chg file class ~~ SourceReference].

	"drop class changes managed by some application"
	seq := seq select: [:chg | chg class ~~ ClassDefinitionChange 
			or: [(man applicationsIncludingDefinition: chg className) isEmpty]].

	"drop selector changes managed by some application"
	seq := seq select: [:chg | chg class ~~ MethodDefinitionChange 
			or: [(man applicationsIncludingSelector: chg selector class: chg className) isEmpty]].

	^seq! !

!SystemApplication methodsFor: 'selectors'!

changeOrIncludeSelector: selector class: aClass 
	"include only if not in another application"

	| app |
	(app := self applicationManagement applicationsIncludingSelector: selector class: aClass) isEmpty
		ifTrue: 
			[Transcript cr; show: aClass printString , '>' , selector printString , ' included in System Application'.
			self includeSelector: selector class: aClass]
		ifFalse: [(app includes: self)
			ifFalse: [^self	"receiver is not responsible for the selector"]].
	self changes changeSelector: selector class: aClass!

excludeSelector: selector class: aClass 
	"forget about the specified selector and any changes on it;
	return the changes which are dropped"

	(self removeSelector: selector inSubsetOf: aClass) isNil
		ifTrue: [^nil].
	^self changes removeSelectorChanges: selector class: aClass!

includeSelector: selector protocol: protocol class: aClass 
	"register the class/selector combination in the application structure; 
	create a backup version in the change sequence"

	(self includesSelector: selector class: aClass)
		ifTrue: [^self].
	self
		addSelector: selector
		protocol: protocol
		inSubsetOf: aClass.
	self changes
		backupSelector: selector
		protocol: protocol
		class: aClass! !

!SystemApplication methodsFor: 'version management'!

latestMinor
	^nil! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!SystemApplication class methodsFor: 'instance creation'!

new
	"note that a new SystemApplication will not automatically 
	be registered"

	self instanceCount > 1 ifTrue: [^self error: 'there is at least 1 instance of me in this image'].
	^super new name: 'SystemChanges'! !



!BinaryObjectStorage methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection
	self writer nextPutClassSubsets: subsetCollection withSource: sourceMode! !



Object subclass: #ApplicationManagement
	instanceVariableNames: 'applications system logging logTo directories '
	classVariableNames: 'ChangeSignal SourceSignal '
	poolDictionaries: ''
	category: 'Framework-Application'!
ApplicationManagement comment:
'This class represents the backbone of the product Application Management. 
The main purpose of the product is organizing applications (sets of class, method
and organizational definitions) in ObjectWorks Smalltalk R4.1.
The main task here is to provide a holder for application structures and forward
all changes passed from the system change mechanism to applications concerned.
See also comment in application ''APL-SystemLink'' to get an idea of how 
Application Management works.

Instance Variables:
	applications	<OrderedCollection of: Application>  registered application structures
	system	<SystemApplication>  the default application, registers changes not automatically mappable to an other application
	logging	<Boolean>  comment
	logTo	<Application | nil>  application recording any change until reset to nil
	directories	<OrderedCollection of: Filename>  search path for retrieving external applications

Class Variables:
	ChangeSignal	<Signal>  raised to get the currently recording application
	SourceSignal	<Signal>  raised to get a source reference for a text'!


!ApplicationManagement methodsFor: 'change management'!

addClass: class 
	"current application has to add class"

	self application addClass: class!

addDoIt: smalltalkExpression 
	"current application has to add a do it"

	self application addDoIt: smalltalkExpression!

addSelector: selector class: aClass 
	"current application has to add a selector"

	self application addSelector: selector class: aClass!

changeClass: aClass 
	"aClass' definition has changed; current application has to include old 
	definition and log change"

	| app |
	logging ifFalse: [^self].
	(app := self application) changeOrIncludeClass: aClass.
	self do: [:ap | ap == app ifFalse: [ap changeClass: aClass]]!

changeSelector: selector class: class 
	"method has changed, inform all applications"

	| app |
	logging ifFalse: [^self].
	(app := self application) changeOrIncludeSelector: selector class: class.
	self do: [:ap | ap == app ifFalse: [ap changeSelector: selector class: class]]!

classifyClass: className under: category 
	"inform all applications to classify class under category"

	logging ifFalse: [^self].
	self do: [:app | 
		app classifyClass: className under: category asSymbol.
		(app includesClass: (className , ' class') asSymbol)
			ifTrue: [app classifyClass: className , ' class' under: category asSymbol]]!

classifySelector: selector under: protocol class: aClass 
	"inform all applications that selector is classified under protocol"

	logging ifFalse: [^self].
	self do: [:app | app classifySelector: selector under: protocol class: aClass]!

commentClass: class 
	"class' comment has changed. Inform applications"

	logging ifFalse: [^self].
	self do: [:app | app commentClass: class]!

excludeCategory: cat 
	"current application has to exclude category"

	self application excludeCategory: cat!

includeCategory: heading 
	"current application has to include category"

	self application includeCategory: heading!

includeProtocol: heading class: class 
	"current application has to include protocol"

	self application includeProtocol: heading class: class!

performMessage: msg 
	"evaluate the Message msg, found during file-in"

	self application perform: msg selector withArguments: msg arguments!

removeCategory: category 
	"exclude category from all applications"

	logging ifFalse: [^self].
	self do: [:app | app excludeCategory: category]!

removeClass: aClass 
	"change management informs that aClass is removed from system."

	logging ifFalse: [^self].
	applications do: [:app | app removeClass: aClass]!

removeProtocol: protocol class: class 
	"discard from all applications"

	logging ifFalse: [^self].
	self do: [:app | app removeProtocol: protocol class: class]!

removeSelector: selector class: class 
	"inform all applications that selector is removed"

	logging ifFalse: [^self].
	self do: [:app | app removeSelector: selector class: class]!

renameCategory: oldName to: newName 
	"inform applications that category is renamed"

	logging ifFalse: [^self].
	self do: [:app | app renameCategory: oldName to: newName]!

renameClass: class as: newName 
	"inform applications that class is renamed"

	logging ifFalse: [^self].
	self do: [:app | app renameClass: class as: newName]!

renameProtocol: oldName to: newName class: class 
	"inform applications that protocol is renamed"

	logging ifFalse: [^self].
	self do: [:app | app renameProtocol: oldName to: newName class: class]!

updateChanges
	"a system change propagated by 'ChangeSet current' will be recorded by 
	concerned applications by creating an appropriate change object and storing 
	this object in a change sequence; often after the change a new source pointer 
	is installed which must be propagated to those change objects:"

	logging ifFalse: [^self].
	applications do: [:app | app changes updateChanges]!

updateCondensedChanges
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	applications do: [:app |
		app changes do: [:chg |
			self debug. (chg class == MethodDefinitionChange and: [chg sourceFileNumber = 2])
				ifTrue: [chg getSource]]]! !

!ApplicationManagement methodsFor: 'application access'!

applicationIdentified: aString 
	"return the application with identifier aString"

	^self applicationIdentified: aString ifAbsent: [nil]!

applicationIdentified: aString ifAbsent: aBlock 
	"return the application with identifier aString 
	(remark: identifiers include the version)"

	^applications detect: [:app | app identifier = aString]
		ifNone: aBlock!

applicationNamed: aString 
	^self applicationNamed: aString ifAbsent: [nil]!

applicationNamed: aString ifAbsent: aBlock
	^applications detect: [:app | aString = app name]
		ifNone: aBlock!

sortedApplicationIdentifiers
	^(applications collect: [:app | app identifier]) asSortedCollection asOrderedCollection! !

!ApplicationManagement methodsFor: 'application creation'!

newApplication
	"prompt for an application name until empty or not yet registered"
	
	[| newName |
	newName := DialogView request: 'enter new application name'
				initialAnswer: ''
				ifEmpty: [^nil].
	(self includesNamed: newName)
		ifTrue: [DialogView warn: 'Application ' , newName , ' already exists']
		ifFalse: [^self newApplicationNamed: newName]] repeat!

newApplicationNamed: applicationName 
	"create a new application with name applicationName, register it"

	| newApp |
	newApp := Application name: applicationName.
	self register: newApp ifRegistered: [newApp := (DialogView confirm: 'Application ' , newApp name , ' is already registered. Add file?')
			ifTrue: [self applicationNamed: applicationName]
			ifFalse: [nil]].
	^newApp! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInAsApplication: filename 
	"file in source code;
	the first chunk must be an application source header holding the application name;
 	redirect the source code pointers to the source file"

	| app old newName stream header conflicts |

	stream := filename asFilename readStream.
	header := stream nextChunk.
	stream close. 
	((Application sourceHeader , '*') storeString match: header)
		ifFalse: [^self error: filename , '
is not an application source file'].
	header := header copyFrom: Application sourceHeader size + 2 to: header size.
	newName := header copyUpTo: $(.	"strip off version string"
	newName isEmpty ifTrue: [^nil].

	(old := self applicationNamed: newName) notNil
		ifTrue: [(DialogView confirm: 'Do you want to overwrite
', old identifier storeString, ' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['
(changes will get lost)']) initialAnswer: false)
			ifTrue:
				[self unregister: old name]
			ifFalse:
				[Transcript show: 'aborted '. ^nil]].

	(app := self newApplicationNamed: newName) isNil
		ifTrue: [^nil].

		"an application source path includes a basic path,
		followed by a major and a minor directory and the source file name;
		strip off the last three ones"
	app label path: filename asFilename head asFilename head asFilename head.
	app version: (VersionDescriptor readFromString: (header copyFrom: (header indexOf: $( ) +1 to: header size)).

	self fileInFrom: stream to: app external: true.
	app setChanges: (app changes select: [:chg | chg class == MethodDefinitionBackup]).
	app clearChanges version: app latestMinor.
	self changed: #version.

	old isNil ifTrue: [^app].
	conflicts := app checkConflictsWith: old.
	(conflicts isNil or: [conflicts isEmpty]) ifTrue: [^app].
	DialogView userInfo: 'Installation is done.

A change browser will open now. Selecting ''replay all''
will remove all classes and methods registered in the
previously installed version but not in the new one.
Be careful!!'.

	conflicts browse.
	^app!

fileInFrom: aStream 
	"file in source code from 'aStream', ask for an application to log all 
	the changes; return the result of the file-in"

	| app external name |
	name := (external := aStream isKindOf: ExternalReadStream)
		ifTrue: [aStream name]
		ifFalse: ['an internal stream'].
	
	(app := self selectApplicationOrNew: 'Select an application to log the file in from
', name) isNil
		ifTrue: ["empty user selection means cancel; simulate scanning complete"
			aStream setToEnd. ^nil].

	external := external & (app ~~ system) & app path isNil
		and: [DialogView confirm: 'Do you want the source references to be kept external?'].

	^self fileInFrom: aStream to: app external: external!

fileInFrom: aStream to: anApplication external: aBoolean 
	"file in source code; 
	if 'aBoolean' is true, all changes will directly be assigned to 'anApplication'
	and the source code pointers will be directed to the source file just read;
	otherwise changes will pass through system change set and reach 'anApplication' "

	| result |
	anApplication == nil ifTrue: [^nil].
	result := anApplication fileInFromStream: aStream startAt: nil stopAt: nil external: aBoolean.
	anApplication cleanUp.
	self changed: #version.
	^result!

fileOutBootScript
	"file out all the class definitions which are part of application management applications"
	"self applicationManagement fileOutBootScript"

	| stream classnames |
	stream := (self applicationNamed: 'APL-Framework') path / 'appman.st0'.
	stream := stream asFilename writeStream.
	stream
		timeStamp;
		nextPutAll: '"The following definitions allow import of application structures
defined by Application Management from a BinaryObjectStorage

Copyright (C) June 1993 by Georg Heeg, Dortmund
"'.
	classnames := #(#Application #StorableObjectAccessor #AccessControlledObjectAccessor #VersionedObjectAccessor #AccessControlList #VersionDescriptor #Categorizer #ClassSubset #ChangeSequence #SourceReference #ClassApplicationChange #MethodApplicationChange).

	classnames do: [:classname | stream cr; cr; nextChunkPut: classname asClass definition].

	(SourceCodeStream on: stream)
		fileOutMessages: #(#classRef: #definition:) for: ClassSubset logging: false;
		fileOutMessages: #(#of:definition:selectors:) for: ClassSubset class logging: false;
		fileOutMessages: #(#setComment:) for: ClassDescription logging: false;
		fileOutMessages: #(#setComment:) for: ClassOrganizer logging: false.

	stream 
		cr cr cr nextPutAll: '"now reset the changes for the classes just filed in"';
		cr; nextPutAll: classnames printString;
		crtab: 2; 
		nextChunkPut: 'do: [:classname | ChangeSet current removeClassChanges: (Smalltalk at: classname)]';
		close!

load: application path: path version: version 
	"load the application (its structure together with class and method definitions)
	whose sources reside in the directory <path>/<application>/<version>"

	| dir bos app old conflicts |
	dir := (path / application / version) asFilename.
	((bos := (dir / 'applic.bos') asFilename) exists
		and: [DialogView confirm: 'Do you want to load in binary mode?'])
		ifFalse: [^self fileInAsApplication: (dir / 's.st') asFilename].

	bos := BinaryObjectStorage onOld: bos readStream.
	Transcript cr; show: 'loading ' , application asString, ' (structure)... '.
	Cursor read showWhile: [app := bos next. bos close].

	(old := self applicationNamed: app name) notNil
		ifTrue: [(DialogView confirm: 'Do you want to overwrite
', old identifier storeString, ' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['
(changes will get lost)']) initialAnswer: false)
			ifTrue:
				[conflicts := app checkConflicts.
				self unregister: app name]
			ifFalse:
				[Transcript show: 'aborted '. ^nil]].

	Transcript show: '(checks)... '.
	self checkRequired: app necessary.

	Transcript show: '(pre-installation)... '.
	app preInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	Transcript show: '(source)... '.
	self logTo: app while: [Cursor read showWhile: [bos nextClasses; close]].
	Transcript show: '(post-installation)... '.
	app postInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	app setChanges: (app changes select: [:chg | chg class == MethodDefinitionBackup]).

	"set the correct path"
	app label path: path.
	app definePoolDictionaries.
	self register: app.
	Transcript show: 'done '.

	(conflicts isNil or: [conflicts isEmpty]) ifTrue: [^app].
	DialogView userInfo: 'Installation is done.

A change browser will open now. Selecting ''replay all''
will remove all classes and methods registered in the
previously installed version but not in the new one.
Be careful!!'.

	conflicts browse.
	^app!

upgradeMinor: anApplication 
	"upgrade anApplication to a higher minor version"

	| latest version |
	((latest := anApplication latestMinor) isNil or: [latest <= anApplication version])
		ifTrue: [^DialogView userInfo: 'No upgrade available'].
	version := DialogView
				request: 'Upgrade up to minor version'
				initialAnswer: latest minor printString
				ifEmpty: [^self].
	(version := Integer readFromString: version) <= anApplication minorVersion
		ifTrue: [^self].
	version := VersionDescriptor major: latest major minor: version.
	version < latest ifTrue: [latest := version] ifFalse: [version := nil].
	anApplication upgradeMinorUpTo: version; version: latest.
	self changed: #version! !

!ApplicationManagement methodsFor: 'initialize-release'!

initialize
	"the receiver will not yet log after this initialization"

	system := SystemApplication new.
	applications := OrderedCollection with: system.
	directories := OrderedCollection new.
	logging := false! !

!ApplicationManagement methodsFor: 'inquiry'!

applicationDirectoriesIn: path 
	"search the directory 'path' for installed application sources; 
	return a collection of triples like (<path>, <app name>, <major version>)"

	| found |
	found := OrderedCollection new.
	path isDirectory ifFalse: [^found].
	path directoryContents do: 
		[:appdir | 
		"search <path> for subdirectories containing applications; 
		an application's directory has the name of the application"
		| s |
		(s := (path / appdir) asFilename) isDirectory
			ifTrue: 
				["search <path>/<appdir> for non-empty major version subdirectories"
				| majors |
				majors := s directoryContents select: [:maj | ('v#*' match: maj)
							and: [| dir | (dir := (path / appdir / maj) asFilename) isDirectory
							and: [(dir directoryContents
								detect: [:fname | 's.st' match: fname] ifNone: []) notNil]]].
				found addAll: (majors collect: [:maj | Array with: path with: appdir with: maj])]].
	^found!

applicationsCallingSelector: aSymbol class: aClass without: anApplication 
	"return a dictionary of (application -> calls) for applications calling class>selector
	(class refs if selector is nil)"

	^Cursor execute showWhile: 
		[| table key | 
		table := Dictionary new.
		key := aSymbol isNil ifTrue: [Smalltalk associationAt: aClass name] ifFalse: [aSymbol].
		self do: [:app | | calls |
			(app ~~ anApplication and: [(calls := app allCallsOn: key) isEmpty not])
				ifTrue: [table at: app put: calls]].
		table]!

applicationsIncludingClass: className 
	^self select: [:app | app includesClass: className]!

applicationsIncludingDefinition: className 
	^self select: [:app | app definesClass: className]!

applicationsIncludingSelector: aSymbol class: class
	^aSymbol isNil
		ifTrue: [self applicationsIncludingClass: class asSymbol]
		ifFalse: [self select: [:app | app includesSelector: aSymbol class: class]]!

availableApplications
	"search the paths given in 'directories' for installed application sources; 
	return a collection of triples like (<path>, <app name>, <major version>)"

	| found |
	found := OrderedCollection new.
	Cursor read showWhile:
		[self directories do: [:d | found addAll: (self applicationDirectoriesIn: d asFilename)]].
	^found!

changedApplications
	"collect applications that have unsaved changes"

	| changed |
	(changed := self select: [:app | app isModified]) isEmpty
		ifTrue: [^changed].
	changed remove: system ifAbsent: [].
	^changed!

upgradeApplications
	"return all applications with new sources which have not been loaded yet"

	^Cursor read showWhile:
		[self select: [:app | app hasUpgrade]]! !

!ApplicationManagement methodsFor: 'accessing'!

directories
	^directories!

systemChanges
	^system! !

!ApplicationManagement methodsFor: 'testing'!

checkRequired: anArray
	"check if the applications specified by the elements of an Array are installed;
	add each element to the set of required applications of the currently logging application"

	| missing s |
	missing := OrderedCollection new.
	anArray do: [:name | | app |
		app := self applicationNamed: name ifAbsent: [missing add: name. nil].
		app notNil
			ifTrue: [self application includeApplication: app]].

	missing isEmpty ifTrue: [^self].
	(s := TextStream on: (String new: 1000))
		cr; nextPutAll: 'Some applications supporting ', self application identifier storeString;
		cr; nextPutAll: 'are missing: '.
	missing do: [:name | s crtab; nextPutAll: name].
	s cr; cr.
	(DialogView confirm: s contents, 'Do you want to proceed?' initialAnswer: false)
		ifFalse: [self halt].

	s
		nextPutAll: 'Try to load them with';
		crtab; emphasis: #italic; nextPutAll: 'load...';
		cr; emphasis: nil; nextPutAll: 'from Application Manager browser'.
	s := s contents.
	ComposedTextView open: (ValueHolder with: s)
			label: 'Missing applications'
			icon: (Icon constantNamed: #workspace)
			extent: s asComposedText extent + 30!

includesNamed: anApplicationName
	"return the first application with name"

	^(applications detect: [:app | anApplicationName = app name]
		ifNone: [nil]) notNil! !

!ApplicationManagement methodsFor: 'logging on/off/redirect'!

application
	"the application to log to. If logging is disabled return a DummyObject. If 
	logTo is set to an Application logging is redirected to that application. Else 
	capture application from signal handle block, that is the application 
	adapted to the current controller. If controller has no handle block, log to 
	system application."

	^logging 
		ifTrue: [logTo isNil
				ifTrue: [self currentApplication]
				ifFalse: [logTo]]
		ifFalse: [DummyObject new]!

currentApplication
	"raise changeSignal to obtain the application associated with the current 
	application controller, if fail use system application"

	^Signal noHandlerSignal 
		handle: [:ex | ex returnWith: system]
		do: [self class changeSignal raiseRequest]!

logging: aBoolean
	"false stops any effects of application management, true resumes work"
	"self applicationManagement logging: true"

	logging := aBoolean!

logTo: anApplication while: aBlock 
	"execute aBlock (typically a file in), log changes into anApplication;
	logging to system application is equivalent to 'logTo==nil' "

	| save logApp |
	logApp := anApplication == system ifFalse: [anApplication].
	(save := logTo) == anApplication	"nothing changed"
		ifTrue: [^aBlock value].

	(anApplication isKindOf: Application)
		ifFalse: [^self error: 'bad application for logging'].

	Transcript show: 'Logging redirected to ' , anApplication identifier; cr	.
	logTo := logApp.
	^aBlock valueNowOrOnUnwindDo: [self resetLoggingTo: save]!

resetLogging
	"log changes into saved system changes application again"

	logTo := nil.
	Transcript show: 'Logging back to ' , system identifier; cr!

resetLoggingTo: anApplication 
	"log changes into anApplication again"

	logTo := anApplication.
	Transcript show: 'Logging back to ' , (anApplication isNil
				ifTrue: [system]
				ifFalse: [anApplication]) identifier; cr! !

!ApplicationManagement methodsFor: 'registration'!

register: anApplication 
	^self register: anApplication ifRegistered: []!

register: anApplication ifRegistered: aBlock 
	"enter anApplication in the application holder; 
	ensure unique application names, evaluate aBlock on violation"

	(self includesNamed: anApplication name)
		ifTrue: [^aBlock value].
	applications add: anApplication.
	self changed: #application!

unregister: name 
	| app |
	app := self applicationIdentified: name ifAbsent: [^nil]. 
	applications remove: app! !

!ApplicationManagement methodsFor: 'enumerating'!

do: aBlock
	applications do: aBlock!

select: aBlock 
	| app |
	^(app := applications select: aBlock) isEmpty
		ifTrue: [(aBlock value: self systemChanges)
				ifTrue: [Array with: self systemChanges]
				ifFalse: [Array new]]
		ifFalse: [app]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!



!ApplicationManagement class methodsFor: 'instance creation'!

new
	self error: 'Create a new ApplicationManager by
"ApplicationManagement install"'! !

!ApplicationManagement class methodsFor: 'class initialization'!

initialize
	self initSignals! !

!ApplicationManagement class methodsFor: 'installation'!

install
	"ApplicationManagement install"

	(Smalltalk includesKey: #ApplicationManager)
		ifTrue: [(DialogView confirm: 'An ApplicationManager already exists. Overwrite ?')
				ifFalse: [^self]].
	Smalltalk at: #ApplicationManager put: (super new initialize logging: true)! !

!ApplicationManagement class methodsFor: 'signal constants'!

changeSignal
	^ChangeSignal!

initSignals

	ChangeSignal := (Signal genericSignal newSignalMayProceed: true)
				notifierString: 'method or class changed';
				nameClass: self message: #changeSignal.

	SourceSignal := (Signal genericSignal newSignalMayProceed: true)
				notifierString: 'source reference changed';
				nameClass: self message: #sourceSignal.!

sourceSignal
	^SourceSignal! !



Change subclass: #CategoryRenameChange
	instanceVariableNames: 'oldname newname '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!


!CategoryRenameChange methodsFor: 'accessing'!

name
	| txt |
	^(txt := self text) copyFrom: 24 to: txt size!

rename: oldName to: newName 
	"arguments should be Symbol or String"

	oldname := oldName.
	newname := newName!

text
	^('Smalltalk organization renameCategory: ' , oldname storeString , ' to: ' , newname) storeString! !



MethodOtherChange subclass: #MethodApplicationChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
MethodApplicationChange comment:
'Changes for methods regarding application management. Additional types are #include, #exclude'!


!MethodApplicationChange methodsFor: 'accessing'!

text
	"avoid doits which cannot be evaluated for users without application 
	management; return a message which can be sent to application 
	management"

	^(Message
		selector: (type , 'Selector:class:') asSymbol
		arguments: (Array with: selector with: className))
		storeString! !



!ChangeSequence methodsFor: 'change management'!

backupClass: aClass 
	"create (and return) a change object which saves the source code info for the 
	specified class;
	after compilation of a class a ClassBuilder informs the change management, 
	inducing a call to this method (via ChangeSet, App...Management, Application);
	to get source before compilation we 'ask someone in the stack'
	(the appropriate handler is expected in ClassBuilder>modifyExistingClass)"

	| src |
	src := Signal noHandlerSignal
		handle: [:ex | ex return]	"no class definition text available"
		do: ["try to get a class definition text"
			self applicationManagement class sourceSignal raiseRequest].
	src isString
		ifFalse: ["self error: 'invalid source reference'." ^nil].
	^src = aClass definition
		ifFalse: [self add: (ClassDefinitionBackup new className: aClass name; text: src)]!

excludeClass: className 
	self add: (ClassApplicationChange new className: className; type: #excludeClassDefinition:)!

excludeSelector: selector class: classname 
	self add: (MethodApplicationChange new type: #exclude; selector: selector; className: classname)!

includeClass: className 
	self add: (ClassApplicationChange new className: className; type: #includeClass:; text: className asClass definition)!

includeSelector: selector protocol: protocol class: className 
	"notify the inclusion (registration) of the specified method in an application; 
	return the new element"

	^self add: (MethodDefinitionInclude new
		className: className;
		selector: selector;
		category: protocol;
		getSource)! !



MethodDefinitionChange subclass: #MethodDefinitionInclude
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
MethodDefinitionInclude comment:
'MethodDefinitionIncludes notify that the definition of the underlying method has been included (e.g. registered) in an application of Application Management. Thus we want to differentiate between added or changed definitions (modelled by MethodDefinitionChange) and the registration (entry in an application''s change sequence) of a definition which already exists.
The file out behavior is inherited. Filing in a filed out MethodDefinitionInclude induces a simple MethodDefinitonChange. Maybe that this is an unpleasant side effect in an environment where the definition does not actually change but only enters an application, but it is safe for those environments where the underlying method does not exist at all.'!


!MethodDefinitionInclude methodsFor: 'accessing'!

type
	^#include! !



!PeekableStream methodsFor: 'fileIn'!

fileInFrom: startVersion to: stopVersion sources: fileHolder
	"file in sources, start reading after encountering startVersion (nil for current position),
	return result of compilation; (for more comment see #fileInUpTo:sources:)" 

	^Cursor read showWhile: [
		[ | chunk |
		startVersion == nil
			ifFalse: [[self atEnd or: [startVersion = (chunk := self nextChunk)]] whileFalse].
		self fileInUpTo: stopVersion sources: fileHolder]
			valueNowOrOnUnwindDo: [self close]]!

fileInUpTo: stopVersion sources: fileHolder
	"(rewritten from self>fileIn, enhanced by checking occurences of special do its) 
	file in sources, stop encountering stopVersion (end of file, if not found);
	if fileHolder isn't nil, compile without logging to changes file and
	construct non-default source code pointers for the generated methods
	(use fileHolder, usually an Application, for access to the source file)" 

	| manager signal evaluator chunk val |
	signal := (manager := self applicationManagement) class sourceSignal.
	evaluator := Object evaluatorClass.
	[self atEnd]
		whileFalse: 
			[ | methods pos |
			methods := self skipSeparators peekFor: $!!.
			pos := self position.
			val := signal
				handle: [:ex | "provide a source reference for some do its (e.g. class comment ...)"
					fileHolder isNil
						ifTrue: [ex reject]
						ifFalse: [ex proceedWith: (SourceReference for: fileHolder position: pos)]]
				do: [evaluator evaluate: (chunk := self nextChunk) logged: false].
			methods
				ifTrue: [val := val scanFrom: self sourceHolder: fileHolder]
				ifFalse:
					[stopVersion = chunk ifTrue: [^self].
					"check for application management commands"
					val class == Message ifTrue: [manager performMessage: val]]
			].
	^val! !



!BOSSWriter methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection withSource: sourceMode 
	"adaption of #nextPutClasses:withSource: to the needs of application management: 
	write current state of class definition and compiled code (usually sourceMode 
	is #reference)"

	| classes ordered inits table |
	useSource := sourceMode.
	table := Dictionary withAll: (subsetCollection collect: [:subset | subset mappedClass -> subset]).
	"don't forget meta subsets if no soleInstance subset is present"
	classes := table keys collect: [:c | c isMeta ifTrue: [c soleInstance] ifFalse: [c]].
	ordered := OrderedCollection new.
	classes copy do: [:c | self enumerateClass: c from: classes to: ordered].
	ordered := (ordered select: [:c | table includesKey: c]) asArray.
	inits := OrderedCollection new.

	self nextPut: ordered do: 
		[ordered do: 
			[:c | | subset cmt selectors |
			currentClass := c.
			subset := table at: c ifAbsent: [ClassSubset new].
			subset includesDefinition
				ifTrue: 
					[self trace: (BOSSContents new object: currentClass).
					cmt := c comment]
				ifFalse: 
					[self trace: currentClass. cmt := ''].
			(cmt isEmpty or: [useSource == #discard])
				ifFalse: 
					[cmt := c organization globalComment class == SourceReference
						ifTrue: [Array with: currentClass with: #setComment: with: c organization globalComment]
						ifFalse: [Array with: currentClass with: #comment: with: cmt].
					self trace: cmt body: cmt class: BinaryObjectStorage indexImportSend].
			selectors := subset selectors.
			c organization elements do: [:s | (selectors includes: s)
				ifTrue: [self trace: (currentClass compiledMethodAt: s)]].
			(currentClass isMeta and: [subset includesSelector: #initialize])
				ifTrue: [inits add: currentClass]].

		inits do: [:c | | init |
			init := MessageSend receiver: c soleInstance selector: #initialize.
			init := Array with: #deferSend: with: init.
			self trace: init body: init class: BinaryObjectStorage indexImportSendSelf]]! !

!BOSSWriter methodsFor: 'private-tracing'!

trace: anObject body: body class: aClass
	" Record anObject in the objects dictionary,
	and trace its dependents and body if needed.
	The caller has already traced the class. "

	| class objx flags deps |
	deps := anObject binaryStoredDependents.
	class := body class.
	(class isBits and: [deps == nil])
		ifTrue:
			[objx := self assign: anObject.
			self record: anObject body: body class: aClass flags: 0 index: objx.
			^objx].
	" Recursively trace the contents of the body.
	If the object has circular references,
	the trace may assign the object index. "
	anObject == nil ifFalse: [objects at: anObject put: -2].
	flags := BinaryObjectStorage flagHasShortOops.
	deps == nil ifFalse:
		[self
			trace: (anObject -> deps)
			body: (Array with: anObject with: #myDependents: with: deps)
			class: BinaryObjectStorage indexImportSend].
	class isBits
		ifFalse:
			[1 to: class instSize do: [:i |
				((self trace: (body instVarAt: i)) between: 0 and: 16rffff)
					ifFalse: [flags := 0]].
			class isVariable ifTrue:
				[1 to: body basicSize do: [:i |
					((self trace: (body basicAt: i)) between: 0 and: 16rffff)
						ifFalse: [flags := 0]]]].
	objx := self encode: anObject.
	objx < 0
		ifTrue: [objx := self assign: anObject]
		ifFalse: [flags := flags + BinaryObjectStorage flagHasForwardReferences].
	self record: anObject body: body class: aClass flags: flags index: objx.
	^objx!

traceDefault: anObject 
	"recursively trace anObject, return its encoding
	(code is the piece from #trace: executed for objects with nil representation)"

	self trace: anObject class.
	^self
		trace: anObject
		body: anObject
		class: anObject class! !



BOSSWriter subclass: #ApplicationBOSSWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Binary Storage'!


!ApplicationBOSSWriter methodsFor: 'private-initialize-release'!

storage: aBOS on: aStream registry: reg 
	"force SourceReferences to represent in the default way"

	super
		storage: aBOS
		on: aStream
		registry: reg.
	self traceSelectorFor: SourceReference is: #traceDefault:! !

!ApplicationBOSSWriter methodsFor: 'tracing-policy'!

traceSelectorFor: aClass is: aSymbol 
	"enters aSymbol as selector for exporting instances of aClass; 
	aSymbol must be a method selector sent to the receiver; 
	Valid arguments are #represent:, #traceDefault:, #traceSkip:, ..."

	^traceClasses at: aClass put: aSymbol! !



ClassOtherChange subclass: #ClassApplicationChange
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Changes'!
ClassApplicationChange comment:
'ClassApplicationChange represents a change to a class regarding application management.
Additional types are #include, #exclude, #includeDefinition and #excludeDefinition.'!


!ClassApplicationChange methodsFor: 'accessing'!

text
	"avoid doits which cannot be evaluated for users without application 
	management; return a message which can be sent to application 
	management"

	^(Message selector: type arguments: (Array with: className)) storeString! !

ApplicationManagement initialize!




' .ApplicationSource APL-Framework(1.1)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 31 August 1993 at 7:35:40 pm'!



"fixes for:
	1) invalid source pointers for include changes after condense
	2) application registration faults
	3) invalid boot script
	4) invalid search paths for application retrieval (load)
	5) invalid source pointers during exclude to system application
	6) invalid minor version after forbidden minor version release
	7) source file protection (not a bug)
	8) improper storing of initializations
	9) impossible binary installation of classes with version

optimizations concerning BOSS storing
"!

!BinaryObjectStorage methodsFor: 'private-initialize-release'!

resizeMaps: newSize
	"resize the reader/writer maps to the specified size (number of slots)"

	(readerMap size max: writerMap size) > newSize
		ifTrue: [^self error: 'invalid size in resizing'].
	readerMap changeCapacityTo: newSize.
	writerMap changeCapacityTo: newSize! !

!ApplicationManagement methodsFor: 'change management'!

updateCondensedChanges
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	applications do: [:app |
		app changes do: [:chg |
			(chg class == MethodDefinitionChange | (chg class == MethodDefinitionInclude)
				and: [chg sourceFileNumber = 2])
				ifTrue: [chg getSource]]]! !

!ApplicationManagement methodsFor: 'registration'!

unregister: anApplication 
	"remove anApplication from the registration structure and return it"

	^applications remove: anApplication
		ifAbsent: [DialogView internalError: 'improper call to unregister']! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInAsApplication: filename 
	"file in source code;
	the first chunk must be an application source header holding the application name;
 	redirect the source code pointers to the source file"

	| app old newName stream header conflicts |

	stream := filename asFilename readStream.
	header := stream nextChunk.
	stream close. 
	((Application sourceHeader , '*') storeString match: header)
		ifFalse: [^self error: filename , '
is not an application source file'].
	header := header copyFrom: Application sourceHeader size + 2 to: header size.
	newName := header copyUpTo: $(.	"strip off version string"
	newName isEmpty ifTrue: [^nil].

	(old := self applicationNamed: newName) notNil
		ifTrue: [(DialogView confirm: 'Do you want to overwrite
', old identifier storeString, ' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['
(changes will get lost)']) initialAnswer: false)
			ifTrue:
				[self unregister: old]
			ifFalse:
				[Transcript show: 'aborted '. ^nil]].

	(app := self newApplicationNamed: newName) isNil
		ifTrue: [^nil].

		"an application source path includes a basic path,
		followed by a major and a minor directory and the source file name;
		strip off the last three ones"
	app label path: filename asFilename head asFilename head asFilename head.
	app version: (VersionDescriptor readFromString: (header copyFrom: (header indexOf: $( ) +1 to: header size)).

	self fileInFrom: stream to: app external: true.
	app setChanges: (app changes select: [:chg | chg class == MethodDefinitionBackup | (chg class == ClassDefinitionBackup)]).
	app clearChanges version: app latestMinor.
	self changed: #version.

	old isNil ifTrue: [^app].
	conflicts := app checkConflictsWith: old.
	(conflicts isNil or: [conflicts isEmpty]) ifTrue: [^app].
	DialogView userInfo: 'Installation is done.

A change browser will open now. Selecting ''replay all''
will remove all classes and methods registered in the
previously installed version but not in the new one.
Be careful!!'.

	conflicts browse.
	^app! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

load: application from: aBOS path: path 
	"load the application (structure with class & method definitions) from aBOS;
	assign the given path after installation"

	| app old conflicts |
	Transcript cr; show: 'loading ' , application asString, ' (structure)... '.
	"avoid growing of the bos' maps"
	aBOS resizeMaps: aBOS nextIndex // 4 * 3 // 2.
	Cursor read showWhile: [app := aBOS next. aBOS close].

	(old := self applicationNamed: app name) notNil
		ifTrue: [(DialogView confirm: 'Do you want to overwrite
', old identifier storeString, ' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['
(changes will get lost)']) initialAnswer: false)
			ifTrue:
				[conflicts := app checkConflicts.
				self unregister: old]
			ifFalse:
				[Transcript show: 'aborted '. ^nil]].

	Transcript show: '(checks)... '.
	self checkRequired: app necessary.

	Transcript show: '(pre-installation)... '.
	app preInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	Transcript show: '(source)... '.
	self logTo: app while: [Cursor read showWhile: [aBOS nextClasses; close]].

	Transcript show: '(post-installation)... '.
	app postInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	app setChanges: (app changes select: [:chg | chg class == MethodDefinitionBackup | (chg class == ClassDefinitionBackup)]).

	"set the correct path"
	app label path: path.
	app definePoolDictionaries.
	self register: app; changed: #version.
	Transcript show: 'done '.

	(conflicts isNil or: [conflicts isEmpty]) ifTrue: [^app].
	DialogView userInfo: 'Installation is done.

A change browser will open now. Selecting ''replay all''
will remove all classes and methods registered in the
previously installed version but not in the new one.
Be careful!!'.

	conflicts browse.
	^app! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

load: application path: path version: version binary: booleanOrNil
	"load the application (its structure together with class and method definitions)
	whose sources reside in the directory <path>/<application>/<version>;
	booleanOrNil switches binary mode on/off, nil inquires mode from user;
	return the new application"

	| dir bos |
	dir := (path / application / version) asFilename.
	^(booleanOrNil ~~ false
		and: [(bos := (dir / 'applic.bos') asFilename) exists
		and: [booleanOrNil == true
			or: [DialogView confirm: 'Do you want to load
', application, ' ', version, '
in binary mode?']]])
		ifTrue: [self load: application from: (BinaryObjectStorage onOld: bos readStream) path: path]
		ifFalse: [self fileInAsApplication: (dir / 's.st') asFilename]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutBootScript
	"file out all the class definitions which are part of application management applications"
	"self applicationManagement fileOutBootScript"

	| stream classnames |
	stream := (self applicationNamed: 'APL-Framework') path / 'appman.st0'.
	stream := stream asFilename writeStream.
	stream
		timeStamp;
		nextPutAll: '"The following definitions allow import of application structures
defined by Application Management from a BinaryObjectStorage

Copyright (C) June 1993 by Georg Heeg, Dortmund
"'.
	classnames := #(#Application #StorableObjectAccessor #AccessControlledObjectAccessor #VersionedObjectAccessor #AccessControlList #VersionDescriptor #ClassSubset #ChangeSequence #SourceReference #ClassApplicationChange #MethodApplicationChange).

	classnames do: [:classname | stream cr; cr; nextChunkPut: classname asClass definition].

	(SourceCodeStream on: stream)
		fileOutMessages: #(#classRef: #definition:) for: ClassSubset logging: false;
		fileOutMessages: #(#of:definition:selectors:) for: ClassSubset class logging: false;
		fileOutMessages: #(#setComment:) for: ClassDescription logging: false;
		fileOutMessages: #(#setComment:) for: ClassOrganizer logging: false.

	stream 
		cr cr cr nextPutAll: '"now reset the changes for the classes just filed in"';
		cr; nextPutAll: classnames printString;
		crtab: 2; 
		nextChunkPut: 'do: [:classname | ChangeSet current removeClassChanges: (Smalltalk at: classname)]';
		close! !

!ApplicationManagement methodsFor: 'inquiry'!

applicationDirectoriesIn: path 
	"search the directory 'path' for installed application sources; 
	return a collection of triples like (<path>, <app name>, <major version>)"

	| found |
	found := OrderedCollection new.
	(path exists and: [path isDirectory]) ifFalse: [^found].
	path directoryContents do: 
		[:appdir | 
		"search <path> for subdirectories containing applications; 
		an application's directory has the name of the application"
		| s |
		(s := (path / appdir) asFilename) isDirectory
			ifTrue: 
				["search <path>/<appdir> for non-empty major version subdirectories"
				| majors |
				majors := s directoryContents select: [:maj | ('v#*' match: maj)
							and: [| dir | (dir := (path / appdir / maj) asFilename) isDirectory
							and: [(dir directoryContents
								detect: [:fname | 's.st' match: fname] ifNone: []) notNil]]].
				found addAll: (majors collect: [:maj | Array with: path with: appdir with: maj])]].
	^found! !

ApplicationManagement removeSelector: #load:path:version:!

!Application methodsFor: 'selectors'!

excludeSelector: selector class: aClass 
	"selector is excluded from application, included into system application; 
	all recorded changes move to system application"

	| chg sys |
	(self removeSelector: selector inSubsetOf: aClass) isNil
		ifTrue: [^nil]. 

	chg := self changes removeSelectorChanges: selector class: aClass.
	(sys := self systemApplication) changeOrIncludeSelector: selector class: aClass.
	sys changes updateChanges.		"assign a valid source ref for the new change"
	"improve this code: it easily duplicates changes"
	sys addChanges: chg.

	"enter the exclude change after removing all others"
	self changes excludeSelector: selector class: aClass.
	self touch! !

!Application methodsFor: 'private-storing'!

fixChanges: newVersion 
	"append source code for changes to source file, move changes to history; 
	redirect source code pointers to source file (usually from image changes file); 
	if 'newVersion' is true, an application header and a time stamp will be written, 
	and the start position for the new minor version sources is stored in a special file"

	| stream startPos |
	Cursor write showWhile:
		[stream := self storeFile.
		[stream cr; cr.
		newVersion ifTrue: 
			[self history addDoIt: '"', self sourceIdentifier, '"'.		"insert a separator"
			self incrementMinorVersion.
			startPos := stream targetFile writePosition.
			self fileOutSourceIdentifierOn: stream.
			stream timeStamp].
		self changes fileOutExternalOn: stream] valueNowOrOnUnwindDo: [stream close].

		self clearChanges.
		startPos == nil ifFalse: [self storeMinorVersionPosition: startPos].
		].
	self storeBinary! !

!Application methodsFor: 'private-storing'!

storeSource
	"file out the current state of all sources of the application,
	move source code references to the output file;
	format: <application identifier> <time stamp> <application comment> 
		<pre installation> <required applications> <source code> <post installation>"

	| stream |
	Cursor write showWhile:
	[
		[self
			fileOutSourceIdentifierOn: (stream := self storeFile);
			fileOutDocumentationOn: stream timeStamp;
			fileOutPreInstallationOn: stream;
			fileOutRequiredOn: stream;
			fileOutSourcesOn: stream;
			fileOutPostInstallationOn: stream
		]
			valueNowOrOnUnwindDo:
				[stream close.
				stream targetFile name asFilename makeUnwritable.
				self releaseSourceFile]
	]! !

!Application methodsFor: 'version management'!

newMinorVersion
	"increment the minor version number; 
	save current changes to source file and history, redirect source code pointers;
	broadcast the change of version to dependent browsers"

	| latest file |
	self version isInitialVersion
		ifTrue: [DialogView userError: 'Minor versions are not allowed
before the first major version release'. ^nil].

	(latest := self latestMinor) > self version
		ifTrue: [DialogView userError:
'This application has already been released to version
', latest printString, '
Try to upgrade and make your release afterwards'. ^nil].

	latest < self version copy untouch
		ifTrue: [(DialogView confirm:
'This application has not been released since version
', latest printString, '
Do you want to proceed?' initialAnswer: false)
			ifFalse: [^nil]].

	(file := self instSourceFilename) makeWritable.
	[self fixChanges: true] valueNowOrOnUnwindDo: [file makeUnwritable].
	self changed: #version! !

!Application methodsFor: 'version management'!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp newMajor major |
	self version isInitialVersion ifFalse: [self sourceFile]. "believe me, it is necessary"

	(newApp := self cleanUp copy setPath) isNil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) isZero
		ifTrue: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor <= major
		ifTrue: [newApp path asString = self path asString
			ifTrue: [DialogView userError: 'You must specify a new path
if an old major number is used'. ^self]
			ifFalse: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]]
		ifFalse: [major + 1 = newMajor
			ifFalse: [(DialogView confirm: 'The major number increment is greater than 1.
Do you want to proceed?' initialAnswer: false)
				ifTrue: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]
				ifFalse: [^self]]].

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	newApp incrementMajorVersion; storeSource; storePools; noChanges; storeBinary.
	self applicationManagement unregister: self; register: newApp.
	self asBackup.
	^newApp! !

!Application methodsFor: 'do its'!

postInstallationClassInits
	"return a collection of non-meta classes whose initializations occur explicitly in the post-installation"

	| classesToInit |
	classesToInit := OrderedCollection new.
	self postInstallation do: [:chg | | tokens |
		tokens := Scanner new scanTokens: chg text.
		2 to: tokens size do: [:i | | class |
			((tokens at: i) == #initialize and: [(class := (tokens at: i - 1) asClass) notNil])
				ifTrue: [classesToInit add: class]]].
	^classesToInit! !

!Application methodsFor: 'fileIn/Out'!

fileOutClasses: classNameList on: aStream 
	"file out the current source code for all registered elements
	(definition, comment, methods, etc.) of the classes in classNameList"

	| classList first |
	first := true.
	aStream deferInitializations.
	classList := classNameList collect: [:name | | class |
		(class := name asClass) isMeta
			ifTrue: [class soleInstance]
			ifFalse: [class]].
	(classList := ChangeSet superclassOrder: classList) do: [:class | 
		first ifTrue: [first := false] ifFalse: [aStream cr; newPage; cr].
		self fileOutClass: class on: aStream].
	aStream removeInitializations: self postInstallationClassInits.
	aStream cr; finishInitializations; cr! !

!Application methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        

!Application methodsFor: 'protocols'!

includeProtocol: protocolName class: aClass 
	"include the given protocol;
	 if class is not in structure, register it (without definition)"

"	self touch.	(adding protocol is no significant change to the structure)"
	^(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass])
		addCategory: protocolName asSymbol! !

BOSSWriter subclass: #ApplicationBOSSWriter
	instanceVariableNames: 'application '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Binary Storage'!

!ApplicationBOSSWriter methodsFor: 'private-tracing'!

traceClassNoVersion: aClass 
	"if the application defines aClass it will be exported; the binary representation 
	version may be erased, cause aClass and any instance of it will match when 
	reading the contents of the receiver; otherwise new classes defining a version 
	could not be installed in a binary mode, cause version validation runs before 
	installation of the binaryReaderBlock methods"

	| assoc |
	assoc := aClass representBinaryOn: self.
	assoc value at: 3 put: nil.
	^assoc! !

!ApplicationBOSSWriter methodsFor: 'private-initialize-release'!

storage: aBOS on: aStream registry: reg 
	"find out the application calling this store;
	force SourceReferences to represent in the default way;
	force classes defined by the application to represent without version"

	| ctx |
	super storage: aBOS on: aStream registry: reg.
	ctx := thisContext.
	[ctx == nil or: [(application := ctx receiver) class == Application]] whileFalse: [ctx := ctx sender].

	self traceSelectorFor: SourceReference is: #traceDefault:.
	application classNames do: [:c | | cls |
		((cls := c asClass) binaryRepresentationVersion notNil
			and: [(application definesClass: c)])
			ifTrue: [self traceSelectorFor: cls class is: #traceClassNoVersion:]]! !

!ApplicationBOSSWriter methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection withSource: sourceMode 
	"adaption of #nextPutClasses:withSource: to the needs of application management: 
	write current state of class definition and compiled code (usually sourceMode 
	is #reference)"

	| classes ordered inits postInits table |
	useSource := sourceMode.
	table := Dictionary withAll: (subsetCollection collect: [:subset | subset mappedClass -> subset]).
	"don't forget meta subsets if no soleInstance subset is present"
	classes := table keys collect: [:c | c isMeta ifTrue: [c soleInstance] ifFalse: [c]].
	ordered := OrderedCollection new.
	classes copy do: [:c | self enumerateClass: c from: classes to: ordered].
	ordered := (ordered select: [:c | table includesKey: c]) asArray.
	inits := OrderedCollection new.
	postInits := application postInstallationClassInits.

	self nextPut: ordered do: 
		[ordered do: 
			[:c | | subset cmt selectors |
			Transcript space; show: c name.
			currentClass := c.
			(subset := table at: c ifAbsent: [ClassSubset new]) includesDefinition
				ifTrue: 
					[self trace: (BOSSContents new object: c). cmt := c comment]
				ifFalse: 
					[self trace: c. cmt := ''].
			(cmt isEmpty or: [useSource == #discard])
				ifFalse: 
					[cmt := c organization globalComment class == SourceReference
						ifTrue: [Array with: c with: #setComment: with: c organization globalComment]
						ifFalse: [Array with: c with: #comment: with: cmt].
					self trace: cmt body: cmt class: BinaryObjectStorage indexImportSend].

			"store selectors as ordered by system"
			selectors := subset selectors.
			c organization elements do: [:s | (selectors includes: s)
				ifTrue: [self trace: (c compiledMethodAt: s)]].
			(c isMeta and: [(selectors includes: #initialize)
				and: [(postInits includes: c soleInstance) not]])
				ifTrue: [inits add: c]].

		inits do: [:c | | init |
			init := MessageSend receiver: c soleInstance selector: #initialize.
			init := Array with: #deferSend: with: init.
			self trace: init body: init class: BinaryObjectStorage indexImportSendSelf]]! !

BOSSWriter removeSelector: #nextPutClassSubsets:withSource:!

!BOSSWriter methodsFor: 'tracing-policy'!

traceSelectorFor: anObject 
	"anObject is the first object of its class encountered so far. Answer the message 
	selector that should be sent to self to trace the object. The default is #represent:, 
	which simply asks the object for its preferred representation.
	Optimized by associating #traceDefault: for objects inheriting #representBinaryOn:
	from Object, leading to the same result as #represent:"

	^(anObject class whichClassIncludesSelector: #representBinaryOn:) == Object
		ifTrue: [#traceDefault:]
		ifFalse: [#represent:]! !

!BOSSWriter methodsFor: 'private-tracing'!

trace: anObject
	" Recursively trace anObject.
	Return the encoding of the object. "

	| objx |
	anObject == nil ifTrue: [^nilIndex].
	objx := objects at: anObject ifAbsent:
		[| ox | (ox := storage indexOf: anObject) == nil
			ifTrue: [-1]
			ifFalse: [objects at: anObject put: ox]].
	objx >= 0 ifTrue: [^objx].
	objx = -1 ifTrue:
		[| sel value |
		sel := traceClasses at: anObject class
			ifAbsent: [traceClasses at: anObject class put: (self traceSelectorFor: anObject)].
		sel == #represent:
			ifTrue:
				["optimize represent: "
				(value := self represent: anObject) isImmediate
					ifTrue:
					[" A special representation method might have
					been invoked.  Check whether the return value
					is actually a correct object index. "
					value isInteger ifTrue:
						[value = nilIndex
							ifTrue:
								[" Substitute nil for the object. "
								objects at: anObject put: value]
							ifFalse:
								[((value anyMask: 2) ifTrue: [value - 2 + baseIndex] ifFalse: [value]) = (objects at: anObject ifAbsent: [nil]) ifFalse:
									[self error: 'Immediate representation is not allowed'.
									value := nil]]]]]
			ifFalse: [value := self perform: sel with: anObject].
		value == nil
			ifTrue:  "use standard representation"
				[self trace: anObject class.
				^self trace: anObject body: anObject class: anObject class].
		value isInteger ifTrue: [^value].  "internal method returned objx"
		" Trace a generic object with special handling. "
		(value isMemberOf: Association)
			ifTrue:  "special reader index"
				["Check to make sure that the representation
				is minimally compatible with the index. "
				| index arg |
				index := value key.
				arg := value value.
				(arg class isBits = index odd)
					ifFalse: [self error: 'Incompatible representation'.  ^nil].
				^self trace: anObject body: arg class: index].
		^self
			trace: anObject
			body: (Array with: value receiver with: value selector) , value arguments
			class: BinaryObjectStorage indexImportSend].
	objx = -2 ifTrue:
		[" Circular / recursive reference.
		Assign the index now; the correct
		flags and body will be supplied when
		we return out of the recursion. "
		^self assign: anObject].
	" Registered object. "
	^(registry at: -2 - objx) traceOn: self! !


' .ApplicationSource APL-Framework(1.2)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 1 September 1993 at 3:27:06 pm'!



(ChangeSequence includesSelector: #changeSelector:class:backup:)
	ifFalse: [self halt: 'Update ''APL-ChangeSequence'' first, please']!

"move the following two selectors from 'ExternalSource' to self
enable backups at every change
fix a bug in conflict checking"!

!ClassCategoryReader methodsFor: 'fileIn/Out'!

scanFrom: aStream sourceHolder: fileHolder 
	"rewritten from self>scanFrom: (executed if fileHolder is nil):
	file in methods from aStream, print name/category of found methods to transcript;
	do NOT append source code to the changes file,
	set source pointers to the original source file"

	| string p |
	fileHolder == nil ifTrue: [^self scanFrom: aStream].

	Transcript nextPutAll: class name; nextPut: $<; nextPutAll: category; cr; endEntry.
	
	[p:= aStream position.
	(string := aStream nextChunk) size > 0]
		whileTrue: ["done when double terminators"
			class
				compile: string
				classified: category
				source: (SourceReference for: fileHolder position: p)
				logging: self applicationManagement
				ifFail: []
			]! !

!ClassDescription methodsFor: 'compiling'!

compile: code classified: heading source: aSourceReference logging: aChangeManager ifFail: failBlock 
	"compile code, the result is a CompiledMethod; 
	classify resulting method under protocol 'heading', 
	set its source pointer to aSourceReference; 
	inform aChangeManager about the change 
	('ChangeSet current' represents the usual system change mechanism, 
	ApplicationManager records the change, but skips the system mechanism)"

	^Cursor execute showWhile: 
		[ | methodNode selector |
		methodNode := self compilerClass new
				compile: code
				in: self
				notifying: nil
				ifFail: failBlock.
		(methodDict includesKey: (selector := methodNode selector))
			ifTrue: [aChangeManager changeSelector: selector class: self]
			ifFalse: [aChangeManager addSelector: selector class: self].
		self addSelector: selector withMethod: (methodNode generate sourcePointer: aSourceReference).

		"application management records the change in change objects;
		the new source pointer must be propagated to those change objects:"
		aChangeManager updateChanges.

		"proceed as usual"
		self organization classify: selector under: heading.
		selector].! !

!SystemApplication methodsFor: 'selectors'!

changeOrIncludeSelector: selector class: aClass 
	"include only if not in another application"

	| app |
	(app := self applicationManagement applicationsIncludingSelector: selector class: aClass) isEmpty
		ifTrue: 
			[Transcript cr; show: aClass printString , '>' , selector printString , ' included in System Application'.
			self includeSelector: selector class: aClass]
		ifFalse: [(app includes: self)
			ifFalse: [^self	"receiver is not responsible for the selector"]].
	self changes changeSelector: selector class: aClass backup: true! !

!Application methodsFor: 'selectors'!

changeOrIncludeSelector: selector class: aClass 
	"a method change has happened, and the receiver has been determined to 
	record the change; if the class/selector combination is not yet registered, do it 
	right now; create a backup version in the change sequence to remember the
	original state of the source code"

	(self includesSelector: selector class: aClass)
		ifFalse: 
			[self includeSelector: selector class: aClass.
			self systemApplication excludeSelector: selector class: aClass].
	self touch.
	self changes changeSelector: selector class: aClass backup: true! !

!Application methodsFor: 'selectors'!

changeSelector: selector class: aClass 
	"Include indication that a method has been edited."

	(self includesSelector: selector class: aClass)
		ifFalse: [^self].
	self changes changeSelector: selector class: aClass backup: true.
	self touch! !

!Application methodsFor: 'selectors'!

includeSelector: selector class: aClass 
	"include class/selector combination to the application structure"

	self
		addSelector: selector
		protocol: (self protocolFor: selector class: aClass)
		inSubsetOf: aClass! !

!Application methodsFor: 'checking conflicts'!

checkConflictsWith: anApplication 
	"return a change sequence with removal of classes and selectors in anApplication
	not present in the receiver; applying this method to anApplication
	which is older than the receiver you will find obsolete classes and methods"

	| conflicts ownClasses |
	conflicts := ChangeSequence new.
	ownClasses := self classNames.
	((anApplication classNames select: [:name | anApplication definesClass: name])
		reject: [:name | ownClasses includes: name])
		do: [:name | 
			conflicts addDoIt: name, ' subclasses isEmpty
	ifFalse: [self notify: ''', name , ' has subclasses'']'.
			conflicts removeClass: name asClass].

	anApplication classNames do: [:name | | classSelectors meta |
		classSelectors := self selectorsInClass: name.
		((anApplication selectorsInClass: name) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: name asClass].
		classSelectors := self selectorsInClass: (meta := name asClass class name).
		((anApplication selectorsInClass: meta) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: meta asClass]].
	^conflicts! !


' .ApplicationSource APL-Framework(1.3)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 29 September 1993 at 2:30:16 pm'!



"fix for: unintended upgrade all (improper stop version for upgrade)
enhanced: new application name on new major version
		file out for upgrade
corrected: call notify: instead of error: in fileInFromStream:...
		rtn-value in fileInUpTo:... was not exact
"!

!Application methodsFor: 'private'!

sourceIdentifierForVersion: aVersionDescriptor 
	"return a store string identifying the receiver with given version in a source file"

	^(self class sourceHeader , self name , $( asString , aVersionDescriptor printString , $) asString) storeString! !

!Application methodsFor: 'version management'!

upgradeMinorUpTo: aVersion
	"file in all the (new) sources in the receiver's source file:
	- start with the receiver's following minor version
	- stop at the beginning of aVersion (file in all if argument is nil)"

	| oldChanges newChanges stream chgList result |
	oldChanges := self changes.
	self noChanges.
	"use a private copy of the source file accessor"
	(stream := self instSourceFilename readStream)
		position: (self getMinorVersionPosition: self version newMinor).
	result := self 
		fileInFromStream: stream
		startAt: (self sourceIdentifierForVersion: self version newMinor)
		stopAt: (aVersion notNil ifTrue: [self sourceIdentifierForVersion: aVersion newMinor])
		external: true.

	oldChanges isEmpty ifFalse:
		[oldChanges
			addDoIt: '"the following changes result from upgrade"';
			addAll: (newChanges := self changes)].
	self clearChanges.	"move upgrade changes to history"
	aVersion notNil
		ifTrue: [self version: aVersion].

	"user interaction for unsaved temporary changes"
	oldChanges isEmpty ifTrue: [^self].
	chgList := (EnhancedChangeList onApplication: self)
			listName: 'Changes conflicts in upgrading ' , self identifier storeString 
			changes: oldChanges
			removed: newChanges.

	DialogView userInfo: 
'The temporary changes have been removed from application.
Select "remove selection" for those you want to forget,
select "reinstall" for those which have been overwritten,
select "accept" to assign the contents of the list to the application.'.
 
	ChangeSequenceView openOn: chgList! !

!Application methodsFor: 'version management'!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp oldname newName newMajor major |

	"- assign a new path ---------------------------"
	(newApp := self cleanUp copy setPath) isNil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	"- assign a new name ---------------------------"
	oldname := self name.
	newName := DialogView
			request: 'Enter the name of the new major version'
			initialAnswer: oldname
			ifEmpty: [^self].
	oldname = newName
		ifFalse: [(DialogView confirm: 'You entered ''', newName, ''' as new application name.
Is this correct?')
			ifFalse: [^self]].
	newApp label name: newName.

	"- assign a new major version number ----------------"
	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) isZero
		ifTrue: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor <= major
		ifTrue: [newApp path / newName = (self path / self name)
			ifTrue: [DialogView userError: 'You must specify a new path
if an old major number is used'. ^self]
			ifFalse: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]]
		ifFalse: [major + 1 = newMajor
			ifFalse: [(DialogView confirm: 'The major number increment is greater than 1.
Do you want to proceed?' initialAnswer: false)
				ifTrue: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]
				ifFalse: [^self]]].

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	"- basic storing sequence ---------------------------"
	newApp incrementMajorVersion; storeSource; storePools; noChanges; storeBinary.
	self applicationManagement unregister: self; register: newApp.
	self asBackup.
	^newApp! !

!Application methodsFor: 'fileIn/Out'!

fileInFromStream: aStream startAt: startVersion stopAt: stopVersion external: flag 
	"file in sources from a stream: 
	start after the first occurence of startVersion (current position if nil), 
	stop at occurence of stopVersion (or at end of stream). 
	All changes are logged by the receiver; 
	if 'flag' is false, all the sources are appended to the changes file; 
	otherwise keep source pointers to aStream (holder for the file is the receiver, 
	so take care that aStream is a stream on the receivers source file!!)"

	flag ifTrue: [self path isNil
			ifTrue: [self label path: aStream name asFilename]
			ifFalse: [self instSourceFilename asString = aStream name asString
				ifFalse: [self notify: 'Attempt to assign a second source file']]].

	^self applicationManagement logTo: self
		while: [aStream fileInFrom: startVersion to: stopVersion sources: (flag ifTrue: [self])]! !

!PeekableStream methodsFor: 'fileIn'!

fileInUpTo: stopVersion sources: fileHolder
	"(rewritten from self>fileIn, enhanced by checking occurences of special do its) 
	file in sources, stop encountering stopVersion (end of file, if not found);
	if fileHolder isn't nil, compile without logging to changes file and
	construct non-default source code pointers for the generated methods
	(use fileHolder, usually an Application, for access to the source file)" 

	| manager signal evaluator chunk val |
	signal := (manager := self applicationManagement) class sourceSignal.
	evaluator := Object evaluatorClass.
	[self atEnd]
		whileFalse: 
			[ | methods pos |
			methods := self skipSeparators peekFor: $!!.
			pos := self position.
			val := signal
				handle: [:ex | "provide a source reference for some do its (e.g. class comment ...)"
					fileHolder isNil
						ifTrue: [ex reject]
						ifFalse: [ex proceedWith: (SourceReference for: fileHolder position: pos)]]
				do: [evaluator evaluate: (chunk := self nextChunk) logged: false].
			methods
				ifTrue: [val := val scanFrom: self sourceHolder: fileHolder]
				ifFalse:
					[stopVersion = chunk ifTrue: [^val].
					"check for application management commands"
					val class == Message ifTrue: [manager performMessage: val]]
			].
	^val! !

!Application methodsFor: 'fileIn/Out'!

fileOutUpgradeFrom: oldMinor to: newMinorOrNil on: aStream 
	"file out released sources (i.e. not the temporary changes) to aStream: 
	- start with minor version oldMinor 
	- stop at the beginning of newMinorOrNil + 1 (end of file if argument is nil)"

	| src pos1 pos2 |
	src := self sourceFile.
	pos1 := self getMinorVersionPosition: oldMinor.
	pos2 := (newMinorOrNil isNil or: [newMinorOrNil minor == self version minor])
				ifTrue: [src name asFilename fileSize]
				ifFalse: [self getMinorVersionPosition: newMinorOrNil newMinor].
	src position: pos1.
	[pos1 < pos2]
		whileTrue: 
			[| read |
			aStream nextPutAll: (src next: (read := pos2 - pos1 min: 8192)).
			pos1 := pos1 + read]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutUpgrade: anApplication 
	"request for start and stop minor versions for which sources should be filed out; 
	request the destination file from user"

	| start stop file |
	anApplication minorVersion == 0
		ifTrue: [^DialogView userInfo: 'Application has not been modified since last major version'].
	start := DialogView
				request: 'Enter minor version to start with'
				initialAnswer: '1'
				ifEmpty: [^self].
	(start := Integer readFromString: start) > 0
		ifFalse: [^DialogView userError: 'Integer greater than zero expected'].
	stop := DialogView
				request: 'Enter minor version to stop with'
				initialAnswer: anApplication minorVersion printString
				ifEmpty: [^self].
	(stop := Integer readFromString: stop) > 0
		ifFalse: [^DialogView userError: 'Integer greater than zero expected'].
	file := DialogView
				requestFileName: 'Enter name of upgrade-file'
				default: anApplication minorDirectory / start printString, '-' , stop printString, '.cs'
				version: #new
				ifFail: [^self].
	start := VersionDescriptor major: anApplication majorVersion minor: start.
	stop := VersionDescriptor major: start major minor: stop.
	Cursor write showWhile: [anApplication
		fileOutUpgradeFrom: start
		to: stop
		on: (file := file asFilename writeStream)].
	file close! !


' .ApplicationSource APL-Framework(1.4)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 5 November 1993 at 11:45:30 am'!



"fix: keeping a class (with definition) in different applications may induce errors during application binary storage in context with class comment source references; inform the user and skip those comments"!

!ApplicationBOSSWriter methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection withSource: sourceMode 
	"adaption of #nextPutClasses:withSource: to the needs of application management: 
	write current state of class definition and compiled code (usually sourceMode 
	is #reference)"

	| classes ordered inits postInits table |
	useSource := sourceMode.
	table := Dictionary withAll: (subsetCollection collect: [:subset | subset mappedClass -> subset]).
	"don't forget meta subsets if no soleInstance subset is present"
	classes := table keys collect: [:c | c isMeta ifTrue: [c soleInstance] ifFalse: [c]].
	ordered := OrderedCollection new.
	classes copy do: [:c | self enumerateClass: c from: classes to: ordered].
	ordered := (ordered select: [:c | table includesKey: c]) asArray.
	inits := OrderedCollection new.
	postInits := application postInstallationClassInits.

	self nextPut: ordered do: 
		[ordered do: 
			[:c | | subset cmt selectors |
			Transcript space; show: c name.
			currentClass := c.
			(subset := table at: c ifAbsent: [ClassSubset new]) includesDefinition
				ifTrue: 
					[self trace: (BOSSContents new object: c). cmt := c comment]
				ifFalse: 
					[self trace: c. cmt := ''].
			(cmt isEmpty or: [useSource == #discard])
				ifFalse: 
					[| cmt0 |
					cmt := (cmt0 := c organization globalComment) class == SourceReference
						ifTrue: [cmt0 sourceFileHolder == application
							ifTrue: [Array with: c with: #setComment: with: cmt0]
							ifFalse: [DialogView warn: 'The comment of class ''', c name, ''' will not be stored.
It is still managed by application 
''', cmt0 sourceFileHolder identifier printString, ''''. nil]]
						ifFalse: [Array with: c with: #comment: with: cmt].
					self trace: cmt body: cmt class: BinaryObjectStorage indexImportSend].

			"store selectors as ordered by system"
			selectors := subset selectors.
			c organization elements do: [:s | (selectors includes: s)
				ifTrue: [self trace: (c compiledMethodAt: s)]].
			(c isMeta and: [(selectors includes: #initialize)
				and: [(postInits includes: c soleInstance) not]])
				ifTrue: [inits add: c]].

		inits do: [:c | | init |
			init := MessageSend receiver: c soleInstance selector: #initialize.
			init := Array with: #deferSend: with: init.
			self trace: init body: init class: BinaryObjectStorage indexImportSendSelf]]! !


' .ApplicationSource APL-Framework(1.5)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 23 March 1994 at 6:43:06 pm'!



"fixes for errors
- invalid source reference for class comments during nested file in
- invalid source reference for class comments after condense changes
- during source file access (filename problems)"!

!PeekableStream methodsFor: 'fileIn'!

fileInUpTo: stopVersion sources: fileHolder
	"(rewritten from self>fileIn, enhanced by checking occurences of special do its) 
	file in sources, stop encountering stopVersion (end of file, if not found);
	if fileHolder isn't nil, compile without logging to changes file and
	construct non-default source code pointers for the generated methods
	(use fileHolder, usually an Application, for access to the source file)" 

	| manager signal evaluator chunk val |
	signal := (manager := self applicationManagement) class sourceSignal.
	evaluator := Object evaluatorClass.
	[self atEnd]
		whileFalse: 
			[ | methods pos |
			methods := self skipSeparators peekFor: $!!.
			pos := self position.
			val := signal
				handle: [:ex | "provide a source reference for some do its (e.g. class comment ...)"
					ex proceedWith: (fileHolder == nil
						ifFalse: [SourceReference for: fileHolder position: pos])]
				do: [evaluator evaluate: (chunk := self nextChunk) logged: false].
			methods
				ifTrue: [val := val scanFrom: self sourceHolder: fileHolder]
				ifFalse:
					[stopVersion = chunk ifTrue: [^val].
					"check for application management commands"
					val class == Message ifTrue: [manager performMessage: val]]
			].
	^val! !

!ApplicationManagement methodsFor: 'change management'!

updateCondensedChanges
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	applications do: [:app |
		app changes do: [:chg |
			chg class == MethodDefinitionChange | (chg class == MethodDefinitionInclude)
				ifTrue: [chg sourceFileNumber = 2 ifTrue: [chg getSource]]
				ifFalse: [chg class == ClassCommentChange ifTrue: [chg file: nil position: nil]]]]! !

!Application methodsFor: 'filenames'!

requestPath: mode
	"the current path is undefined or invalid; request it from user;
	return a valid path/filename object (nil if user cancels)
	(special mode for initial versions, whose path may be a source file name);
	case mode of
	- nil : request a filename for initial versions with own path, a directory name for others
	- #path : request a directory name unconditionally
	- #error : like nil mode + 'invalid' message + pass new path to other applications holding the invalid one"

	| p old msg initial err |
	initial := mode ~~ #path & self version isInitialVersion.
	p := old := self path == nil
		ifTrue: [initial := false. Filename defaultDirectoryName]
		ifFalse: [self path asString].
	msg := initial
		ifTrue: ['source file name']
		ifFalse: ['path to application directory'].
	msg := 'Enter the new ', msg, '\of application ''' withCRs, self identifier, ''''.
	(err := mode == #error) ifTrue: [msg := (msg, '\(''', old, ''' is invalid)') withCRs].

	[p := DialogView requestFileName: msg default: p asString version: #mustBeOld ifFail: [^nil].
	(p := p asFilename) isDirectory ~~ initial]
		whileFalse:
			[DialogView warn: 'You must specify a ', (initial ifTrue: ['filename, not a directory'] ifFalse: ['directory'])].

	self setPath: p.
	err ifTrue: [self applicationManagement do: [:app | (app path notNil and: [app path asString = old])
		ifTrue: [app setPath: p]]].
	^p! !

!Application methodsFor: 'filenames'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  

!Application methodsFor: 'filenames'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            

!Application methodsFor: 'filenames'!

instBossFilename
	"return a Filename for the binary file holding application structure, classes and compiled methods"

	^(self minorDirectory / 'applic.bos') asFilename! !

!Application methodsFor: 'filenames'!

instPoolFilename
	"return the pool file name"

	^(self minorDirectory / ('p' , self minorVersion printString , '.pol')) asFilename! !

!Application methodsFor: 'filenames'!

instSourceFilename
	"return a Filename for the Smalltalk80 source file"

	^(self minorDirectory / 's.st') asFilename! !

!Application methodsFor: 'filenames'!

minorDirectory
	"return a Filename for the directory keeping minor versions 
	('<path>/<application name>/<v_>' , where _ is the major version number)"

	^(self directory / ('v' , self version major printString)) asFilename! !

!Application methodsFor: 'filenames'!

versionFileName
	"return a Filename for the version map file"

	^(self minorDirectory / 'versions') asFilename! !

!Application methodsFor: 'accessing-source'!

sourceFile
	"return (and cache) the file stream containing the sources (open it if necessary);
	handle OSErrorHolder inaccessibleSignal (includes nonexistentSignal), cause
	filename may be invalid due to image platform migration; try to repair filename
	specification interactively if errors occur"

	sourceFile == nil ifFalse: [^sourceFile].

	^OSErrorHolder inaccessibleSignal
		handle:
			[:ex | self checkPath == nil 
				ifTrue:
					["return a dummy, so that continuing without source file is possible"
					sourceFile := '' readStream]
				ifFalse:
					["repeat source file access with new path"
					ex restart]]
		do:
			[| fname |
			fname := self version isInitialVersion ifTrue: [self path] ifFalse: [self instSourceFilename].
			sourceFile := fname asFilename readStream]! !

!Application methodsFor: 'accessing-source'!

stringAt: anInteger ifAbsent: aBlock 
	"return the string stored in the receiver's source file at position anInteger; 
	return the result of aBlock if the source file is not accessible"

	^Stream positionOutOfBoundsSignal
		handle:
			[:ex | "handle dummy source file (an empty ReadStream)"
			ex returnWith: aBlock value]
		do: [OSErrorHolder inaccessibleSignal
				handle: 
					[:ex | "handle incompatible source file due to platform migration"
					self releaseSourceFile.
					ex restart]
				do: [self sourceFile position: anInteger; nextChunk]]! !

!Application methodsFor: 'initialize-release'!

releaseSourceFile
	| f |
	sourceFile == nil ifTrue: [^self].
	f := sourceFile.
	sourceFile := nil.
	f close! !

"organizational changes"
!

Application organization renameCategory: #'class subset access' to: #'accessing-class subsets'!

Application organization renameCategory: #categories to: #'changing-categories'!

Application organization renameCategory: #classes to: #'changing-classes'!

Application organization renameCategory: #protocols to: #'changing-protocols'!

Application organization renameCategory: #selectors to: #'changing-selectors'!

"fixes for errors
- during conflict check with new classes of a new major version
- during file in with include/exclude messages (symbol arguments)
- class include did not transmit the class definition
- due to class category renames
"!

!Application methodsFor: 'private'!

allClassNamesIn: aCollection 
	"Only non-empty class subsets are part of the class info structure. 
	This method accounts for the case that there is an entry for a metaclass but not 
	for the non-metaclass and returns the names of all classes part of the info 
	structure in any way (meta or not), mapped to the names of the non-metaclasses"

	^aCollection collect: [:className | | class |
			(class := className asClass) == nil
				ifTrue: [nil	"dangerous if the receiver is not a set"]
				ifFalse: [class isMeta
					ifTrue: [class soleInstance name]
					ifFalse: [className]]]! !

!Application methodsFor: 'inquiry'!

ownClassNames
	"return the names of all non-meta classes defined by the receiver"

	^self classNames select: [:name | self definesClass: name]! !

!Application methodsFor: 'checking conflicts'!

checkConflictsWith: anApplication 
	"return a change sequence with removal of classes and selectors in anApplication
	not present in the receiver; applying this method to anApplication
	which is older than the receiver you will find obsolete classes and methods"

	| conflicts ownClasses |
	conflicts := ChangeSequence new.
	ownClasses := self classNames.
	(anApplication ownClassNames reject: [:name | ownClasses includes: name])
		do: [:name | 
			conflicts addDoIt: name, ' subclasses isEmpty
	ifFalse: [self notify: ''', name , ' has subclasses'']'.
			conflicts removeClass: name asClass].

	anApplication classNames do: [:name | | classSelectors meta |
		classSelectors := self selectorsInClass: name.
		((anApplication selectorsInClass: name) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: name asClass].
		classSelectors := self selectorsInClass: (meta := name asClass class name).
		((anApplication selectorsInClass: meta) reject: [:sel | classSelectors includes: sel])
			do: [:sel | 
				conflicts removeSelector: sel class: meta asClass]].
	^conflicts! !

!Application methodsFor: 'changing-categories'!

removeAllCategories
	"remove ALL classes from system (meta and soleInstance) 
	(restricted to the corresponding class subsets of the receiver)"

	| classList cats org |
	cats := self categories.
	classList := (self allClassNamesIn: self classNames)
				collect: [:cname | cname asClass].
	Cursor execute showWhile:
		[(ChangeSet superclassOrder: classList)
			reverseDo: [:c | self removeFromSystem: c]].

	"remove empty categories"
	org := Smalltalk organization.
	cats do: [:cat | (org listAtCategoryNamed: cat) isEmpty
		ifTrue: [org removeCategory: cat. ChangeSet current reorganizeSystem]].
! !

!Application methodsFor: 'changing-categories'!

renameCategory: oldName to: newName 
	"category oldName has been renamed to newName; 
	add a change if the receiver defines some class in that category"

	(Smalltalk organization listAtCategoryNamed: newName)
		detect: [:clsname | self definesClass: clsname]
		ifNone: [^self].
	self changes add: (CategoryRenameChange new rename: oldName to: newName).
	self touch! !

!Application methodsFor: 'changing-classes'!

excludeClass: aClass 
	"exclude aClass, its definition and all its methods from application"

	| subset sys c |
	(subset := self removeClassSubset: aClass) isNil
		ifTrue: [^nil].

	(sys := self systemApplication)
		addChanges: (self changes removeClassChanges: (c := aClass asClass)).

	"enter exclude changes after removing all others"
	subset selectors do: [:sel |
		sys changeOrIncludeSelector: sel class: c.
		self changes excludeSelector: sel class: c name].
	subset includesDefinition
		ifTrue:
			[sys includeClass: c.
			self changes excludeClass: c name].
	self touch! !

!Application methodsFor: 'changing-classes'!

excludeClassDefinition: aClass 
	| c |
	(self classSubsetOf: aClass ifAbsent: [^nil])
		removeDefinition ifFalse: [^nil].
	self systemApplication
		changeOrIncludeClass: (c := aClass asClass);
		addChanges: (self changes removePureClassChanges: c).
	self touch! !

!Application methodsFor: 'changing-classes'!

includeClass: aClass 
	"aClass is included (structure and definition);
	includes all changes on aClass which are not managed by a selector change"

	| someChanges c |
	(self definesClass: aClass) ifTrue: [^aClass].

	c := aClass asClass.
	(self classSubsetOf: c ifAbsent: [self addClassSubset: c]) addDefinition.
	((someChanges := self systemApplication excludeClassDefinition: c) isNil
		or: [someChanges isEmpty])
		ifTrue:
			["assume metaclasses will never be included without its soleInstance;
			so we don't need a change entry"
			c isMeta ifFalse: [self changes includeClass: c]]
		ifFalse:
			[self changes addAll: someChanges].
	self touch.
	^aClass! !

!Application methodsFor: 'changing-classes'!

includeOrChangeClass: aString
	"aString is the definition string of a class;
	create/change the class if necessary; include it to the receiver's registry;
	return the class object
	(sent during file in only, when the receiver is the currently logging application)"

	| cls |
	cls := (ReadStream on: aString) skipThrough: $#; upTo: Character cr.
	^((cls := cls asClass) == nil or: [cls definition ~= aString])
		ifTrue: [Object evaluatorClass evaluate: aString]
		ifFalse: [self includeClass: cls]! !

!ChangeSequence methodsFor: 'changing classes'!

includeClass: className 
	self add: (ClassApplicationChange new className: className; type: #includeOrChangeClass:; text: className asClass definition)! !

!ClassApplicationChange methodsFor: 'accessing'!

text
	"avoid doits which cannot be evaluated for users without application 
	management; return a message which can be sent to application 
	management"

	| arg |
	arg := type == #includeOrChangeClass:
		ifTrue: [super text]
		ifFalse: [className].
	^(Message selector: type arguments: (Array with: arg)) storeString! !

!CategoryRenameChange methodsFor: 'accessing'!

text
	^'Smalltalk organization renameCategory: ' , oldname storeString , ' to: ' , newname storeString! !


' .ApplicationSource APL-Framework(1.6)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 14 April 1994 at 1:07:54 pm'!



"fix: import of methods with equal code did not move source references"!

!BOSSCompiledCodeHolder methodsFor: 'accessing'!

sourcePointer
	^source! !

!BOSSReader methodsFor: 'reading-imports'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    

"fix: moving classes from App A1 to A2 (A2 include, A1 exclude) entered class definitions to system, although A2 held them"!

!SystemApplication methodsFor: 'classes'!

includeClass: aClass 
	"include aClass (structure and definition) if no other application holds its definition"

	(self applicationManagement applicationsIncludingDefinition: aClass) isEmpty
		ifTrue: [(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass])
				addDefinition].
	^aClass! !

"sort registered applications"
Object errorSignal handle: [:ex | ] do: [| apm apps |
	apm := self applicationManagement.
	apps := apm instVarAt: 1.
	apm instVarAt: 1 put: (apps asSortedCollection: [:x :y | x name < y name]) asOrderedCollection]!

!ApplicationManagement methodsFor: 'registration'!

register: anApplication ifRegistered: aBlock 
	"enter anApplication in the application holder; 
	ensure unique application names, evaluate aBlock on violation"

	(self includesNamed: anApplication name)
		ifTrue: [^aBlock value].
	applications add: anApplication.
	applications := (applications asSortedCollection: [:x :y | x name < y name]) asOrderedCollection.
	self changed: #application! !

!ApplicationManagement methodsFor: 'application access'!

sortedApplicationIdentifiers
	^applications collect: [:app | app identifier]! !

!ApplicationManagement methodsFor: 'application access'!

applicationIdentified: aString ifAbsent: aBlock 
	"return the application with identifier aString 
	(remark: identifiers include the version)"

	^applications detect: [:app | app identifier = aString asString]
		ifNone: aBlock! !

"- factorize conflicts browsing (#browseConflicts:)
- support cancel on required application check (#do:onCancel:)
- move changes during load application to history
- set platform compatible filename for application during #fileInAsApplication:
- support application preview from binary application file"!

!ApplicationManagement methodsFor: 'private'!

browseConflicts: aChangeSequence 
	"the argument contains changes for conflicts between two applications; 
	show a dialog with instructions how to treat conflicts and open a ChangeList"

	aChangeSequence isEmpty ifTrue: [^self].
	DialogView userInfo: 'Installation is done.\\A change browser will open now. Selecting ''replay all''\will remove all classes and methods registered in the\previously installed version but not in the new one.\Be careful!!' withCRs.
	aChangeSequence browse! !

!ApplicationManagement methodsFor: 'private'!

do: aBlock onCancel: cancelBlock 
	"evaluate aBlock; on call of <halt: 'cancel'> execute cancelBlock"

	^Object haltSignal handle: [:ex | ex errorString = 'cancel'
			ifTrue: [cancelBlock value]
			ifFalse: [ex reject]]
		do: [aBlock value]! !

!ApplicationManagement methodsFor: 'testing'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             

!ApplicationManagement methodsFor: 'fileIn/Out'!

loadInfo: application path: path version: version
	"load the application structure from its binary storage, return info/documentation"

	^Cursor read showWhile:
		[| file aBOS app |
		OSErrorHolder errorSignal handle: [:ex | ^nil]
			do: [file := (path / application / version / 'applic.bos') asFilename readStream].
		aBOS := BinaryObjectStorage onOld: file.
		app := [aBOS next] valueNowOrOnUnwindDo: [aBOS close].
		app information]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInAsApplication: aFilename 
	"file in source code from a file named by the given filename object;
	the first chunk must be an application source header holding the application name;
 	redirect the source code pointers to the source file"

	| app old newName stream header pathAndName |

	stream := aFilename readStream.
	header := stream nextChunk.
	stream close. 
	((Application sourceHeader , '*') storeString match: header)
		ifFalse: [^self error: aFilename asString, '\is not an application source file' withCRs].
	header := header copyFrom: Application sourceHeader size + 2 to: header size.
	newName := header copyUpTo: $(.	"strip off version string"
	newName isEmpty ifTrue: [^nil].

	(old := self applicationNamed: newName) == nil
		ifFalse: [(DialogView confirm: ('Do you want to overwrite\''', old identifier, ''' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['\(changes will get lost)'])) withCRs initialAnswer: false)
			ifTrue: [self unregister: old]
			ifFalse: [Transcript show: 'aborted '. ^nil]].

	(app := self newApplicationNamed: newName) == nil
		ifTrue: [^nil].

		"an application source path includes a basic path,
		followed by a major and a minor directory and the source file name;
		strip off the last three ones"
	pathAndName := aFilename head asFilename head asFilename.
	app label
		path: pathAndName head;
		setFilename: pathAndName tail forPlatform: OSHandle currentOS.
	app version: (VersionDescriptor readFromString: (header copyFrom: (header indexOf: $( ) +1 to: header size)).

	self do: [self fileInFrom: stream to: app external: true]
		onCancel:
			[self unregister: app.
			old == nil ifFalse: [self register: old].
			Transcript show: 'aborted '. ^nil].
	app setChanges: (app changes select: [:chg | chg isBackup]); clearChanges; version: app latestMinor.
	self changed: #version.

	old == nil ifFalse: [self browseConflicts: (app checkConflictsWith: old)].
	^app! !

!Application methodsFor: 'fileIn/Out'!

fileOutAllOn: aSourceCodeStream
	"file out the current state of all sources of the application;
	format: <application identifier> <time stamp> <application comment> 
		<required applications> <pre installation> <source code> <post installation>"

	Cursor write showWhile:
		[[self
			fileOutSourceIdentifierOn: aSourceCodeStream;
			fileOutDocumentationOn: aSourceCodeStream timeStamp;
			fileOutRequiredOn: aSourceCodeStream;
			fileOutPreInstallationOn: aSourceCodeStream;
			fileOutSourcesOn: aSourceCodeStream;
			fileOutPostInstallationOn: aSourceCodeStream]
				valueNowOrOnUnwindDo: [aSourceCodeStream close]]! !

!Application methodsFor: 'private-storing'!

fixChanges: newVersion 
	"append source code for changes to source file, move changes to history; 
	redirect source code pointers to source file (usually from image changes file); 
	if 'newVersion' is true, an application header and a time stamp will be written, 
	and the start position for the new minor version sources is stored in a special file"

	| stream startPos |
	self releaseSourceFile.	"avoid conflicts 'found' by MS-DOS SHARE.EXE"
	Cursor write showWhile:
		[stream := self storeFile.
		[stream cr; cr.
		newVersion ifTrue: 
			[self history addDoIt: '"', self sourceIdentifier, '"'.		"insert a separator"
			self incrementMinorVersion.
			startPos := stream targetFile writePosition.
			self fileOutSourceIdentifierOn: stream.
			stream timeStamp].
		self changes fileOutExternalOn: stream] valueNowOrOnUnwindDo: [stream close].

		self clearChanges.
		startPos == nil ifFalse: [self storeMinorVersionPosition: startPos].
		].
	self storeBinary! !

!Application methodsFor: 'private-storing'!

storeSource
	"file out the current state of all sources of the application, 
	use a target stream moving source code references to the output file"

	[self fileOutAllOn: self storeFile]
		valueNowOrOnUnwindDo: 
			[self instSourceFilename makeUnwritable.
			self releaseSourceFile]! !

"- error during #latestMinor for initial applications
- negative major version number from user, binary storage near end"!

!Application methodsFor: 'version management'!

latestMinor
	"return a VersionDescriptor for the latest version string in the source file"

	| p string |
	self version isInitialVersion ifTrue: [^self version].
	self versionFileName exists ifFalse: [^self version].
	p := ChangeList new findLast: self class sourceHeader , self name , '(' in: self sourceFile.
	p isNil ifTrue: [^nil].	"file does not include minor version descriptors"

	string := self sourceFile position: p; nextChunk.
	string := string copyFrom: 1 to: string size -1.	"strip off trailing ) "
	^VersionDescriptor readFromString: string! !

!Application methodsFor: 'version management'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              

"set modification flag when changing the doits"!

!Application methodsFor: 'do its'!

setDoits: aChangeSequence 
	"assign a collection of doits which are filed out before any other source"

	| old |
	old := doits.
	doits := aChangeSequence isEmpty ifFalse: [aChangeSequence].
	old = doits ifFalse: [self touch]! !

"introducing traceability"!

!Application methodsFor: 'prerequesites'!

requiresApplication: anApplication 
	"return true iff the receiver refers to anApplication as a necessary application"

	^self necessary includes: anApplication name! !

!Application methodsFor: 'prerequesites'!

useOfClasses: classNames do: aBlock 
	"evaluate aBlock with each own class or own class' superclass included in classNames; 
	evaluate aBlock for each own selector whose code refers to the given classes"

	self classNames do: 
		[:cname | | requiredClass |
		requiredClass := (self definesClass: cname)
					ifTrue: [cname asClass superclass name]
					ifFalse: [cname].
		(classNames includes: requiredClass)
			ifTrue: [aBlock value: cname value: requiredClass]].

	classNames do: 
		[:cname | | calls |
		(calls := self allCallsOn: (Smalltalk associationAt: cname ifAbsent: [])) isEmpty
			ifFalse: [aBlock value: cname value: calls]]! !

!Application methodsFor: 'prerequesites'!

testUseOfApplication: anApplication 
	"check if there are references to classes defined in anApplication"

	anApplication == self ifTrue: [^false].
	^Cursor execute showWhile: 
		[self useOfClasses: anApplication ownClassNames do: [:className :ignore | ^true].
		false]! !

!Application methodsFor: 'prerequesites'!

reportUseOfApplication: anApplication on: aStream 
	"report references to classes defined in anApplication"

	anApplication == self ifTrue: [^self].
	aStream cr; emphasis: #bold; nextPutAll: 'Use of application '; store: anApplication identifier; nextPut: $:; emphasis: nil.
	self useOfClasses: anApplication ownClassNames do: [:className :references |
		references isString
			ifTrue: 
				[aStream cr.
				references = className
					ifTrue: [aStream nextPutAll: 'refers to class '; nextPutAll: className]
					ifFalse: [aStream nextPutAll: className; nextPutAll: 'has superclass '; nextPutAll: references]]
			ifFalse: [references do: [:assoc |
				aStream cr; nextPutAll: assoc key; nextPut: $>; nextPutAll: assoc value;
					tab; nextPutAll: 'refers to class '; nextPutAll: className]]]! !

!Application methodsFor: 'prerequesites'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              

!Application methodsFor: 'prerequesites'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                

"optimization"!

!Application methodsFor: 'retrieving'!

allCallsOn: aLiteral 
	"return a Collection of: (class name -> selector) for all selectors that call
	on aLiteral (restricted to elements registered by the receiver)"

	| lit |
	lit := aLiteral isBehavior
		ifTrue: [Smalltalk associationAt: aLiteral name ifAbsent: [aLiteral]]
		ifFalse: [aLiteral].
	^Cursor execute showWhile: 
		[| calls |
		calls := OrderedCollection new.
		classes keysAndValuesDo: [:cname :subset | (cname asClass whichSelectorsReferTo: lit)
			do: [:sel | (subset includesSelector: sel)
					ifTrue: [calls add: cname -> sel]]].
		calls]! !

!Application methodsFor: 'initialize-release'!

release
	"move all methods in my source file to system changes and release external 
	references"

	self moveMethodsInSourcefileToChanges.
	self asBackup.
	super release! !

!Application methodsFor: 'filenames'!

setPath
	self requestPath: nil! !

!Application methodsFor: 'filenames'!

filenameForPlatform: aString
	^self label filenameForPlatform: aString! !

"code maintenance: remove some obsolete selectors"!

Application removeSelector: #=!

Application removeSelector: #hash!

Application removeSelector: #isEmpty!

Application removeSelector: #installationAllowed!

Application removeSelector: #includesCategory:!

Application removeSelector: #fileOutChanges!

Application removeSelector: #fileOutPools!

Application removeSelector: #fileOutOrder:!

Application removeSelector: #changesFilename!

Application removeSelector: #instDocFilename!

Application removeSelector: #excludeApplication:!

Application class removeSelector: #fromFileNamed:!

ApplicationManagement removeSelector: #includeCategory:!

ApplicationManagement removeSelector: #removeCategory:!

ApplicationManagement removeSelector: #excludeCategory:!

"improving binary storage of applications:
- default storage mechanism for the owner of the binary file
- expression for others (thus breaking application reference chains)"!

!ApplicationBOSSWriter methodsFor: 'private-tracing'!

traceApplication: anApplication 
	"if the argument is the main application currently being stored then store as 
	usual (recursively); otherwise return an expression which requests the 
	argument from existing applications"

	^anApplication == application
		ifTrue: [self traceDefault: anApplication]
		ifFalse: 
			[| string |
			string := anApplication name storeString.
			DialogView warn: 'Application ', string, '\is to be stored in the binary file\of application ''' withCRs, application name, ''''.
			string := 'self applicationManagement applicationNamed: % ifAbsent: [%]'
				copyReplaceAll: '%' with: string.
			MessageSend
				receiver: Object evaluatorClass
				selector: #evaluate:
				arguments: (Array with: string)]! !

!ApplicationBOSSWriter methodsFor: 'private-initialize-release'!

storage: aBOS on: aStream registry: reg 
	"find out the application calling this store;
	force Application other than this to store a generic expression;
	force SourceReferences to represent in the default way;
	force classes defined by the application to represent without version"

	| ctx |
	super storage: aBOS on: aStream registry: reg.
	ctx := thisContext.
	[ctx == nil or: [(application := ctx receiver) class == Application]] whileFalse: [ctx := ctx sender].

	self traceSelectorFor: SourceReference is: #traceDefault:.
	self traceSelectorFor: Application is: #traceApplication:.
	application classNames do: [:c | | cls |
		((cls := c asClass) binaryRepresentationVersion notNil
			and: [(application definesClass: c)])
			ifTrue: [self traceSelectorFor: cls class is: #traceClassNoVersion:]]! !

!Application methodsFor: 'private-storing'!

storeBinaryOn: aBOS 
	"store the application structure and the current state of all sources of the 
	application on binary object storage aBOS 
	format: <application structure> <class definitions and sources>"

	aBOS
		writerClass: ApplicationBOSSWriter;
		nextPut: self releaseSourceFile;
		sourceMode: #reference;
		nextPutClassSubsets: classes values.

	OSErrorHolder volumeFullSignal 
		handle: [:ex | | fn |
			Transcript cr; show: 'volume full during writing ', (fn := self instBossFilename) asString.
			Object errorSignal handle: [:x | x return] do: [fn delete]]
		do: [aBOS close]! !

!Application methodsFor: 'private-storing'!

storeBinary
	"store the application structure and the current state of all sources of the 
	application on a BinaryObjectStorage 
	format: <application structure> <class definitions and sources>"

	self changes isEmpty ifFalse:
		["avoid inconsistency arising from unsaved changes"
		DialogView userError: 'Binary storage must be rejected
if the application has temporary changes'.
		^self].

	Transcript cr; show: 'storing binary'.
	Cursor write showWhile:
		[self storeBinaryOn: (BinaryObjectStorage onNew: self instBossFilename writeStream)]! !

!Application methodsFor: 'support'!

recoverFrom: aFileStream 
	"construct a change list from the contents of aFileStream, but collect only
	those selectors which are registered in the receiver"

	| list dummy |
	list := ChangeSequence new.
	dummy := ClassSubset new.
	[ChangeScanner new scanFile: aFileStream do: [:chg |
		(chg class == MethodDefinitionChange
			and: [(self classSubsetOf: chg className ifAbsent: [dummy])
				includesSelector: chg selector])
			ifTrue: [list add: chg]]]
		valueNowOrOnUnwindDo: [aFileStream close].
	^list! !

!Application methodsFor: 'support'!

recoverDefault
	^Cursor read showWhile: [self recoverFrom: (SourceFileManager default fileAt: 1) reset]! !

!Application methodsFor: 'support'!

recoverOwn
	^self recoverFrom: self sourceFile reset! !

!Application methodsFor: 'support'!

reinstallBackups
	"scan the full history for any method definition backups referring to an 
	application different from the receiver; restore the states defined by such 
	backups and exclude the corresponding selectors from the receiver"

	self fullHistory reverseDo: [:chg | | sysRef |
		(chg isBackup
			and: [(sysRef := chg file class ~~ SourceReference)
				or: [chg file sourceFileHolder ~~ self]])
			ifTrue: 
				[chg restore.	"recompile; restore source reference"
				self excludeSelector: chg selector class: chg className.
				sysRef ifTrue:
					[self systemApplication excludeSelector: chg selector class: chg className]]]! !

!Application methodsFor: 'support'!

deinstall
	"restore backup states if necessary and remove all methods and classes 
	belonging to the receiver"

	Transcript cr; show: 'restoring...'.
	self reinstallBackups.
	Transcript show: 'removing...'.
	self removeAllCategories.
	Transcript show: 'done '.
	self applicationManagement unregister: self.
	self asBackup! !

"use functionality moved to application APL-ChangeSequence(1.5)"!

!ApplicationManagement methodsFor: 'change management'!

updateCondensedChanges
	"condensing changes invalidates source references of change objects with 
	code in the changes file; update correct source references"

	applications do: [:app | app changes updateCondensed]! !

!ApplicationManagement methodsFor: 'change management'!

updateChanges
	"a system change propagated by 'ChangeSet current' will be recorded by 
	concerned applications by creating an appropriate change object and storing 
	this object in a change sequence; often after the change a new source pointer 
	is installed which must be propagated to those change objects:"

	logging ifFalse: [^self].
	applications do: [:app | app changes updateLastChange]! !

!Application methodsFor: 'changing-selectors'!

excludeSelector: selector class: aClass 
	"selector is excluded from application, included into system application; 
	all recorded changes move to system application"

	| chg sys |
	(self removeSelector: selector inSubsetOf: aClass) isNil
		ifTrue: [^nil]. 

	chg := self changes removeSelectorChanges: selector class: aClass.
	(sys := self systemApplication) changeOrIncludeSelector: selector class: aClass.
	sys changes updateLastChange.		"assign a valid source ref for the new change"
	"improve this code: it easily duplicates changes"
	sys addChanges: chg.

	"enter the exclude change after removing all others"
	self changes excludeSelector: selector class: aClass.
	self touch! !

"after unregister (with moving source code), there were SourceRefs pointing to the eliminated application (owned by MethodDefInc, which have been moved to another application (MutateClass -> Patches) )"!

!ApplicationManagement methodsFor: 'change management'!

updateChanges: selectorSet class: className 
	"moving sources to the image changes file invalidates source references of 
	change objects; update correct source references"

	applications do: [:app | app changes updateSelectors: selectorSet class: className]! !

!Application methodsFor: 'private'!

moveMethodsInSourcefileToChanges
	"move all methods in my source file to system changes"

	classes do: 
		[:subset | 
		| cls set |
		SourceFileManager default
			fileOutMessages: (set := subset selectors)
			for: (cls := subset mappedClass)
			logging: false.
		"care for valid source references"
		self applicationManagement updateChanges: set class: cls name]! !

!Application methodsFor: 'inquiry'!

selectorIntersection
	"return a Dictionary key: Application value: (Set of: selector -> class name)
	for each other application including class>selector"

	^Cursor execute showWhile:
		[| am map |
		am := self applicationManagement.
		map := Dictionary new.
		self selectors do: [:assoc | am do: [:ap | ap == self
			ifFalse: [(ap includesSelector: assoc key class: assoc value)
				ifTrue: [(map at: ap ifAbsentPut: [Set new]) add: assoc]]]].
		map]! !

!Application methodsFor: 'inquiry'!

information
	"return an Array with
	- application identifier (name/version)
	- a Text with application identifier and required applications
	- application comment"

	| id info |
	info := TextStream on: (String new: 500).
	info nextPutAll: 'Application '; emphasis: #bold; nextPutAll: (id := self identifier).
	self necessary isEmpty
		ifFalse:
			[info cr; cr; emphasis: #italic; nextPutAll: 'required applications:'; emphasis: nil.
			self necessary do: [:name | info crtab; nextPutAll: name]].

	^Array with: id with: info contents with: self documentation! !

!ApplicationManagement methodsFor: 'inquiry'!

applicationDirectoriesIn: path 
	"search the directory 'path' for installed application sources; 
	return a collection of triples like (<path>, <app name>, <major versions>)
	where the last component is again an Array of Strings like #('v1' 'v2' ...)"

	| result files ls |
	ls := [:dirname | OSErrorHolder errorSignal
			handle: [:ex | ex returnWith: #()	"skip invalid directory specifications"]
			do: [dirname directoryContents asSortedCollection]].

	(files := ls value: path) isEmpty ifTrue: [^files].
	result := OrderedCollection new.
	files do: [:appname |
		"search <path> for subdirectories containing applications; 
		an application directory has the name of the application"
		| appdir majors |
		majors := (ls value: (appdir := (path / appname) asFilename)) asArray select:
			[:maj |
			"search <path>/<appname> for non-empty major version subdirectories"
			('v#*' match: maj)
				and: [((ls value: (appdir / maj) asFilename)
						detect: [:fname | 's.st' match: fname] ifNone: []) notNil]].
		majors isEmpty ifFalse: [result add: (Array with: path with: appname with: majors)]].
	^result! !

!ApplicationManagement methodsFor: 'inquiry'!

availableApplications
	"search the paths given in 'directories' for installed application sources; 
	return a collection of triples (for more see also #applicationDirectoriesIn:)"

	^Cursor read showWhile: 
		[| found |
		found := OrderedCollection new.
		self directories do: [:d | found addAll: (self applicationDirectoriesIn: d asFilename)].
		found]! !

!ApplicationManagement class methodsFor: 'installation'!

productVersion
	"return the version number of this Application Management"

	^1.1! !

!ApplicationManagement class methodsFor: 'installation'!

productIdentifier
	"return the name and version number of this Application Management"

	^'Application Management ', self productVersion printString! !

!ApplicationManagement class methodsFor: 'installation'!

findBackupsFor: anApplication 
	"find (and return) original versions of selectors in the given application;
	if a file <app.name>.bak exists (contents = storeString of a ChangeSequence)
	then use the versions found there; otherwise scan the system source file"

	| bak f |
	Transcript cr; show: 'recovering original sources for ', anApplication name , '...'.
	bak := (f := (anApplication name , '.bak') asFilename) exists
		ifTrue:
			[| sys |
			sys := SourceFileManager default fileAt: 1.
			([ChangeSequence readFrom: (f := f readStream)] valueNowOrOnUnwindDo: [f close])
				do: [:chg | chg instVarAt: 1 put: sys]]
		ifFalse:
			[| b |
			b := MethodDefinitionBackup new.
			anApplication recoverDefault do: [:chg | chg changeClassToThatOf: b]].
	Transcript show: 'done '.
	^bak! !

!ApplicationManagement class methodsFor: 'installation'!

uninstall
	"unregister all known applications;
	remove Application Management applications with all classes and methods
	(APL-Interface, APL-SystemLink, ExternalSource, APL-ChangeSequence, 
	APL-Framework,
	but not BASE, X_Changes, Version, AccessControl, StorableObject)"

	| apm app selectors classes recover |
	(apm := self applicationManagement) logging: false.

	recover := [:ap | ap setChanges: (self findBackupsFor: ap)].

	(ApplicationBrowser allInstances, ApplicationManagementBrowser allInstances) do:
		[:brw | brw dependents isEmpty
			ifFalse: [brw dependents first topComponent controller close]].

	#('APL-Interface' 'APL-SystemLink') do: [:appname | | ap |
		recover value: (ap := apm applicationNamed: appname).
		ap deinstall].

	recover value: (app := apm applicationNamed: 'ExternalSource').
	app reinstallBackups.
	selectors := app selectors.
	classes := app ownClassNames collect: [:cname | cname asClass].

	app := apm applicationNamed: 'APL-ChangeSequence'.
	selectors addAll: app selectors.
	classes addAll: (app ownClassNames collect: [:cname | cname asClass]).

	app := apm applicationNamed: 'APL-Framework'.
	apm sortedApplicationIdentifiers do: [:id | | ap |
		((ap := apm applicationIdentified: id) == app or: [ap == apm systemChanges])
			ifFalse: [apm unregister: ap asBackup]].
	app
		removeClassSubset: Object;
		removeClassSubset: BinaryObjectStorage;
		removeClassSubset: BOSSWriter.
	app classNames do: [:cname | (app definesClass: cname)
			ifFalse: [app removeFromSystem: cname asClass]].
	classes addAll: (app classNames collect: [:cname | cname asClass]).

	"ExternalSource, APL-ChangeSequence, APL-Framework"
	(ChangeSet superclassOrder: classes)
		reverseDo: [:c | c removeFromSystem].
	selectors do: [:assoc | | c | (c := assoc value asClass) == nil
		ifFalse: [c removeSelector: assoc key]].

	Smalltalk removeKey: #ApplicationManager! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

timeStamp: aStream 
	"write the tool product release id and current time on aStream"

	| dateTime |
	dateTime := Time dateAndTimeNow.
	aStream nextPutAll: self class productIdentifier; nextPutAll: ' on '; print: dateTime first; nextPutAll: ' at '; print: dateTime last! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                   

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             


' .ApplicationSource APL-Framework(1.7)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 9 May 1994 at 3:11:59 pm'!



"fix: backup must be done if imported code is equal"!

!BOSSReader methodsFor: 'reading-imports'!

importCompiledCode: info 
	"convert info (an Array) to a BOSSCompiledCodeHolder;
	add it to compiledCode (if it defines new or different code)"

	self change: info toInstanceOf: BOSSCompiledCodeHolder.
	info useSource: useSource; computeCodeWithTransformation: (storage readerBlockFor: info codeClass).
	readingClasses
		ifTrue: 
			[| oldCode |
			(oldCode := info existingCode) = info code ifTrue:
				["same code already installed; don't disturb it, but update source pointer"
				| sel |
				sel := oldCode who last.
				self applicationManagement application changes
					backupSelector: sel
					protocol: (oldCode mclass organization categoryOfElement: sel)
					class: oldCode mclass.
				^useSource == #keep
					ifTrue: [oldCode	"do not change current source"]
					ifFalse: [oldCode sourcePointer: info sourcePointer]]]
		ifFalse: [info dontInstall].

	"let info stand in for the actual compiled method until final commit"
	compiledCode == nil ifTrue: [compiledCode := OrderedCollection new].
	^compiledCode addLast: info! !

"drop certain selectors during file out"!

!ClassSubset methodsFor: 'file out'!

filesOutProtocol: protocolName 
	"return true if the given protocol should be filed out
	(default is false for any protocol starting with #generated)"

	^('generated*' match: protocolName) not! !

!ClassSubset methodsFor: 'file out'!

fileOutSourceOn: aSourceStream 
	"copied and modified from ClassDescription"

	| class |
	class := self mappedClass.
	self includesDefinition
		ifTrue: 
			[(class isMeta and: [class instVarNames isEmpty])
				ifFalse: [aSourceStream cr; cr; emphasis: #bold; store: class definition].
			class organization storeCommentOn: aSourceStream.
			aSourceStream cr].

	class organization categories do: [:p | (self filesOutProtocol: p)
		ifTrue: [self fileOutProtocol: p on: aSourceStream]
		ifFalse: [aSourceStream willUpdateSourcePointers
			ifTrue: [(self selectorsInProtocol: p) do:
				[:sel | (class compiledMethodAt: sel) sourcePointer: nil]]]].

	(class isMeta and: [self includesSelector: #initialize])
		ifTrue: [aSourceStream initializeThisClass: class soleInstance]! !

!ClassSubset methodsFor: 'file out'!

selectorsToFileout
	"return all registered selectors which should be filed out, ordered as defined by 
	the underlying class' organization"

	| all org |
	org := self mappedClass organization.
	all := OrderedCollection new: org elements size.
	org categories do: [:cat | (self filesOutProtocol: cat)
			ifTrue: 
				[| subcat |
				subcat := self at: cat ifAbsent: [#()].
				(org listAtCategoryNamed: cat)
					do: [:sel | (subcat includes: sel) ifTrue: [all add: sel]]]].
	^all! !

!ApplicationBOSSWriter methodsFor: 'writing'!

nextPutClassComment: aClass 
	"store the comment of aClass; assumes source mode set properly;
	special mode for comments held by applications other than the receiver's "

	| org cmtRef array |
	(useSource == #discard or: [(org := aClass organization) hasNoComment])
		ifTrue: [^self].
	array := (cmtRef := org globalComment) class == SourceReference
			ifTrue: [cmtRef sourceFileHolder == application
				ifTrue:
					[Array with: aClass with: #setComment: with: cmtRef]
				ifFalse: 
					[DialogView warn: 'The comment of class ''' , aClass name , ''' will not be stored.\It is still managed by application \' withCRs , cmtRef sourceFileHolderId storeString.
					^self]]
			ifFalse: 
				[Array with: aClass with: #comment: with: aClass comment].
	self
		trace: array
		body: array
		class: BinaryObjectStorage indexImportSend! !

!ApplicationBOSSWriter methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection withSource: sourceMode 
	"adaption of #nextPutClasses:withSource: to the needs of application management: 
	write current state of class definition and compiled code (usually sourceMode 
	is #reference)"

	| classes ordered inits postInits table |
	useSource := sourceMode.
	table := Dictionary withAll: (subsetCollection collect: [:subset | subset mappedClass -> subset]).
	"don't forget meta subsets if no soleInstance subset is present"
	classes := table keys collect: [:c | c isMeta ifTrue: [c soleInstance] ifFalse: [c]].
	ordered := OrderedCollection new.
	classes copy do: [:c | self enumerateClass: c from: classes to: ordered].
	ordered := (ordered select: [:c | table includesKey: c]) asArray.
	inits := OrderedCollection new.
	postInits := application postInstallationClassInits.

	self nextPut: ordered do: 
		[ordered do: 
			[:cls | | subset |
			Transcript space; show: cls name.
			currentClass := cls.
			(subset := table at: cls ifAbsent: [ClassSubset new]) includesDefinition
				ifTrue: 
					[self trace: (BOSSContents new object: cls).
					self nextPutClassComment: cls]
				ifFalse: 
					[self trace: cls].

			"store selectors as ordered by system"
			subset selectorsToFileout do: [:s | self trace: (cls compiledMethodAt: s)].
			(cls isMeta
				and: [(subset includesSelector: #initialize)
				and: [(postInits includes: cls soleInstance) not]])
				ifTrue: [inits add: cls]].

		inits do: [:c | | init |
			init := MessageSend receiver: c soleInstance selector: #initialize.
			init := Array with: #deferSend: with: init.
			self trace: init body: init class: BinaryObjectStorage indexImportSendSelf]]! !

"improve installation script creation: drop elements from required applications"!

!Application methodsFor: 'prerequesites'!

subApplicationsDo: aBlock level: level skip: applications 
	"evaluate aBlock for each of the required applications and level 
	(recursively through all the required applications, skip those and all 
	sub-applications occurring in 'applications')"

	necessary asSortedCollection do: 
		[:appName | | sub |
		sub := self applicationManagement applicationNamed: appName
				ifAbsent: [self notify: 'Application ''' , appName , ''' is not loaded.'. nil].
		(applications includes: sub)
			ifTrue: ["break recursion cycles"]
			ifFalse: 
				[aBlock value: sub value: level.
				sub subApplicationsDo: aBlock level: level + 1 skip: (applications copyWith: sub)]]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallScript: sortedApplications platform: aString on: aFilename
	"write out code which installs applications in given order; if aString is nil,
	loading will search the path of the installation file at execution time;
	otherwise the paths of the applications will be used"

	| export specs stamp aplfram template |
	export := aString notNil.
	specs := sortedApplications collect: [:app | Array
				with: (export ifTrue: [app label filenameForPlatform: aString] ifFalse: [app name])
				with: app majorVersion
				with: (export ifFalse: [app path asString])].
	self timeStamp: (stamp := WriteStream on: (String new: 100)).

	aplfram := self applicationNamed: 'APL-Framework'.
	template := OSErrorHolder errorSignal
		handle:
			[:ex | 
			DialogView warn: 'Could not read file ''install.tmp''\in path of application ''' withCRs, aplfram name, ''''.
			^nil]
		do: [(aplfram minorDirectory / 'install.tmp') asFilename contentsOfEntireFile].

	template := ((template copyReplaceAll: '$APPID' with: sortedApplications last identifier)
				copyReplaceAll: '$TIMESTAMP' with: stamp contents)
				copyReplaceAll: '$APPLICATIONSPECS' with: specs asArray printString.
	aFilename writeStream nextPutAll: template; close! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallation: anApplication 
	"request target platform/path from user, create installation file <path>/'install.st'; 
	if the target platform is not nil, copy sources to <path> 
	(see also #fileOutInstallation:on:path:platform:)"

	| param path instFilename platform |
	param := ApplicationBrowser
				request: 'Enter parameters for installation' asText allBold
				path: anApplication path asString
				filename: 'install.st'
				onCancel: [^self].

	(path := param first asFilename) exists
		ifTrue: [path isDirectory ifFalse: [^DialogView warn: path asString, '\must be a directory.' withCRs]]
		ifFalse: [^DialogView warn: path asString, '\does not exist.' withCRs].

	(instFilename := param at: 2) == nil
		ifFalse: [((instFilename := (path / instFilename) asFilename) checkExistMode: #new)
			ifFalse: [^self]].

	(platform := param last) == nil
		ifFalse: [(path asString findString: anApplication directory asString startingAt: 1) == 0
			ifFalse: [^DialogView warn: 'Installation is not allowed in the application directory\if you want to copy sources.' withCRs]].

	self
		fileOutInstallation: anApplication
		on: instFilename
		path: path
		platform: platform! !

"fix: applications did not register if errors occurred during load"!

!ApplicationManagement methodsFor: 'fileIn/Out'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                     

!ApplicationManagement methodsFor: 'testing'!

checkRequired: anArray
	"check if the applications specified by the elements of an Array are installed;
	add each element to the set of required applications of the currently logging application"

	| missing app s |
	missing := OrderedCollection new.
	app := self application.
	anArray do: [:name | 
		self applicationNamed: name ifAbsent: [missing add: name].
		app includeApplication: name].

	missing isEmpty ifTrue: [^self].
	(s := TextStream on: (String new: 1000))
		cr; nextPutAll: 'Some applications supporting '; store: app identifier;
		cr; nextPutAll: 'are missing: '.
	missing do: [:name | s crtab; nextPutAll: name].
	s cr; cr.
	(DialogView confirm: s contents, 'Do you want to proceed?' initialAnswer: false)
		ifFalse: [self halt: 'cancel'].

	s
		nextPutAll: 'Try to load them with';
		crtab; emphasis: #italic; nextPutAll: 'load...'; emphasis: nil;
		cr; nextPutAll: 'from Application Manager browser'.

	ComposedTextView open: (ValueHolder with: (s := s contents))
			label: 'Missing applications'
			icon: (Icon constantNamed: #workspace)
			extent: s asComposedText extent + 30! !

!Application methodsFor: 'prerequesites'!

includeApplication: stringOrApplication 
	"register the use of stringOrApplication"

	| n |
	(n := stringOrApplication) isString ifFalse: [n := n name].
	n = self name ifTrue: [^nil].
	necessary == nil ifTrue: [necessary := OrderedCollection new].
	^(necessary includes: n)
		ifFalse: [necessary add: n]! !

"fix: handle insufficient file access right during #newMinorVersion
- provide version change broadcast after new minor/major version"!

!Application methodsFor: 'version management'!

newMinorVersion
	"increment the minor version number; 
	save current changes to source file and history, redirect source code pointers;
	broadcast the change of version to dependent browsers;
	return nil if any error or cancel from user occurs"

	| latest file |
	self version isInitialVersion ifTrue:
		[DialogView userError: 'Minor versions are not allowed\before the first major version release' withCRs. ^nil].

	self changes isEmpty ifTrue:
		[(DialogView confirm: 'There are no changes.\Do you want a new minor version anyway?' withCRs initialAnswer: false)
			ifFalse: [^nil]].

	(latest := self latestMinor) > self version ifTrue:
		[DialogView userError: ('This application has already been released to version\', latest printString, '\Try to upgrade and make your release afterwards') withCRs. ^nil].

	latest < self version copy untouch
		ifTrue: [(DialogView confirm: ('This application has not been released since version\', latest printString, '\Do you want to proceed?') withCRs initialAnswer: false)
			ifFalse: [^nil]].

	file := OSErrorHolder noPermissionsSignal
		handle: [:ex |
			DialogView warn: ex signal errorString, '.\You are not allowed to create the new minor version' withCRs. ^nil]
		do: [self instSourceFilename makeWritable].

	[self fixChanges: true] valueNowOrOnUnwindDo: [file makeUnwritable]! !

!Application methodsFor: 'version management'!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp newName newMajor major |

	(DialogView confirm: ('Do you really want to create a new version of\''' , self identifier , ''' ?') withCRs)
		ifFalse: [^self].

	"- assign a new path ---------------------------"
	((newApp := self cleanUp copy) requestPath: #path) == nil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	"- assign a new name ---------------------------"
	newName := DialogView
			request: 'Enter the name of the new major version'
			initialAnswer: self name
			ifEmpty: [^self].
	self name = newName
		ifFalse: [(DialogView confirm: 'You entered ''', newName, '''\as new application name.\Is this correct?' withCRs)
			ifFalse: [^self]].
	newApp label name: newName.

	"- assign a new major version number ----------------"
	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) > 0
		ifFalse: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor <= major
		ifTrue: [newApp path / newName = (self path / self name)
			ifTrue: [DialogView userError: 'You must specify a new path
if an old major number is used'. ^self]
			ifFalse: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]]
		ifFalse: [major + 1 = newMajor
			ifFalse: [(DialogView confirm: 'The major number increment is greater than 1.
Do you want to proceed?' initialAnswer: false)
				ifTrue: [newApp version: (VersionDescriptor major: newMajor - 1 minor: 0)]
				ifFalse: [^self]]].

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	"- basic storing sequence ---------------------------"
	newApp incrementMajorVersion; storeSource; storePools; noChanges.
	self applicationManagement unregister: self; register: newApp.
	self asBackup; changed: #version.
	newApp storeBinary.	"place this near the end to minimize errors"
	^newApp! !

Application removeSelector: #storeForDOS!

Application removeSelector: #checkConflicts!

ApplicationManagement removeSelector: #fileOutInstallation:path:platform:!

"remove selectors used in class Application only;
avoid adding changes to Application 'BASE' "

self applicationManagement
	logTo: self applicationManagement systemChanges
	while: [Filename class removeSelector: #path:name:; removeSelector: #path:name:extension:]!


' .ApplicationSource APL-Framework(1.8)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 18 May 1994 at 11:28:55 am'!



Application removeSelector: #requestCompatibleName:!

Application removeSelector: #setNameForDos!

!Application methodsFor: 'filenames'!

 !                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        

!Application methodsFor: 'filenames'!

checkPath
	"image platform migration or file reorganizations may invalidate the path 
	specifications; determine an invalid part (path or application directory name); 
	correct it with interaction of the user; return nil if the current specification 
	remains unmodified"

	| path ok |
	ok := (path := self path) == nil
			ifTrue: [true]
			ifFalse: [(SignalCollection with: OSErrorHolder inaccessibleSignal with: OSErrorHolder invalidArgumentsSignal)
				handle: [:ex | ex returnWith: false]
				do: [path exists]].
	ok ifFalse: [^self requestPath: #error].
	self label filenameForPlatform: OSHandle currentOS
		ifNone: [self requestPlatformNamesOnCancel: [^nil]].
	^self directory exists ifFalse: [self requestPath: #error]! !

"assign actual path earlier"!

!ApplicationManagement methodsFor: 'fileIn/Out'!

load: application from: aBOS path: path 
	"load the application (structure with class & method definitions) from aBOS;
	assign the given path after installation"

	| app old conflicts |
	Transcript cr; show: 'loading ' , application asString, ' (structure)... '.
	aBOS resizeMaps: aBOS nextIndex // 4 * 3 // 2.	"avoid growing of the bos' maps"
	app := Cursor read showWhile: [[aBOS next] valueNowOrOnUnwindDo: [aBOS close]].

	(old := self applicationNamed: app name) == nil
		ifFalse: [(DialogView confirm: ('Do you want to overwrite\''', old identifier, ''' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['\(changes will get lost)'])) withCRs initialAnswer: false)
			ifTrue:
				[conflicts := app checkConflictsWith: old.
				self unregister: old]
			ifFalse: [Transcript show: 'aborted '. ^nil]].

	"set the correct path"
	app label path: path.
	Transcript show: '(checks)... '.
	self do: [self logTo: app while: [self checkRequired: app necessary]]
		onCancel:
			[old == nil ifFalse: [self register: old].
			Transcript show: 'aborted '. ^nil].

	[Transcript show: '(pre-installation)... '.
	app preInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	Transcript show: '(source)... '.
	self logTo: app while: [Cursor read showWhile: [aBOS sourceMode: #reference; nextClasses; close]].

	Transcript show: '(post-installation)... '.
	app postInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	app setChanges: (app changes select: [:chg | chg isBackup]); clearChanges; definePoolDictionaries.
	Transcript show: 'done ']

		valueNowOrOnUnwindDo: [self register: app; changed: #version].

	conflicts == nil ifFalse: [self browseConflicts: conflicts].
	^app! !

!Application methodsFor: 'fileIn/Out'!

exportForPlatform: aString destinationPath: dirFilename
	"copy the directory of the receiver to directory dirFilename;
	return nil if errors occur (self otherwise)"

	| src dest srcfile |
	(src := self directory) = (dest := (dirFilename / (self filenameForPlatform: aString)) asFilename)
		ifTrue:
			[DialogView userError: '''', dirFilename asString, '''\is invalid as export directory.' withCRs.
			^nil].
	"careful if this has been done in a previous export"
	srcfile := (dest / ('v', self majorVersion printString) / 's.st') asFilename.
	OSErrorHolder errorSignal handle: [:ignore ] do: [srcfile makeWritable].
	[src copyRecursiveTo: dest] valueNowOrOnUnwindDo: [srcfile makeUnwritable]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallation: anApplication on: instFilename path: dirFilename platform: aString 
	"create a file <instFilename> (if argument is not nil) with code to install anApplication 
	and (recursively) all its sub-applications; if aString is 
	- nil : do not copy the sources, use application paths during installation 
	- platform specifier : copy the application directory tree to <dirFilename>, obeying 
	platform characteristics, loading will search the path of the installation file at 
	execution time"

	| required drop apps abort |
	required := (apps := anApplication necessary) isEmpty
		ifTrue: [apps]
		ifFalse: [ApplicationBrowser multipleSelect: apps prompt: 'include applications' onCancel: [^self]].

	abort := [:msg | (DialogView confirm: (msg, '\Proceed without it?') withCRs) ifFalse: [^self]].
	apps := apps collect: [:appName | self applicationNamed: appName
				ifAbsent: [abort value: 'Application ''' , appName , ''' is not loaded.'. nil]].
	drop := apps removeAllSuchThat: [:app | app == nil or: [(required includes: app name) not]].
	required := apps.

	"recursively collect required applications; sort wrt. sub-application property"
	apps := Dictionary with: anApplication -> 0.
	required do: [:app | apps at: app put: 1].
	Object notifySignal
		handle: [:ex | abort value: ex errorString]
		do: [required do: [:app | app subApplicationsDo: [:ap :lev | apps at: ap put: lev] level: 2 skip: drop]].
	apps := (apps associations asSortedCollection: [:x :y | x value > y value])
				collect: [:assoc | assoc key].

	"make sure that the target platform filenames are defined"
	aString == nil ifFalse: [apps do: [:app | app label filenameForPlatform: aString ifNone: [app requestPlatformNamesOnCancel: [DialogView userInfo: 'Installation file out aborted.'. ^nil]]]].

	Cursor write showWhile:
		[instFilename == nil
			ifFalse: [(self fileOutInstallScript: apps platform: aString on: instFilename) == nil
				ifTrue: [^self]].
		aString == nil ifTrue: [^self].
		Transcript cr; show: 'copying sources...'.
		apps do: [:app | 
			Transcript crtab; show: app name, '...'.
			((app exportForPlatform: aString destinationPath: dirFilename) == nil
				and: [DialogView confirm: 'Do you want to abort the installation file out?'])
				ifTrue: [^self]].
		Transcript cr; show: 'done.']! !

!Application methodsFor: 'prerequesites'!

reportUseOfApplications: applicationNames 
	"open a workspace reporting references to classes defined in 
	applicationNames"

	| st apps |
	apps := applicationNames collect: [:appName |
		self applicationManagement applicationNamed: appName
			ifAbsent: [(DialogView confirm: 'The required application ''' , appName , ''' is not loaded.\Do you want to proceed without it?' withCRs)
						ifFalse: [^self]]].
	(apps := apps copyWithout: nil) isEmpty
		ifTrue: [DialogView userInfo: 'Nothing to report.'. ^self].

	st := TextStream on: (String new: 1000).
	Cursor execute showWhile:
		[apps do: [:app | self reportUseOfApplication: app on: st. st cr]].

	ComposedTextView
		open: (ValueHolder with: (st := st contents))
		label: 'Application dependency of ''' , self identifier , ''''
		icon: (Icon constantNamed: #workspace)
		extent: 500 @ 300! !

"optimization"!

!Application methodsFor: 'initialize-release'!

cleanUpProtocols
	"remove empty class protocols"

	| removed |
	removed := false.
	classes do: [:subset | subset removeEmptyCategories ifTrue: [removed := subset isEmpty]].
	removed ifTrue: [self cleanUpClasses]! !

!Application methodsFor: 'private'!

protocolFor: selector class: aClass 
	"system protocol for selector, if none return the one registered, if none return 
	default"

	| prt |
	^(prt := aClass asClass organization categoryOfElement: selector) == nil
		ifTrue: 
			[| def |
			def := ClassOrganizer defaultProtocol.
			(self classSubsetOf: aClass ifAbsent: [^def])
				keysAndValuesDo: [:cat :set | (set includes: selector)
						ifTrue: [^cat]].
			def]
		ifFalse: [prt]! !


' .ApplicationSource APL-Framework(1.9)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 20 May 1994 at 12:37:19 pm'!



!Application methodsFor: 'filenames'!

requestPlatformNamesOnCancel: aBlock
	"open a dialog to define application names compatible to supported platforms;
	return the name map (result of aBlock on cancel)"

	| map |
	map := ApplicationBrowser
				request: 'Enter platform compatible names for\''' withCRs , self identifier , ''''
				platformNames: self label platformNameMap
				default: self name 
				onCancel: [^aBlock value].
	map keysAndValuesDo: [:pf :pfname | pfname isEmpty
		ifFalse: [self label setFilename: pfname forPlatform: pf]].
	^map! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallScript: sortedApplications commonPath: aBoolean on: aFilename
	"write out code which installs applications in given order; if aBoolean is true,
	loading will search the path of the installation file at execution time;
	otherwise the paths of the applications will be used"

	| specs stamp aplfram template |
	specs := sortedApplications collect: [:app | Array
				with: app name
				with: app majorVersion
				with: (aBoolean ifFalse: [app path asString])].
	self timeStamp: (stamp := WriteStream on: (String new: 100)).

	aplfram := self applicationNamed: 'APL-Framework'.
	template := OSErrorHolder errorSignal
		handle:
			[:ex | 
			aplfram checkPath == nil
				ifTrue:
					[DialogView warn: 'Could not read file ''install.tmp''\in path of application ''' withCRs, aplfram name, ''''.
					^nil]
				ifFalse: [ex restart]]
		do: [(aplfram minorDirectory / 'install.tmp') asFilename contentsOfEntireFile].

	template := ((template copyReplaceAll: '$APPID' with: sortedApplications last identifier)
				copyReplaceAll: '$TIMESTAMP' with: stamp contents)
				copyReplaceAll: '$APPLICATIONSPECS' with: specs asArray printString.
	aFilename writeStream nextPutAll: template; close! !


' .ApplicationSource APL-Framework(1.10)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 September 1994 at 8:37:15 pm'!



"fixes:
- return nil instead of empty chunks on source access
- error during use of classes search when classes with superclass nil occur
- releasing applications left SourceRefences on class comments
- apply a unique algorithm for defining class orderings during binary and normal file out
- use appl. fileString (instead of name) in specification for install script
- use lineEndTransparent mode for application source files
- DosFilename separators were converted to CRs during #requestPath:
- do not change other application paths after changing an invalid path
- mnu (#parameters) on applying the same-filter on CategoryRenameChanges

add ons:
- application usage query operator
- change paths operator
- release unused source files operator
- defining ordering of classes for file-in/out with an own operator
- log file-in from files matching install* to system changes
- no transcript output upon logging redirection to system changes"!

!Application methodsFor: 'accessing-source'!

stringAt: anInteger ifAbsent: aBlock 
	"return the string stored in the receiver's source file at position anInteger; 
	return the result of aBlock if the source file is not accessible"

	^Stream positionOutOfBoundsSignal
		handle:
			[:ex | "handle dummy source file (an empty ReadStream)"
			ex returnWith: aBlock value]
		do: [OSErrorHolder inaccessibleSignal
				handle: 
					[:ex | "handle incompatible source file due to platform migration"
					self releaseSourceFile.
					ex restart]
				do:
					[| chk | 
					(chk := self sourceFile position: anInteger; nextChunk) isEmpty
						ifTrue: aBlock
						ifFalse: [chk]]]! !

!ApplicationBOSSWriter methodsFor: 'private-initialize-release'!

storage: aBOS on: aStream registry: reg 
	"find out the application calling this store;
	force Application other than this to store a generic expression;
	force SourceReferences to represent in the default way;
	force classes defined by the application to represent without version"

	| ctx |
	super storage: aBOS on: aStream registry: reg.
	ctx := thisContext.
	[application := ctx == nil
		ifTrue: [self applicationManagement currentApplication]
		ifFalse: [ctx receiver].
	application isKindOf: Application] whileFalse: [ctx := ctx sender].

	self traceSelectorFor: SourceReference is: #traceDefault:.
	self traceSelectorFor: Application is: #traceApplication:.
	application classNames do: [:c | | cls |
		((cls := c asClass) binaryRepresentationVersion notNil
			and: [(application definesClass: c)])
			ifTrue: [self traceSelectorFor: cls class is: #traceClassNoVersion:]]! !

!ApplicationManagement methodsFor: 'inquiry'!

applicationsRequiring: anApplication 
	^applications select: [:app | app requiresApplication: anApplication]! !

!Application methodsFor: 'prerequesites'!

useOfClasses: classNames do: aBlock 
	"evaluate aBlock with each own class or own class' superclass included in classNames; 
	evaluate aBlock for each own selector whose code refers to the given classes"

	self classNames do: 
		[:cname | | requiredClass |
		requiredClass := (self definesClass: cname)
					ifTrue: [| sup | (sup := cname asClass superclass) == nil
						ifTrue: []
						ifFalse: [sup name]]
					ifFalse: [cname].
		(classNames includes: requiredClass)
			ifTrue: [aBlock value: cname value: requiredClass]].

	classNames do: 
		[:cname | | calls |
		(calls := self allCallsOn: (Smalltalk associationAt: cname ifAbsent: [])) isEmpty
			ifFalse: [aBlock value: cname value: calls]]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

releaseSourceFiles
	"release source files of registered applications;
	return true if at least one of them could be released (false otherwise)"

	| files active |
	files := IOAccessor registry size.
	active := self application.
	applications do: [:app | app == active ifFalse: [app releaseSourceFile]].
	^IOAccessor registry size < files! !

!Application methodsFor: 'private'!

moveMethodsInSourcefileToChanges
	"move all method source code and comments in my source file to system changes"

	classes do: 
		[:subset | | cls cmtref set |
		((cmtref := (cls := subset mappedClass) organization globalComment) class == SourceReference
			and: [cmtref sourceFileHolder == self])
			ifTrue:
				[cls comment: cls comment	"reassign comment, writes to changes file"].
		SourceFileManager default
			fileOutMessages: (set := subset selectors)
			for: cls
			logging: false.
		"care for valid source references, especially for elements in change sequences"
		self applicationManagement updateChanges: set class: cls name]! !

!Application methodsFor: 'private'!

classDefinitionOrder: collectionOfClasses 
	"define and return the ordering doing best during filing in the given classes"

	^(Smalltalk at: #SystemUtils
		ifAbsent: [^ChangeSet superclassOrder: collectionOfClasses])
		sortForLoading: collectionOfClasses! !

!Application methodsFor: 'fileIn/Out'!

fileOutClasses: classNameList on: aStream 
	"file out the current source code for all registered elements 
	(definition, comment, methods, etc.) of the classes in classNameList"

	| classList first |
	first := true.
	aStream deferInitializations.
	classList := classNameList collect: 
			[:name | | class |
			(class := name asClass) isMeta
				ifTrue: [class soleInstance]
				ifFalse: [class]].
	(classList := self classDefinitionOrder: classList) do: 
			[:class | 
			first
				ifTrue: [first := false]
				ifFalse: [aStream cr; newPage; cr].
			self fileOutClass: class on: aStream].
	aStream removeInitializations: self postInstallationClassInits.
	aStream cr; finishInitializations; cr! !

!Application methodsFor: 'print out'!

printOutClasses: classNameList on: aStream 
	| realClasses last |
	realClasses := classNameList collect: [:cName | cName asClass].
	realClasses := realClasses select: [:class | class isMeta not].
	realClasses := self classDefinitionOrder: realClasses.
	last := realClasses last.
	realClasses do: 
		[:class | 
		self printOutClassWithMeta: class on: aStream.
		class == last ifFalse: [aStream cr; nextPut: Character newPage]].
	aStream cr; cr.
	realClasses do: [:class | self printOutClassInit: class on: aStream]! !

!Application methodsFor: 'changing-categories'!

removeAllCategories
	"remove ALL classes from system (meta and soleInstance) 
	(restricted to the corresponding class subsets of the receiver)"

	| classList cats org |
	cats := self categories.
	classList := (self allClassNamesIn: self classNames)
				collect: [:cname | cname asClass].
	Cursor execute showWhile:
		[(self classDefinitionOrder: classList)
			reverseDo: [:c | self removeFromSystem: c]].

	"remove empty categories"
	org := Smalltalk organization.
	cats do: [:cat | (org listAtCategoryNamed: cat) isEmpty
		ifTrue: [org removeCategory: cat. ChangeSet current reorganizeSystem]].! !

!Application methodsFor: 'changing-categories'!

removeCategory: categoryName 
	"remove all classes in specified category from system (meta and soleInstance) 
	(restricted to the corresponding class subsets of the receiver)"

	| classList |
	classList := (self classNamesInCategory: categoryName)
				collect: [:name | name asClass].
	(self classDefinitionOrder: classList)
		reverseDo: [:c | self removeFromSystem: c]! !

!ApplicationManagement class methodsFor: 'installation'!

uninstall
	"unregister all known applications;
	remove all classes and methods of Application Management applications
	(APL-Interface, APL-SystemLink, ExternalSource, APL-ChangeSequence, 
	APL-Framework,
	but not BASE, X_Changes, Version, AccessControl, StorableObject)"

	| apm app selectors classes recover |
	(apm := self applicationManagement) logging: false.

	recover := [:ap | "ap setChanges: (self findBackupsFor: ap)"].

	(ApplicationBrowser allInstances, ApplicationManagementBrowser allInstances) do:
		[:brw | brw dependents isEmpty
			ifFalse: [brw dependents first topComponent controller close]].

	#('APL-Interface' 'APL-SystemLink') do:
		[:appname | | ap |
		recover value: (ap := apm applicationNamed: appname).
		ap deinstall].

	recover value: (app := apm applicationNamed: 'ExternalSource').
	app reinstallBackups.
	selectors := app selectors.
	classes := app ownClassNames collect: [:cname | cname asClass].

	app := apm applicationNamed: 'APL-ChangeSequence'.
	selectors addAll: app selectors.
	classes addAll: (app ownClassNames collect: [:cname | cname asClass]).

	app := apm applicationNamed: 'APL-Framework'.
	apm sortedApplicationIdentifiers do: [:id | | ap |
		((ap := apm applicationIdentified: id) == app or: [ap == apm systemChanges])
			ifFalse: [apm unregister: ap asBackup]].
	app
		removeClassSubset: Object;
		removeClassSubset: BinaryObjectStorage;
		removeClassSubset: BOSSWriter.
	app classNames do: [:cname | (app definesClass: cname)
			ifFalse: [app removeFromSystem: cname asClass]].
	classes addAll: (app classNames collect: [:cname | cname asClass]).

	"ExternalSource, APL-ChangeSequence, APL-Framework"
	(app classDefinitionOrder: classes)
		reverseDo: [:c | c removeFromSystem].
	selectors do: [:assoc | | c | (c := assoc value asClass) == nil
		ifFalse: [c removeSelector: assoc key]].

	Smalltalk removeKey: #ApplicationManager! !

!BinaryObjectStorage methodsFor: 'private-initialize-release'!

resizeMaps: newSize
	"resize the reader/writer maps to the specified size (number of slots)"

	(readerMap size max: writerMap size) > newSize
		ifTrue: [^self error: 'invalid size in resizing'].
	readerMap == nil ifFalse: [readerMap changeCapacityTo: newSize].
	writerMap == nil ifFalse: [writerMap changeCapacityTo: newSize]! !

!ApplicationBOSSWriter methodsFor: 'writing'!

nextPutClassSubsets: subsetCollection withSource: sourceMode 
	"adaption of #nextPutClasses:withSource: to the needs of application management: 
	write current state of class definition and compiled code (usually sourceMode 
	is #reference)"

	| classes inits postInits table |
	useSource := sourceMode.
	table := Dictionary new: (classes := subsetCollection size) * 2.
	classes := OrderedCollection new: classes.
	subsetCollection do: [:subset | table at: (classes add: subset mappedClass) put: subset].
	classes := application classDefinitionOrder: classes.
	inits := OrderedCollection new.
	postInits := application postInstallationClassInits.

	self nextPut: classes do: 
		[classes do: 
			[:cls | | subset |
			Transcript space; show: cls name.
			currentClass := cls.
			(subset := table at: cls ifAbsent: [ClassSubset new]) includesDefinition
				ifTrue: 
					[self trace: (BOSSContents new object: cls).
					self nextPutClassComment: cls]
				ifFalse: 
					[self trace: cls].

			"store selectors as ordered by system"
			subset selectorsToFileout do: [:s | self trace: (cls compiledMethodAt: s)].
			(cls isMeta
				and: [(subset includesSelector: #initialize)
				and: [(postInits includes: cls soleInstance) not]])
				ifTrue: [inits add: cls]].

		inits do: [:c | | init |
			init := MessageSend receiver: c soleInstance selector: #initialize.
			init := Array with: #deferSend: with: init.
			self trace: init body: init class: BinaryObjectStorage indexImportSendSelf]]! !

ApplicationManagement removeSelector: #fileOutInstallScript:commonPath:on:!

!ApplicationManagement class methodsFor: 'installation'!

findBackupsFor: anApplication 
	"find (and return) original versions of selectors in the given application;
	if a file <app.name>.bak exists (contents = storeString of a ChangeSequence)
	then use the versions found there; otherwise scan the system source file"

	| bak f |
	Transcript cr; show: 'recovering original sources for ', anApplication name , '...'.
	bak := (f := (anApplication fileString , '.bak') asFilename) exists
		ifTrue:
			[| sys |
			sys := SourceFileManager default fileAt: 1.
			([ChangeSequence readFrom: (f := f readStream)] valueNowOrOnUnwindDo: [f close])
				do: [:chg | chg instVarAt: 1 put: sys]]
		ifFalse:
			[| b |
			b := MethodDefinitionBackup new.
			anApplication recoverDefault do: [:chg | chg changeClassToThatOf: b]].
	Transcript show: 'done '.
	^bak! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallScript: sortedApplications platform: aString on: aFilename
	"write out code which installs applications in given order; if aString is nil,
	loading will search the path of the installation file at execution time;
	otherwise the paths of the applications will be used"

	| export specs stamp aplfram template |
	export := aString notNil.
	specs := sortedApplications collect: [:app | Array
				with: (export ifTrue: [app label filenameForPlatform: aString] ifFalse: [app fileString])
				with: app majorVersion
				with: (export ifFalse: [app path asString])].
	self timeStamp: (stamp := WriteStream on: (String new: 100)).

	aplfram := self applicationNamed: 'APL-Framework'.
	template := OSErrorHolder errorSignal
		handle:
			[:ex | 
			DialogView warn: 'Could not read file ''install.tmp''\in path of application ''' withCRs, aplfram name, ''''.
			^nil]
		do: [(aplfram minorDirectory / 'install.tmp') asFilename contentsOfEntireFile].

	template := ((template copyReplaceAll: '$APPID' with: sortedApplications last identifier)
				copyReplaceAll: '$TIMESTAMP' with: stamp contents)
				copyReplaceAll: '$APPLICATIONSPECS' with: specs asArray printString.
	aFilename writeStream nextPutAll: template; close! !

(Message selector: #includeOrChangeClass: arguments: #('ExternalSourceCodeStream subclass: #ApplicationSourceStream
	instanceVariableNames: ''application ''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''Kernel-Support''' ))!

!ApplicationSourceStream methodsFor: 'initialize-release'!

on: anApplication 
	application := anApplication.
	super on: anApplication instSourceFilename appendStream lineEndTransparent! !

!Application methodsFor: 'accessing-source'!

sourceFile
	"return (and cache) the file stream containing the sources (open it if necessary);
	handle OSErrorHolder inaccessibleSignal (includes nonexistentSignal), cause
	filename may be invalid due to image platform migration; try to repair filename
	specification interactively if errors occur"

	sourceFile == nil ifFalse: [^sourceFile].

	^OSErrorHolder inaccessibleSignal
		handle:
			[:ex | self checkPath == nil 
				ifTrue:
					["return a dummy, so that continuing without source file is possible"
					sourceFile := '' readStream]
				ifFalse:
					["repeat source file access with new path"
					ex restart]]
		do:
			[| fname |
			fname := self version isInitialVersion ifTrue: [self path] ifFalse: [self instSourceFilename].
			sourceFile := fname asFilename readStream lineEndTransparent]! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInFrom: aStream 
	"file in source code from 'aStream', ask for an application to log all 
	the changes; return the result of the file-in"

	| app external name |
	name := (external := aStream isKindOf: ExternalReadStream)
		ifTrue: [aStream name]
		ifFalse: ['an internal stream'].
	
	app := ('install*' match: name)
		ifTrue: [system]
		ifFalse: [self selectApplicationOrNew: 'Select an application to log the file-in from\' withCRs, name].
	app == nil
		ifTrue: ["empty user selection means cancel; simulate scanning complete"
			aStream setToEnd. ^nil].

	external := external & (app ~~ system) & app path isNil
		and: [DialogView confirm: 'Do you want the source references to be kept external?'].

	^self fileInFrom: aStream to: app external: external! !

!ApplicationManagement methodsFor: 'logging on/off/redirect'!

logTo: anApplication while: aBlock 
	"execute aBlock (typically a file in), log changes into anApplication;
	logging to system application is equivalent to 'logTo==nil' "

	| save logApp |
	logApp := anApplication == system ifFalse: [anApplication].
	(save := logTo) == logApp	"nothing changed"
		ifTrue: [^aBlock value].

	(anApplication isKindOf: Application)
		ifFalse: [^self error: 'bad application for logging'].

	Transcript show: 'Logging redirected to ' , anApplication identifier; cr	.
	logTo := logApp.
	^aBlock valueNowOrOnUnwindDo: [self resetLoggingTo: save]! !

!Application methodsFor: 'filenames'!

requestPath: mode
	"the current path is undefined or invalid; request it from user;
	return a valid path/filename object (nil if user cancels)
	(special mode for initial versions, whose path may be a source file name);
	case mode of
	- nil : request a filename for initial versions with own path, a directory name for others
	- #path : request a directory name unconditionally
	- #error : like nil mode + 'invalid' message + pass new path to other applications holding the invalid one"

	| p old msg initial |
	initial := mode ~~ #path & self version isInitialVersion.
	p := old := self path == nil
		ifTrue: [initial := false. Filename defaultDirectoryName]
		ifFalse: [self path asString].
	msg := initial
		ifTrue: ['source file name']
		ifFalse: ['path to application directory'].
	msg := 'Enter the new ', msg, '\of application ''' withCRs, self identifier, ''''.
	mode == #error ifTrue: [msg := msg, '\(''' withCRs, old, ''' is invalid)'].

	[p := DialogView requestFileName: msg default: p asString version: #mustBeOld ifFail: [^nil].
	(p := p asFilename) isDirectory ~~ initial]
		whileFalse:
			[DialogView warn: 'You must specify a ', (initial ifTrue: ['filename, not a directory'] ifFalse: ['directory'])].

	self setPath: p.
	^p! !

!ApplicationManagement methodsFor: 'accessing'!

changePaths
	"show a menu of paths used by registered applications; selecting one opens a 
	dialog for entering a new path, all references to the selected path will be 
	replaced by the new one"

	| choices oldpath newpath |
	choices := Set new.
	applications do: [:app | (oldpath := app path) == nil
		ifFalse:
			[| p |
			p := oldpath asString.
			(OSErrorHolder errorSignal handle: [:ex | ex returnWith: false] do: [oldpath exists])
				ifFalse: [p := p asText allBold].
			choices add: p]].
	oldpath := choices asSortedCollection select: 'Select a path to modify' fromUserIfEmpty: [^nil].
	oldpath := oldpath asString.
	choices := applications select: 
				[:app | | p |
				(p := app path) notNil and: [p asString = oldpath]].
	choices := choices collect: [:app | app name].
	choices := ApplicationBrowser
				multipleSelect: choices
				prompt: 'Select applications to change path \''' withCRs , oldpath , ''''
				onCancel: [^nil].
	newpath := DialogView
				requestFileName: 'Enter the new path'
				default: oldpath
				version: #directory
				ifFail: [^nil].
	applications do: [:app | (choices includes: app name) ifTrue: [app setPath: newpath]]! !

!CategoryRenameChange methodsFor: 'accessing'!

parameters
	^self text! !


' .ApplicationSource APL-Framework(1.11)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 21 September 1994 at 8:44:34 pm'!



!ApplicationSourceStream methodsFor: 'private'!

sourceFileHolder
	^application! !


' .ApplicationSource APL-Framework(1.12)'!
'From Objectworks\Smalltalk(R), Release 4.1 of 15 April 1992 on 22 September 1994 at 3:05:11 pm'!



"modifications for VisualWorks 2.0"!

!BOSSReader methodsFor: 'reading-imports'!

importCompiledCode: info 
	"convert info (an Array) to a BOSSCompiledCodeHolder;
	add it to compiledCode (if it defines new or different code)"

	self change: info toInstanceOf: BOSSCompiledCodeHolder.
	info useSource: useSource.
	info computeCodeForStorage: storage.
	readingClasses
		ifTrue: 
			[| oldCode |
			(oldCode := info existingCode) = info code ifTrue:
				["same code already installed; don't disturb it, but update source pointer"
				| sel |
				sel := oldCode who last.
				self applicationManagement application changes
					backupSelector: sel
					protocol: (oldCode mclass organization categoryOfElement: sel)
					class: oldCode mclass.
				^useSource == #keep
					ifTrue: [oldCode	"do not change current source"]
					ifFalse: [oldCode sourcePointer: info sourcePointer]]]
		ifFalse: [info dontInstall].

	"let info stand in for the actual compiled method until final commit"
	compiledCode == nil ifTrue: [compiledCode := OrderedCollection new].
	^compiledCode addLast: info! !

!BOSSWriter methodsFor: 'private-tracing'!

trace: anObject body: body class: aClass
	" Record anObject in the objects dictionary,
	and trace its dependents and body if needed.
	The caller has already traced the class. "

	| class objx flags deps |
	deps := anObject binaryStoredDependents.
	class := body class.
	(class isBits and: [deps == nil])
		ifTrue:
			[objx := self assign: anObject.
			self record: anObject body: body class: aClass flags: 0 index: objx.
			^objx].
	" Recursively trace the contents of the body.
	If the object has circular references,
	the trace may assign the object index. "
	anObject == nil ifFalse: [objects at: anObject put: -2].
	flags := BinaryObjectStorage flagHasShortOops.
	deps == nil ifFalse:
		[self
			trace: (anObject -> deps)
			body: (Array with: anObject with: #myDependents: with: deps)
			class: BinaryObjectStorage indexImportSend].
	class isBits
		ifFalse:
			[1 to: class instSize do: [:i |
				((self trace: (body instVarAt: i)) bitAnd: -16r10000) == 0
					ifFalse: [flags := 0]].
			class isVariable ifTrue:
				[1 to: body basicSize do: [:i |
					((self trace: (body basicAt: i)) bitAnd: -16r10000) == 0
						ifFalse: [flags := 0]]]].
	objx := self encode: anObject.
	objx < 0
		ifTrue: [objx := self assign: anObject]
		ifFalse: [flags := flags + BinaryObjectStorage flagHasForwardReferences].
	self record: anObject body: body class: aClass flags: flags index: objx.
	^objx! !

!Behavior methodsFor: 'fileIn/Out'!

binaryStoredDependents
	"(extracted from BOSSWriter>trace:body:class:) 
	classes should not store dependents"

	^nil! !


' .ApplicationSource APL-Framework(1.13)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 27 September 1994 at 6:05:47 am'!



!BufferedExternalStream methodsFor: 'accessing'!

detectLineEndConvention
	"scan the receiver from the beginning for occurrence of CRLF, CR, or LF
	and set the line end convention character appropriately"

	| pos byte |
	pos := self position.
	self reset; binary.
	lineEndConvention := nil.
	[lineEndConvention == nil and: [(byte := self next) notNil]]
		whileTrue:
			[byte = 10
				ifTrue: [lineEndConvention := LineEndLF]
				ifFalse: [byte = 13 ifTrue: [lineEndConvention := (self peekFor: 10)
							ifTrue: [LineEndCRLF]
							ifFalse: [LineEndTransparent]]]].
	lineEndConvention == nil ifTrue: [lineEndConvention := LineEndTransparent].
	self text; position: pos; setLineEndCharacter! !

!Application methodsFor: 'accessing-source'!

sourceFile
	"return (and cache) the file stream containing the sources (open it if necessary);
	handle OSErrorHolder inaccessibleSignal (includes nonexistentSignal), cause
	filename may be invalid due to image platform migration; try to repair filename
	specification interactively if errors occur"

	sourceFile == nil ifFalse: [^sourceFile].

	^OSErrorHolder inaccessibleSignal
		handle:
			[:ex | self checkPath == nil 
				ifTrue:
					["return a dummy, so that continuing without source file is possible"
					sourceFile := '' readStream]
				ifFalse:
					["repeat source file access with new path"
					ex restart]]
		do:
			[| fname |
			fname := self version isInitialVersion ifTrue: [self path] ifFalse: [self instSourceFilename].
			(sourceFile := fname asFilename readStream) detectLineEndConvention]! !

!ApplicationSourceStream methodsFor: 'initialize-release'!

on: anApplication 
	| file |
	application := anApplication.
	file := anApplication instSourceFilename asFilename readAppendStream.
	file detectLineEndConvention.
	super on: file! !


' .ApplicationSource APL-Framework(1.14)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 27 September 1994 at 8:12:14 am'!



!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInFrom: aStream 
	"file in source code from 'aStream', ask for an application to log all 
	the changes; return the result of the file-in"

	| app external name |
	name := (external := aStream isKindOf: ExternalReadStream)
		ifTrue:
			[('install*' match: aStream name asFilename tail) ifTrue: [app := system].
			aStream name]
		ifFalse: ['an internal stream'].
	
	app == nil ifTrue:
		[(app := self selectApplicationOrNew: 'Select an application to log the file-in from\' withCRs, name) == nil
			ifTrue:
				["empty user selection means cancel; simulate scanning complete"
				aStream setToEnd. ^nil]].

	external := external & (app ~~ system) & app path isNil
		and: [DialogView confirm: 'Do you want the source references to be kept external?'].

	^self fileInFrom: aStream to: app external: external! !


' .ApplicationSource APL-Framework(1.15)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 24 January 1995 at 11:10:39 am'!



"fix: invalid source pointers after exclude class/category (Source code not available)"!

!SystemApplication methodsFor: 'classes'!

includeClass: aClass 
	"return nil if another application holds the definition of aClass;
	otherwise include aClass (structure and definition) and return it"

	(self applicationManagement applicationsIncludingDefinition: aClass) isEmpty
		ifFalse: [^nil].
	(self classSubsetOf: aClass ifAbsent: [self addClassSubset: aClass]) addDefinition.
	^aClass! !

!SystemApplication methodsFor: 'selectors'!

includeSelector: selector class: aClass changes: aChangeSequence
	"register class>selector in the application structure (if no other application holds it); 
	remove appropriate changes from aChangeSequence and enter necessary changes 
	for the receiver"

	| selChgs |
	selChgs := aChangeSequence removeSelectorChanges: selector class: aClass.
	(self applicationManagement applicationsIncludingSelector: selector class: aClass) isEmpty
		ifFalse:
			["assume each app found (incl. sys) has all changes due to continuous change logging"
			^self].

	Transcript cr; show: aClass name , '>' , selector , ' included in System Application'.
	self includeSelector: selector class: aClass.
	"enter changes for aClass/selector (include change if none)"
	selChgs isEmpty
		ifTrue: [self changes includeSelector: selector protocol: (self protocolFor: selector class: aClass) class: aClass]
		ifFalse: [self changes addAll: selChgs]! !

!Application methodsFor: 'changing-classes'!

includeOrChangeClass: aString
	"aString is the definition string of a class;
	create/change the class if necessary; include it to the receiver's registry;
	return the class object
	(sent during file in only, when the receiver is the currently logging application)"

	| cls |
	cls := (ReadStream on: aString) skipThrough: $#; upTo: Character cr.
	^((cls := cls asClass) == nil or: [cls definition ~= aString])
		ifTrue: [Object evaluatorClass evaluate: aString]
		ifFalse: [self includeClass: cls. cls]! !

!Application methodsFor: 'changing-classes'!

excludeClass: aClass 
	"exclude aClass, its definition and all its methods from application;
	move selectors and changes not held in other applications to system app.;
	enter exclude changes after removing all others"

	| subset cls clsChgs sys srcChgs sysOldChgs |
	(subset := self removeClassSubset: aClass) == nil ifTrue: [^nil].
	clsChgs := self changes removeClassChanges: (cls := aClass asClass).
	srcChgs := clsChgs shallowCopy.
	sysOldChgs := (sys := self systemApplication) changes shallowCopy.

	"move class definition, selectors and changes to sys (if not held elsewhere)"
	subset selectors do:
		[:sel | 
		sys includeSelector: sel class: cls changes: clsChgs.
		self changes excludeSelector: sel class: cls name].
	subset includesDefinition
		ifTrue:
			[(sys includeClass: cls) == nil ifFalse: [sys addChanges: clsChgs].
			self changes excludeClass: cls name].
	self touch.

	"keep original ordering of class changes"
	(clsChgs := sys changes copyFrom: sysOldChgs size + 1 to: sys changes size) isEmpty
		ifTrue: [^self].
	srcChgs do: [:chg | | idx |
		(idx := clsChgs indexOf: chg) > 0
			ifTrue: [sysOldChgs add: chg. clsChgs at: idx put: nil]].
	clsChgs do: [:chg | chg == nil ifFalse: [sysOldChgs add: chg]].
	sys setChanges: sysOldChgs! !

!Application methodsFor: 'changing-selectors'!

excludeSelector: selector class: aClass 
	"selector is excluded from application, included into system application; 
	all recorded changes move to system application"

	(self removeSelector: selector inSubsetOf: aClass) == nil ifTrue: [^nil]. 
	self systemApplication includeSelector: selector class: aClass changes: self changes.
	"enter an exclude change after removing all others"
	self changes excludeSelector: selector class: aClass.
	self touch! !

"bug fixes:
- misbehavior/errors in 'set directories' for initial applications with ext. source
- release source file after changing the platform compatible name
- filing in class definitions entered changes in peculiar ordering (includeOrChange + backup + change)
- mistakes in scanning the version map file"!

!ApplicationManagement methodsFor: 'accessing'!

changePaths
	"show a menu of paths used by registered applications; selecting one opens a 
	dialog for entering a new path, all references to the selected path will be 
	replaced by the new one"

	| allpaths oldpath newpath apps selectedapps initial |
	allpaths := Set new.
	applications do: [:app | (oldpath := app path) == nil
		ifFalse:
			[| p |
			p := oldpath asString.
			(OSErrorHolder errorSignal handle: [:ex | ex returnWith: false] do: [oldpath exists])
				ifFalse: [p := p asText allBold].
			allpaths add: p]].
	oldpath := allpaths asSortedCollection select: 'Select a path to modify' fromUserIfEmpty: [^nil].
	oldpath := oldpath asString.
	apps := applications select:
				[:app | | p |
				(p := app path) notNil and: [p asString = oldpath]].
	selectedapps := ApplicationBrowser
				multipleSelect: (apps collect: [:app | app name])
				prompt: 'Select applications to change path \''' withCRs , oldpath , ''''
				onCancel: [^nil].
	selectedapps isEmpty ifTrue: [^nil].
	apps := apps select: [:app | selectedapps includes: app name].
	initial := apps first version isInitialVersion.
	newpath := DialogView
				requestFileName: 'Enter the new ', (initial ifTrue: ['file'] ifFalse: ['path'])
				default: oldpath
				version: (initial ifTrue: [#mustBeOld] ifFalse: [#directory])
				ifFail: [^nil].
	apps do: [:app | app setPath: newpath]! !

!Application methodsFor: 'filenames'!

requestPlatformNamesOnCancel: aBlock 
	"open a dialog to define application names compatible to supported platforms; 
	return the name map (result of aBlock on cancel)"

	| map |
	map := ApplicationBrowser
				request: 'Enter platform compatible names for\''' withCRs , self identifier , ''''
				platformNames: self label platformNameMap
				default: self name
				onCancel: [^aBlock value].
	map keysAndValuesDo: [:pf :pfname | pfname isEmpty
			ifFalse: 
				[self label setFilename: pfname forPlatform: pf.
				pf == OSHandle currentOS ifTrue: [self releaseSourceFile]]].
	^map! !

!ChangeSequence methodsFor: 'change management'!

backupClass: aClass 
	"create a change object saving the source code info for the given class;
	after compilation of a class a ClassBuilder informs the change management, 
	inducing a call to this method (via ChangeSet, App...Management, Application);
	to get source before compilation we 'ask someone in the stack'
	(the appropriate handler is expected in ClassBuilder>modifyExistingClass);
	return nil if no backup was possible, a dummy change if definition is equal,
	the newly entered backup change otherwise"

	| oldsrc |
	oldsrc := Signal noHandlerSignal
		handle: [:ex | ex return]	"no class definition text available"
		do: ["try to get a class definition text"
			self applicationManagement class sourceSignal raiseRequest].
	^oldsrc isString ifTrue: [oldsrc = aClass definition
			ifTrue: [OtherChange new]
			ifFalse: [self add: (ClassDefinitionBackup new className: aClass name; text: oldsrc)]]! !

!Application methodsFor: 'changing-classes'!

changeOrIncludeClass: aClass 
	"definition of aClass changed in the scope of the receiver; include it with 
	definition if necessary, and make backup, include/change entries to changes"

	| bak |
	bak := self changes backupClass: aClass.
	(self includesClass: aClass)
		ifTrue: [self changes changeClass: aClass]
		ifFalse: 
			[self includeClass: aClass.
			(bak == nil or: [bak isBackup])
				ifTrue: 
					["class is new or truely changed; replace the include change"
					| chg |
					((chg := self changes last) className == aClass name
						and: [chg type == #includeOrChangeClass:])
						ifTrue: [self changes removeLast; changeClass: aClass]]]! !

!Application methodsFor: 'private'!

getMinorVersionPosition: aVersion 
	"return a file index for the beginning of sources for aVersion;
	return source file size if aVersion is not yet written"

	^InputState default shiftDown
		ifTrue: 
			[| stream pos |
			stream := self versionFileName readStream.
			pos := nil.
			[pos == nil and: [stream atEnd not]]
				whileTrue: 
					[(Integer readFrom: stream) = aVersion minor
						ifTrue: [pos := Integer readFrom: (stream skip: 1)]
						ifFalse: [stream skipThrough: Character cr]].
			stream close.
			pos]
		ifFalse: 
			[| pos key |
			key := self class sourceHeader , self name , '(' , aVersion printString , ')'.
			(pos := ChangeList new findLast: key in: self sourceFile) == nil
				ifTrue: [self sourceFile size]
				ifFalse: [pos - key size - 1]]! !

"enhanced product version access"!

!ApplicationManagement class methodsFor: 'installation'!

producerAddress
	"return the address of company Georg Heeg Objektorientierte Systeme"

	^'Georg Heeg Objektorientierte Systeme\Baroper Str. 337\44227 Dortmund' withCRs! !

!ApplicationManagement class methodsFor: 'installation'!

producerLogo
	"return the logo of company Georg Heeg Objektorientierte Systeme"

	| im shape palClass colorClass |
	palClass := Smalltalk at: #VariablePalette ifAbsent: [MappedPalette].
	colorClass := Smalltalk at: #MappedColorValue ifAbsent: [ColorValue].
	im := Image
				extent: 46 @ 36
				depth: 2
				palette: (palClass
						with: colorClass white
						with: (colorClass red: 0.6 green: 0.1 blue: 0.1)
						with: (colorClass brightness: 0.7))
				bits: (ByteArray fromPackedString: '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@UUUUT@@@@@@@@@UUP@@@ET@@@@@@@UT@@@@E@E@@@@@@UT@@@@@U@E@@@@@ET@@@@@ATDT@@@@AT@@@@@@APUP@@@@UP@@@@@AUUU@@@@AT@@@@@@@UUP@@@@EP@@@@@@ATE@@@@@U@@@@U@@EPU@@@@AT@@@ET@@EAT@@@@AP@@ATPP@UAP@@@@AT@AUQQ@ATEP@@@@@UUUPATEUUUUUUU@@EUP@EPUUUUUUUT@@@@@@U@@UAT@@@@@@@@@AT@ATEP@@@@@@@@@EP@EPU@@@@@@@@@@U@@EQU@@@@@@@@@AT@@UAT@@@@@@@@@EP@ATEP@@@@@@@@@UP@EPU@@@@@@@@@AU@@EPU@@@@@@@@@EE@@UAT@@@@@@@@@TT@ATAP@@@@@@@@APP@EPEP@@@@@@@@AQQTE@UAT@@@@@@@AUATE@TAT@@@@@@@@T@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@B**************@B**************@@@@@@@@@@@@@@@@@').
	shape := im
				convertToPalette: CoveragePalette monoMaskPalette
				renderedBy: NearestPaint new
				paintTransfer: [:pix | pix red = 1
						ifTrue: [CoverageValue transparent]
						ifFalse: [CoverageValue opaque]].
	^OpaqueImage figure: (CachedImage on: im)
		shape: (CachedImage on: shape)! !

!ApplicationManagement class methodsFor: 'installation'!

productInfo
	"open a dialog showing the product info text (product name, version, date, producer, etc)"
	"ApplicationManagement productInfo"

	| dialog addLabel |
	(dialog := DialogView model: self)
		width: 300;
		addVerticalSpace: 24;
		addVisual: self producerLogo fromX: 0 toX: 0.2;
		yPosition: 24.

	addLabel := [:string :title | | txt |
		txt := string asText emphasizeAllWith: (title ifTrue: [#(large bold)]).
		dialog addVisual: txt asComposedText fromX: 0.2 toX: 1].

	addLabel value: self productIdentifier value: true.
	dialog addVerticalSpace: 24.
	addLabel value: ' 1993-1995 by' value: false.
	dialog addVerticalSpace: 8.
	addLabel value: self producerAddress value: false.
	dialog
		addVerticalSpace: 24;
		addWrapper: (BoundedWrapper on: (Button trigger
			model: ((PluggableAdaptor on: dialog controller) performAction: #accept);
			label: 'OK')) atX: 0.5;
		addVerticalSpace: 8;
		open! !

!ApplicationManagement class methodsFor: 'installation'!

productVersion
	"return the version number of this Application Management (i.e. 1.2)"

	^VersionDescriptor major: 1 minor: 2! !

"code maintenance:
- write section headings into source file
- do not store doit change text externally"!

!Application methodsFor: 'private-fileIn/Out'!

fileOutHeading: aString on: aSourceStream 
	"store the name of a section in an application source code file 
	format: '- <section name> of <appl. name and version> -' "

	| str |
	str := ('- <section> of <id> -'
				copyReplaceAll: '<section>' with: aString)
				copyReplaceAll: '<id>' with: self identifier.
	aSourceStream cr; cr; store: str storeString! !

!Application methodsFor: 'private-fileIn/Out'!

fileOutSourcesOn: fileStream 
	self fileOutHeading: 'sources' on: fileStream.
	self fileOutClasses: self classNames on: fileStream! !

!Application methodsFor: 'private-fileIn/Out'!

fileOutDocumentationOn: aSourceStream 
	"write a message which can be sent to application management"

	documentation == nil ifTrue: [^self].
	self fileOutHeading: 'comment' on: aSourceStream.
	aSourceStream cr; cr; store: (Message selector: #documentation: arguments: (Array with: self documentation)) storeString! !

!Application methodsFor: 'private-fileIn/Out'!

fileOutRequiredOn: aSourceStream 
	"avoid doits which cannot be evaluated for users without application 
	management; return a message which can be sent to application 
	management"

	self necessary isEmpty ifTrue: [^self].
	self fileOutHeading: 'required applications' on: aSourceStream.
	aSourceStream cr; cr; store: (Message selector: #checkRequired: arguments: (Array with: self necessary asArray)) storeString! !

!Application methodsFor: 'private-fileIn/Out'!

fileOutPreInstallationOn: aSourceStream 
	"write the list of expressions which prepare the installation of the receiver"

	| list |
	(list := self preInstallation) isEmpty ifTrue: [^self].
	self fileOutHeading: 'installation' on: aSourceStream.
	list fileOutOn: aSourceStream targetFile cr cr! !

!Application methodsFor: 'private-fileIn/Out'!

fileOutPostInstallationOn: aSourceStream 
	"write the list of expressions which finalize the installation of the receiver"

	| list |
	(list := self postInstallation) isEmpty ifTrue: [^self].
	self fileOutHeading: 'post-installation' on: aSourceStream.
	list fileOutOn: aSourceStream targetFile cr cr! !

SystemApplication removeSelector: #latestMinor!


' .ApplicationSource APL-Framework(1.16)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 16 February 1995 at 11:48:13 am'!



"forgotten changes for bug fix against invalid source pointers after exclude class/category"!

!Application methodsFor: 'changing-selectors'!

excludeSelector: selector class: classOrSymbol 
	"selector is excluded from application, included into system application; 
	all recorded changes move to system application"

	(self removeSelector: selector inSubsetOf: classOrSymbol) == nil ifTrue: [^nil].
	self systemApplication
		includeSelector: selector
		class: classOrSymbol asClass
		changes: self changes.
	"enter an exclude change after removing all others"
	self changes excludeSelector: selector class: classOrSymbol.
	self touch! !

!MethodApplicationChange methodsFor: 'accessing'!

text
	"avoid doits which cannot be evaluated for users without application management;
	return a message which can be sent to application management"

	type == #exclude
		ifFalse: [self error: 'Unsupported type of MethodApplicationChange: ' , type].
	^(Message
		selector: #excludeSelector:class:
		arguments: (Array with: self selector with: self className)) storeString! !

"fix: avoid outdated application list"!

!ApplicationManagement methodsFor: 'change management'!

updateApplicationList
	"broadcast a change on the application registry"

	self changed: #version! !

!ApplicationManagement methodsFor: 'registration'!

register: anApplication ifRegistered: aBlock 
	"enter anApplication in the application holder; 
	ensure unique application names, evaluate aBlock on violation"

	(self includesNamed: anApplication name)
		ifTrue: [^aBlock value].
	applications add: anApplication.
	applications := (applications asSortedCollection: [:x :y | x name < y name]) asOrderedCollection.
	self updateApplicationList! !

!Application methodsFor: 'version management'!

version: aVersionDescription 
	self label version: aVersionDescription.
	self changed: #version.
	self applicationManagement updateApplicationList! !

!Application methodsFor: 'accessing-changes'!

setChanges: changeSequence 
	"absolute assignment to changes, keeping version consistent"

	| vers |
	(changes := changeSequence) isEmpty == (vers := self version) modified
		ifTrue: [self version: (vers perform: (changes isEmpty
					ifTrue: [#untouch]
					ifFalse: [#touch]))]
		ifFalse: ["modification state matches empty changes"]! !

"improved file-in (loading and upgrade):
- reorg. responsibility for file-in functionality (PeekableStream -> Application)
- pass version specifiers in the file to application and ApplicationManager
- evaluate section headings to catch pre/post-installation doits"!

!Application methodsFor: 'private-fileIn/Out'!

evaluateSpecial: aStringOrMessage chunk: codeString mode: aValueHolder 
	"aStringOrMessage has been encountered on a file-in stream; 
	check if it is a valid command, execute it and return its result;
	register changes with respect to source file section given by aValueHolder"

	| string section chglist |
	(string := aStringOrMessage) isString
		ifTrue:
			["check for version or source file section specifiers"
			| key |
			(string findString: (key := self class sourceHeader) startingAt: 1) == 1
				ifTrue:
					["encountering a version specifier"
					^((string := string readStream) skip: key size; upTo: $() = self name
						ifTrue:
							[aValueHolder value: 'patches'.
							VersionDescriptor readFrom: string]
						ifFalse: [nil	"source header of some other application"]].
			(string isEmpty not and: [string first == $- and: ['- * of ', self name, '* -' match: string]])
				ifTrue:
					["encountering a section heading"
					string := string readStream skip: 2; upTo: Character space.
					aValueHolder value: string.
					^string]]
		ifFalse: [(aStringOrMessage isKindOf: Message)
			ifTrue: [^self applicationManagement performMessage: aStringOrMessage]].

	"register changes with respect to current source file section"
	section := #("'comment' 'required'" 'installation' "'sources'" 'post-installation' 'patches')
				indexOf: aValueHolder value ifAbsent: [^string].
	chglist := self perform: (#(#preInstallation #postInstallation #changes) at: section).
	(codeString includes: $.)
		ifTrue: [chglist addDoIt: codeString]
		ifFalse: [ChangeScanner new scanFile: codeString readStream
					do: [:chg | chglist add: chg]].
	section <= 2 ifTrue:
		[self perform: (#(#setPreInstallation: #setPostInstallation:) at: section) with: chglist].
	^string! !

!Application methodsFor: 'private-fileIn/Out'!

fileInExternal: aStream stop: stopVersion
	"(rewritten from PeekableStream>fileIn + checking occurences of special do its) 
	file in from aStream, start at current position, new version spec should follow immediately,
	stop encountering stopVersion (end of file, if not found);
	compile w/o logging to changes file, 
	construct non-default source code pointers for generated methods;
	update version at each occurrence of version specifiers" 

	| signal evaluator val prevVers nextVers section |
	signal := self applicationManagement class sourceSignal.
	evaluator := Object evaluatorClass.
	prevVers := nil.
	nextVers := [:vers | "vers has been encountered -> previous version is complete"
			prevVers == nil ifFalse: [self version: prevVers].
			prevVers := vers].
	section := aStream -> nil.

	[aStream skipSeparators; atEnd]
		whileFalse: 
			[| scanMode pos chunk |
			scanMode := aStream peekFor: $!!.
			pos := aStream position.
			chunk := aStream nextChunk.
			val := signal
				handle: [:ex | "provide source ref for some do its (e.g. class comment)"
					ex proceedWith: (SourceReference for: self position: pos)]
				do: [evaluator evaluate: chunk logged: false].
			scanMode
				ifTrue: [val scanFrom: aStream sourceHolder: self]
				ifFalse:
					["check for special commands"
					((val := self evaluateSpecial: val chunk: chunk mode: section)
						isKindOf: VersionDescriptor)
						ifTrue: [stopVersion = (nextVers value: val) ifTrue: [^val]]]
			].
	nextVers value: prevVers.
	^val! !

!Application methodsFor: 'private-fileIn/Out'!

fileInFrom: aStream stop: stopVersion external: aBoolean 
	"if aBoolean is true perform a special file-in (see #fileInExternal:stop:);
	otherwise file in up to end of file (rewritten from PeekableStream>fileIn)"

	| val signal compiler |
	aBoolean ifTrue: [^self fileInExternal: aStream stop: stopVersion].
	signal := self applicationManagement class sourceSignal.
	compiler := Object evaluatorClass.
	[aStream skipSeparators; atEnd]
		whileFalse: 
			[| scanMode |
			scanMode := aStream peekFor: $!!.
			val := signal
					handle: [:ex | 
						"handle here to inhibit handler search higher in the stack
						(further handlers can be found during nested file-in)"
						ex proceed]
					do: [compiler evaluate: aStream nextChunk logged: scanMode not].
			scanMode ifTrue: [val scanFrom: aStream]].
	^val! !

!Application methodsFor: 'fileIn/Out'!

fileInFrom: aStream start: startVersion stop: stopVersion external: aBoolean
	"file in from aStream: 
	start at first occurence of startVersion (current position if nil), 
	stop at occurence of stopVersion (or at end of stream); 
	changes are logged by the receiver; 
	keep source pointers to aStream (holder for the file is the receiver, 
	so take care that aStream is a stream on the receivers source file)"

	aBoolean ifTrue: [self path == nil
			ifTrue: [self label path: aStream name]
			ifFalse: [self instSourceFilename asString = aStream name
				ifFalse: [self notify: 'Attempt to assign a second source file']]].
	startVersion == nil
		ifFalse: [aStream position: (self getMinorVersionPosition: startVersion)].

	^self applicationManagement logTo: self while:
		[Cursor read showWhile:
			[[self fileInFrom: aStream stop: stopVersion external: aBoolean]
				valueNowOrOnUnwindDo: [aStream close]]]! !

!Application methodsFor: 'version management'!

upgradeMinorUpTo: aVersion
	"file in all the new sources in the receiver's source file:
	- start with the receiver's following minor version
	- stop at the beginning of aVersion (file in all if argument is nil)"

	| oldChanges newChanges chgList |
	oldChanges := self changes.
	self noChanges.
	"use a private copy of the source file accessor"
	self 
		fileInFrom: self instSourceFilename readStream
		start: self version newMinor
		stop: (aVersion == nil ifFalse: [aVersion newMinor])
		external: true.

	oldChanges isEmpty ifFalse:
		[oldChanges
			addDoIt: '"the following changes result from upgrade"';
			addAll: (newChanges := self changes)].
	self clearChanges.	"move upgrade changes to history"

	"user interaction for unsaved temporary changes"
	oldChanges isEmpty ifTrue: [^self].
	chgList := (EnhancedChangeList onApplication: self)
			listName: 'Changes conflicts in upgrading ' , self identifier storeString 
			changes: oldChanges
			removed: newChanges.

	DialogView userInfo: 
'The temporary changes have been removed from application.
Select "remove selection" for those you want to forget,
select "reinstall" for those which have been overwritten,
select "accept" to assign the contents of the list to the application.'.
 
	ChangeSequenceView openOn: chgList! !

!Application methodsFor: 'version management'!

upgradeMinor
	"file in all new sources in the receiver's source file: 
	start with the receiver's following minor version, stop at end of file"

	self hasUpgrade ifFalse: [^self].	"nothing to upgrade"
	self upgradeMinorUpTo: nil! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

upgradeMinor: anApplicationOrNil 
	"upgrade the given application to a higher minor version;
	if argument is nil show applications with new versions and request from user"

	| app latest version |
	(app := anApplicationOrNil) == nil 
		ifTrue: [app := self upgradeApplications select: 'Select an application to upgrade' fromUserIfEmpty: [^nil]].
	(latest := app latestMinor) <= app version
		ifTrue: [^DialogView userInfo: 'No upgrade available'].
	version := DialogView
				request: 'Upgrade application ''', app identifier, '''\up to minor version' withCRs
				initialAnswer: latest minor printString
				ifEmpty: [^self].
	(version := Integer readFromString: version) <= app minorVersion
		ifTrue: [^self].
	
	app upgradeMinorUpTo: (version >= latest minor ifFalse: [VersionDescriptor major: latest major minor: version])! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInFrom: aStream to: anApplication external: aBoolean 
	"file in source code; 
	if 'aBoolean' is true, all changes will directly be assigned to 'anApplication'
	and the source code pointers will be directed to the source file just read;
	otherwise changes will pass through system change set and reach 'anApplication' "

	| result |
	result := anApplication fileInFrom: aStream start: nil stop: nil external: aBoolean.
	self updateApplicationList.
	^result! !

Application removeSelector: #fileInFromStream:startAt:stopAt:external:!

PeekableStream removeSelector: #fileInFrom:to:sources:!

PeekableStream removeSelector: #fileInUpTo:sources:!

"bug fixes:
- unh.ex. ERROR_SHARING_VIOLATION during minor versioning (DOS only)
- mnu #isEmpty during major version with rename
- invalid application browser models after major version with rename
- request platform names on major version with rename
- improper installation ordering for multiply required applications

modifications:
- save the confirm dialog during major versioning"!

!Application methodsFor: 'version management'!

newMinorVersion
	"increment the minor version number; 
	save current changes to source file and history, redirect source code pointers;
	broadcast the change of version to dependent browsers;
	return nil if any error or cancel from user occurs"

	| latest file |
	self version isInitialVersion ifTrue:
		[DialogView userError: 'Minor versions are not allowed\before the first major version release' withCRs. ^nil].

	self changes isEmpty ifTrue:
		[(DialogView confirm: 'There are no changes.\Do you want a new minor version anyway?' withCRs initialAnswer: false)
			ifFalse: [^nil]].

	(latest := self latestMinor) > self version ifTrue:
		[DialogView userError: ('This application has already been released to version\', latest printString, '\Try to upgrade and make your release afterwards') withCRs. ^nil].

	latest < self version copy untouch
		ifTrue: [(DialogView confirm: ('This application has not been released since version\', latest printString, '\Do you want to proceed?') withCRs initialAnswer: false)
			ifFalse: [^nil]].

	self releaseSourceFile.	"avoid problems with SHARE.EXE"
	file := OSErrorHolder noPermissionsSignal
		handle: [:ex |
			DialogView warn: ex signal errorString, '.\You are not allowed to create the new minor version' withCRs. ^nil]
		do: [self instSourceFilename makeWritable].

	[self fixChanges: true] valueNowOrOnUnwindDo: [file makeUnwritable]! !

!Application methodsFor: 'version management'!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp newName newMajor major |
	"- assign a new name ---------------------------"
	newName := DialogView
			request: 'Enter the name of the new major version of\''' withCRs, self identifier , ''''
			initialAnswer: self name
			ifEmpty: [^self].
	newApp := self cleanUp; copy.
	self name = newName
		ifFalse: [(DialogView confirm: 'You entered ''', newName, '''\as new application name.\Is this correct?' withCRs)
			ifTrue:
				[newApp label name: newName.
				newApp requestPlatformNamesOnCancel: [DialogView userInfo: 'Operation cancelled'. ^self]]
			ifFalse: [^self]].

	"- assign a new path ---------------------------"
	(newApp requestPath: #path) == nil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	"- assign a new major version number ----------------"
	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) > 0
		ifFalse: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor - major > 1
		ifTrue: [(DialogView confirm: 'The major number increment is greater than 1.\Do you want to proceed?' withCRs initialAnswer: false)
			ifFalse: [^self]]
		ifFalse: [newMajor <= major
			ifTrue: [newApp directory = self directory
				ifTrue: [DialogView userError: 'You must specify a new path\if an old major number is used.' withCRs. ^self]]].
	newApp version: (VersionDescriptor major: newMajor - 1 minor: 0).

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	"- basic storing sequence ---------------------------"
	newApp incrementMajorVersion; storeSource; storePools; noChanges.
	self applicationManagement unregister: self; register: newApp.
	self asBackup; changed: #application with: newApp.
	newApp storeBinary.
	^newApp! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileInAsApplication: aFilename 
	"file in source code from a file named by the given filename object;
	the first chunk must be an application source header holding the application name;
 	redirect the source code pointers to the source file"

	| stream header appName appVers app old pathAndName |

	header := String readFrom: (stream := aFilename readStream).
	stream nextChunk; close. 
	(Application sourceHeader , '*(*.*)' match: header)
		ifFalse: [^self error: aFilename asString, '\is not an application source file' withCRs].
	(header := header readStream) skip: Application sourceHeader size.
	appName := header upTo: $(.
	appVers := VersionDescriptor readFrom: header.

	(old := self applicationNamed: appName) == nil
		ifFalse: [(DialogView confirm: ('Do you want to overwrite\''', old identifier, ''' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['\(changes will get lost)'])) withCRs initialAnswer: false)
			ifTrue: [self unregister: old]
			ifFalse: [Transcript show: 'aborted '. ^nil]].

	(app := self newApplicationNamed: appName) == nil
		ifTrue: [^nil].

		"an application source path includes a basic path,
		followed by a major and a minor directory and the source file name;
		strip off the last three ones"
	pathAndName := aFilename head asFilename head asFilename.
	app label
		path: pathAndName head;
		setFilename: pathAndName tail forPlatform: OSHandle currentOS.
	app version: appVers.

	self do: [self fileInFrom: stream to: app external: true]
		onCancel:
			[self unregister: app.
			old == nil ifFalse: [self register: old].
			Transcript show: 'aborted '. ^nil].
	app setChanges: (app changes select: [:chg | chg isBackup]); clearChanges.

	old == nil ifFalse:
		[old changed: #application with: app.
		self browseConflicts: (app checkConflictsWith: old)].
	^app! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

load: application from: aBOS path: path 
	"load the application (structure with class & method definitions) from aBOS;
	assign the given path after installation"

	| app old conflicts |
	Transcript cr; show: 'loading ' , application asString, ' (structure)... '.
	aBOS resizeMaps: aBOS nextIndex // 4 * 3 // 2.	"avoid growing of the bos' maps"
	app := Cursor read showWhile: [[aBOS next] valueNowOrOnUnwindDo: [aBOS close]].

	(old := self applicationNamed: app name) == nil
		ifFalse: [(DialogView confirm: ('Do you want to overwrite\''', old identifier, ''' ?', (old changes isEmpty ifTrue: [''] ifFalse: ['\(changes will get lost)'])) withCRs initialAnswer: false)
			ifTrue:
				[conflicts := app checkConflictsWith: old.
				self unregister: old]
			ifFalse: [Transcript show: 'aborted '. ^nil]].

	"set the correct path"
	app label path: path.
	Transcript show: '(checks)... '.
	self do: [self logTo: app while: [self checkRequired: app necessary]]
		onCancel:
			[old == nil ifFalse: [self register: old].
			Transcript show: 'aborted '. ^nil].

	[Transcript show: '(pre-installation)... '.
	app preInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	Transcript show: '(source)... '.
	self logTo: app while: [Cursor read showWhile: [aBOS sourceMode: #reference; nextClasses; close]].

	Transcript show: '(post-installation)... '.
	app postInstallation do: [:chg | Compiler evaluate: chg text logged: true].

	app setChanges: (app changes select: [:chg | chg isBackup]); clearChanges; definePoolDictionaries.
	Transcript show: 'done ']

		valueNowOrOnUnwindDo: [self register: app].

	old == nil ifFalse:
		[old changed: #application with: app.
		self browseConflicts: conflicts].
	^app! !

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallation: anApplication on: instFilename path: dirFilename platform: aString 
	"create a file <instFilename> (if argument is not nil) with code to install 
	anApplication and (recursively) all its sub-applications; if aString is 
	- nil : do not copy the sources, use application paths during installation 
	- platform specifier : copy the application directory tree to <dirFilename>, 
		obeying platform characteristics, loading will search the path of the 
		installation file at execution time"

	| required drop apps abort |
	required := (apps := anApplication necessary) isEmpty
		ifTrue: [apps]
		ifFalse: [ApplicationBrowser multipleSelect: apps prompt: 'include applications' onCancel: [^self]].

	abort := [:msg | (DialogView confirm: (msg , '\Proceed without it?') withCRs)
				ifFalse: [^self]].
	apps := apps collect: [:appName | self applicationNamed: appName
				ifAbsent: [abort value: 'Application ''' , appName , ''' is not loaded.']].
	drop := apps removeAllSuchThat: [:app | app == nil or: [(required includes: app name) not]].
	required := apps.

	"recursively collect required applications; sort wrt. sub-application property"
	(apps := Dictionary new: required size * 4) at: anApplication put: 0.
	required do: [:app | apps at: app put: 1].
	Object notifySignal handle: [:ex | abort value: ex errorString]
		do: [required do: [:app | app
				subApplicationsDo: [:ap :lev | apps at: ap put: (lev max: (apps at: ap ifAbsent: [lev]))]
				level: 2
				skip: drop]].
	apps := (apps associations asSortedCollection: [:x :y | x value > y value])
				collect: [:assoc | assoc key].

	aString == nil ifFalse: ["make sure that the target platform filenames are defined"
		apps do: [:app | app label filenameForPlatform: aString ifNone: [app
					requestPlatformNamesOnCancel: 
						[DialogView userInfo: 'Installation file out aborted.'.
						^nil]]]].

	Cursor write showWhile: 
		[instFilename == nil ifFalse:
			[(self fileOutInstallScript: apps platform: aString on: instFilename) == nil ifTrue: [^self]].
		aString == nil ifTrue: [^self].
		Transcript cr; show: 'copying sources...'.
		apps do: 
			[:app | 
			Transcript crtab; show: app name , '...'.
			((app exportForPlatform: aString destinationPath: dirFilename) == nil 
				and: [DialogView confirm: 'Do you want to abort the installation file out?'])
				ifTrue: [^self]].
		Transcript cr; show: 'done.']! !

"code maintenance"!

!Application methodsFor: 'version management'!

latestMinor
	"return a VersionDescriptor for the latest version string in the source file"

	^(self version isInitialVersion or: [self versionFileName exists not])
		ifTrue: [self version]
		ifFalse: 
			[| pos |
			pos := ChangeList new findLast: (self class sourceHeader , self name copyWith: $() in: self sourceFile.
			VersionDescriptor readFrom: (self sourceFile position: pos)]! !

!Application methodsFor: 'testing'!

hasUpgrade
	"return true if there are new sources since the last file in"

	^self latestMinor > self version! !

!Application methodsFor: 'private'!

touch
	"mark that a change occurred"

	self version touch! !

!ApplicationManagement methodsFor: 'logging on/off/redirect'!

logTo: anApplication while: aBlock 
	"execute aBlock (typically a file in), log changes into anApplication;
	logging to system application is equivalent to 'logTo==nil' "

	| save logApp |
	logApp := anApplication == system ifFalse: [anApplication].
	(save := logTo) == logApp	"nothing changed"
		ifTrue: [^aBlock value].

	Transcript cr; nextPutAll: 'Logging to '; print: anApplication; space; endEntry.
	logTo := logApp.
	^aBlock valueNowOrOnUnwindDo:
		[logTo := save.
		Transcript cr; nextPutAll: 'Logging back to '; print: (save == nil ifTrue: [system] ifFalse: [save]); space; endEntry]! !

ApplicationManagement removeSelector: #resetLogging!

ApplicationManagement class removeSelector: #findBackupsFor:!


' .ApplicationSource APL-Framework(1.17)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 16 February 1995 at 3:04:12 pm'!



!Application methodsFor: 'private'!

incrementMinorVersion
	self version: self version newMinor! !

"no dialog on binary storage of foreign applications"!

!ApplicationBOSSWriter methodsFor: 'private-tracing'!

traceApplication: anApplication 
	"if the argument is the main application currently being stored then store as 
	usual (recursively); otherwise return an expression which requests the 
	argument from existing applications"

	^anApplication == application
		ifTrue: [self traceDefault: anApplication]
		ifFalse: 
			[| expr |
			Transcript nextPutAll: ' (reference on '''; print: anApplication; show: ''')'.
			expr := 'self applicationManagement applicationNamed: % ifAbsent: [%]'
				copyReplaceAll: '%' with: anApplication name storeString.
			MessageSend
				receiver: Object evaluatorClass
				selector: #evaluate:
				arguments: (Array with: expr)]! !


' .ApplicationSource APL-Framework(1.18)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 3 March 1995 at 5:36:35 pm'!



"feed installation scripts with better parameters"!

!ApplicationManagement methodsFor: 'fileIn/Out'!

fileOutInstallScript: sortedApplications platform: aString on: aFilename
	"write out code which installs applications in given order; if aString is nil,
	loading will search the path of the installation file at execution time;
	otherwise the paths of the applications will be used"

	| export specs stamp template |
	export := aString notNil.
	specs := sortedApplications collect: [:app | Array
				with: app name
				with: app majorVersion
				with: (export ifFalse: [app path asString])
				with: (export ifTrue: [app label filenameForPlatform: aString] ifFalse: [app fileString])].
	self timeStamp: (stamp := WriteStream on: (String new: 100)).

	template := (self applicationNamed: 'APL-Framework') minorDirectory / 'insttmpl.txt'.
	template := OSErrorHolder errorSignal
		handle:
			[:ex | 
			DialogView warn: 'Could not read template file ''insttmpl.txt''\in path of ''APL-Framework''\(' withCRs, ex errorString, ')'.
			^nil]
		do: [template asFilename contentsOfEntireFile].

	template := ((template copyReplaceAll: '$APPID' with: sortedApplications last identifier)
				copyReplaceAll: '$TIMESTAMP' with: stamp contents)
				copyReplaceAll: '$APPLICATIONSPECS' with: specs asArray printString.
	aFilename writeStream nextPutAll: template; close! !


' .ApplicationSource APL-Framework(1.19)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 10 April 1995 at 4:53:36 pm'!



"bug fix:
- major versioning sometimes left old application references
	in changes of other applications (e.g. major versions with rename)
- class renames have to be propagated immediately to all change sequences"!

!ApplicationManagement methodsFor: 'change management'!

updateChangesForMajor: anApplication previous: prevApplication 
	"prevApplication has been saved by a new major version (anApplication); 
	update source code references held by other applications (usually backups)"

	| hist |
	hist := anApplication history.
	applications do: [:app | app changes removeAllSuchThat:
		[:chg | | srcref |
		(srcref := chg file) class == SourceReference
			and: [srcref sourceFileHolder == prevApplication
			and: 
				[| txt chg2 |
				txt := chg text.
				(chg2 := hist findLast: [:ch | ch text = txt]) = 0
					or: 
						[chg file: (hist at: chg2) file copy position: nil.
						false]]]]]! !

!ApplicationManagement methodsFor: 'version management'!

newMajorVersion: anApplication previous: prevApplication
	"prevApplication has been saved by a new major version (anApplication); 
	update references to the old version"

	self
		unregister: prevApplication;
		register: anApplication;
		updateChangesForMajor: anApplication previous: prevApplication! !

!Application methodsFor: 'version management'!

newMajorVersion
	"create (and return) a new application, leave the old one for history references;
	create a subdirectory structure for storing a new major version,
	make a full store of all the associated stuff (e.g. sources etc.);
	forget changes since last minor version release, they won't get
	reliable source references"

	| newApp newName newMajor major |
	"- assign a new name ---------------------------"
	newName := DialogView
			request: 'Enter the name of the new major version of\''' withCRs, self identifier , ''''
			initialAnswer: self name
			ifEmpty: [^self].
	newApp := self cleanUp; copy.
	self name = newName
		ifFalse: [(DialogView confirm: 'You entered ''', newName, '''\as new application name.\Is this correct?' withCRs)
			ifTrue:
				[newApp label name: newName.
				newApp requestPlatformNamesOnCancel: [DialogView userInfo: 'Operation cancelled'. ^self]]
			ifFalse: [^self]].

	"- assign a new path ---------------------------"
	(newApp requestPath: #path) == nil
		ifTrue: [DialogView userInfo: 'Operation cancelled'. ^self].

	"- assign a new major version number ----------------"
	newMajor := DialogView
				request: 'New major version?'
				initialAnswer: ((major := self majorVersion) + 1) printString
				ifEmpty: [^self].
	(newMajor := Integer readFromString: newMajor) > 0
		ifFalse: [DialogView userError: 'Major numbers must be greater than zero'. ^self].

	newMajor - major > 1
		ifTrue: [(DialogView confirm: 'The major number increment is greater than 1.\Do you want to proceed?' withCRs initialAnswer: false)
			ifFalse: [^self]]
		ifFalse: [newMajor <= major
			ifTrue: [newApp directory = self directory
				ifTrue: [DialogView userError: 'You must specify a new path\if an old major number is used.' withCRs. ^self]]].
	newApp version: (VersionDescriptor major: newMajor - 1 minor: 0).

	newApp history isEmpty
		ifFalse: [(DialogView confirm: 'Do you want to the history be forgotten?')
			ifTrue: [newApp setHistory: ChangeSequence new]].

	"- basic storing sequence ---------------------------"
	newApp incrementMajorVersion; storeSource; storePools; noChanges.
	self applicationManagement newMajorVersion: newApp previous: self.
	self asBackup; changed: #application with: newApp.
	newApp storeBinary.
	^newApp! !

!ApplicationManagement methodsFor: 'change management'!

renameClass: class as: newName 
	"inform applications that class is renamed"

	logging ifFalse: [^self].
	applications do: [:app | app renameClass: class as: newName]! !

!Application methodsFor: 'changing-classes'!

renameClass: aClass as: newName 
	"rename aClass' subset and reenter it under the new key; 
	add a rename change if the receiver defines aClass"

	| rename subset metaName |
	rename := false.
	(subset := self removeClassSubset: aClass) isNil
		ifFalse: 
			[subset renameTo: newName.
			classes at: newName asSymbol put: subset.
			rename := subset includesDefinition].
	(subset := self removeClassSubset: aClass class) isNil
		ifFalse: 
			[metaName := (newName , ' class') asSymbol.
			subset renameTo: metaName.
			classes at: metaName put: subset.
			rename := rename | subset includesDefinition].
	rename
		ifTrue: 
			[self changes renameClass: aClass name to: newName.
			self touch]
		ifFalse: [self changes renameAllClass: aClass name to: newName]! !

"code maintenance"!

Application removeSelector: #sourceIdentifierForVersion:!

ApplicationManagement removeSelector: #changedApplications!

"ensure that binary storage does not use source code references pointing to the changes file"!

!BOSSWriter methodsFor: 'representation'!

representCompiledMethod: aCompiledMethod 
	"This is the representation method for compiled code objects, invoked from 
	CompiledMethod>>representBinaryOn:. It manages the saving and restoring 
	of the source code or source pointer."

	| mth src cls sel cat |
	src := (mth := aCompiledMethod shallowCopy) sourcePointer.
	mth sourcePointer: nil.
	sel := currentClass selectorAtMethod: aCompiledMethod setClass: [:c] ifAbsent: [].
	cls := aCompiledMethod mclass.
	cat := Object errorSignal handle: [:ex | ex return]
				do: [cls whichCategoryIncludesSelector: sel].
	src == nil ifFalse: [useSource == #keep
		ifTrue: [src := aCompiledMethod getSource]
		ifFalse: [useSource == #reference
			ifTrue: [(SourceFileManager default fileIndexFor: src) = 2
				ifTrue:
					["the changes file is not safe for reference transfer"
					self notify: ('Invalid source code reference\for %>%'
								withArgs: (Array with: cls name with: sel)).
					src := nil]]
			ifFalse: [src := nil]]].
	^BinaryObjectStorage indexImportCompiledCode -> (BOSSCompiledCodeHolder new
			class: cls
			selector: sel
			category: cat
			codeClass: aCompiledMethod class
			codeContents: (mth changeClassToThatOf: #())
			source: src)! !


' .ApplicationSource APL-Framework(1.20)'!
'From VisualWorks(R) Release 2.0 of 4 August 1994 on 21 April 1995 at 3:17:25 pm'!



"copy latest major version only during installation export"!

!Application methodsFor: 'fileIn/Out'!

exportForPlatform: aString destinationPath: dirFilename 
	"copy the directory of the receiver (without old major versions) 
	to directory dirFilename; return nil if errors occur (self otherwise)"

	| src dest maj srcfile |
	(src := self directory) = (dest := (dirFilename / (self filenameForPlatform: aString)) asFilename)
		ifTrue: 
			[DialogView userError: ('''%''\is invalid as export directory.' withArgs: dirFilename).
			^nil].
	OSErrorHolder existingReferentSignal handle: [:ignore] do: [dest makeDirectory].
	src := src / (maj := 'v' , self majorVersion printString).
	dest := dest / maj.
	srcfile := (dest / 's.st') asFilename.
	"careful if some previous export created a read-only file"
	OSErrorHolder errorSignal handle: [:ignore] do: [srcfile makeWritable].
	[src asFilename copyRecursiveTo: dest]
		valueNowOrOnUnwindDo:
			[OSErrorHolder errorSignal handle: [:ignore] do: [srcfile makeUnwritable]]! !

"bug fix: mnu ClassDefinitionChange>type during class redefinition"!

!Application methodsFor: 'changing-classes'!

changeOrIncludeClass: aClass 
	"definition of aClass changed in the scope of the receiver; include it with 
	definition if necessary, and make backup, include/change entries to changes"

	| bak |
	bak := self changes backupClass: aClass.
	(self includesClass: aClass)
		ifTrue: [self changes changeClass: aClass]
		ifFalse: 
			[self includeClass: aClass.
			(bak == nil or: [bak isBackup])
				ifTrue: 
					["class is new or truely changed; replace the include change"
					| chg |
					((chg := self changes last) className == aClass name
						and: [chg class == ClassApplicationChange])
						ifTrue: [self changes removeLast; changeClass: aClass]]]! !